!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


!==============================================================================
!
!  Name   :  Biomass
!
!  Remarks:  contains the subroutine
!
!            Biomass   (<GetStd<TreeMig)
!
!==============================================================================


SUBROUTINE Biomass(dbh, a1, a2, c1, bm)

!<Passed variables>--------------------------------------------------------------------------------
REAL,INTENT(IN)  :: dbh, &  ! tmp var to calculate diameters                       [=diam/nextDiam]
                    a1,  &  ! Parameters for LA calcs                                 [=a1(sTypeN)]
                    a2,  &  ! Parameters for LA calcs                                 [=a2(sTypeN)]
                    c1      ! Parameters for biomass calcs                            [=c1(sTypeN)]
REAL,INTENT(OUT) :: bm      ! calculated biomas                              [=BioMassAtHeight(..)]

!<Local variables>---------------------------------------------------------------------------------
REAL :: lndiam, &     ! tmp var to calculate LA                                       [=rLnDiam]
        stmvol, &     ! Stem volume             	                             [=stemvol]
        stmwgt, &     ! Stem weight                                                     [=stemW]
        folwgt        ! Foliage weight                                                   [=folW]

!<Here the subroutine starts>**********************************************************************
  IF (dbh .le. 0.0) THEN
    lndiam = 0.0
    folwgt = 0.0
    stmvol = 0.0
  ELSE
    lndiam = LOG (dbh * 100.0)
    folwgt = c1 * a1 * EXP (lndiam * a2)
    stmvol = 0.12 * EXP (lndiam * 2.4)
  END IF

   !NZ: if output should only be stem volume (timber harvesting), comment next 2 lines out
   ! and set bm = stmvol instead
  stmwgt = 0.5 * stmvol
  bm = stmwgt + folwgt

END SUBROUTINE Biomass
!****<Here the subroutine ends>********************************************************************
