!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------



!==============================================================================
!
!  Name   :  ChestnutCult
!
!  Purpose:   calculones the calculonion of chestnuts in Ticino
!
!  Remarks:  contains the subroutine
!
!            ChestnutCult (LocalForestDynamics<DoLocalWork<DistrLocalDynThisTime<TimeLoop<TreeMig)
!
!==============================================================================

SUBROUTINE ChestnutCult( lat, lon, year, idsp,thiscell)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: maxspc, maxhc, maxlc, slow, &
                   ltfrq,  htcl, ltcl,   &
                   spec, stockability,    &
                   disturbances, disturbProb, disturbIntensity ,&
                   CurrStateInCell,&
                   latrealstart, lonrealstart, simustartyear         ! Use control parameters
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN) ::  lat, lon, year     ! Passed number of species                   [=spec]

Logical, INTENT(IN) :: idsp

!<Local variables>---------------------------------------------------------------------------------
REAL ::  randomvalue,               & ! random value for disturbance calculonion
        cult_startyear, cult_endyear, cult_area_lonmin, cult_area_lonmax,&
              cult_area_latmin, cult_area_latmax

TYPE(CurrStateInCell):: thiscell
  cult_startyear=  1800-simustartyear
  cult_endyear  =  1950-simustartyear
  cult_area_latmin= (-125000.-latrealstart)/200. +1
  cult_area_latmax= (-45000.-latrealstart)/200. +1
  cult_area_lonmin= (80000.-lonrealstart)/200. +1
  cult_area_lonmax= (140000.-lonrealstart)/200. +1

   !  Start Chestnut cultivation for randomly selected cells in Ticino
  IF (year.lt.cult_startyear)thiscell%chestnutcult=.FALSE.
  IF (year.eq.cult_startyear  &
        .and.lat.gt.cult_area_latmin .and. lat .lt. cult_area_latmax &
        .and.lon.gt.cult_area_lonmin .and. lon .lt. cult_area_lonmax &
        .and. thiscell%ddegs>1200. ) THEN
      CALL RANDOM_NUMBER (randomvalue)
      IF (randomvalue .le. 0.333)  thiscell%chestnutcult=.TRUE.
!      print *, 'start chestnutcult',year,lat,lon,  thiscell%chestnutcult,cult_area_latmin, &
!             cult_area_latmax,cult_area_lonmin, cult_area_lonmax
  END IF
  IF (year.ge.cult_endyear )  thiscell%chestnutcult=.FALSE.
 END SUBROUTINE ChestnutCult
!****<Here the subroutine ends>********************************************************************
