!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!==============================================================================
!  Name   : ChilldaysAndGDD5
!
!  Auxiliary subroutine for updating the temperature indices used in the leaf
!  phenology module (LeafPhenology.f90).
!==============================================================================

SUBROUTINE ChilldaysAndGDD5(temp,chd,gdd)

use prevahglobals
use forhycsglobals, only: gdd5,chilldays,dtemp31,mtemp

implicit none

REAL :: temp, gdd
INTEGER :: chd

REAL :: mtemp_last
INTEGER :: d

!In midwinter, reset GDD counter for summergreen phenology (northern hemisphere)
IF ((TAG.eq.15).and.(MON.eq.1)) gdd5(xcoor,ycoor) = 0

!Calculate growing degree days
gdd5(xcoor,ycoor) = MAX(0.0,temp-5.0) + gdd5(xcoor,ycoor)

!Update chilldays
IF (temp.lt.5.0.and.chilldays(xcoor,ycoor).lt.365)THEN
	chilldays(xcoor,ycoor) = chilldays(xcoor,ycoor)+1
END IF

! Save yesterday's mean temperature for the last month
mtemp_last= mtemp(xcoor,ycoor)

! Update daily temperatures and mean overall temperature, for last 31 days
mtemp(xcoor,ycoor) =temp
DO d= 1, 30
	dtemp31(xcoor,ycoor,d) = dtemp31(xcoor,ycoor,d+1);
	mtemp(xcoor,ycoor) = dtemp31(xcoor,ycoor,d) + mtemp(xcoor,ycoor)
END DO

dtemp31(xcoor,ycoor,31)=temp
mtemp(xcoor,ycoor) = mtemp(xcoor,ycoor)/31.0

!Reset chilldays and growing degree days below mean temperature of last 31 days below limit of 5 celsius degrees
IF(mtemp_last.ge.5.0 .and.mtemp(xcoor,ycoor).lt.5.0) THEN
	IF (MON .gt. 7) gdd5(xcoor,ycoor)=0.0
	IF (MON .gt. 7) chilldays(xcoor,ycoor)=0.0    ! possibly resetting only once per year
END IF

 gdd=gdd5(xcoor,ycoor)
 chd=chilldays(xcoor,ycoor)

END SUBROUTINE ChilldaysAndGDD5
