!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


!==============================================================================
!
!  Name   :  EvaluateResultFuntions
!
!  Remarks:  contains the subroutines + functions
!
!       (F)  BiodivIndex (<EvaluateResults<DoLocalWork<DistrLocalDynThisTime<
!                           TimeLoop<TreeMig)
!            FindDominantSpecies (<EvaluateResults<DoLocalWork<DistrLocalDynThisTime<
!                                   TimeLoop<TreeMig)
!
!==============================================================================


REAL FUNCTION BiodivIndex(nspc, bio, sumbio)

!<CALL modules for type definitions>---------------------------------------------------------------
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN) :: nspc          ! Passed number of species
REAL, INTENT(IN)    :: bio(nspc), &  ! species biomass
                       sumbio

!<Local variables>---------------------------------------------------------------------------------
INTEGER :: i                         ! loop-counter
REAL    :: diversity, rnspc, value

!<Here the subroutine starts>**********************************************************************
  !----- Calculation of diversity per species
  rnspc     = nspc
  diversity = 0.0

  IF (ABS (sumbio) < 1.E-20) THEN
    diversity = 0.0
  ELSE
    BySpc2: DO i = 1, nspc
       value = bio(i) / sumbio
       IF (value .gt. 1.E-5 ) THEN
         value     = -value * LOG (value)
         diversity = diversity + value / LOG (rnspc)
       END IF
    END DO BySpc2
  END IF

  BiodivIndex = diversity

END FUNCTION BiodivIndex
!****<Here the subroutine ends>********************************************************************




SUBROUTINE FindDominantSpecies(nspc, nonzeroSpecs, domspec)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: spec

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN)  :: nspc           ! Passed number of species
INTEGER, INTENT(OUT) :: nonzeroSpecs, &
                        domspec(3)

!<Local variables>---------------------------------------------------------------------------------
INTEGER :: i,j
REAL    :: bio, biomax, biomaxmax, sumbio

!<Here the subroutine starts>**********************************************************************
  domspec      = 0
  nonzeroSpecs = 0
  biomax       = 0.1
  biomaxmax    = 1.E20
  sumbio       = 0

  DO i = 1, nspc
    sumbio = sumbio + spec(i)%sumtimebio
  END DO

  IF (ABS (sumbio) < 1.E-20) RETURN

  DO i = 1, nspc
    IF ((spec(i)%sumtimebio / sumbio) > 0.01) nonzeroSpecs = nonzeroSpecs + 1
  END DO

  DO j = 1, Min (nonzeroSpecs, 3)
    DO i = 1 , nspc
      bio = spec(i)%sumtimebio
      IF ((bio > biomax) .AND. (bio < biomaxmax)) THEN
        domspec(j) = i
        biomax     = bio
      END IF
    END DO

    biomaxmax = biomax
    biomax    = 0.1
  END DO

END SUBROUTINE FindDominantSpecies
!****<Here the subroutine ends>********************************************************************
