!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!==============================================================================
!
!  Name   :  EvaluateResults
!
!  Remarks:  contains the subroutine
!
!            EvaluateResults (<DoLocalWork<DistrLocalDynThisTime<TimeLoop<TreeMig)
!
!==============================================================================


SUBROUTINE EvaluateResults(nspc, year, lat, lon)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: cell, spec, spc,   tabsepOutput, pltsiz, writeOutput,                    &
                   latEvalCell, lonEvalCell, numyrs,                                                  &
                   avdiversity80, avdiversity160, avdiversity240, avdiversity500, avdiversityEq,      &
                   nonzeroSpecs80, nonzeroSpecs160, nonzeroSpecs240, nonzeroSpecs500, nonzeroSpecsEq, &
                   domspec80, domspec160, domspec240, domspec500, domspecEq, sumDiversity,            &
                   biocrit, specAppearance, specMigSpeed              !----- for appearance
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN) :: nspc,  &   ! Passed number of species
                       year,  &   ! year of calculation [=..]
                       lat,   &
                       lon

!<Local variables>---------------------------------------------------------------------------------
INTEGER :: height, i, ibiocrit  ! loop-counters
REAL    :: bio(nspc),    &      ! species biomass
           sumbio,       &
           BiodivIndex


!<Here the subroutine starts>**********************************************************************
!----- for appearance
  IF (year .eq. 0) THEN
    specAppearance = -999.0
!    specMigSpeed   = -999.0
    biocrit(1)     = 0.0001
    biocrit(2)     = 0.001
    biocrit(3)     = 0.01
    biocrit(4)     = 0.1
    biocrit(5)     = 0.5
    biocrit(6)     = 1.0
    biocrit(7)     = 5.
    biocrit(8)     = 10.
    biocrit(9)     = 20.
    biocrit(10)    = 50.
  END IF

  DO i = 1 , nspc
    bio(i) = 0

    DO height = 1, spec(i)%maxhtc
      bio(i) = bio(i) + (cell(lat,lon)%sp(i)%numin(height)) * (spc(i)%bioin(height))
    END DO

    DO ibiocrit = 1, 10
      IF ((specAppearance(ibiocrit, i, lat, lon) .eq. -999.0) .AND. &
          (bio(i) .ge. biocrit(ibiocrit))) THEN
        specAppearance(ibiocrit,i,lat,lon) = year
      END IF
    END DO !----- ibiocrit
  END DO !----- i=1,nspc

  IF ((lat == latEvalCell) .AND. (lon == lonEvalCell)) THEN
    sumbio = 0
    !----- Calculate biomass and density sums over species in heights and total
    BySpc: DO i = 1, nspc
      sumbio             = sumbio + bio(i)
      spec(i)%sumtimebio = spec(i)%sumtimebio + bio(i)
    END DO BySpc
    sumDiversity = sumDiversity + BiodivIndex(nspc, bio, sumbio)

    IF ((year .eq. 0)) THEN
      sumDiversity    = 0
      avdiversity80   = 0.0
      avdiversity160  = 0.0
      avdiversity240  = 0.0
      avdiversity500  = 0.0
      avdiversityEq   = 0.0
      nonzeroSpecs80  = 0
      nonzeroSpecs160 = 0
      nonzeroSpecs240 = 0
      nonzeroSpecs500 = 0
      nonzeroSpecsEq  = 0
      domspec80       = 0
      domspec160      = 0
      domspec240      = 0
      domspec500      = 0
      domspecEq       = 0
    End IF

    IF (year .eq. 80) THEN
      CALL FindDominantSpecies (nspc, nonzeroSpecs80, domspec80)
      avdiversity80   = sumDiversity / 80
      sumDiversity    = 0
      spec%sumtimebio = 0
   ELSE IF (year .eq. 160) THEN
      CALL FindDominantSpecies (nspc, nonzeroSpecs160, domspec160)
      avdiversity160  = sumDiversity / 80
      sumDiversity    = 0
      spec%sumtimebio = 0
    ELSE IF (year .eq. 240) THEN
      CALL FindDominantSpecies (nspc, nonzeroSpecs240, domspec240)
      avdiversity240  = sumDiversity / 80
      sumDiversity    = 0
      spec%sumtimebio = 0
    ELSE IF (year .eq. 500) THEN
      CALL FindDominantSpecies (nspc, nonzeroSpecs500, domspec500)
      avdiversity500  = sumDiversity / 260
      sumDiversity    = 0
      spec%sumtimebio = 0
    ELSE IF (year .eq. numyrs - 1) THEN
     CALL FindDominantSpecies (nspc, nonzeroSpecsEq, domspecEq)
      avdiversityEq   = sumDiversity / (numyrs - 1 - 500)
      spec%sumtimebio = 0
   END IF
 END IF !----- lat == latEvalCell....
END SUBROUTINE EvaluateResults
! ****<Here the subroutine ends>*******************************************************************
