!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


!==============================================================================
!
!  Name   :  FileHandling
!
!  Remarks:  contains the subroutines
!
!            OpenControlParsFileList }   (<TreeMig)
!            OpenResultsFile         }
!            CloseResultsFile        }
!            OpenLogfile             }
!            OpenStateFile           }
!            OpenStateFileForWrite   }
!            OpenStateFileForRead (<ReadState<TreeMig)
!            OpenDispKernFile (<TreeMig)
!            CloseFiles (<TreeMig)
!            OpenExistingFile (<OpenControlParsFileList<TreeMig AND/OR
!                                <ReadControlPars<TreeMig AND/OR
!                                <GetConstEnvironment<TreeMig AND/OR
!                                <GetSpc<TreeMig AND/OR
!                                <ReadImmigrationData<GetSpc>TreeMig)
!
!==============================================================================


!****<Here the subroutine OpenControlParsFileList starts>******************************************
SUBROUTINE OpenControlParsFileList(pathletter1, pathletter2, readDispersalKernel)

USE All_Par, Only:  unitOfSpatialData
use forhycsglobals, only: TMdir
!<CALL modules for type definitions>---------------------------------------------------------------

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
CHARACTER(2), INTENT(OUT) :: pathletter1
CHARACTER(1), INTENT(OUT) :: pathletter2
LOGICAL, INTENT(OUT)      :: readDispersalKernel

!<Local variables>---------------------------------------------------------------------------------
CHARACTER(1) :: system
INTEGER      :: errcp
CHARACTER(50):: dummy

!<Here the subroutine starts>
  CALL OpenExistingFile (trim(TMdir)//'listOfCtrlPars.txt', 1010, errcp)
  print *, 'nach open listofctrlpars.txt', errcp
  IF (errcp .ne. 0) STOP
  READ (1010,*) system, dummy

  IF (system .eq. 'u') THEN                               !----- UNIX, Linux, Macintosh (X)
    pathletter1 = "./"
    pathletter2 = "/"
  ELSE IF ((system .eq. "w") .OR. (system .eq. "n")) THEN !----- Windows
    pathletter1 = ".\"
    pathletter2 = "\"
  ELSE IF (system .eq. "m") THEN                          !----- Macintosh (classic)
    pathletter1 = " :"
    pathletter2 = ":"
  ELSE
    PRINT *, 'error reading system ', system
!    PAUSE
    STOP
  END IF

  READ (1010,*) readDispersalKernel, dummy
  READ (1010,*) dummy                      !----- for Bioclim- and Stockability- data, in m
END SUBROUTINE OpenControlParsFileList
!****<Here the subroutine ends>*******************************************************************


!****<Here the subroutine OpenResultsFile starts>**************************************************
!--------------------------------------------------------------------------------------------------
SUBROUTINE OpenResultsFile(pathletter1, pathletter2)

!<CALL modules for type definitions>---------------------------------------------------------------
use forhycsglobals, only: TMdir

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
CHARACTER(2), INTENT(IN) :: pathletter1
CHARACTER(1), INTENT(IN) :: pathletter2

!<Local variables>---------------------------------------------------------------------------------
LOGICAL       :: there
INTEGER       ::   err, count
character(256):: expPath,appPath

!<Here the subroutine starts>

  expPath = trim(TMdir)//pathletter1//'R'//pathletter2//'expresults.txt'
  appPath = trim(TMdir)//pathletter1//'R'//pathletter2//'appresults.txt'
  INQUIRE(FILE=trim(expPath), EXIST=there, IOSTAT=err)

  count = 0
  IF (there) THEN
    OPEN(UNIT=1075, FILE=trim(expPath), STATUS="old", IOSTAT= err, position="append")
  ELSE
    OPEN(UNIT=1075, FILE=trim(expPath), STATUS="new", IOSTAT= err)
   ! CALL WriteHeaderForIntegratedResults
  END IF

   WRITE(1070,*) 'expresults opened'

  INQUIRE(FILE=trim(appPath) ,EXIST=there,IOSTAT=err)
  count = 0
  IF (there) THEN
    OPEN(UNIT=1059, FILE=trim(appPath), STATUS="old", IOSTAT= err, position="append")
  ELSE
    OPEN(UNIT=1059, FILE=trim(appPath), STATUS="new", IOSTAT= err)
!    CALL WriteHeaderForAppearanceResults
  END IF

  WRITE(1070,*) 'appresults opened'
END SUBROUTINE OpenResultsFile
!****<Here the subroutine ends>*******************************************************************


!****<Here the subroutine CloseResultsFile starts>*************************************************
!--------------------------------------------------------------------------------------------------
SUBROUTINE CloseResultsFile
!<Here the subroutine starts>
  Close(1075)
  Close(1059)
END SUBROUTINE CloseResultsFile
!****<Here the subroutine ends>*******************************************************************


!****<Here the subroutine OpenLogfile starts>******************************************************
!--------------------------------------------------------------------------------------------------
SUBROUTINE OpenLogfile
use forhycsglobals, only: TMdir
IMPLICIT NONE
  close(1070)
!<Here the subroutine starts>
  OPEN(UNIT=1070, FILE=trim(TMdir)//'logfile.txt', STATUS="replace")
END SUBROUTINE OpenLogfile
!****<Here the subroutine ends>*******************************************************************


!****<Here the subroutine OpenStateFile starts>****************************************************
!--------------------------------------------------------------------------------------------------
SUBROUTINE OpenStateFile
use forhycsglobals, only: TMdir
IMPLICIT NONE

!<Here the subroutine starts>
  OPEN(UNIT=1071, FILE=trim(TMdir)//'statefile.txt', STATUS="replace")
  OPEN(UNIT=1073, FILE=trim(TMdir)//'statefile2.txt', STATUS="replace")
  WRITE(1070,*) 'statefiles opened'
END SUBROUTINE OpenStateFile
!****<Here the subroutine ends>*******************************************************************


!****<Here the subroutine OpenStateFileForWrite starts>********************************************
!--------------------------------------------------------------------------------------------------
SUBROUTINE OpenStateFileForWrite
!<CALL modules for type definitions>---------------------------------------------------------------
use forhycsglobals, only: TMdir
IMPLICIT NONE
 INTEGER ::  err

!<Here the subroutine starts>
  OPEN(UNIT=1071, FILE=trim(TMdir)//'statefile.txt', STATUS="replace", IOSTAT=err)
  IF (err .ne. 0) THEN
     print *,'statefile.txt not successfully opened'
     STOP
  END IF
  OPEN(UNIT=1073, FILE=trim(TMdir)//'statefile2.txt', STATUS="replace",  IOSTAT=err)
  IF (err .ne. 0) THEN
     print *,'statefile2.txt not successfully opened'
     STOP
  END IF

  WRITE(1070,*) 'statefiles opened for write'
END SUBROUTINE OpenStateFileFORWrite
!****<Here the subroutine ends>*******************************************************************


!****<Here the subroutine OpenStateFileForRead starts>********************************************
!--------------------------------------------------------------------------------------------------
SUBROUTINE OpenStateFileForRead
!<CALL modules for type definitions>---------------------------------------------------------------
use forhycsglobals, only: TMdir
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
!<Local variables>---------------------------------------------------------------------------------
INTEGER ::  err

!<Here the subroutine starts>
!  OPEN(UNIT=1071, FILE='statefile_O16_06_Obs.txt', STATUS="old", IOSTAT=err)
  OPEN(UNIT=1071, FILE=trim(TMdir)//'statefile_O16_06T.txt', STATUS="old", IOSTAT=err)
   IF (err .ne. 0) THEN
     print *,'statefile.txt not successfully opened'
     STOP
  END IF

  print *,  'statefile opened for read'
END SUBROUTINE OpenStateFileForRead
!****<Here the subroutine ends>*******************************************************************


!****<Here the subroutine OpenDispKernFile starts>
!--------------------------------------------------------------------------------------------------
SUBROUTINE OpenDispKernFile
use forhycsglobals, only: TMdir
IMPLICIT NONE

  OPEN(UNIT=1110, FILE=trim(TMdir)//'newdispkernels.txt', STATUS="replace")
!  WRITE(1070,*) 'newdispkernels opened'
END SUBROUTINE OpenDispKernFile
!****<Here the subroutine ends>*******************************************************************


!****<Here the subroutine CloseFiles starts>*******************************************************
!--------------------------------------------------------------------------------------------------
SUBROUTINE CloseFiles

IMPLICIT NONE

!<Local variables>---------------------------------------------------------------------------------
INTEGER :: iunit

!<Here the subroutine starts>
  CLOSE(1070)
  CLOSE(1071)
  CLOSE(1073)
  DO iunit=1011,1015
    CLOSE(iunit)
    CLOSE(iunit + 40)
  END DO
  DO iunit=1020,1041
   CLOSE(iunit)
  END DO

  CLOSE (1100)
END SUBROUTINE CloseFiles
!****<Here the subroutine ends>*******************************************************************


!****<Here the subroutine OpenExistingFile starts>*************************************************
SUBROUTINE OpenExistingFile(fn, iunit, err)

use MPIVars, only: rank

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
CHARACTER(len=*), INTENT(IN) :: fn
INTEGER, INTENT(IN)          :: iunit
INTEGER, INTENT(OUT)         :: err

!<Local variables>---------------------------------------------------------------------------------
LOGICAL :: there

!<Here the subroutine starts>
  err = 0
  INQUIRE(FILE=fn, EXIST=there, IOSTAT=err)

  print*, "fn", fn, rank
  print*, "iostat", err, rank

  Check_There: IF (.NOT. there) THEN
    WRITE(1070,"(A)") " The file ", fn , "is not there!"
    err = 1
    RETURN
  END IF Check_There

  Check_File: IF (err /= 0) THEN
    WRITE(1070,"(A)") "Error reading file ", fn, "!"
    IF (err < 0) THEN
      WRITE(1070,"(A)") "The file", fn, " is empty!"
      RETURN
    END IF
  END IF Check_File

!----- First, open file for reading species immigration informations
  OPEN(UNIT=iunit, FILE=fn, STATUS="OLD", ACTION="READ", DELIM="NONE")
END SUBROUTINE OpenExistingFile
!****<Here the subroutine ends>********************************************************************
