!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


!==============================================================================
!
!  Name   :  GetConstEnvironment
!
!  Purpose:  It calculates the spatial forest dynamics on a grid with or
!            without seed dispersal
!
!  Remarks:  contains the subroutine
!
!            GetConstEnvironment (<TreeMig)
!
!==============================================================================


SUBROUTINE GetConstEnvironment(stockfileisokay)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: stockAreafilename, experimentID, maxlat, maxlon,                    &
                   stockability, latRealStart, lonRealStart, pathletter1, pathletter2, &
                   unitOfSpatialData,cellSideLength, stocksum,stocksumcum
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
LOGICAL, INTENT(OUT):: stockfileisokay

!<Local variables>--------------------------------------------------------------------------------
INTEGER      :: err, err1,                  &
                lat,                   &    ! looping variable for latitude  [=..]
                lon  ,iunit ,ilat,ilon ,irow,ilon2     ! looping variable for longitude [=..]
REAL         :: rlon, rlat, value, rlonread, rlatread,rrlon
CHARACTER*11 :: stockAreafilename11
CHARACTER*15 :: stockAreafilename15
CHARACTER*100 :: header
LOGICAL::foundilon256
!<Here the subroutine starts>**********************************************************************
  stockfileisokay     = .true.
  ! only for small area tests

  stockability=1.
  RETURN
  !only for small area tests
  err                 = 0
  iunit               = 1015
  stockAreafilename11 = stockAreafilename
  stockAreafilename15 = pathletter1//'E'//pathletter2//stockAreafilename11
 foundilon256=.FALSE.
   OPEN(UNIT=1055, file=pathletter1//'R'//pathletter2//experimentID//'_' &
       //stockAreafilename11, IOSTAT=err1, STATUS='unknown')
  IF (err1 .ne. 0) THEN
     RETURN
  END IF

   CALL OpenExistingFile (stockAreafilename15, iunit, err)
  IF (err .ne. 0) THEN
    stockfileisokay = .false.
    print *, err
    print *, 'stockfile not opened ',pathletter1//'E'//pathletter2//stockAreafilename11
    RETURN
  END IF
  READ (iunit,'(A100)') header
!  print *, 'header of stockfile: ', header


  stockability = 0.0
  irow=0
  print *, err
  DO WHILE (err == 0)
       READ(iunit,*,IOSTAT=err) rlonRead,  rlatRead, value                ! for Observation-Simus
 !      READ(iunit,*,IOSTAT=err) rlatRead,  rlonRead, value               !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! for CC-Simus, sequence different
    !   print *, value, rlatRead,  rlonRead
 ! Coordinates are rounded to 0.1 m
     rlon=ANINT(rlonRead*10.)/10.
     rlat =ANINT(rlatRead *10.)/10.

     IF (err .ne. 0) THEN
      print *, 'error or ready reading stockability, err', err
      stocksumcum(1)=0
      DO ilat=1, maxlat
        stocksum(ilat)=0
        DO ilon=1, maxlon
          stocksum(ilat)=stocksum(ilat)+stockability(ilat,ilon)
        end DO ! ilon
        if (ilat.gt.1) THEN
          stocksumcum(ilat)=stocksumcum(ilat-1)+stocksum(ilat)
        ELSE
          stocksumcum(1)= stocksum(1)
        END IF
!       if (stocksum(ilat)>0.) print *, 'ilat, stocksum(ilat), stocksumcum(ilat)',&
 !       ilat, stocksum(ilat), stocksumcum(ilat)
      END DO !ilat

      RETURN
    END IF

    !----- spatial coordinates are transformed to m
 !   lat = rlat  *  unitOfSpatialData
 !   lon = rlon * unitOfSpatialData
 !   !----- Standard calculation scale are km s
 !   ilat =  1 + (lat -   latRealStart*unitOfSpatialData) / cellSideLength
 !   ilon = 1 + (lon - lonRealStart*unitOfSpatialData) / cellSideLength
  !----- spatial coordinates are transformed to m
    rlat = rlat  *  unitOfSpatialData
    rlon = rlon * unitOfSpatialData
    !----- Standard calculation scale are km s
    ilat =  ANINT(1. + (rlat -   latRealStart*unitOfSpatialData) / cellSideLength )
    ilon = ANINT(1. + (rlon - lonRealStart*unitOfSpatialData) / cellSideLength)
 !   if (value.gt.0.) print *, 'stockability, rlonRead, rlatRead, value, err ', rlonRead, rlatRead, lat, lon, ilat, ilon, value, err

    irow=irow+1
    IF ((ilat .le. 0) .OR. (ilat .gt. maxlat) .OR. (ilon .le. 0) .OR. (ilon .gt. maxlon)) THEN
      print *,'bei readstocks falsche dimensionen ,rlat,ilat,lat,latstart',rlat, ilat,lat,latRealStart
      print *,'bei readstocks falsche dimensionen,rlon, ilon,lon,lonstart,value,irow',&
                   rlon, ilon,lon, lonRealstart,value,irow
      stockfileisokay = .false.
      STOP
    END IF

    ! stockability(ilat, ilon) = value
 END DO


  stockfileisokay = .true.

END SUBROUTINE GetConstEnvironment
!****<Here the subroutine ends>********************************************************************
