!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


!==============================================================================
!
!  Name   :  GetEnvFactors
!
!  Purpose:  It calculates the spatial forest dynamics on a grid with or
!            without seed dispersal
!
!  Remarks:  contains the subroutine
!
!            GetEnvFactors (<DistrLocalDynThisTime<TimeLoop<TreeMig)
!
!==============================================================================


SUBROUTINE GetEnvFactors( year,   lat, lon, ddegs, wtemp, drstr, avnit, brwpr, smday)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: ienv,  envFromFile, stockability,oldBioClim

IMPLICIT NONE

! <Passed variables>-------------------------------------------------------------------------------
INTEGER,INTENT(IN) :: year,     &    ! Passed simulation year
                      lat, lon       ! Passed coordinates
REAL, INTENT(OUT)  :: ddegs, wtemp,drstr,  avnit, brwpr, smday


!<Here the subroutine starts>*********************************************************************
  !----- Check whether site data drive dynamics ----------------------------------------------------
     IF (oldBioClim%bc(lat, lon)%muDD <0.0  )  THEN
       stockability(lat, lon) = 0.0
       ddegs=-999.0
     END IF
    IF (stockability(lat, lon) == 0.0) RETURN


  Include_ENV: IF (ienv) THEN

    !----- Read environmental (site) information per grid-cell -----------------------
      IF (envFromFile .AND. (stockability(lat, lon) > 0.0)) THEN                                                                             !
 !     CALL GetBioClim (year, lat, lon, ddegs, wtemp, drstr)

 ! GetBioClimSimple updates the variables ddegs, wtemp, drstr
 ! by assigning them the corresponding value from newBioClim
       CALL GetBioClimSimple (year, lat, lon, ddegs, wtemp, drstr,smday)
    ELSE
    END IF !----- envFromFile
  END IF Include_ENV
    brwpr =   8.000
    avnit = 100.000

  END SUBROUTINE GetEnvFactors
!****<Here the subroutine ends>********************************************************************
