!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


!==============================================================================
!
!  Name   :  GetConstEnvironment
!
!  Purpose:  It calculates the spatial forest dynamics on a grid with or
!            without seed dispersal
!
!  Remarks:  contains the subroutine
!
!            GetConstEnvironment (<TreeMig)
!
!==============================================================================


SUBROUTINE GetLandCoverFilter(stockfileisokay)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: landCoverFilterFilename, experimentID, maxlat, maxlon,                    &
                   landCoverFilter, latRealStart, lonRealStart, pathletter1, pathletter2, &
                   unitOfSpatialData,cellSideLength, stocksum,stocksumcum
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
LOGICAL, INTENT(OUT):: stockfileisokay

!<Local variables>--------------------------------------------------------------------------------
INTEGER      :: err, err1,                  &
                lat,                   &    ! looping variable for latitude  [=..]
                lon  ,iunit ,ilat,ilon ,irow,ilon2     ! looping variable for longitude [=..]
REAL         :: rlon, rlat, value, rlonread, rlatread,rrlon
CHARACTER*11 :: landCoverFilterFilename11
CHARACTER*15 :: landCoverFilterFilename15
CHARACTER*100 :: header
LOGICAL::foundilon256
!<Here the subroutine starts>**********************************************************************
  stockfileisokay     = .true.

  ! only for small area tests

  landCoverFilter = 1.
  RETURN
  !only for small area tests

  err                 = 0
  iunit               = 1015
  landCoverFilterFilename11 = landCoverFilterFilename
  landCoverFilterFilename15 = pathletter1//'E'//pathletter2//landCoverFilterFilename11
 foundilon256=.FALSE.
   CALL OpenExistingFile (landCoverFilterFilename15, iunit, err)
  IF (err .ne. 0) THEN
    stockfileisokay = .false.
    print *, err
    print *, 'stockfile not opened ',pathletter1//'E'//pathletter2//landCoverFilterFilename11
    RETURN
  END IF
  READ (iunit,'(A100)') header
  OPEN(UNIT=1055, file=pathletter1//'R'//pathletter2//experimentID//'_' &
       //landCoverFilterFilename11, IOSTAT=err1, STATUS='unknown')
  IF (err1 .ne. 0) THEN
     RETURN
  END IF

  landCoverFilter = 0.0
  irow=0
  DO WHILE(err == 0)
 !   READ(iunit,*,IOSTAT=err) rlatRead,  rlonRead, value  ! For CC-Simus, also in GetConstEnvironment
    READ(iunit,*,IOSTAT=err) rlonRead,  rlatRead, value   ! For Obs-Simus
   ! print *, rlon, rlat, value
 ! Coordinates are rounded to 0.1 m
     rlon=ANINT(rlonRead*10.)/10.
     rlat =ANINT(rlatRead *10.)/10.

     IF (err .ne. 0) THEN  ! we get here when reading is finished
      print *, 'error or ready reading landcoverFilter'
      RETURN
    END IF  ! err.ne.0
    !----- spatial coordinates are transformed to m
 !   lat = rlat  *  unitOfSpatialData
 !   lon = rlon * unitOfSpatialData
 !   !----- Standard calculation scale are km s
 !   ilat =  1 + (lat -   latRealStart*unitOfSpatialData) / cellSideLength
 !   ilon = 1 + (lon - lonRealStart*unitOfSpatialData) / cellSideLength
  !----- spatial coordinates are transformed to m
    rlat = rlat  *  unitOfSpatialData
    rlon = rlon * unitOfSpatialData
    !----- Standard calculation scale are km s
    ilat =  ANINT(1. + (rlat -   latRealStart*unitOfSpatialData) / cellSideLength )
    ilon = ANINT(1. + (rlon - lonRealStart*unitOfSpatialData) / cellSideLength)

    irow=irow+1
    IF ((ilat .le. 0) .OR. (ilat .gt. maxlat) .OR. (ilon .le. 0) .OR. (ilon .gt. maxlon)) THEN
      print *,'bei readstocks falsche dimensionen ,rlat,ilat,lat,latstart',rlat, ilat,lat,latRealStart
      print *,'bei readstocks falsche dimensionen,rlon, ilon,lon,lonstart,value,irow',&
                   rlon, ilon,lon, lonRealstart,value,irow
      stockfileisokay = .false.
      STOP
    END IF

    landCoverFilter(ilat, ilon) = value

 END DO


  stockfileisokay = .true.

END SUBROUTINE GetLandCoverFilter
!****<Here the subroutine ends>********************************************************************
