!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


!==============================================================================
!
!  Name   :  Grow
!
!  Purpose:
!
!  Remarks:  contains the subroutine
!
!            Grow (<LocalForestDynamics<DoLocalWork<DistrLocalDynThisTime<
!                    TimeLoop<TreeMig)
!
!==============================================================================


SUBROUTINE Grow(spp, height, light, srvinhl,   minhc, thiscell)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY:  spc,   hcwidth, birht, maxhc,  CurrStateInCell  ! Use control and species parameters

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN) :: spp,    &     ! number of species                                       [=spec]
                       height, &     ! height class                                          [=height]
                       light,  &     ! lightclass                                            [=ilight]
                       minhc         ! lowest height class (0 for dispersal, 1 for local simus)  [=..]
REAL, INTENT(IN)    :: srvinhl       ! remaining ind. per spec/height/light class          [=survNInH]

!<Local variables>---------------------------------------------------------------------------------
INTEGER :: newlwhc,     &      ! new lower class number of growing trees [=newBotHeight]
           newuphc             ! new upper class number of growing trees [=newTopHeight]
REAL    :: propbot,     &      ! proportion of individuals remaining in lower class          [=proportionBot]
           gr,          &      ! growth rate
           inHC1, delta

TYPE(CurrStateInCell):: thiscell
!<Here the subroutine starts>**********************************************************************
  !----- <Calc. rel. growth  in height classes
  gr = spc(spp)%hlgro(height,light)

  IF (height == 0) THEN  !----- this is the complicated case, because 0th height class is smaller than others
    IF (gr <= birht) THEN
      newlwhc = 0
      newuphc = 1
      propbot = (birht - gr) / birht
    ELSE
      newlwhc = MAX (MIN (1 + FLOOR ((gr - birht) / hcwidth), maxhc), minhc)
      newuphc = MAX (MIN (1 + FLOOR (gr / hcwidth), maxhc), minhc)
      IF (newlwhc == newuphc) THEN
        propbot = 1.0
      ELSE
        propbot = (spc(spp)%htmin(newuphc) - gr) / birht
      END IF
    END IF
  ELSE
  ! Standard case (adult trees)
    newlwhc = MAX (MIN (height + FLOOR (gr / hcwidth), maxhc), minhc)
    newuphc = MAX (MIN (1 + newlwhc, maxhc), minhc)
    propbot = (hcwidth - gr) / hcwidth
  END IF

  IF ((propbot < 0.) .OR. (propbot > 1.))THEN
    WRITE(1070,*) 'propbot=', propbot
  END if

  !----- <Subtract from heightclass what leaves given the current looping condition
  inHC1                           = thiscell%sp(spp)%numin(1)

  thiscell%sp(spp)%numin(height)  = thiscell%sp(spp)%numin(height) - srvinhl
  thiscell%sp(spp)%numin(newlwhc) = thiscell%sp(spp)%numin(newlwhc) + (propbot*srvinhl)
  thiscell%sp(spp)%numin(newuphc) = thiscell%sp(spp)%numin(newuphc) + ((1.0 - propbot) * srvinhl)

 IF (height == 0) THEN
    delta = thiscell%sp(spp)%numin(1) - inHC1
    ! thiscell%sp(...)%ingrowth was set to 0 in LocalForestDynamics ?!?
    thiscell%sp(spp)%ingrowth  = thiscell%sp(spp)%ingrowth + delta
  END IF

  IF (thiscell%sp(spp)%numin(height) < 0) THEN
    WRITE(1070,*) 'neg.trees in grow 2', height, thiscell%sp(spp)%numin(height)
    thiscell%sp(spp)%numin(height) = 0.0
  END IF

END SUBROUTINE Grow
!****<Here the subroutine ends>********************************************************************
