!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


!-------------------------------------------------------------------------------
! Sets cells in which forest is allowed to grow during spin-up, as determined by
! the control parameter LCDYN
!-------------------------------------------------------------------------------


subroutine LCPreSpinup

  use forhycsglobals, only: lcdyn
  use All_par, only: stockability
  use prevahglobals, only: landin, ncol, nrow

  implicit none

  integer :: xcoor, ycoor, land

!=======================================================================
  print*, "in LCSettings"
  print*, "ncol, nrow", ncol, nrow
  if(lcdyn == 0) then
  ! No cells converted to forest, neither in spin-up nor during simulation
    do ycoor = nrow,1,-1
    do xcoor = 1,ncol
      land = landin(xcoor,ycoor)
      select case(land)
        case(3,4,5,8)
          stockability(ycoor,xcoor) = 1.0
        case default
          stockability(ycoor,xcoor) = 0.0
      end select

      if((xcoor==20).and.(ycoor==20)) then
        print*, "land", land
        print*, "stockability", stockability(ycoor,xcoor)
      end if
    end do
    end do

  else if(lcdyn == 1) then
  ! No cells converted to forest during spin-up
    do ycoor = nrow,1,-1
    do xcoor = 1,ncol
      land = landin(xcoor,ycoor)
      select case(land)
        case(3,4,5,8)
          stockability(ycoor,xcoor) = 1.0
        case default
          stockability(ycoor,xcoor) = 0.0
      end select
    end do
    end do

  else if(lcdyn == 2) then
  ! Forest is allowed to grow in all potentially forested cells, also during
  ! spin-up

    do ycoor = nrow,1,-1
    do xcoor = 1,ncol
      land = landin(xcoor,ycoor)
      select case(land)
        case(3,4,5,8,21,23,24,25,26)
          stockability(ycoor,xcoor) = 1.0
        case default
          stockability(ycoor,xcoor) = 0.0
      end select
    end do
    end do

  end if

end subroutine LCPreSpinup

!=======================================================================
!=======================================================================
subroutine LCPostSpinup

! Sets cells in which forest is allowed to grow during the actual simulation (after spin-up),
! as determined by the control parameter LCDYN

  use forhycsglobals, only: lcdyn
  use All_par, only: stockability
  use prevahglobals, only: landin, ncol, nrow

  implicit none

  integer :: xcoor, ycoor, land

!=======================================================================
  if(lcdyn == 0) then
  ! No cells converted to forest, neither in spin-up nor during simulation
    do ycoor = nrow,1,-1
    do xcoor = 1,ncol
      land = landin(xcoor,ycoor)
      select case(land)
        case(3,4,5,8)
          stockability(ycoor,xcoor) = 1.0
        case default
          stockability(ycoor,xcoor) = 0.0
      end select
    end do
    end do

  else if(lcdyn == 1) then
  ! No cells converted to forest during spin-up
    do ycoor = nrow,1,-1
    do xcoor = 1,ncol
      land = landin(xcoor,ycoor)
      select case(land)
        case(3,4,5,8,21,23,24,25,26)
          stockability(ycoor,xcoor) = 1.0
        case default
          stockability(ycoor,xcoor) = 0.0
      end select
    end do
    end do

  else if(lcdyn == 2) then
  ! Forest is allowed to grow in all potentially forested cells, also during
  ! spin-up
    do ycoor = nrow,1,-1
    do xcoor = 1,ncol
      land = landin(xcoor,ycoor)
      select case(land)
        case(3,4,5,8,21,23,24,25,26)
          stockability(ycoor,xcoor) = 1.0
        case default
          stockability(ycoor,xcoor) = 0.0
      end select
    end do
    end do

  end if

end subroutine LCPostSpinup
