!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Adaptation to FORHYCS MPI structure

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------



!==============================================================================
!
!  Name   :  LocalWork
!
!  Remarks:  contains the subroutines
!
!            DistrLocalDynThisTime (<TimeLoop<TreeMig)
!            DoLocalWork (<DistrLocalDynThisTime<TimeLoop<TreeMig)
!
!==============================================================================


!=================== For non parallel (i.e.) standard TreeMig======================================================
SUBROUTINE DistrLocalDynThisTime( year,counter,currLatStart, currLatEnd, &
                         currLonStart, currLonEnd,nspc, idsp)


!<CALL modules for type definitions>---------------------------------------------------------------
 use MPIvars, only: rank, root, doParallel
 use all_par, only: sumseeds3
 use forhycsglobals, only: ecor     ! MS, Apr 2017 - Output of structure for each cell of an ecoregion
 IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN)    :: year,                               &  ! current year
                          currLatStart, currLatEnd, currLonStart, currLonEnd, &  ! start and end of simulated areaINTEGER :: nspc
                           nspc ! number of species involved [=spec]
 LOGICAL, INTENT(IN)    :: idsp                                  ! dispersal or not?
INTEGER, INTENT(INOUT) :: counter                                ! simulation counter for output
integer :: actyear            ! MS, Apr 2017

!<Local variables>--------------------------------------------------------------------------------
INTEGER :: lat,           &    ! looping variable for latitude              [=..]
           lon                 ! looping variable for longitude
REAL:: ddegs, wtemp, drstr, avnit, brwpr, smday
INTEGER:: Iam
!<Here the subroutine starts>
    iam = 0    ! Remainder from parallel implementation of stand-alone TreeMig
    sumseeds3 = 0
    print*, "nspc in LocalWork", nspc, rank
    if ((rank .ne. root) .or. (doParallel==.false.)) then
    LoopLatLocaDyn: DO lat=currLatStart,currLatEnd
      LoopLonLocaDyn: DO lon=currLonStart,currLonEnd
          CALL GetEnvFactors (  year,  lat, lon, ddegs,  wtemp,  drstr,  avnit, brwpr,smday)
          CALL DoLocalWork( Iam, year,lat, lon, nspc, ddegs, wtemp, drstr, avnit, brwpr,smday, idsp)
          actyear = year+1500
          !if(ecor(lon,lat)==1103) call cellStructOut(lon,lat,actyear)  ! MS, Apr 2017
        END DO LoopLonLocaDyn
    END DO LoopLatLocaDyn
    end if  ! Slaves
    print*, "sumseeds3", sumseeds3, rank
    Call UpdateNewSeeds (nspc,iam,idsp,year,currLatStart,currLatEnd,currLonStart,currLonEnd )             ! In Interact.f90
    if ((rank .ne. root) .or. (doParallel==.false.)) then
      Call SeedsFromOutsSimRegion (nspc,  idsp,year, currLatStart, currLatEnd, currLonStart, currLonEnd ) ! In Interact.f90
    end if

    call stemnum

END SUBROUTINE DistrLocalDynThisTime
!****<Here the subroutine ends>********************************************************************




!========================= For parallel and sequential ===============================
SUBROUTINE DoLocalWork( Iam, year,lat, lon, nspc,ddegs, wtemp, drstr, avnit, brwpr,smday, idsp )
!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY:  stockability,   CurrStateInCell  ,cell, maxlat, maxlon
IMPLICIT NONE
!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN)    :: Iam, year, nspc, lat, lon             ! number of species involved [=spec]
LOGICAL, INTENT(IN)    :: idsp                                  ! dispersal or not?
REAL,    INTENT(IN)    :: ddegs, wtemp, drstr, avnit, brwpr,smday

 TYPE(CurrStateInCell):: thiscell
!<Here the subroutine starts>
        if ( (lat>maxlat) .or. (lat <1) .or. (lon>maxlon) .or. (lon<1) )then
           print *, 'wrong coordinates in dolocalwork, lat, maxlat, lon, maxlon', lat, maxlat, lon, maxlon
            stop
        end if

        if(year == 1) then
          print*, "stock before", stockability(lat,lon), lat, lon
          print*, "ddegs", ddegs, lat, lon
        end if

        IF (ddegs<0.0.and.stockability(lat, lon)>0.0) THEN
          stockability(lat, lon)=0.0
        END IF

        if(year == 1) then
          print*, "stock after", stockability(lat,lon), lat, lon
        end if



        IF (stockability(lat, lon) > 1.E-20) THEN
          CALL CalcCurrentDepFunctsInCell(Iam,year, lat, lon, nspc, ddegs, wtemp, drstr, avnit, brwpr,smday)
          CALL StoreCellStateToThisCell(year,lat, lon,  nspc, idsp,ddegs, wtemp, drstr, avnit,brwpr,thiscell)     ! store the current cell to the structure thiscell  (StoringBackAndForth.f90)
          CALL LocalForestDynamics(lat, lon, Iam, nspc,year,idsp,thiscell)
          call bioKaufmann(lon, lat, thiscell, 4)     ! Matthias, Dec 2017
          CALL StoreThisCellSpecStatesToCell(  lat, lon, nspc, thiscell )     ! store the current structure thiscell back to the cell, 	(StoringBackAndForth.f90)
          if (idsp) CALL SeedsDispersedFromThisCell(nspc, lat, lon,year, Iam) ! calculate the seed dispersal from this cell (Interact.f90)
        END IF  !stockability(lat, lon) > 1.E-20

 	CALL Report (nspc, year, lat, lon,Iam)


END SUBROUTINE DoLocalWork
!========================================================================
