!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Grids to be written: Bioclimatic indices calculated in PREVAH (DrStr, MuWit, DDSum)
! and given by BioClim, for comparison.
!-------------------------------------------------------------------------------

! 1. Drought Stress FORHYCS
      if(rank .ne. root) then
        wrgrd = drstrOld
        call MPI_SEND(wrgrd(:,firstRow:lastRow),ncol*nrowLocal,
     *    MPI_REAL,root,22,MPI_COMM_WORLD,ierr)
      else  ! Master
        wrgrd = rnodat

        do j=1,numslaves
          call MPI_RECV(wrgrd(:,firstRowArray(j):lastRowArray(j)),
     *          ncol*nrowLocalArray(j),MPI_REAL,j,22,
     *          MPI_COMM_WORLD,status,ierr)
        end do

         namgrd=diroutput(1:posi3)//idnam(posiid1:posiid2)//
     *          treeyyyy//"drStr.fyc"
        OPEN (732,FILE=namgrd,STATUS='unknown',ERR=999,FORM='BINARY')

        uwas=732
	 include 'wstsp.inc'

      end if


! 2. Minimum winter temperature FORHYCS

      if(rank .ne. root) then

        wrgrd = muwint(:,:,4)
        call MPI_SEND(wrgrd(:,firstRow:lastRow),ncol*nrowLocal,
     *    MPI_REAL,root,22,MPI_COMM_WORLD,ierr)
      else  ! Master
        wrgrd = rnodat

        do j=1,numslaves
          call MPI_RECV(wrgrd(:,firstRowArray(j):lastRowArray(j)),
     *          ncol*nrowLocalArray(j),MPI_REAL,j,22,
     *          MPI_COMM_WORLD,status,ierr)
        end do

         namgrd=diroutput(1:posi3)//idnam(posiid1:posiid2)//
     *          treeyyyy//"muwit.fyc"
        OPEN (732,FILE=namgrd,STATUS='unknown',ERR=999,FORM='BINARY')

        uwas=732
	 include 'wstsp.inc'

      end if


! 3. Degree-day sum FORHYCS

      if(rank .ne. root) then
        wrgrd = ddegsum5old
        call MPI_SEND(wrgrd(:,firstRow:lastRow),ncol*nrowLocal,
     *    MPI_REAL,root,22,MPI_COMM_WORLD,ierr)
      else  ! Master
        wrgrd = rnodat

        do j=1,numslaves
          call MPI_RECV(wrgrd(:,firstRowArray(j):lastRowArray(j)),
     *          ncol*nrowLocalArray(j),MPI_REAL,j,22,
     *          MPI_COMM_WORLD,status,ierr)
        end do

         namgrd=diroutput(1:posi3)//idnam(posiid1:posiid2)//
     *          treeyyyy//"dd5.fyc"
        OPEN (732,FILE=namgrd,STATUS='unknown',ERR=999,FORM='BINARY')

        uwas=732
	 include 'wstsp.inc'

      end if


! 4. Drought Stress BioClim
      if(rank .ne. root) then
        wrgrd = compBioClim(:,:,1)
        call MPI_SEND(wrgrd(:,firstRow:lastRow),ncol*nrowLocal,
     *    MPI_REAL,root,22,MPI_COMM_WORLD,ierr)
      else  ! Master
        wrgrd = rnodat

        do j=1,numslaves
          call MPI_RECV(wrgrd(:,firstRowArray(j):lastRowArray(j)),
     *          ncol*nrowLocalArray(j),MPI_REAL,j,22,
     *          MPI_COMM_WORLD,status,ierr)
        end do

         namgrd=diroutput(1:posi3)//idnam(posiid1:posiid2)//
     *          treeyyyy//"drStr.bcl"
        OPEN (732,FILE=namgrd,STATUS='unknown',ERR=999,FORM='BINARY')

        uwas=732
	 include 'wstsp.inc'

      end if


! 5. Minimum winter temperature BioClim

      if(rank .ne. root) then
        print*, "muwintTEST", compBioClim(1,firstRow,2), rank, firstRow
        wrgrd = compBioClim(:,:,2)
        call MPI_SEND(wrgrd(:,firstRow:lastRow),ncol*nrowLocal,
     *    MPI_REAL,root,22,MPI_COMM_WORLD,ierr)
      else  ! Master
        wrgrd = rnodat

        do j=1,numslaves
          call MPI_RECV(wrgrd(:,firstRowArray(j):lastRowArray(j)),
     *          ncol*nrowLocalArray(j),MPI_REAL,j,22,
     *          MPI_COMM_WORLD,status,ierr)
        end do

         namgrd=diroutput(1:posi3)//idnam(posiid1:posiid2)//
     *          treeyyyy//"muwit.bcl"
        OPEN (732,FILE=namgrd,STATUS='unknown',ERR=999,FORM='BINARY')

        uwas=732
	 include 'wstsp.inc'

      end if


! 6. Degree-day sum BioClim

      if(rank .ne. root) then
        wrgrd = compBioClim(:,:,3)
        call MPI_SEND(wrgrd(:,firstRow:lastRow),ncol*nrowLocal,
     *    MPI_REAL,root,22,MPI_COMM_WORLD,ierr)
      else  ! Master
        wrgrd = rnodat

        do j=1,numslaves
          call MPI_RECV(wrgrd(:,firstRowArray(j):lastRowArray(j)),
     *          ncol*nrowLocalArray(j),MPI_REAL,j,22,
     *          MPI_COMM_WORLD,status,ierr)
        end do

         namgrd=diroutput(1:posi3)//idnam(posiid1:posiid2)//
     *          treeyyyy//"dd5.bcl"
        OPEN (732,FILE=namgrd,STATUS='unknown',ERR=999,FORM='BINARY')

        uwas=732
	 include 'wstsp.inc'

      end if
