!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Output of TreeMig Report for parallelized FORHYCS version.
! Included in subroutine Timeloop
!-------------------------------------------------------------------------------

write(chyear,'(I4)') simustartyear + year
print*, "chyear", chyear

tableDir = trim(TMdir)//'R/'//chyear
gridDir = trim(TMdir)//'G/'//chyear


! Create directories that will contain output for the current year
if (rank == root) then
  result = makedirqq(trim(tableDir))
  if(result == .false.) print*, "Failed to create directory", trim(tableDir)
  result = makedirqq(trim(gridDir))
  if(result == .false.) print*, "Failed to create directory", trim(gridDir)
end if

numvars = 7
do i = 1,numvars
  select case(i)
    case(1)
      writeOut = writeoutput%biomass
      writeTable = biomassTable
      write2d = biomass2d
      varName = "biomass"

    case(2)
      writeOut = writeoutput%number
      writeTable = numberTable
      write2d = number2d
      varName = "number"

    case(3)
      writeOut = writeoutput%seeds
      writeTable = seedsTable
      write2d = seeds2d
      varName = "seeds"

    case(4)
      writeOut = writeoutput%ingrowth
      writeTable = ingrowthTable
      write2d = ingrowth2d
      varName = "ingrowth"

    case(5)
      writeOut = writeoutput%antagonists
      writeTable = antagonTable
      write2d = antagon2d
      varName = "antagon"

    case(6)
      writeOut = writeoutput%pollen
      writeTable = pollenTable
      write2d = pollen2d
      varName = "pollen"

    case(7)
      writeOut = writeoutput%leafarea
      writeTable = laTable
      write2d = la2d
      varName = "leafarea"
  end select

! Writing TABLES
! Slaves: send their part of the output table to Master

  if(writeOut .and. reptTables) then
    if (rank .ne. root) then
      call MPI_SEND(writeTable(:,tableStart:tableStart+ncol*nrowLocal-1),ncol*nrowLocal*(nspc+1),MPI_REAL,&
      root,22,MPI_COMM_WORLD,ierr)
    else
! Master: receives output from slaves (first all lines
! from 1st slave, then all lines from 2nd, etc.) and writes them to file.
      do j = 1,numslaves
        call MPI_RECV(writeTable(:,tableStartArray(j):tableStartArray(j)+ncol*nrowLocalArray(j)),&
        ncol*nrowLocalArray(j)*(nspc+1),MPI_REAL,j,22,MPI_COMM_WORLD,status,ierr)
      end do  ! Loop over slaves

! Once all slaves sent their tables, write to file
      tableName = trim(tableDir)//'/'//chyear//trim(varname)//'.tmt'
!      open(1080,file=trim(tableName),status='new')
!
!      do j = 1,ncol*nrow
!        write(1080,*) latTable(j),lonTable(j),jahr,(writeTable(k,j), k=1,nspc+1)
!      end do
!
!      close(1080)

    end if  ! Master / Slaves
  end if  ! writeOut .and. reptTables

! Writing GRIDS

  if(writeOut .and. reptGrids) then
    if(rank .ne. root) then
      call MPI_SEND(write2d(:,firstRow:lastRow,:),nrowLocal*ncol*(nspc+1),MPI_REAL,root,23,MPI_COMM_WORLD,ierr)
    else
      do j = 1,numslaves
        call MPI_RECV(write2d(:,firstRowArray(j):lastRowArray(j),:),nrowLocalArray(j)*ncol*(nspc+1),MPI_REAL,j,23,&
        MPI_COMM_WORLD,status,ierr)
      end do  ! Looop over slaves
      do j = 1,nspc+1
        if((j .ge. 1) .and. (j .le. nspc)) then
            specName = spec(j)%ns
        else if (j == nspc + 1) then
            specName = 'XX'
        end if
        gridName = trim(gridDir)//'/'//chyear//specName//varName//'.tmg'
        open(1081,file=trim(gridName),status='new',form='binary')

        wstats(1) = count(write2d .ne. rnodat)
        wstats(2) = minval(write2d, mask=write2d .ne. rnodat)
        wstats(3) = maxval(write2d, mask=write2d .ne. rnodat)
        wstats(4) = sum(write2d, mask=write2d .ne. rnodat)
        wstats(5) = wstats(4) / wstats(1)
        wstats(6) = -9999

	write (1081) rcol
	write (1081) rrow
	write (1081) rxll
	write (1081) ryll
	write (1081) rgridsize
	write (1081) rnodat

	do rloop=1,6
	    write (1081) wstats(rloop)
	end do

	do ycoor=nrow,1,-1
         write (1081) (write2D(xcoor,ycoor,j),xcoor=1,ncol)
	end do

	close(1081)
      end do  ! Loop over species
    end if  ! Master / Slaves

  end if
end do  ! Loop over variables
