!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Enable spinup with bootstrapped years

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------



!==============================================================================
!
!  Name   :  PrepareBioclimInput
!
!  Remarks:  contains the subroutine
!
!            PrepareBioclimInput (<TreeMig)
!
!==============================================================================


SUBROUTINE PrepareBioclimInput (bioclimfileisokay)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: bioclimfilename1, bioclimfilename2, experimentID, pathletter1, pathletter2, newBioClim
USE forhycsglobals, ONLY: PREVAHBC, TMdir
use MPIvars, only: rank, root

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
LOGICAL, INTENT(OUT) :: bioclimfileisokay

!<Local variables>---------------------------------------------------------------------------------
INTEGER      :: err
CHARACTER*80 :: text
CHARACTER*11 :: bioclimfilename11
character*256 :: bcPath
!character*3 :: dTchar, dPchar

!<Here the subroutine starts>**********************************************************************

! print *, 'in preparebiocliminput ', bioclimfilename1, bioclimfilename2


 !IF (prevahBC.eq.1) then
  !bioclimfilename11 = bioclimfilename1
 !ELSE
  bioclimfilename11 = bioclimfilename2
!	bioclimfilename11 = bioclimfilename1
 !END IF

  bcPath=trim(TMdir)//'E/'//bioclimfilename11


!  OPEN(UNIT=1014, file= pathletter1//'E'//pathletter2//bioclimfilename11, IOSTAT=err, STATUS='OLD')
  OPEN(UNIT=1014, file= trim(bcPath), IOSTAT=err, STATUS='OLD')
  IF (err .ne. 0) THEN
    WRITE(1070,*) 'problems opening file', bioclimfilename11, rank
   print*,  'problems opening file, return', bioclimfilename11, rank
    bioclimfileisokay = .false.
    RETURN
  ELSE
    bioclimfileisokay = .true.
     print *,' bioclimfile ', pathletter1//'E'//pathletter2//bioclimfilename11 ,' successfully opened', rank
  END IF

  Read(1014,*) text
  if (rank==root) WRITE (1070,*) text   ! Matthias
  !print *, 'header of bioclimfile ', text
!  CALL ReadNewBioClim  !MARC necessary?

END SUBROUTINE PrepareBioclimInput
!****<Here the subroutine ends>********************************************************************




!==============================================================================
! Unklar wie Jahre gezaehlt werden!  vermutlich noch climatestartyear einfuegen und einlesen
SUBROUTINE ReadFirst30Years (bioclimfileisokay)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: experimentID, pathletter1, pathletter2, newBioClim, &
     climatestartyear,simustartyear, oldBioClim, spi_sample_yrs, &
     BioClimSU1, BioClimSU2, BioClimSU3, BioClimSU4, BioClimSU5, &
     BioClimSU6, BioClimSU7, BioClimSU8, BioClimSU9, BioClimSU10, &
     BioClimSU11, BioClimSU12, BioClimSU13, BioClimSU14, BioClimSU15, &
     BioClimSU16, BioClimSU17, BioClimSU18, BioClimSU19, BioClimSU20, &
     BioClimSU21, BioClimSU22, BioClimSU23, BioClimSU24, BioClimSU25, &
     BioClimSU26, BioClimSU27, BioClimSU28, BioClimSU29, BioClimSU30
use MPIVars, only: rank

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
LOGICAL, INTENT(OUT) :: bioclimfileisokay

!<Local variables>---------------------------------------------------------------------------------
INTEGER      :: err, iyear
CHARACTER*80 :: text
CHARACTER*11 :: bioclimfilename11

    print *, 'in Readfirst30Years, vor PrepareBioclimInput', rank
  ! Eigene Subroutine, danach nochmal PrepareBioclimInput aufrufen
     Call PrepareBioclimInput(bioclimfileisokay)
     print *, 'in  Readfirst30Years, nach PrepareBioclimInput', rank

   DO iyear=1, (spi_sample_yrs-1)
      CALL ReadNewBioclimIfNecessary(climatestartyear-simustartyear+iyear-1,.FALSE.)
    !  print*, 'yearsBC', climatestartyear-simustartyear+iyear-1
       !IF (iyear.eq.1) ThirtyYearBioClim(iyear)=oldBioClim
       ! IF (iyear.eq.1) ThirtyYearBioClim(iyear)=newBioClim   ! Kludge


      !ThirtyYearBioClim(iyear+1)=newBioClim !     Deklarieren, genauso wie newbioclim

      print*, "iyear", iyear

      select case(iyear)
        case(1)
          BioClimSU1 = newBioClim
        case(2)
          BioClimSU2 = newBioClim
        case(3)
          BioClimSU3 = newBioClim
        case(4)
          BioClimSU4 = newBioClim
        case(5)
          BioClimSU5 = newBioClim
        case(6)
          BioClimSU6 = newBioClim
        case(7)
          BioClimSU7 = newBioClim
        case(8)
          BioClimSU8 = newBioClim
        case(9)
          BioClimSU9 = newBioClim
        case(10)
          BioClimSU10 = newBioClim
        case(11)
          BioClimSU11 = newBioClim
        case(12)
          BioClimSU12 = newBioClim
        case(13)
          BioClimSU13 = newBioClim
        case(14)
          BioClimSU14 = newBioClim
        case(15)
          BioClimSU15 = newBioClim
        case(16)
          BioClimSU16 = newBioClim
        case(17)
          BioClimSU17 = newBioClim
        case(18)
          BioClimSU18 = newBioClim
        case(19)
          BioClimSU19 = newBioClim
        case(20)
          BioClimSU20 = newBioClim
        case(21)
          BioClimSU21 = newBioClim
        case(22)
          BioClimSU22 = newBioClim
        case(23)
          BioClimSU23 = newBioClim
        case(24)
          BioClimSU24 = newBioClim
        case(25)
          BioClimSU25 = newBioClim
        case(26)
          BioClimSU26 = newBioClim
        case(27)
          BioClimSU27 = newBioClim
        case(28)
          BioClimSU28 = newBioClim
        case(29)
          BioClimSU29 = newBioClim
        case(30)
          BioClimSU30 = newBioClim
      end select

   END DO
   ! print *,'fertig 30 Y LOOP', iyear,  newBioClim%cctime, rank
    !pause
   close(1014)


END SUBROUTINE ReadFirst30Years
!****<Here the subroutine ends>********************************************************************
