!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------



!==============================================================================
!
!  Name   :  ReadControlPars
!
!  Remarks:  contains the subroutines
!
!            ReadControlPars (<TreeMig)
!            RWLine     } (<ReadControlPars<TreeMig)
!            RWInt      |
!            RWReal     |
!            RWLog      |
!            RWText1    |
!            RWText15   }
!
!==============================================================================


SUBROUTINE ReadControlPars( controlfileisokay,ready)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: numyrs, rptint, rptdist,                                                    &
                   maxlat, maxlon, latRealStart, latRealEnd, lonRealStart, lonRealEnd,         &
                   startlatStart, startlatEnd, startlonStart, startlonEnd,                     &
                   latEvalCell, lonEvalCell,                                                   &
                   starttime, readStateVector,                                                 &
                   ienv, envFromfile, bioclimfilename1, bioclimfilename2, stockAreafilename,   &
                   kernelType, alpha, epsKernel, cellSideLength, active, pltsiz,               &
                   doStochSeedDisp, seedsDifferent, mathgtDifferent,                           &
                   dispersalDifferent, mastSeeding, seedBank, seedlcrowndiamInCm,              &
                   disturbances, disturbProb, disturbIntensity,                                &
                   withSeedAntagonists, seedAntaGraz, seedAntaMort, seedAntaEff, seedAntaRain, &
                   withSeedCarrCap, seedCarrCap,                                               &
                   boundaries, fcvarInit, fcvarDyn, estabEqHC0, contLightDepGerm,              &
                   specfilename ,immifilename, immigration, simustartyear,                     &
                   experimentID, tabsepOutput, pathletter1, pathletter2, OuterSeeds,           &
                   writeOutput, unitOfSpatialData, landCoverFilterFilename,                    &
                   maxrowscols, maxDataInPlot            ! Use control and species parameters

use prevahglobals, ONLY: diroutput, ncol, nrow, rxll, ryll
use forhycsglobals, only: tpswitch,TMdir

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
LOGICAL, INTENT(OUT):: controlfileisokay, ready

!<Local variables>---------------------------------------------------------------------------------
CHARACTER*15 :: controlparfile
CHARACTER*11 :: controlparfile11
character*256:: cpPath
Character*80 :: line
INTEGER      :: err, errcp, cell2cellType,ic,tmpos

!<Here the subroutine starts>**********************************************************************
  controlfileisokay = .TRUE.
  READ(1010,*,IOSTAT=errcp) controlparfile

  IF(errcp .ne. 0) THEN
    ready = .true.
    print *, 'konnte contolparfilename nicht lesen'
    RETURN
  ELSE
    ready = .false.

  END IF

   controlparfile11 = controlparfile
   cpPath = trim(TMdir)//'C/'//controlparfile11

!   CALL OpenExistingFile(pathletter1//'C'//pathletter2//controlparfile11,1011, err)
   CALL OpenExistingFile(trim(cpPath),1011, err)
 !print *,  trim(cpPath),' opened', err

  IF (err .ne. 0) THEN
    controlfileisokay = .FALSE.
    RETURN
  END IF

   READ(1011,"(A80)") line
   READ(1011,"(A)") experimentID    !----- identification of sim.exp. for outputfiles
   DO ic=1, 12
     if (experimentID(ic:ic).eq.' ') experimentID(ic:ic) ='_'
   END DO
   CALL POSNAM (diroutput,1,tmpos)
   OPEN(UNIT=1051, FILE=diroutput(1:tmpos)//'treemig'//pathletter2//'R'//pathletter2//experimentID//'_'//tpswitch//'_control.pars',STATUS='UNKNOWN')
  !----- Time
  CALL RWLine
  CALL RWInt (numyrs, err)
  IF (err .ne. 0) RETURN
  CALL RWInt (simustartyear, err)
  IF (err .ne. 0) RETURN
  CALL RWInt (rptint, err)
  IF (err .ne. 0) RETURN
  CALL RWInt (starttime, err)
  IF (err .ne. 0) RETURN
  CALL RWLog( readStateVector, err)
  IF (err .ne. 0) RETURN

  !----- Space
  CALL RWLine
  CALL RWReal (pltsiz, err)         !----- = 833.0; size of patches, reference size for distributions
  IF (err .ne. 0) RETURN
  CALL RWInt (maxlat, err)
  IF (err .ne. 0) RETURN
  CALL RWInt (maxlon, err)
  IF (err .ne. 0) RETURN

  CALL RWReal (cellSideLength, err)
  IF (err .ne. 0) RETURN
  CALL RWInt (unitOfSpatialData, err)
  IF (err .ne. 0) RETURN
  CALL RWReal (lonRealStart, err)   !----- bottom left edge of real grid (->cell1,1)
  IF (err .ne. 0) RETURN
  CALL RWReal (lonRealEnd, err)
     IF (err .ne. 0) RETURN
  CALL RWReal (latRealStart,err)
  IF (err .ne. 0) RETURN
  CALL RWReal (latRealEnd, err)
  ! Replace maxlat and maxlon, as well as reallat/lon, with the dimensions of PREVAH grids
  ! (so that the TM control file does not need to be updated every time for new study areas) - MS
  maxlon = ncol
  maxlat = nrow
  lonRealStart = rxll
  lonRealEnd = rxll + (ncol-1) * cellSideLength
  latRealStart = ryll
  latRealEnd = ryll + (nrow-1) * cellSideLength

  IF (err .ne. 0) RETURN
  CALL RWInt (startlatStart, err)
  IF (err .ne. 0) RETURN
  CALL RWInt (startlatEnd, err)
  IF (err .ne. 0) RETURN
  CALL RWInt (startlonStart, err)
  IF (err .ne. 0) RETURN
  CALL RWInt (startlonEnd, err)
  IF (err .ne. 0) RETURN
  CALL RWInt (latEvalCell, err)
  IF (err .ne. 0) RETURN
  CALL RWInt (lonEvalCell, err)
  IF (err .ne. 0) RETURN
  CALL RWInt (rptdist, err)
  IF (err .ne. 0) RETURN
  CALL RWText1 (boundaries, err)     !-----  = 'c' ! 'c':cyclic, i.e torus, 'a':absorbing
  IF (err .ne. 0) RETURN

  ! Not sure what startLat.../startLon... are for, but their values correspond to 1 and maxlat/lon
  ! in the control file. Overriding the values from control file so that they match the values
  ! read in / taken from PREVAH (see above) - MS
  startLatStart = 1
  startLatEnd = maxlat
  startLonStart = 1
  startLonEnd = maxlon

  !----- Environment
  CALL RWLine
  IF (err .ne. 0) RETURN
  !----- Site included, and how?
  CALL RWLog (ienv, err)                  !----- = .TRUE.  ! include environmental info for calcs?   [=..] !ENV
  IF (err .ne. 0) RETURN
  CALL RWLog (envFromfile, err)           !----- = .FALSE. ! read environmental info from file?     [=..] !ENV
  IF (err .ne. 0) RETURN
  CALL RWText15 (bioclimfilename1, err)    !----- file for bioclim !FORHYCS - 2 bioclimfiles, TreeMig BC and Prevah BC, important for spinup
  IF (err .ne. 0) RETURN
  CALL RWText15 (bioclimfilename2, err)    !----- file for bioclim
  IF (err .ne. 0) RETURN
  CALL RWText15 (stockAreafilename, err)  !----- file for percentage of stockable area
  IF (err .ne. 0) RETURN
 CALL RWText15 (landCoverFilterFilename, err)  !----- file for filtering out land uses prohibiting forests at start of simulation
  IF (err .ne. 0) RETURN

  !----- Dispersal
  CALL RWLine
  !----- Seed dispersal included, and how?
  CALL RWLog (dispersalDifferent, err)  !----- dispersal distances different
  IF (err .ne. 0) RETURN
  CALL RWInt (kernelType, err)          !----- 0: circular, 1: single-expon. 2: double-expon.                       [=..] !DSP
  IF (err .ne. 0) RETURN
  CALL RWInt (cell2cellType, err)       !----- not actual anymore, included only to keep old input files dispersal from cell center to center (0) or from each point to each point (1)
  IF (err .ne. 0) RETURN
  CALL RWReal (epsKernel, err)          !----- =  cut-off threshold for kernel (in 1/1000)
  IF (err .ne. 0) RETURN
  epsKernel = epsKernel * 1.E-10
  CALL RWReal (alpha, err)              !----- = 100.0    ! Alpha-value of kernel [=..] !DSP
  IF (err .ne. 0) RETURN
  CALL RWLog (active, err)              !----- = .TRUE.   ! Active or passive seed dispersal (calculations)           [=..] !DSP
  IF (err .ne. 0) RETURN
  CALL RWLog (doStochSeedDisp, err)     !----- = .FALSE.  ! Stochastic or deterministic seed dispersal (calculations) [=..] !DSP
  IF (err .ne. 0) RETURN
  CALL RWInt (OuterSeeds, err)          !----- = 30       ! No. of seeds (per patch) coming from outerspace
  IF (err .ne. 0) RETURN

  !----- Seed production and seed bank dynamics
  CALL RWLine
  CALL RWLog (seedsDifferent, err)      !----- =  .FALSE.   ! different seed production
  IF (err .ne. 0) RETURN
  CALL RWLog (mathgtDifferent, err)     !----- different maturation height
  IF (err .ne. 0) RETURN
  CALL RWLog (mastSeeding, err)         !----- =  .FALSE.   ! periodic seed production include (calculations)   [=..] !DSP
  IF (err .ne. 0) RETURN
  CALL RWLog (seedBank, err)            !----- =  .FALSE.   ! seedbank dynamics included?
  IF (err .ne. 0) RETURN


  !----- Density regulation
  IF (err .ne. 0) RETURN
  CALL RWLine
  IF (err .ne. 0) RETURN
  CALL RWReal (seedlcrowndiamInCm, err)    !----- = 25.0    ! Alpha-value of kernel [=..] !DSP
  IF (err .ne. 0) RETURN
  CALL RWLog (withSeedAntagonists, err)    !----- = .FALSE. ! seed antagonists (predators or pathogens) included?
  IF (err .ne. 0) RETURN
  CALL RWReal (seedAntaGraz, err)          !----- = 0.1     ! grazing rate of seed antagonists
  IF (err .ne. 0) RETURN
  CALL RWReal (seedAntaEff, err)           !----- = 0.1     ! efficiency of grazing of seed antagonists
  IF (err .ne. 0) RETURN
  CALL RWReal (seedAntaMort, err)          !----- = 0.1     ! mortality of seed antagonists
  IF (err .ne. 0) RETURN
  CALL RWReal (seedAntaRain, err)          !----- constant influx of seed antagonists
  IF (err .ne. 0) RETURN
  CALL RWLog (withSeedCarrCap, err)        !----- = .FALSE. ! seed carrying capacity included?
  IF (err .ne. 0) RETURN
  CALL RWReal (seedCarrCap, err)           !----- carrying capacity for seeds
  IF (err .ne. 0) RETURN
  seedCarrCap = seedCarrCap * pltsiz / 833.0

  !----- Immigration
  CALL RWLine
  CALL RWLog (immigration, err)            !----- =  .TRUE. immigration included?
  IF (err .ne. 0) RETURN
  CALL RWText15 (immifilename, err)        !----- file for species immigration
  IF (err .ne. 0) RETURN

  !----- Disturbances
  CALL RWLine
  IF (err .ne. 0) RETURN
  CALL RWLog (disturbances, err)           !----- = .TRUE.   disturbances included?
  IF (err .ne. 0) RETURN
  CALL RWReal (disturbProb, err)           !----- =0.1  disturbance probability per cell and year
  IF (err .ne. 0) RETURN
  CALL RWReal (disturbIntensity, err)      !----- =0.1  disturbance intensity (proportion of trees which die)
  IF (err .ne. 0) RETURN

  !----- Clumping
  CALL RWLine
  IF (err .ne. 0) RETURN
  CALL RWReal (fcvarInit, err)            !----- = 1.0  Fac.variability for light freq.  initial simulations
  IF (err .ne. 0) RETURN
  CALL RWReal (fcvarDyn, err)             !----- = 1.0  Fac.variability for light freq.  spatial simulations
  IF (err .ne. 0) RETURN


  !----- Light dependence
  CALL RWLine
  IF (err .ne. 0) RETURN
  CALL RWLog (estabEqHC0, err)            !----- = .FALSE.  pply establ. light dep. for seedlings life (TRUE) or germination  ?
  IF (err .ne. 0)    RETURN
  CALL RWLog( contLightDepGerm, err)      !----- use a continuous light dependence for germination  ?
  IF (err .ne. 0) RETURN

  !----- Species input file
  CALL RWLine
  CALL RWText15 (specfilename, err)       !----- file for species parameters
  IF (err .ne. 0) RETURN



  !----- Output
  CALL RWLine
  CALL RWLog (writeOutput%biomass, err)      !----- write biomass?
  IF (err .ne. 0) RETURN
  CALL RWLog (writeOutput%number, err)       !----- write number?
  IF (err .ne. 0) RETURN
  CALL RWLog (writeOutput%hstruct, err)      !----- write height structure?
  IF (err .ne. 0) RETURN
  CALL RWLog (writeOutput%seeds, err)        !----- write seeds?
  IF (err .ne. 0) RETURN
  CALL RWLog (writeOutput%antagonists, err)  !----- write antagonists?
  IF (err .ne. 0) RETURN
  CALL RWLog (writeOutput%pollen, err)       !----- write pollen?
  IF (err .ne. 0) RETURN
  CALL RWLog (writeOutput%ingrowth, err)     !----- write ingrowth?
  IF (err .ne. 0) RETURN
  CALL RWLog (writeOutput%biodiv, err)       !----- write ingrowth?
  IF (err .ne. 0) RETURN
  CALL RWLog (writeOutput%leafarea, err)       !----- write leaf area? FORHYCS
  IF (err .ne. 0) RETURN
  CALL RWLog (tabsepOutput, err)             !----- file for species parameters
  IF (err .ne. 0) RETURN
  controlfileisokay = .TRUE.
  ! Plotting
  CALL RWLine
  CALL RWLog (writeOutput%plot, err)         !----- plot during simulation?
  IF (err .ne. 0) RETURN
  CALL RWInt (maxrowscols, err)               !----- max. rows and colums in plot
  IF (err .ne. 0) RETURN
  CALL RWReal (maxDataInPlot, err)             !-----  max biomass for each stacked column in plot


  IF (err .ne. 0) RETURN



  CLOSE(UNIT=1011)
  CLOSE(UNIT=1051)
  print *, 'normal exit of readcontrolpars'


END SUBROUTINE ReadControlPars
! ****<Here the subroutine ends>******************************************************************



! ****<Here the subroutine CALL RWLine for input and output of integers starts>*******************
!-------------------------------------------------------------------------------------------------
SUBROUTINE RWLine

!<CALL modules for type definitions>---------------------------------------------------------------
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------

!<Local variables>---------------------------------------------------------------------------------
Character*80 :: line

!<Here the subroutine starts>**********************************************************************
  READ(1011,"(A80)") line
  WRITE(1051,"(A80)") line

END SUBROUTINE RWLine
! ****<Here the subroutine ends>*******************************************************************




! ****<Here the subroutine RWInt for input and output of integers starts>**************************
!--------------------------------------------------------------------------------------------------
SUBROUTINE RWInt(ivalue, err)

!<CALL modules for type definitions>---------------------------------------------------------------
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(OUT) :: ivalue, err

!<Local variables>---------------------------------------------------------------------------------
Character*70 :: text

!<Here the subroutine starts>**********************************************************************
  READ(1011,100,iostat=err) ivalue, text
  IF (err .ne. 0) RETURN
  WRITE(1051,100) ivalue, text

  100  FORMAT(I15,A70)
END SUBROUTINE RWInt
! ****<Here the subroutine ends>*******************************************************************



! ****<Here the subroutine RWReal for input and output of reals starts>****************************
!--------------------------------------------------------------------------------------------------
SUBROUTINE RWReal(rvalue, err)

!<CALL modules for type definitions>---------------------------------------------------------------
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
REAL, INTENT(OUT)    :: rvalue
INTEGER, INTENT(OUT) :: err

!<Local variables>---------------------------------------------------------------------------------
Character*70 :: text

!<Here the subroutine starts>**********************************************************************
  READ(1011,200,iostat=err) rvalue, text
  IF (err .ne. 0) RETURN
  WRITE(1051,200) rvalue, text

  200 FORMAT(F15.3,A70)
END SUBROUTINE RWReal
! ****<Here the subroutine ends>*******************************************************************




! ****<Here the subroutine RWLog for input and output of logicals starts>**************************
SUBROUTINE RWLog(lvalue,err)

!<CALL modules for type definitions>---------------------------------------------------------------
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(OUT) :: err
LOGICAL, INTENT(OUT) :: lvalue

!<Local variables>---------------------------------------------------------------------------------
Character*70 :: text

!<Here the subroutine starts>**********************************************************************
  READ(1011,400,iostat=err) lvalue, text
  IF (err .ne. 0)  RETURN
  WRITE(1051,400) lvalue, text

  400 FORMAT(14X,L1,A70)
END SUBROUTINE RWLog
! ****<Here the subroutine ends>*******************************************************************



! ****<Here the subroutine RWLog for input and output of logicals starts>**************************
!--------------------------------------------------------------------------------------------------
SUBROUTINE RWText1(text1,err)

!<CALL modules for type definitions>---------------------------------------------------------------
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(OUT)     :: err
Character*1, INTENT(OUT) :: text1

!<Local variables>---------------------------------------------------------------------------------
Character*70 text

!<Here the subroutine starts>**********************************************************************
  READ(1011,500,iostat=err) text1, text
  IF (err .ne. 0) RETURN
  WRITE(1051,500) text1, text

  500 FORMAT(14X,A1,A70)
END SUBROUTINE RWText1
! ****<Here the subroutine ends>*******************************************************************



! ****<Here the subroutine RWLog for input and output of logicals starts>**************************
!--------------------------------------------------------------------------------------------------
SUBROUTINE RWText15(text15,err)

!<CALL modules for type definitions>---------------------------------------------------------------
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(OUT)      :: err
Character*15, INTENT(OUT) :: text15

!<Local variables>---------------------------------------------------------------------------------
Character*70 :: text

!<Here the subroutine starts>**********************************************************************
  READ(1011,600,iostat=err) text15, text
  IF (err .ne. 0) RETURN
  WRITE(1051,600) text15, text

  600 FORMAT(A15,A70)
END SUBROUTINE RWText15
! ****<Here the subroutine ends>*******************************************************************
