!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------



!==============================================================================
!
!  Name   :  ReadImmigrationData
!
!  Purpose:  Reads species parameters and calculates constant species specific
!            values from them
!
!  Remarks:  contains the subroutine
!
!            ReadImmigrationData (<GetSpc<TreeMig)
!
!==============================================================================
! Comments:
! Author  Date      Description
! ------  ----      -----------
! tjl     11.11.02  text1 is read, but never used
!
!==============================================================================

SUBROUTINE ReadImmigrationData(nspc, errtypimmi)

!<CALL modules for typedefinitions>---------------------------------------------------------------
USE All_par, ONLY: simustartyear, experimentID,                     &
                   spec, spc,                                       &
                   immigration, immiPeriod, maxImmigrationsPerSpec, &
                   immifilename,                                    &
                   tabsepOutput, pathletter1, pathletter2

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER,INTENT(IN) :: nspc        ! number of species   [=spec]
INTEGER,INTENT(OUT):: errtypimmi  ! number of species   [=spec]

!<Local variables>---------------------------------------------------------------------------------
INTEGER       :: i,j, &           ! looping variable for species and immigration dates
                 err, &
                 imdat(maxImmigrationsPerSpec)
CHARACTER(27) :: text1

!<Here the subroutine starts>**********************************************************************
  errtypimmi = 0
  WRITE(1070,*) 'in readimmidata', immigration

  !-----<Check for file (opening) errors>
  IF (immigration) THEN
    CALL OpenExistingFile (pathletter1//'P'//pathletter2//immifilename, 1013, err)
    IF (err .ne. 0) THEN
      print *, 'immifile not opened '
      errtypimmi = 3
      RETURN
    END IF
    OPEN(UNIT=1053, FILE=pathletter1//'R'//pathletter2//experimentID//'_'//'immi.pars.txt', STATUS='UNKNOWN')

   !----- Read immigration period
   READ(1013,"(A)")
   READ(1013, *) immiPeriod

    !----- Skip first
    READ(1013,"(A)")
    READ_SpecImmi: DO i = 1, nspc
      READ (1013,*,IOSTAT=err) text1, spec(i)%immi%n,&
                             (imdat(j), spec(i)%immi%dat(j)%lat, spec(i)%immi%dat(j)%lon, j=1, spec(i)%immi%n)

      IF (spec(i)%immi%n .gt. maxImmigrationsPerSpec) THEN
        WRITE(1070,*) 'more immigration points ', spec(i)%immi%n, 'than allowed ', maxImmigrationsPerSpec
        errtypimmi = 3
        RETURN
      END IF

      !----- If EOF, then exit the read loop
      Exit_Loop1: IF (err /= 0) THEN
        EXIT Read_SpecImmi
      END IF Exit_Loop1

      DO j = 1, spec(i)%immi%n
        spec(i)%immi%dat(j)%year = - simustartyear + imdat(j)
!----->OLD spec(i)%immi%dat(j)%lat,spec(i)%immi%dat(j)%lon
      END DO

      WRITE(1053,*) spec(i)%immi%n, (spec(i)%immi%dat(j)%year,&
                  spec(i)%immi%dat(j)%lat,spec(i)%immi%dat(j)%lon,j = 1, spec(i)%immi%n)
    END DO Read_SpecImmi

    CLOSE(1013)
    CLOSE(1053)
  ELSE   !----- immigration

    spec%immi%n = 0

  END IF !----- immigration
END SUBROUTINE ReadImmigrationData
!****<Here the subroutine ends>********************************************************************
