!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: New output format (tables or grids)

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!==============================================================================
!
!  Name   :  Report
!
!  Remarks:  contains the subroutine
!
!            Report (<DoLocalWork<DistrLocalDynThisTime<TimeLoop<TreeMig)
!
!==============================================================================
! design          : H. Lischke, N. Zimmermann
! author(s)       : H. Lischke, N. Zimmermann
! implementation  : H. Lischke, N. Zimmermann
! cleaner         : T.J. Loeffler
! copyright       : (c) 1999, 03 by H. Lischke
!==============================================================================

SUBROUTINE Report(nspc, year, lat, lon, Iam)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: cell, seedrain,spec, spc,   tabsepOutput, pltsiz, writeOutput, maxhc,&
                        stockability, rptdist, rptint,numyrs,simustartyear,&
                        avbio0, lonRealStart, latRealStart! ,avbio1,avbio2,avbio3

use forhycsglobals, ONLY: ddegsum5old,muwint,drstrold,drstrold2,meantold,sumpold,EBJSout,EPJSout,EIJSout,&
                          biomass2d,number2d,seeds2d,ingrowth2d,antagon2d,pollen2d,la2d,reptTables,reptGrids,&
                          biomassTable,numberTable,seedsTable,ingrowthTable,antagonTable,pollenTable,laTable
use prevahglobals, only: fslope,faspect,nrow,ncol
use MPIvars, only: firstRow,tableStart

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN)    :: nspc, &    ! Passed number of species
                          year, &    ! year of calculation           [=..]
                          lat,lon, Iam

!<Local variables>---------------------------------------------------------------------------------
INTEGER     :: height, &  ! loop-counter
               i,      &  ! loop-counter
               iunit ,&     ! format variable
               rpti

REAL        :: bio(nspc),                       &  ! species biomass
               num(nspc),                       &  ! species number
               pollen(nspc), pol4perc(nspc),    &  ! pollen amount and 4*percent
               sumpollen,                       &
               sumbio, diversity,               &
               sumnumber, sumseeds, sumantagon,sumingrowth, &
               BiodivIndex, sumnumbh(0:maxhc),lai(nspc),sumlai, rbio,yr, &
               la(nspc), sumla, lonreal, latreal !FORHYCS
real :: nums(0:maxhc)
integer, parameter, dimension(16) :: hcinds = (/0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15/)

CHARACTER*1 :: tab
character(21) :: smpl
character(4) :: chyr
character(6) :: chcell
integer :: nhcls
character(7) :: charspc
!<Here the subroutine starts>**********************************************************************
     IF (tabsepOutput) THEN
	  tab = CHAR(9)
	 ELSE
	  tab=','
	 END IF

 !----- Calculate av. biomasses
  yr=year+simustartyear
   !if ((lat==1).and.(lon==1))   print *, 'year, yr, simustartyear in report', year,yr, simustartyear
   BySp : DO i=1,nspc
           DO height=1,spec(i)%maxhtc
 	         rbio= (cell(lat,lon)%sp(i)%numin(height )) * (spc(i)%bioin(height))
	!		 if ( yr>=2003.0  )  print *, 'yr, lat, lon, i, height, rbio, ' ,yr, lat, lon, i,  height,rbio
 	         if (year ==1) THEN
 	          avbio0(lat,lon,i)=0.
  !            avbio1(lat,lon,i)=0.
  !            avbio2(lat,lon,i)=0.
 !             avbio3(lat,lon,i)=0.
             END IF
             if ((yr>=1980.0).and.(yr<=2005.0)) THEN
                       avbio0(lat,lon,i)= avbio0(lat,lon,i) + rbio/26.
!                       if (rbio>0.) print *, 'yr, lat, lon, i, rbio, avbio0 ' ,yr, lat, lon, i, rbio, avbio0(lat,lon,i)
             END IF
 	!         if (yr>=2020.and.yr<=2049) avbio1(lat,lon,i)= avbio1(lat,lon,i) + rbio/30.
 	 !        if (yr>=2045.and.yr<=2074) avbio2(lat,lon,i)= avbio2(lat,lon,i) + rbio/30.
 	  !       if (yr>=2070.and.yr<=2099) avbio3(lat,lon,i)= avbio3(lat,lon,i) + rbio/30.
	  END DO !height
      END DO BySp

      if (yr ==2005) THEN
           sumbio=0.
         DO i=1,nspc
           sumbio=sumbio+avbio0 (lat,lon,i)
         END DO
        IF (sumbio>0.) WRITE(1244,100)  lat, tab, lon, tab, year, tab, (avbio0(lat,lon,i) ,tab, i=1,nspc), sumbio
      END IF ! yr== 2005
!      if (yr ==2049) THEN
!         sumbio=0.
!         DO i=1,nspc
!           sumbio=sumbio+avbio1 (lat,lon,i)
!         END DO
!         WRITE(1241,100)  lat, tab, lon, tab, year, tab, (avbio1(lat,lon,i) ,tab, i=1,nspc), sumbio
!     END IF ! yr== 2049
!     if (yr ==2074) THEN
!         sumbio=0.
!         DO i=1,nspc
!           sumbio=sumbio+avbio2 (lat,lon,i)
!         END DO
!         WRITE(1242,100)  lat, tab, lon, tab, year, tab, (avbio2(lat,lon,i) ,tab, i=1,nspc), sumbio
!     END IF ! yr== 2074
!     if (yr ==2099) THEN
!         sumbio=0.
!         DO i=1,nspc
!           sumbio=sumbio+avbio3(lat,lon,i)
!         END DO
!         WRITE(1243,100)  lat, tab, lon, tab, year, tab, (avbio3(lat,lon,i) ,tab, i=1,nspc), sumbio
!  END IF ! yr== 2099


! Output whole structure (trees per species and height class) for selected cells

! write(chyr,'(I4)') int(yr)
! charspc="Species"
! smpl = "sample_54133_"//chyr//".txt"
! nhcls = maxhc+1
! if((lat==133).and.(lon==54)) then
!   open(888,file=smpl,status="new")
!   write(888,'(A7,X,<nhcls>(I2,X))') charspc,hcinds
!   do i=1,30
!   nums = 0.0
!     do height = 0,maxhc
!       nums(height) = cell(lat,lon)%sp(i)%numin(height)
!     end do
!     write(888,'(I2,X,<nhcls>(f12.5,X))') i, nums
!
!   end do
!
! end if
!
! smpl = "sample_72172_"//chyr//".txt"
! nhcls = maxhc+1
! if((lat==172).and.(lon==72)) then
!   open(889,file=smpl,status="new")
!   write(889,'(A7,X,<nhcls>(I2,X))') charspc,hcinds
!   do i=1,30
!   nums = 0.0
!     do height = 0,maxhc
!       nums(height) = cell(lat,lon)%sp(i)%numin(height)
!     end do
!     write(889,'(I2,X,<nhcls>(f12.5,X))') i, nums
!
!   end do
!
! end if
!


! Output more frequent between 1950 and 2100
  IF (year>=1500.and.year<=1700)  THEN
      rpti=5
   ELSE
      rpti=rptint
  END IF

 IF (   (MOD (year, rpti ) == 0)   &
            .AND.   (MOD (lat, rptdist) ==  0)  .AND.  (stockability(lat,lon) > 0.0)    ) THEN
          sumbio     = 0
          sumnumber  = 0
	  sumseeds   = 0
	  sumantagon = 0
	  sumpollen  = 0
	  sumingrowth= 0
	  sumnumbh   = 0
          sumlai     = 0
          	sumla =0 !FORHYCS

	  !----- Calculate biomass and density sums over species in heights and total
	  BySpc: DO i=1,nspc
	    bio(i)    = 0
	    num(i)    = 0
	    pollen(i) = 0
            lai(i)    = 0
            	la(i) = 0 !FORHYCS



	    DO height=1,spec(i)%maxhtc
 	      bio(i)    = bio(i) + (cell(lat,lon)%sp(i)%numin(height )) * (spc(i)%bioin(height))
	      num(i)    = num(i) + cell(lat,lon)%sp(i)%numin(height )
	      pollen(i) = pollen(i) + cell(lat,lon)%sp(i)%numin(height ) * &
	                  spc(i)%mxsdht(height) * spec(i)%repFac
	      sumnumbh(height)=sumnumbh(height)+num(i)
              lai(i)    =lai(i)+ cell(lat,lon)%sp(i)%numin(height )* spc(i)%lain(height)/833.0
              	la(i)    =la(i)+ cell(lat,lon)%sp(i)%numin(height )* spc(i)%lain(height)/2.0 !FORHYCS
 	    END DO

 	    if((lon==54).and.(lat==133)) then
 	      print*, "In REPORT"
 	      print*, "bio", i, bio(i), year

 	    end if

!	    sumseeds   = sumseeds + cell(lat,lon)%sp(i)%seedbed
          if(i .le. nspc -1) then
	      sumseeds   = sumseeds + seedrain(lat,lon)%sp(i)%newseeds
	      sumingrowth= sumingrowth + cell(lat,lon)%sp(i)%ingrowth
	      sumantagon = sumantagon + cell(lat,lon)%sp(i)%antagonist
	      sumbio     = sumbio + bio(i)
	      sumnumber  = sumnumber + num(i)
	      sumpollen  = sumpollen + pollen(i)
             sumlai     = sumlai+ lai(i)
             sumla  = sumla + la(i)/2.0  !FORHYCS
          end if
	  END DO BySpc

	  !----- Calculate pollen percent
	  IF (ABS (sumpollen) < 1.E-20) THEN
	    pol4perc = 0.0
	  ELSE
	    pol4perc = 100.0 * pollen / sumpollen
	  END IF

	 IF (tabsepOutput) THEN
	  tab = CHAR(9)
	 ELSE
	  tab=','
	 END IF

	 !----- Output of diversity per species
	 IF (writeOutput%biodiv) THEN
	    diversity = BiodivIndex (nspc, bio, sumbio)
	  !  WRITE(1042,100)   tab, lat, tab, lon, tab, year, tab, diversity
	  END IF

	 !----- Output of biomass or density per species

	 IF (reptTables .and. writeOutput%biomass) then
 	  ! WRITE(1020,100)    lat, tab, lon, tab, year, tab, &
	  !               (bio(i),tab, i=1,nspc), sumbio
	    biomassTable(1:nspc,tableStart+((lat-firstRow)*ncol)+lon-1) = bio(1:nspc)
	    biomassTable(nspc+1,tableStart+((lat-firstRow)*ncol)+lon-1) = sumbio
	 end if

	 !if(reptGrids .and. writeOutput%biomass) then

	 ! Modified Dec 2017 - Matthias
	 ! biomass2d is now updated in the new function bioKaufmann
	 ! -> commented out the 2 following lines:

	   !biomass2d(lon,lat,1:nspc) = bio(1:nspc)
	   !biomass2d(lon,lat,nspc+1) = sumbio

	 !end if
	                 !NZ: if biomass was set to stem volume in subroutine Biomass,
	                 ! 'sumbio' here will be dm3/ha (transformation from m3 per plot to dm3/ha
	                 ! is in subroutine GetStd)
	                 ! divide by 1000 if you are using stem volume and need m3 instead of dm3

	 !IF (writeOutput%number) &
	 !  WRITE(1021,100)   lat, tab, lon, tab, year, tab, &
	 !                (num(i)*10000/pltsiz,tab, i=1,nspc), sumnumber * 10000 / pltsiz   ! number per ha

	 IF (reptTables .and. writeOutput%number) then
	    numberTable(1:nspc,tableStart+((lat-firstRow)*ncol)+lon-1) = num(1:nspc)*10000/pltsiz
	    numberTable(nspc+1,tableStart+((lat-firstRow)*ncol)+lon-1) = sumnumber*10000/pltsiz
	 end if

	 if(reptGrids .and. writeOutput%number) then
	   number2d(lon,lat,1:nspc) = num(1:nspc)*10000/pltsiz
	   number2d(lon,lat,nspc+1) = sumnumber * 10000 / pltsiz
	 end if

	 !----- output of the seedbed
!	 IF (writeOutput%seeds) &
!	   WRITE(1022,100)  lat, tab, lon, tab, year, tab, &
	!----->               (seedbed(i,lat,lon), tab, i=1,nspc), tab, sumseeds
!	                 (cell(lat,lon)%sp(i)%sb, tab, i=1,nspc), sumseeds

	 IF (reptTables .and. writeOutput%seeds) then
	    seedsTable(1:nspc,tableStart+((lat-firstRow)*ncol)+lon-1) = cell(lat,lon)%sp(1:nspc)%sb
	    seedsTable(nspc+1,tableStart+((lat-firstRow)*ncol)+lon-1) = sumseeds
	 end if

	 if(reptGrids .and. writeOutput%seeds) then
	   seeds2d(lon,lat,1:nspc) = cell(lat,lon)%sp(1:nspc)%sb
	   seeds2d(lon,lat,nspc+1) = sumseeds
	 end if

	 !----- output of the ingrowth
!	  IF (writeOutput%ingrowth) &
!	    WRITE(1025,100)   lat, tab, lon, tab, year, tab, &
!	                  (cell(lat,lon)%sp(i)%ingrowth , tab, i=1,nspc), sumingrowth

	 IF (reptTables .and. writeOutput%ingrowth) then
	    ingrowthTable(1:nspc,tableStart+((lat-firstRow)*ncol)+lon-1) = cell(lat,lon)%sp(1:nspc)%ingrowth
	    ingrowthTable(nspc+1,tableStart+((lat-firstRow)*ncol)+lon-1) = sumingrowth
	 end if

	 if(reptGrids .and. writeOutput%ingrowth) then
	   ingrowth2d(lon,lat,1:nspc) = cell(lat,lon)%sp(1:nspc)%ingrowth
	   ingrowth2d(lon,lat,nspc+1) = sumingrowth
	 end if

	  !----- output of the antagonists
!	  IF (writeOutput%antagonists)&
!	    WRITE(1023,100)   lat, tab, lon, tab, year, tab, &
!	                  (cell(lat,lon)%sp(i)%antagonist , tab, i=1,nspc), sumantagon

	 IF (reptTables .and. writeOutput%antagonists) then
	    antagonTable(1:nspc,tableStart+((lat-firstRow)*ncol)+lon-1) = cell(lat,lon)%sp(1:nspc)%antagonist
	    antagonTable(nspc+1,tableStart+((lat-firstRow)*ncol)+lon-1) = sumantagon
	 end if

	 if(reptGrids .and. writeOutput%antagonists) then
	   antagon2d(lon,lat,1:nspc) = cell(lat,lon)%sp(1:nspc)%antagonist
	   antagon2d(lon,lat,nspc+1) = sumantagon
	 end if

	  !----- output of the pollen percents
!	  IF (writeOutput%pollen) &
!	    WRITE(1024,100)   lat, tab, lon, tab, year, tab, &
!	                    (pol4perc(i),tab, i=1,nspc), sumpollen
!	      !(lai(i),tab,  i=1,nspc), sumlai

	 IF (reptTables .and. writeOutput%pollen) then
	    pollenTable(1:nspc,tableStart+((lat-firstRow)*ncol)+lon-1) = pol4perc(1:nspc)
	    pollenTable(nspc+1,tableStart+((lat-firstRow)*ncol)+lon-1) = sumpollen
	 end if

	 if(reptGrids .and. writeOutput%pollen) then
	   pollen2d(lon,lat,1:nspc) = pol4perc(1:nspc)
	   pollen2d(lon,lat,nspc+1) = sumpollen
	 end if

	  !----- output of the leaf area  FORHYCS
!	  IF (writeOutput%leafarea) &
!	    WRITE(1026,100)   lat, tab, lon, tab, year, tab, &
!	      (la(i),tab,  i=1,nspc), sumla

	 IF (reptTables .and. writeOutput%leafarea) then
	    laTable(1:nspc,tableStart+((lat-firstRow)*ncol)+lon-1) = la(1:nspc)
	    laTable(nspc+1,tableStart+((lat-firstRow)*ncol)+lon-1) = sumla
	 end if

	 if(reptGrids .and. writeOutput%leafarea) then
	   la2d(lon,lat,1:nspc) = la(1:nspc)
	   la2d(lon,lat,nspc+1) = sumla
	 end if


	  !----- output of the height structure
	  IF(writeOutput%hstruct) THEN
	    DO i=1, nspc
	      sumnumbh(0)=sumnumbh(0)+cell(lat,lon)%sp(i)%numin(0 )
	    END DO
	    DO height=0,maxhc
	      iunit = 1166 + height
	      WRITE(iunit,100)  lat, tab, lon, tab, year, tab, &
	            (cell(lat,lon)%sp(i)%numin(height ), tab, i=1,nspc) , sumnumbh(height)
	    END DO
	  END IF
    END IF ! ((MOD (year, rptint) == 0) .AND ...
!  100 FORMAT ((I8,A1),3(I5,A1), 31(E18.7,A1))
 100 FORMAT ( 3(I5,A1), 31(E13.7,A1))


!FORHYCS

lonreal=lon*200+lonRealStart-200
latreal=lat*200+latRealStart-200

!if ( (int(yr+1).ge.1976).and.(int(yr+1).le.2010) ) then
if ( (int(yr).ge.1971).and.(int(yr).le.2010) ) then

 if ((lonreal.lt.100000).and.(latreal.ge.100000))  WRITE(1237,101) int(lonreal),",",int(latreal),",",int(yr),",",ddegsum5old(lon,lat),",",muwint(lon,lat,4),",",drstrold(lon,lat),",",drstrold2(lon,lat),",",EBJSout(lon,lat),",",EPJSout(lon,lat),",",EIJSout(lon,lat)
 if ((lonreal.ge.100000).and.(latreal.lt.100000))  WRITE(1237,102) int(lonreal),",",int(latreal),",",int(yr),",",ddegsum5old(lon,lat),",",muwint(lon,lat,4),",",drstrold(lon,lat),",",drstrold2(lon,lat),",",EBJSout(lon,lat),",",EPJSout(lon,lat),",",EIJSout(lon,lat)
 if ((lonreal.lt.100000).and.(latreal.lt.100000))  WRITE(1237,103) int(lonreal),",",int(latreal),",",int(yr),",",ddegsum5old(lon,lat),",",muwint(lon,lat,4),",",drstrold(lon,lat),",",drstrold2(lon,lat),",",EBJSout(lon,lat),",",EPJSout(lon,lat),",",EIJSout(lon,lat)
 if ((lonreal.ge.100000).and.(latreal.ge.100000))  WRITE(1237,104) int(lonreal),",",int(latreal),",",int(yr),",",ddegsum5old(lon,lat),",",muwint(lon,lat,4),",",drstrold(lon,lat),",",drstrold2(lon,lat),",",EBJSout(lon,lat),",",EPJSout(lon,lat),",",EIJSout(lon,lat)

 if ((lonreal.lt.100000).and.(latreal.ge.100000))  WRITE(1238,105) int(lonreal),",",int(latreal),",",int(yr),",",fslope(lon,lat),",",faspect(lon,lat),",",meantold(lon,lat,1),",",meantold(lon,lat,2),",",meantold(lon,lat,3),",",meantold(lon,lat,4),",",meantold(lon,lat,5),",",meantold(lon,lat,6),",",meantold(lon,lat,7),",",meantold(lon,lat,8),",",meantold(lon,lat,9),",",meantold(lon,lat,10),",",meantold(lon,lat,11),",",meantold(lon,lat,12),",",sumpold(lon,lat,1),",",sumpold(lon,lat,2),",",sumpold(lon,lat,3),",",sumpold(lon,lat,4),",",sumpold(lon,lat,5),",",sumpold(lon,lat,6),",",sumpold(lon,lat,7),",",sumpold(lon,lat,8),",",sumpold(lon,lat,9),",",sumpold(lon,lat,10),",",sumpold(lon,lat,11),",",sumpold(lon,lat,12)
 if ((lonreal.ge.100000).and.(latreal.lt.100000))  WRITE(1238,106) int(lonreal),",",int(latreal),",",int(yr),",",fslope(lon,lat),",",faspect(lon,lat),",",meantold(lon,lat,1),",",meantold(lon,lat,2),",",meantold(lon,lat,3),",",meantold(lon,lat,4),",",meantold(lon,lat,5),",",meantold(lon,lat,6),",",meantold(lon,lat,7),",",meantold(lon,lat,8),",",meantold(lon,lat,9),",",meantold(lon,lat,10),",",meantold(lon,lat,11),",",meantold(lon,lat,12),",",sumpold(lon,lat,1),",",sumpold(lon,lat,2),",",sumpold(lon,lat,3),",",sumpold(lon,lat,4),",",sumpold(lon,lat,5),",",sumpold(lon,lat,6),",",sumpold(lon,lat,7),",",sumpold(lon,lat,8),",",sumpold(lon,lat,9),",",sumpold(lon,lat,10),",",sumpold(lon,lat,11),",",sumpold(lon,lat,12)
 if ((lonreal.lt.100000).and.(latreal.lt.100000))  WRITE(1238,107) int(lonreal),",",int(latreal),",",int(yr),",",fslope(lon,lat),",",faspect(lon,lat),",",meantold(lon,lat,1),",",meantold(lon,lat,2),",",meantold(lon,lat,3),",",meantold(lon,lat,4),",",meantold(lon,lat,5),",",meantold(lon,lat,6),",",meantold(lon,lat,7),",",meantold(lon,lat,8),",",meantold(lon,lat,9),",",meantold(lon,lat,10),",",meantold(lon,lat,11),",",meantold(lon,lat,12),",",sumpold(lon,lat,1),",",sumpold(lon,lat,2),",",sumpold(lon,lat,3),",",sumpold(lon,lat,4),",",sumpold(lon,lat,5),",",sumpold(lon,lat,6),",",sumpold(lon,lat,7),",",sumpold(lon,lat,8),",",sumpold(lon,lat,9),",",sumpold(lon,lat,10),",",sumpold(lon,lat,11),",",sumpold(lon,lat,12)
 if ((lonreal.ge.100000).and.(latreal.ge.100000))  WRITE(1238,108) int(lonreal),",",int(latreal),",",int(yr),",",fslope(lon,lat),",",faspect(lon,lat),",",meantold(lon,lat,1),",",meantold(lon,lat,2),",",meantold(lon,lat,3),",",meantold(lon,lat,4),",",meantold(lon,lat,5),",",meantold(lon,lat,6),",",meantold(lon,lat,7),",",meantold(lon,lat,8),",",meantold(lon,lat,9),",",meantold(lon,lat,10),",",meantold(lon,lat,11),",",meantold(lon,lat,12),",",sumpold(lon,lat,1),",",sumpold(lon,lat,2),",",sumpold(lon,lat,3),",",sumpold(lon,lat,4),",",sumpold(lon,lat,5),",",sumpold(lon,lat,6),",",sumpold(lon,lat,7),",",sumpold(lon,lat,8),",",sumpold(lon,lat,9),",",sumpold(lon,lat,10),",",sumpold(lon,lat,11),",",sumpold(lon,lat,12)

101 FORMAT ((I5,A1),(I6,A1),(I4,A1),7(E13.7,A1))
102 FORMAT ((I6,A1),(I5,A1),(I4,A1),7(E13.7,A1))
103 FORMAT ((I5,A1),(I5,A1),(I4,A1),7(E13.7,A1))
104 FORMAT ((I6,A1),(I6,A1),(I4,A1),7(E13.7,A1))
105 FORMAT ((I5,A1),(I6,A1),(I4,A1),26(E13.7,A1))
106 FORMAT ((I6,A1),(I5,A1),(I4,A1),26(E13.7,A1))
107 FORMAT ((I5,A1),(I5,A1),(I4,A1),26(E13.7,A1))
108 FORMAT ((I6,A1),(I6,A1),(I4,A1),26(E13.7,A1))
end if




END SUBROUTINE Report
!****<Here the subroutine ends>********************************************************************
