!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!==============================================================================
!
!  Name   :  SpecImmigration
!
!  Purpose:  determines the immigration of seedlings
!
!  Remarks:  contains the subroutine
!
!            SpecImmigration (<LocalForestDynamics<DoLocalWork<
!                               DistrLocalDynThisTime<TimeLoop<TreeMig)
!
!==============================================================================


SUBROUTINE SpecImmigration(nspc, year, idsp,  thiscell)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: potbirth, pltsiz,                      &
                   maxlat, maxlon,                                      &
                   spec, immiPeriod,                      &
                    immigration  , CurrStateInCell
                                                   ! Use control, species and stand  parameters
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN) :: nspc , year           ! number of species                                  [=spec]
Logical, INTENT(IN) :: idsp

!<Local variables>---------------------------------------------------------------------------------
INTEGER :: immilatcoord, immiloncoord, &      !  looping variable for species and light
           iimm, i
TYPE(CurrStateInCell):: thiscell

!<Here the subroutine starts>**********************************************************************
  BySpecies: DO i = 1, nspc
   IF (idsp) THEN
     IF (immigration) THEN
      !----- Immigration (Seed inflow) at certain times and cells
      DO iimm = 1, spec(i)%immi%n
        immilatcoord = INT (MAX (1.0, maxlat * spec(i)%immi%dat(iimm)%lat))
        immiloncoord = INT (MAX (1.0, maxlon * spec(i)%immi%dat(iimm)%lon))

        IF ((thiscell%lat == immilatcoord)                              .and. &
            (thiscell%lon == immiloncoord)                              .and. &
            ( year >= spec(i)%immi%dat(iimm)%year - immiPeriod) .and. &
            ( year <  spec(i)%immi%dat(iimm)%year + immiPeriod)) THEN
           thiscell%sp(i)%numin(0) = thiscell%sp(i)%numin(0) + 1000.0 * pltsiz / 833.0
        END IF
      END DO !----- iimm
     END IF !----- immigration
   ELSE !----- not idsp
     IF (immigration) THEN
       IF ( year >= spec(i)%immi%dat(1)%year) THEN
         spec(i)%birth = potbirth
       ELSE
         spec(i)%birth = 0
       END IF
     ELSE !----- not immigration
       spec(i)%birth = potbirth
     END IF !----- IF immigration
   END IF !----- immigration
  END DO BySpecies

END Subroutine SpecImmigration
!****<Here the subroutine ends>********************************************************************
