!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!==============================================================================
!
!  Name   :  StoringBackAndForth (sequential only; MPI is in separate file)
!
!  Remarks:  contains the subroutines
!
!            StoreCellStateToThisCell (<DoLocalWork<DistrLocalDynThisTime<
!                                        TimeLoop<TreeMig)
!            StoreThisCellSpecStatesToCell (<DoLocalWork<DistrLocalDynThisTime<
!                                             TimeLoop<TreeMig)
!
!==============================================================================
! design          : H. Lischke, N. Zimmermann
! author(s)       : H. Lischke, N. Zimmermann
! implementation  : H. Lischke, N. Zimmermann
! cleaner         : T.J. Loeffler
! copyright       : (c) 1999, 03 by H. Lischke
!==============================================================================

  SUBROUTINE StoreCellStateToThisCell(year,lat, lon,  nsp, idsp,ddegs, wtemp, drstr, avnit,brwpr, thiscell)
      USE All_par, ONLY:  cell,seedrain,CurrStateInCell,seedrain
      implicit none

      TYPE(CurrStateInCell):: thiscell
      integer:: isp, lat, lon, nsp, year
      logical, intent(in) :: idsp
      real:: ddegs, wtemp, drstr, avnit,brwpr

             do isp=1,nsp
                 thiscell%sp(isp)=cell(lat,lon)%sp(isp)
                 thiscell%sp(isp)%newseeds=seedrain(lat,lon)%sp(isp)%newseeds
                 thiscell%sp(isp)%verynewseeds=seedrain(lat,lon)%sp(isp)%verynewseeds
             end do             ! store the actual cell to the structure thiscell
 ! store additional, cell related information to thiscell
             thiscell%lat=lat
             thiscell%lon=lon
             thiscell%brwpr=brwpr
             thiscell%ddegs=ddegs
             thiscell%wtemp=wtemp
             thiscell%drstr=drstr
             thiscell%avnit=avnit
             thiscell%chestnutcult=cell(lat,lon)%chestnutcult
 end subroutine StoreCellStateToThisCell
!==============================================================================
   Subroutine StoreThisCellSpecStatesToCell(  lat, lon,  nspc ,thiscell)
      USE All_par, ONLY:  cell,seedrain,CurrStateInCell
      implicit none
      TYPE(CurrStateInCell)::  thiscell
      integer:: isp, lat, lon, nspc

             do isp=1,nspc
                 cell(lat,lon)%sp(isp) = thiscell%sp(isp)
                 seedrain(lat,lon)%sp(isp)%newseeds= thiscell%sp(isp)%newseeds
                 seedrain(lat,lon)%sp(isp)%verynewseeds= thiscell%sp(isp)%verynewseeds
              end do
              cell(lat,lon)%chestnutcult=  thiscell%chestnutcult
end subroutine StoreThisCellSpecStatesToCell
!==============================================================================
