!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Adaptation to FORHYCS MPI structure

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!==============================================================================
!
!  Name   :  TimeLoop
!
!  Purpose:  It calculates the spatial forest dynamics on a grid with or
!            without seed dispersal
!
!  Remarks:  contains the subroutine
!
!            TimeLoop (<TreeMig)
!                     (developed from SpatialSimu)
!
!==============================================================================


SUBROUTINE TimeLoop(yearStart, yearEnd,      &
                       latStart, latEnd, lonStart, lonEnd,&
                       currLatStart,currLatEnd ,currLonStart,currLonEnd,&
                       facvar,        &
                       counter,            &
                       nspc, idsp,  spatsimuisokay, ncallSpatSimu)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY:     writeOutput,   thisyear,   rptint, spec,simustartyear
use prevahglobals, only: rnodat, rcol,rrow,rxll,ryll,rgridsize,wstats,ncol,nrow, jahr
use forhycsglobals, only: biomass2d,number2d,seeds2d,ingrowth2d,antagon2d,pollen2d,la2d,&
                          biomassTable,numberTable,seedsTable,ingrowthTable,antagonTable,pollenTable,laTable,&
                          TMdir,reptTables,reptGrids,latTable,lonTable,nspc_g,prevahBC
use MPIvars
use mpi
use ifport  ! for MAKEDIRQQ
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN)    :: yearStart, yearEnd,                 &  ! start and end of simulated period
                          latStart, latEnd, lonStart, lonEnd, &  ! start and end of simulated areaINTEGER :: nspc
                          currLatStart,currLatEnd ,currLonStart,currLonEnd,&  ! part of total simulation area currently simulated, e.g. for spin-up. Is not subarea! (MAtthias: now it is)
                           nspc,ncallSpatSimu                                  ! number of species involved [=spec]
REAL, INTENT(IN)       :: facvar
LOGICAL, INTENT(IN)    :: idsp                                   ! flag: simulate dispersal?
INTEGER, INTENT(INOUT) :: counter                                ! simulation counter for output
LOGICAL, INTENT(OUT)   :: spatsimuisokay

!<Local variables>--------------------------------------------------------------------------------
INTEGER :: year,          &    ! looping variable for simulated year
           istatefile           ! actual statefile to be written to

! Variables for MPITMReport.inc; Matthias
integer :: i,j,k,rloop,xcoor,ycoor  ! loop indices
integer :: numVars
logical :: writeOut
character*2 :: specName
character*4 :: chyear
character*8 :: varName
character*256 :: tableDir, gridDir, tableName, gridName
real, dimension(ncol,nrow,nspc+1) :: write2d,wrgrd
real, dimension(nspc+1,ncol*nrow) :: writeTable
logical :: result  ! output of function MAKEDIRQQ (ifport)


 !<Here the subroutine starts>
  spatsimuisokay = .true.
  istatefile     = 1071
     LoopTime: DO year=yearStart,yearEnd
       print *, 'year ', year
      REWIND(1070)
      CALL ReadNewBioClimIfNecessary (year,.TRUE.)
      include 'resetTablesAndGrids.inc'  ! Matthias
      if((rank.ne.root).or.(doParallel==.false.))call adjHmax      ! MS, March 2017: adjustment of species-specific max height to local drought conditions
      CALL DistrLocalDynThisTime(year,counter,currLatStart,currLatEnd, & ! in LocalWork
                       currLonStart,currLonEnd,nspc,idsp)
      !if(prevahBC==0) call ecoGridWrite(year+simustartyear-1)
      !include 'MPITMReport.inc'    ! Matthias

    END DO LoopTime

END SUBROUTINE TimeLoop
!****<Here the subroutine ends>********************************************************************
