!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: New drought tolerance parameters & adaptation for different
! model configurations (with or without coupling).

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!==============================================================================
!
!  Name   :  TreeMig (TREE MIGration simulator)
!
!  Purpose:  main subroutine for the simulation of tree migration
!
!  Remarks:  contains the subroutine
!
!            TreeMig (<StartTreeMig)
!
!==============================================================================


!SUBROUTINE TreeMig(doParallel,JAHR)
SUBROUTINE TreeMig(JAHR)


!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: rptint, maxlat, maxlon,simustartyear,            &
                   fcvarInit, fcvarDyn, facvar,                &
                   startlatStart, startlatEnd, startlonStart, startlonEnd, starttime,     &
                   readStateVector, experimentID,  &
                   writeoutput,stockability,&
                   maxarraylat, maxarraylon ,&
                   spec, CurrStateInCell, cell
use forhycsglobals, only: nspc_g, counter_g, PRtoTM, prevahBC, cdtol, readFH
use MPIVars, only: rank, root, firstRow, lastRow
use prevahglobals, only : jahra, jahre    ! Start and end year of PREVAH simulations
IMPLICIT NONE

!LOGICAL, INTENT(IN):: doParallel
INTEGER, INTENT(IN):: JAHR                       ! FORHYCS year


!<Passed on variables>--------------------------------------------------------------------------------
INTEGER :: nspc,                          &    ! number of species                                    [=spec]
           yearStartEqRun, yearEndEqRun,  &    ! year of calculation for initial run into equilibrium [=..]
           yearStart, yearEnd,            &    ! year of calculation                                  [=..]
           stateYear,                     &    ! year of stored state
           currLatStart, currLatEnd,              &    ! looping variable for latitude                        [=..]
           currLonStart, currLonEnd,              &    ! looping variable for longitude                       [=..]
           counter,                       &
           reportInt, ii                      ! report Interval
LOGICAL :: idsp, spatsimuisokay
real, dimension(31) :: newKdrt
integer :: i
integer :: lat,lon

!logical :: readStateFH, writeStateFH  ! In here for now, make them control pars later
type(CurrStateInCell) :: tempcell

!<Local variables>--------------------------------------------------------------------------------

!<Here the program starts>*************************************************************************



!          currLatStart = 1
!          currLatEnd   = startlatEnd ! Only for testing
          currLatStart = firstRow
          currLatEnd = lastRow
          currLonStart = 1
           currLonEnd   = startlonEnd  ! Only for testing
          !IF (readStateVector) THEN
           !print *, 'vor readstatevector'
           ! CALL ReadState (stateYear, currLatStart, currLatEnd, currLonStart, currLonEnd, nspc)
          !             print *, 'nach readstatevector'
          !   print *, 'stateYear ', stateYear
          !   stateYear=0
         !   yearStart = stateYear + 1
         ! END IF !(readStateVector)

          !readStateFH = .FALSE.
         ! writeStateFH = .FALSE.


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!years before FORHYCS coupling !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!IF (JAHR.eq.1975) THEN
!IF (((JAHR.eq.1971) .and.(prevahBC==0)) .or. ((prevahBC==1).and. (jahr==1941))) THEN
IF ((JAHR.eq.1971) .and.(prevahBC==0) .and. readFH==0) THEN

nspc=nspc_g

!   if(readstateFH) then
!     call ReadInitState
!     yearStartEqRun = 0
!     yearEndEqRun   = starttime + yearStartEqRun
!     currLatStart = firstRow
!     currLatEnd = lastRow
!     currLonStart  = startlonStart
!     currLonEnd    = startlonEnd
!     counter   = 0
!     reportInt = rptint
!     facvar    = fcvarInit

    !if(prevahBC == 1) then
   ! In case of two-way coupling: coupled TM-PREVAH runs start in year 1941
   !   starttime = starttime-30
   !   yearEndEqRun = yearEndEqRun-30
   ! end if


!   else

      OneExp: DO ii=1,1
      print*, "JAHR", JAHR, rank
      counter = 0

          !----- Run into equilibrium in only one cell, with constant seed pool
          idsp           = .FALSE.
          yearStartEqRun = 0
          yearEndEqRun   = starttime + yearStartEqRun

! Years before FORHYCS coupling: spin-up with bootstrapped Bioclim data.
! Seed dispersal is switched off for a short period at the beginning,
! then enabled.

          IF ((yearStartEqRun < yearEndEqRun) .AND. (readFH==0)) THEN
            currLatStart = firstRow
            currLatEnd = lastRow
            currLonStart  = startlonStart
            currLonEnd    = startlonEnd
            counter   = 0
            reportInt = rptint
            facvar    = fcvarInit
      print *, 'vor timeloop, yearStartEqRun,yearEndEqRun',yearStartEqRun,yearEndEqRun
	  print *, ' maxlat,maxlon,currLatStart, currLatEnd, currLonStart, currLonEnd,', &
	                   maxlat,maxlon,currLatStart, currLatEnd, currLonStart, currLonEnd
	 print *, 'idsp',idsp


            CALL TimeLoop (yearStartEqRun, yearStartEqRun+5,         &
                              1, maxlat, 1, maxlon, &
                              currLatStart, currLatEnd, currLonStart, currLonEnd,&
                              facvar, counter, nspc,idsp, spatsimuisokay,1)


            IF (.not. spatsimuisokay) THEN
              WRITE(1070, *) experimentID, 'had problems in initial phase, next experiment'
              EXIT OneExp
            END IF  ! .not. spatsimuisokay

! Second part of spin-up: seed dispersal is now enabled, but no FORHYCS
! coupling yet.

             idsp = .true.  ! Enabling seed dispersal
             print*, "nspc in Treemig ayayay", nspc
             CALL TimeLoop (yearStartEqRun+6, yearEndEqRun,         &
                              1, maxlat, 1, maxlon, &
                              currLatStart, currLatEnd, currLonStart, currLonEnd,&
                              facvar, counter, nspc,idsp, spatsimuisokay,1)

          END IF    !  (yearStartEqRun < yearEndEqRun) .AND. (readFH==0)




      END DO OneExp

!     end if  ! readStateFH or not



          !----- End of spin-up OR state file is loaded.
          idsp     = .TRUE.    !----- IF no seed dispersal but a spatial simulation is wished, set to .FALSE.

          currLatStart = firstRow   ! Matthias
          currLatEnd = lastRow
          currLonStart = 1
          currLonEnd   = maxlon
          currLonEnd   = startlonEnd  ! Only for testing
          yearStart = yearEndEqRun+1
	    yearEnd   = JAHR -simustartyear !FORHYCS
          reportInt = rptint
          facvar    = fcvarDyn

print*,'yearStart,yearEnd',yearStart,yearEnd

! First TreeMig simulation year after spin-up
!           CALL TimeLoop (yearStart, yearEnd,                   &
!                               1, maxlat, 1, maxlon,&
!                               currLatStart, currLatEnd, currLonStart, currLonEnd,&
!                               facvar, counter, nspc,idsp, spatsimuisokay,2)
!           IF (.not. spatsimuisokay) THEN
!             WRITE(1070, *) experimentID, 'had problems in spatial phase, next experiment'
!             !EXIT OneExp
!           END IF
      !if(readStateFH) then
      !  if(rank .ne. root) then
      !  do lon = 1,maxlon
      !  do lat = firstRow,lastRow
      !    tempcell = cell(lat,lon)
      !    call vegPars(lon,lat,tempcell,nspc_g)
      !  end do
      !  end do
      !  end if  ! Slaves only
      !end if
counter_g=counter
!END IF !(JAHR.eq.1971)





!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!from here on FORHYCS coupling !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

else IF (JAHR.ge.(jahra+1) .or. ((prevahBC==1) .and. (jahr.ge.jahra)) .or. (readFH==1)) THEN

counter=counter_g
nspc=nspc_g

! Modify drought stress tolerance parameter values after spin-up, as
! drought stress calculated in PREVAH is based on AT/PT instead of AET/PET.
! ONLY in the case of PREVAH -> TreeMig coupling!

  if(PRtoTM == 1) then
    ! New drought stress parameters after Lischke & Zierl (2002)
!     newKdrt = (/0.37,0.45,0.41,0.43,0.4,0.55,0.39,0.46,0.42,0.34,0.37,0.34,0.37,0.35, &
!      0.46,0.3,0.46,0.37,0.39,0.3,0.41,0.44,0.4,0.44,0.39,0.38,0.27,0.4,0.4,0.35,0.5/)

  if(cdtol==1) then

! Modified parameter set, to make Quercus pubescens more drought resistant than Pinus
! sylvestris (Ps parameter value was artificially increased in Lischke & Zierl 2002)
    newKdrt = (/0.37,&  ! Abies alba
    0.45, &             ! Larix decidua
    0.41, &             ! Picea abies
    0.43, &             ! Pinus cembra
    0.4, &              ! Pinus montana
    0.5, &              ! Pinus sylvestris
    0.39, &             ! Taxus baccata
    0.46, &             ! Acer campestre
    0.42, &             ! Acer platanoides
    0.34, &             ! Acer pseudoplatanus
    0.37, &             ! Alnus glutinosa
    0.34, &             ! Alnus incana
    0.37, &             ! Alnus viridis
    0.35, &             ! Betula pendula
    0.46, &             ! Carpinus betulus
    0.3, &              ! Castanea sativa
    0.46, &             ! Corylus avellana
    0.37, &             ! Fagus sylvatica
    0.39, &             ! Fraxinus excelsior
    0.3, &              ! Populus nigra
    0.41, &             ! Populus tremula
    0.44, &             ! Quercus petraea
    0.55, &             ! Quercus pubescens
    0.44, &             ! Quercus robur
    0.39, &             ! Salix alba
    0.38, &             ! Sorbus aria
    0.27, &             ! Sorbus aucuparia
    0.4, &              ! Tilia cordata
    0.4, &              ! Tilia platyphyllos
    0.35, &             ! Ulmus scabra
    0.5/)

    ! TEST with lowered drought tolerance parameters (*0.7)
    !newKdrt = newKdrt*0.8

    else
! New drought tolerance parameter set, mapping the values of Niinemets & Valladares (2006)
! to the range of Lischke & Zierl (2002)

! Adapted to increase the level of drought tolerance of Ld and Pa, and decrease it for
! As.

    newKdrt = (/0.28,&  ! Abies alba
    !0.32, &             ! Larix decidua
    0.42, &
    ! 0.27, &             ! Picea abies
    ! 0.35, &
    0.32, &
    !0.38, &             ! Pinus cembra
    0.45, &
    0.49, &              ! Pinus montana
    !0.5, &              ! Pinus sylvestris
    0.45, &
    0.38, &             ! Taxus baccata
    0.37, &             ! Acer campestre
    0.36, &             ! Acer platanoides
    !0.36, &             ! Acer pseudoplatanus
    0.32, &
    0.31, &             ! Alnus glutinosa
    0.28, &             ! Alnus incana
    0.33, &             ! Alnus viridis
    0.28, &             ! Betula pendula
    0.35, &             ! Carpinus betulus
    0.42, &              ! Castanea sativa
    0.38, &             ! Corylus avellana
    0.33, &             ! Fagus sylvatica
    0.34, &             ! Fraxinus excelsior
    0.31, &              ! Populus nigra
    0.37, &             ! Populus tremula
    0.38, &             ! Quercus petraea
    0.48, &             ! Quercus pubescens
    0.38, &             ! Quercus robur
    0.29, &             ! Salix alba
    0.43, &             ! Sorbus aria
    0.3, &             ! Sorbus aucuparia
    0.36, &              ! Tilia cordata
    0.34, &              ! Tilia platyphyllos
    0.33, &             ! Ulmus scabra
    0.5/)

    end if  ! cdtol

    do i = 1,nspc
      spec(i)%kdrt = newKdrt(i)
    end do

  end if

  currLatStart = firstRow   ! Matthias
  currLatEnd = lastRow
  currLonStart = 1
  currLonEnd   = maxlon
  currLonEnd   = startlonEnd  ! Only for testing

  reportInt = rptint
  facvar    = fcvarDyn


  ! First five years: ubiquitous cells; no dispersal

      if((jahr.lt.(jahra+5)).and.(readFH==0)) then

          idsp     = .FALSE.    !----- IF no seed dispersal but a spatial simulation is wished, set to .FALSE.

          print*, "jahr,simustartyear, hihihi", jahr, simustartyear

          CALL TimeLoop (JAHR-1-simustartyear, JAHR-1-simustartyear,                   &
                              1, maxlat, 1, maxlon,&
                              currLatStart, currLatEnd, currLonStart, currLonEnd,&
                              facvar, counter, nspc,idsp, spatsimuisokay,2)
          IF (.not. spatsimuisokay) THEN
            WRITE(1070, *) experimentID, 'had problems in spatial phase, next experiment'
          END IF


      else  ! More than 5 years after start: switch on seed dispersal


          idsp     = .TRUE.    !----- IF no seed dispersal but a spatial simulation is wished, set to .FALSE.

          print*, "jahr,simustartyear, hohoho", jahr, simustartyear

          CALL TimeLoop (JAHR-1-simustartyear, JAHR-1-simustartyear,                   &
                              1, maxlat, 1, maxlon,&
                              currLatStart, currLatEnd, currLonStart, currLonEnd,&
                              facvar, counter, nspc,idsp, spatsimuisokay,2)
          IF (.not. spatsimuisokay) THEN
            WRITE(1070, *) experimentID, 'had problems in spatial phase, next experiment'
          END IF
      end if  ! Seed dispersal off/on

          !if((prevahBC==1) .and. (jahr==1970)) then
          !  call writeInitState
         ! end if

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!last year FORHYCS coupling !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

IF (JAHR.eq.jahre) THEN
          CALL CloseFiles
      CALL CloseResultsFile

END IF !(JAHR.eq.2009)


END IF !(JAHR.ge.1972)

END Subroutine TreeMig
!****<Here the program ends>***********************************************************************
