!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: New output format (tables or grids) & adaptation to FORHYCS
! MPI structure.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!==============================================================================
!
!  Name   :  TreeMig (TREE MIGration simulator)
!
!  Purpose:  main subroutine for the simulation of tree migration
!
!  Remarks:  contains the subroutine
!
!            TreeMig (<StartTreeMig)
!
!==============================================================================

SUBROUTINE TreeMig_init

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: maxlat, maxlon,                         &
                   experimentID, tabsepOutput, pathletter1, pathletter2, &
                   readDispersalKernel, writeoutput,                                       &
                   cell, maxarraylat, maxarraylon, &
                   specAppearance, specMigSpeed, OldBioClim, NewBioClim, & !ThirtyYearBioClim, & ! Allocatable arrays from All_par
                   stockability, landcoverfilter, stocksum, stocksumcum, avbio0, maxspc, BIOCLIMDATA, seedrain, &
                   BioClimSU1, BioClimSU2, BioClimSU3, BioClimSU4, BioClimSU5, &
                   BioClimSU6, BioClimSU7, BioClimSU8, BioClimSU9, BioClimSU10, &
                   BioClimSU11, BioClimSU12, BioClimSU13, BioClimSU14, BioClimSU15, &
                   BioClimSU16, BioClimSU17, BioClimSU18, BioClimSU19, BioClimSU20, &
                   BioClimSU21, BioClimSU22, BioClimSU23, BioClimSU24, BioClimSU25, &
                   BioClimSU26, BioClimSU27, BioClimSU28, BioClimSU29, BioClimSU30
use MPIVars, only: rank, root, doParallel, ierr
use prevahglobals, only: ncol, nrow
use forhycsglobals, only: nspc_g,reptTables,reptGrids, latTable, lonTable, prevahBC
use mpi

IMPLICIT NONE

!<Local variables>--------------------------------------------------------------------------------
INTEGER :: counter,errtyp,allocation_status
integer :: countTable, lat, lon  ! used in initReptTables.inc
LOGICAL :: idsp, stockfileisokay, bioclimfileisokay, controlfileisokay, ready



!<Here the program starts>*************************************************************************

    include 'allocateTM.inc'

    !allocate (cell(1:maxArrayLat ,1:maxArrayLon), stat=allocation_status)
    allocate (cell(1:nrow ,1:ncol), stat=allocation_status)
    IF (allocation_status>0) Then
      print *, 'allocation of cell-array in TreeMig did not work '
    stop
    end if

      print *, 'vor OpenControlParsFileList ', rank
    CALL OpenControlParsFileList (pathletter1, pathletter2, readDispersalKernel )
      print *, 'vor OpenDispKernFile ', rank
    CALL OpenDispKernFile

      print *, 'vor OpenLogFile ', rank
    if (rank==root) CALL OpenLogFile   ! Matthias
      print *, 'nach OpenLogFile ', rank

!   CALL OpenStateFile
!   CALL OpenResultsFile (pathletter1, pathletter2)
!     print *, 'nach OpenResultsFile '

  !----- Loop over different sites (isite is passed to GetEnv)
    CALL ReadControlPars (controlfileisokay, ready)
      print *, 'nach ReadControlPars ', rank
    IF (ready) THEN
      write(*,*) 'experiment is ready ?? - strange comment'
    END IF
    IF (.not. controlfileisokay) THEN
      WRITE(1070, *) experimentID, 'had problems with control file, next experiment'
      WRITE(*, *) experimentID, 'had problems with control file, next experiment'
    END IF

  !----- Initialize the program
      !print *, 'vor getconstenv', rank
    CALL GetConstEnvironment (stockfileisokay)  ! Not touching this for now - Matthias
    IF (.not. stockfileisokay) THEN
      WRITE(1070, *) experimentID, 'had problems with stock file, next experiment'
    END IF

      print *, 'nach getconstenv', rank
      !print *, 'vor GetLandcoverFilter  ', rank
    CALL GetLandcoverFilter (stockfileisokay)  ! Not touching this for now - Matthias
      print *, 'nach GetLandcoverFilter', rank
    IF (.not. stockfileisokay) THEN
      WRITE(1070, *) experimentID, 'had problems with Landcoverfilter file, next experiment'
    END IF

      print *, 'vor getstpc ', rank
    CALL GetSpc (nspc_g, errtyp)
      print *, 'nach getstpc ', rank
    IF (errtyp .ne. 0) THEN
            IF (errtyp .eq. 1) print*, experimentID, 'had problems with dispersal file, next experiment' !WRITE(1070, *) experimentID, 'had problems with dispersal file, next experiment'
            IF (errtyp .eq. 2) print*, experimentID, 'had problems with spec file, next experiment' !WRITE(1070, *) experimentID, 'had problems with spec file, next experiment'
            IF (errtyp .eq. 3) print*, experimentID, 'had problems with immigration file, next experiment' !WRITE(1070, *) experimentID, 'had problems with immigration file, next experiment'
            stop
    END IF

   !----- Initialize the plotting if wished (if librabry pllib not included, e.g. on UNIX, comment out
   !!! PLOT ROUTINE
!#ifdef WITHPLOT
!      IF (writeOutput%plot) CALL InitPlotSpecs(nspc_g,maxlat, maxlon )
!#endif

!!! END PLOT ROUTINE
      print *, 'vor getstd ', rank
    if(rank .ne. root) CALL GetStd (nspc_g)  ! Matthias
      print *,' nach GetStd ', rank
      !print*,' vor PrepareOutput '
    if(rank==root) CALL PrepareOutput (tabsepOutput, experimentID, nspc_g,0)   ! Matthias
      print *,' nach PrepareOutput '

    if(prevahBC == 0) then
      CALL ReadFirst30years (bioclimfileisokay)                    ! only for obs; file PrepareBioclimInput.f90
    end if

      print *,' vor PrepareBioclimInput ', rank
    CALL PrepareBioclimInput (bioclimfileisokay)
      print *, 'nach PrepareBioclimInput ', rank

    IF (.not. bioclimfileisokay) THEN
      WRITE(1070, *) experimentID, 'had problems opening bioclim file, next experiment'
    END IF

! Hard-coded for now
! Maybe include them in control files later?
  reptTables = .false.
  reptGrids = .false.

  if(rank == root) include 'initReptTables.inc'

    print*, "before MPI_barrier", rank
  call MPI_BARRIER(MPI_COMM_WORLD,ierr)
    print*, "ierr", ierr, rank

  if(rank.ne.root) then
    print*, "TreeMig initialization done", rank
  end if

   END Subroutine TreeMig_init
!****<Here the program ends>***********************************************************************
