!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!==============================================================================
!
!  Name   :  WriteIntegratedResults
!
!  Remarks:  contains the subroutines
!
!            WriteIntegratedResults (<TreeMig)
!            WriteHeaderForIntegratedResults (<OpenResultsFile<TreeMig)
!            WriteHeaderForAppearanceResults (<OpenResultsFile<TreeMig)
!
!==============================================================================
! Implementation and Revisions:
!------------------------------------------------------------------------------
! Author  Date      Description
! ------  ----      -----------
!
!==============================================================================
! Ideas, Wishes etc.:
!------------------------------------------------------------------------------
! Author  Date      Description
! ------  ----      -----------
!
!==============================================================================
SUBROUTINE WriteIntegratedResults

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: avdiversity80, avdiversity160, avdiversity240, avdiversity500, avdiversityEq,      &
                   nonzeroSpecs80, nonzeroSpecs160, nonzeroSpecs240, nonzeroSpecs500, nonzeroSpecsEq, &
                   domspec80, domspec160, domspec240, domspec500, domspecEq, experimentID,            &
                   maxlat, maxlon, specAppearance, specMigSpeed, biocrit, maxspc, spec, cellSideLength
IMPLICIT NONE

!<Local variables>---------------------------------------------------------------------------------
INTEGER :: i,                         &        ! loop-counter
           ibiocrit, ispec, lat, lon

!<Here the subroutine starts>**********************************************************************
  Write(1075,100) experimentID, ':', nonzeroSpecs80, nonzeroSpecs160, nonzeroSpecs240, &
                                   nonzeroSpecs500, nonzeroSpecsEq,'|',              &
                                   (domspec80(i),i=1,3),'|',                         &
                                   (domspec160(i),i=1,3),'|',                        &
                                   (domspec240(i),i=1,3),'|',                        &
                                   (domspec500(i),i=1,3),'|',                        &
                                   (domspecEq(i),i=1,3),'|',                         &
                                   avdiversity80, avdiversity160, avdiversity240, avdiversity500, avdiversityEq

  100 FORMAT (A12,A1,5I3,A1,5(3I3,A1),5F8.4)

  DO ispec = 1, maxspc
    IF ((ispec .eq. 1) .or. (ispec .eq. 18) .or. (ispec .eq. 20)) THEN
      DO lat = 1, maxlat
        DO lon = 1, maxlon
          IF(specAppearance(1, ispec, lat, lon) > 0.0) THEN
            WRITE(1059,200) experimentID, ' ', spec(ispec)%nl, ' ', lat, lon, lat*cellSideLength,&
                           (specAppearance(ibiocrit, ispec, lat, lon), ibiocrit = 1, 10), cellSideLength
          END IF
        END DO
      END DO
    END IF
  END DO

  200 FORMAT(A12, A1, A20, A1, 2I4, 21F15.6)
END SUBROUTINE WriteIntegratedResults
!****<Here the subroutine ends>********************************************************************



!****<Here the subroutine WriteHeaderForIntegratedResults starts>**********************************
SUBROUTINE WriteHeaderForIntegratedResults
 Write(1075,101) 'Experiment :  nonzeroSpecs80  nonzeroSpecs160 nonzeroSpecs240 nonzeroSpecs500 nonzeroSpecsEq  | ', &
               'domspec80_1  domspec80_2, domspec80_3  | ', &
               'domspec160_1 domspec160_2,domspec160_3 | ', &
               'domspec240_1 domspec240_2,domspec240_3 | ', &
               'domspec500_1 domspec500_2,domspec500_3 | ', &
               'domspecEq_1  domspecEq_2, domspecEq_3  | ', &
               'avdiversity80 avdiversity160 avdiversity240 avdiversity500 avdiversityEq'

  101 FORMAT (A96,5A41,A72)
END SUBROUTINE WriteHeaderForIntegratedResults
!****<Here the subroutine ends>********************************************************************



!****<Here the subroutine WriteHeaderForAppearanceResults starts>**********************************
SUBROUTINE WriteHeaderForAppearanceResults
  Write(1059,102) 'Experiment species lat lon app1 app2 app3 app4 app5 app6 app7 app8 app9 app10',&
                 ' speed1 speed2 speed3 speed4 speed5 speed6 speed7 speed8 speed9 speed10      '
  102 FORMAT (2A77)
END SUBROUTINE WriteHeaderForAppearanceResults
!****<Here the subroutine ends>********************************************************************
