!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2003 Joachim Gurtz, Massimiliano Zappa
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Adaptation to the disaggregated soil evaporation and
! transpiration calculation.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Kahan summation for daily means, to ensure accurate results.
! Current sum (currSum) and correction term (cor) are stored and carried.
! Matthias Speich, Jan 2015
!-------------------------------------------------------------------------------

      subroutine kahandm(currSum,newTerm,cor)

      ! Kahan summation for calculation of daily means

        use fp_ops

        implicit none

        real(kind=dblP),intent(inout):: currSum, cor
        real,intent(in) :: newTerm

        real(kind=dblP) :: sum, corD, newSum, corNext

        sum = currSum
        corD = cor

        corNext = newTerm + corD
        newSum = sum + cornext
        corD = corNext - (newSum-sum)
        sum = newSum

        currSum = sum
        cor = corD

      end subroutine kahandm

! ------------------------------------------------------------------------------


! ------------------------------------------------------------------------------
! Main subroutine ADMSYHYD
!
! Authors: J.Gurtz, M.Zappa et al. 1995-2003
! ------------------------------------------------------------------------------

      SUBROUTINE admshydPatch  (xcoor,ycoor,ATFL,ncol,nrow,
     *          PNDS,PAKDS,WSADS,EPDS,ERDS,EIDS,EBDS,R0DS,
     *          R1DS,R2DS,RGEDS,petDS,eSoilDS,eTransDS,
     *          eSnowDS,eIntDS,eWaterDS,peSoilDS, peTransDS,
     *          PNKSM,PAKSM,WSASM,EPSM,ERSM,EISM,EBSM,R0SM,R1SM,
     *          R2SM,RGESSM,
     *          PNKMS,PAKMS,WSAMS,EPMS,ERMS,EIMS,EBMS,R0MS,R1MS,
     *          R2MS,RGEMS,petMS,eSoilMS,eTransMS,eSnowMS,
     *          eIntMS,eWaterMS,peSoilMS, peTransMS,
     *          PNKJS,PAKJS,WSAJS,EPJS,ERJS,EIJS,EBJS,EBJS2,R0JS,
     *          R1JS,R2JS,RGEJS,petJS,eSoilJS,eTransJS,
     *          eSnowJS,eIntJS,eWaterJS, peSoilJS,peTransJS,
     *          PNKS,PAKS,WSAS,EPS,ERS,EIS,EBS,R0S,R1S,R2S,RGESS,
     *          GWNBS,GWNBDS,GWNBJS,GWNBMS,GLACS,GLACMS,GLACJS,
     *          GLACDS,GLACSS,wstats)

!     This subroutine computes builds daily, monthly, yearly and total
!     sums adding subsequently the values of each HRU (xcoor,ycoor). ncol.nrow
!     is the total number of HRUs.
!     The meaning of the other variables is declared in the main program.
!--------------------------------------------------------------------------

      use fp_ops
      use MPIvars, only: rank
      use forhycsglobals, only: pnksmc, paksmc, wsasmc, epsmc,
     *   ersmc, eismc, ebsmc, r0smc,r1smc, r2smc, glacssc, rgessmc,
     * pnksmp, paksmp, wsasmp, epsmp, ersmp, eismp, ebsmp, r0smp,
     * r1smp, r2smp, glacssp, rgessmp,
     * eSoilsmp, eTranssmp, eIntsmp, eSnowsmp, eWatersmp,
     * eSoilsmc, eTranssmc, eIntsmc, eSnowsmc, eWatersmc

      implicit none

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------
      INTEGER xcoor,ycoor,ncol,nrow
      REAL ATFL(ncol,nrow),
     *     PNDS,PAKDS,WSADS,EPDS,ERDS,EIDS,EBDS,R0DS,
     *     R1DS,R2DS,RGEDS,
     *     PNKSM,PAKSM,WSASM,EPSM,ERSM,EISM,EBSM,R0SM,R1SM,
     *     R2SM,RGESSM
      REAL PNKMS(ncol,nrow),PAKMS(ncol,nrow),WSAMS(ncol,nrow),
     *     EPMS(ncol,nrow),ERMS(ncol,nrow),
     *     EIMS(ncol,nrow),EBMS(ncol,nrow),R0MS(ncol,nrow),
     *     R1MS(ncol,nrow),R2MS(ncol,nrow),
     *     RGEMS(ncol,nrow)
      REAL PNKJS(ncol,nrow),PAKJS(ncol,nrow),WSAJS(ncol,nrow),
     *     EPJS(ncol,nrow),ERJS(ncol,nrow),
     *     EIJS(ncol,nrow),EBJS(ncol,nrow),R0JS(ncol,nrow),
     *     R1JS(ncol,nrow),R2JS(ncol,nrow),EBJS2(ncol,nrow),
     *     RGEJS(ncol,nrow)
      REAL PNKS(ncol,nrow),PAKS(ncol,nrow),WSAS(ncol,nrow),
     *     EPS(ncol,nrow),ERS(ncol,nrow),
     *     EIS(ncol,nrow),EBS(ncol,nrow),R0S(ncol,nrow),R1S(ncol,nrow),
     *     R2S(ncol,nrow),RGESS(ncol,nrow)
      real :: petDS,eSoilDS,eTransDS,eSnowDS,eIntDS,eWaterDS,
     *  peSoilDS, peTransDS
      real, dimension(ncol,nrow) :: petMS,eSoilMS,eTransMS,eSnowMS,
     *          eIntMS, eWaterMS,petJS,eSoilJS,eTransJS,eSnowJS,eIntJS,
     *          eWaterJS, peSoilMS, peTransMS, peSoilJS, peTransJS
      REAL ATFLK,GLACSS,wstats
      REAL GWNBS(ncol,nrow),GWNBDS,GWNBJS(ncol,nrow),GWNBMS(ncol,nrow)
      REAL GLACS(ncol,nrow),GLACDS,GLACJS(ncol,nrow),GLACMS(ncol,nrow)

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------
! -----------------------------------------------------------------------
!     End of day, adding to monthly, yearly and total sums
! -----------------------------------------------------------------------

! Daily mean -SM for whole catchment
      call kahandm(pnksmp,pnds/wstats,pnksmc)
      call kahandm(paksmp,pakds/wstats,paksmc)
      call kahandm(wsasmp,wsads/wstats,wsasmc)
      call kahandm(epsmp,epds/wstats,epsmc)
      call kahandm(ersmp,erds/wstats,ersmc)
      call kahandm(eismp,eids/wstats,eismc)
      call kahandm(ebsmp,ebds/wstats,ebsmc)
      call kahandm(r0smp,r0ds/wstats,r0smc)
      call kahandm(r1smp,r1ds/wstats,r1smc)
      call kahandm(r2smp,r2ds/wstats,r2smc)
      call kahandm(glacssp,glacds/wstats,glacssc)
      call kahandm(rgessmp,rgeds/wstats,rgessmc)


! montly sum -MS(xcoor,ycoor) for each HRU
      PNKMS(xcoor,ycoor)=PNKMS(xcoor,ycoor)+PNDS
      PAKMS(xcoor,ycoor)=PAKMS(xcoor,ycoor)+PAKDS
      WSAMS(xcoor,ycoor)=WSAMS(xcoor,ycoor)+WSADS

      peSoilMS(xcoor,ycoor) = peSoilMS(xcoor,ycoor) + peSoilDS
      peTransMS(xcoor,ycoor) = peTransMS(xcoor,ycoor) + peTransDS
      eSoilMS(xcoor,ycoor) = eSoilMS(xcoor,ycoor) + eSoilDS
      eTransMS(xcoor,ycoor) = eTransMS(xcoor,ycoor) + eTransDS
      eIntMS(xcoor,ycoor) = eIntMS(xcoor,ycoor) + eIntDS
      eSnowMS(xcoor,ycoor) = eSnowMS(xcoor,ycoor) + eSnowDS
      eWaterMS(xcoor,ycoor) = eWaterMS(xcoor,ycoor) + eWaterDS
      R0MS(xcoor,ycoor)=R0MS(xcoor,ycoor)+(R0DS)
      R1MS(xcoor,ycoor)=R1MS(xcoor,ycoor)+(R1DS)
      R2MS(xcoor,ycoor)=R2MS(xcoor,ycoor)+(R2DS)
      RGEMS(xcoor,ycoor)=RGEMS(xcoor,ycoor)+(RGEDS)
      GWNBMS(xcoor,ycoor)=GWNBMS(xcoor,ycoor)+(GWNBDS)
      GLACMS(xcoor,ycoor)=GLACMS(xcoor,ycoor)+(GLACDS)

! Yearly sums -JS(xcoor,ycoor) for each HRU
      PNKJS(xcoor,ycoor)=PNKJS(xcoor,ycoor)+PNDS
      PAKJS(xcoor,ycoor)=PAKJS(xcoor,ycoor)+PAKDS
      WSAJS(xcoor,ycoor)=WSAJS(xcoor,ycoor)+WSADS
      peSoilJS(xcoor,ycoor) = peSoilJS(xcoor,ycoor) + peSoilDS
      peTransJS(xcoor,ycoor) = peTransJS(xcoor,ycoor) + peTransDS
      eSoilJS(xcoor,ycoor) = eSoilJS(xcoor,ycoor) + eSoilDS
      eTransJS(xcoor,ycoor) = eTransJS(xcoor,ycoor) + eTransDS
      eIntJS(xcoor,ycoor) = eIntJS(xcoor,ycoor) + eIntDS
      eSnowJS(xcoor,ycoor) = eSnowJS(xcoor,ycoor) + eSnowDS
      eWaterJS(xcoor,ycoor) = eWaterJS(xcoor,ycoor) + eWaterDS
      R0JS(xcoor,ycoor)=R0JS(xcoor,ycoor)+(R0DS)
      R1JS(xcoor,ycoor)=R1JS(xcoor,ycoor)+(R1DS)
      R2JS(xcoor,ycoor)=R2JS(xcoor,ycoor)+(R2DS)
      RGEJS(xcoor,ycoor)=RGEJS(xcoor,ycoor)+(RGEDS)
      GWNBJS(xcoor,ycoor)=GWNBJS(xcoor,ycoor)+(GWNBDS)
      GLACJS(xcoor,ycoor)=GLACJS(xcoor,ycoor)+GLACDS

! FORHYCS
      EBJS2(xcoor,ycoor)=EBJS2(xcoor,ycoor)
     *  +min(EBDS, EPDS-EIDS)
! Total sums -S(xcoor,ycoor) for each HRU
      PNKS(xcoor,ycoor)=PNKS(xcoor,ycoor)+PNDS
      PAKS(xcoor,ycoor)=PAKS(xcoor,ycoor)+PAKDS
      WSAS(xcoor,ycoor)=WSAS(xcoor,ycoor)+WSADS
      EPS(xcoor,ycoor)=EPS(xcoor,ycoor)+(EPDS)
      ERS(xcoor,ycoor)=ERS(xcoor,ycoor)+(ERDS)
      EIS(xcoor,ycoor)=EIS(xcoor,ycoor)+(EIDS)
      EBS(xcoor,ycoor)=EBS(xcoor,ycoor)+(EBDS)
      R0S(xcoor,ycoor)=R0S(xcoor,ycoor)+(R0DS)
      R1S(xcoor,ycoor)=R1S(xcoor,ycoor)+(R1DS)
      R2S(xcoor,ycoor)=R2S(xcoor,ycoor)+(R2DS)
      RGESS(xcoor,ycoor)=RGESS(xcoor,ycoor)+(RGEDS)
      GWNBS(xcoor,ycoor)=GWNBS(xcoor,ycoor)+(GWNBDS)
      GLACS(xcoor,ycoor)=GLACS(xcoor,ycoor)+(GLACDS)

! -----------------------------------------------------------------------

      RETURN
      END
