!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!     -----------------------------------------------------------------
      SUBROUTINE ANEFFIS (effnam,diroutput,bestrun,
     *                    iteri,stepa,OKI4,OKR4,bestold,ITLIM,
     *                    LINE,OPT,likelihood)

!     -----------------------------------------------------------------
!     Analyses statistical parameters for runoff simulations:
!     Input : 5 Statistics Files, 4 for linear Values, 1 for ln-Values
!     Output : Score for this model run
!
!	Optional processing of peak flow scores if corresponding control
!     file "FLS.EOPT" is found.
!
!     Literature: LETGATES, D.R., McCABE, G.J., 1999. Evaluating
!                    the use of goodness_of_fit measures in
!                    hydrologi! and hydroclimati! model validation.
!                    Water Resources Research, vol. 35, no.1, pp 233-241.
!                 DYCK, S., PESCHKE, G. (1995). Grundlagen der Hydrologie.
!                    Verlag f�r Bauwesen, 3. stark bearbeitete Auflage,
!                    Berlin, pp 123-127.
!                 SEIBERT, J. - MCDONNELL, J.J.: On the dialog between
!                    experimentalist and modeler in catchment hydrology:
!                    Use of soft data for multi-criteria model calibration.
!                    Water Resour. Res., 38(11), 2002,
!                    DOI:10.1029/2001WR000978.
!
!     Version 1.0: Massimiliano Zappa, May 2003
!     Actual Version 2.0 Massimiliano Zappa, September 2003
!     Fully implemented in PREVAH in September 2003
!     Version 2.1, adapted to Windows GUI requirments - January 2004
!     Calibration on peak flow scores added: Daniel Viviroli, December 2004
!     -----------------------------------------------------------------

      implicit none

      INTEGER I,LN,LN1,LN2,JA,JE,iteri,itlim,wnext,flln,pos1,pospin
      INTEGER MONSEA(12),MS,SCOL,RSCOL,OCOL,ROCOL,SZK,OZK,SVID,stvar
      INTEGER kaln,kalyy(100),crit,pairs,docrit(9),LN3,OPT(2),dstbest
      INTEGER iter,pos,af,afl,bestrun,stepa,OKI4(2,4),LINE(2),lobeg
      REAL SV(100,4,10),SV2(100,5,3),wcrit(9),limcrit(2,9)
      REAL best,OKR4(2,4),bestold,OKN4(2,4),likelihood(20)
      REAL SCORE(9),ratio(100),wstats(6),value(100),scorew(4),wall(4)
	REAL tyx(4),tyy(4),plopo,wink,a,b,dstjoker(4,2),dstmin
      CHARACTER*256 filehin,fileh,fileeffi
      CHARACTER*256 fileprev,fileout,effnam
      CHARACTER heada(5)*100,headb(5)*100,zkcrit(9)*75
      CHARACTER*100 bestzk,messrun,aktzk(4)
      CHARACTER*256 diroutput,filed,filedd,cppin,nextnam
      CHARACTER*2 atyp,itt
      CHARACTER*1 seas,yby,kaly,dm
      CHARACTER*8 addfa(3),addfb(2),addfc !*DV
	INTEGER	FDUMMY,FRNK(99) !*DV
	REAL FNSE(99), FWAE(99), FPQT(99), FPSAE(99), FSPMR(99) !*DV
	REAL FNSE_WT,  FWAE_WT,  FPQT_WT,  FPSAE_WT,  FSPMR_WT,F_WTS !*DV
	REAL FNSE_LAST,FWAE_LAST,FPQT_LAST,FPSAE_LAST,FSPMR_LAST!*DV
	REAL FSC(99),FSUM(99),FZZYSC(99)!*DV
	REAL FTOTSCORE(99),FOVERALL_WT !*DV
	LOGICAL FDO,FABORT,ISFLOG !*DV
	CHARACTER*255 TEXT !*DV

!     --------------------------------------
!     Input Parameters
!     --------------------------------------
      CALL POSNAM (diroutput,1,pos1)

	call posnam(effnam,1,pos)
	OPEN (701, file=effnam(1:pos), status='old')
      iter=1
      read (701,'(A1)') dm
	  DO I=1,256
	  fileprev(I:I)=' '
	  fileeffi(I:I)=' '
	  fileout(I:I)=' '
	  END DO
      read (701,'(A)') fileprev
      read (701,'(A)') fileeffi
	read (701,*) pairs
      crit=9
	read (701,'(A)') fileout
	OPEN (446, file='auto.opt', status='unknown')
	do I=1,crit
	read (701,'(A75)') zkcrit(I)
	call messagerun (2,zkcrit(I))
	read (701,*) docrit(I)
	read (701,*) wcrit(I)
	read (701,*) limcrit(1,I),limcrit(2,I)
	end do
	read (701,'(A1)',end=211) dm
      close (701)
	READ (446,*,end=211) iter

211    CONTINUE
      ITER=ITER+1
	close (446)
	write (itt,'(I2.2)') (ITER-1)
      call messagerun (1,'[Iteration number '//itt//']')
	call posnam (fileout,1,pos)
      OPEN (444, file=fileout(1:pos), status='unknown',
     * position='append')
	write (444,'(A)') ' '
	write (444,'(A)') ' '
	write (444,'(A)') ' '
	if (line(1) .gt. 0) write (444,'(A)') '[Parameter Pair '//itt//']'
	if (line(1) .lt. 0)
     *    write (444,'(A11,I6,A1)') '[Iteration ',stepa,']'
	if (line(1) .gt. 0)
     *    write (444,'(A11,I2,A1)') '[Iteration ',stepa,']'

	OPEN (446, file='auto.opt', status='unknown')

!     --------------------------------------
!     Efficiency Parameters
!     --------------------------------------
      call posnam (fileeffi,1,pos)
      OPEN (443, file=fileeffi(1:pos), status='old',err=888)
      read (443,'(A1)') dm
!      print* ,'Begin Year ? [YYYY]'
      read (443,*) JA

!      print* ,'End Year ? [YYYY]'
      read (443,*) JE

      yby='N'
!      print* ,'Input files Year by Year [Y,N] ?'
	read (443,'(A1)') yby

!      print* ,'Simulated File'
	read (443,'(A)') filehin

!	print* ,'Headers, Data Columns and Columns for Runoff?'
      read (443,*) SZK,SCOL,RSCOL

!      print* ,'Observed File'
	read (443,'(A)') fileh

!	print* ,'Headers, Data Columns and Columns for Runoff?'
      read (443,*) OZK,OCOL,ROCOL

!	print* ,'HH=Hourly Data,  DD=Daily Data,   DH= Daily Data from
!     * Hourly Data'
	read (443,'(A2)') atyp

!	print* ,'4 Chars for Output File'
	read (443,'(A4)') filed(1:4)

!	print* ,'Season [Y,N]'
	read (443,'(A1)') seas

!      print* ,'How many months for the season ?'
	read (443,*) ms

!      print* ,'Which months for the season ? (space separated)'
	read (443,*) (MONSEA(I),I=1,MS)

!	print* ,'Exclude calibration years [Y,N]'
	read (443,'(A1)') kaly

!      print* ,'How many calibration years ?'
	read (443,*) kaln

	IF (kaly .eq. 'n') GOTO 900
	IF (kaly .eq. 'N') GOTO 900

!      print* ,'Which years ? (YYYY, space separated)'
	read (443,*) (kalyy(I),I=1,kaln)
	close (443)

900   CONTINUE

!     -------------------------------------- !*DV
!     read peak flow score parameters from ctrl file
!     --------------------------------------
      INQUIRE (FILE="FLS.EOPT",EXIST=FDO)
	IF (.NOT.FDO) GOTO 4779

      CALL MESSAGERUN (1,'Calibration on peak flow active')

      OPEN (447,FILE="FLS.EOPT")
	READ (447,*)
	READ (447,*) FOVERALL_WT
	READ (447,*) FPSAE_WT
	READ (447,*) FWAE_WT
	READ (447,*) FNSE_WT
	READ (447,*) FPQT_WT
	READ (447,*) FSPMR_WT
	CLOSE (447)

	IF (FOVERALL_WT.EQ.0) FDO=.FALSE. !Skip if FLS overall weight is 0...
	FNSE_LAST =  -99
	FWAE_LAST =10E15
	FPQT_LAST =10E15
	FPSAE_LAST=10E15
	FSPMR_LAST=   -1

4779	CONTINUE

!     --------------------------------------
!     Read the analysis file
!     --------------------------------------

      best=-1.0
	if (line(1) .gt. 0) then
	flln=4
	lobeg=1
	end if
      if (line(1) .lt. 0) then
	flln=8
	lobeg=stepa
	end if

      likelihood=0

      DO AFL=lobeg,99999

      AF=MIN(AFL,4)

      if (line(1) .gt. 0) WRITE (FILED(1:4),'(A2,I2.2)') 'ou',AFL
      if (line(1) .lt. 0) WRITE (FILED(1:8),'(A2,I6.6)') 'ou',AFL

!	call messagerun (1,'filed  =  '//FILED(1:flln))

      SVID=JE-JA+1-kaln+2+12
	IF (seas .eq. 'N') svid=svid-1

      addfa(1)='_ost.dat'
      addfa(2)='_sst.dat'
	addfa(3)='_dst.dat'

	addfb(1)='_eli.dat'
      addfb(2)='_eln.dat'

	addfc   ='_fls.dat' !*DV

      DO LN2=1,3
	   filedd=diroutput(1:pos1)//filed(1:flln)//addfa(LN2)
	   if (line(1) .gt. 0) then
	   OPEN (443, file=filedd, status='old',ERR=1999)
         else
	   OPEN (443, file=filedd, status='old',ERR=299)
	   goto 298
299      continue
	   cycle
298      continue
	   end if
	   READ (443,'(A100)') Heada(LN2)
	   READ (443,'(A100)') Headb(LN2)
	   DO LN1=1,SVID
	   READ (443,*) LN3,(SV(LN1,LN2,1)),(SV(LN1,LN2,I),I=2,9)
	   END DO
	   CLOSE (443,status='delete')
	END DO

      write (444,'(A20)') '[SCORES '//FILED(1:flln)//']'

	DO LN1=1,SVID
	   ratio(LN1)=ABS(SV(LN1,2,5)/SV(LN1,1,5)-1)
      END DO


      DO LN2=2,3
	   filedd=diroutput(1:pos1)//filed(1:flln)//addfb(LN2-1)
	   OPEN (443, file=filedd, status='old')
	   READ (443,'(A100)') Heada(LN2+2)
	   READ (443,'(A100)') Headb(LN2+2)
	   DO LN1=1,SVID
	   READ (443,*) LN3,(SV2(LN1,1,1)),(SV2(LN1,I,LN2),I=1,4)
	   END DO
	   CLOSE (443,status='delete')
	END DO

! open and read floodstat-efficiency file !*DV
!	IF (FDO) THEN
		FILEDD=DIROUTPUT(1:POS1)//FILED(1:flln)//ADDFC
		OPEN (443, FILE=FILEDD, STATUS='OLD',err=855)
		READ (443,'(A100)')
		READ (443,'(A100)')
		READ (443,*) FDUMMY,FDUMMY,
     *				 FNSE(AF),FWAE(AF),FPQT(AF),
     *				 FPSAE(AF),FSPMR(AF)
	    likelihood(14)=FNSE(AF)
	    likelihood(15)=FWAE(AF)
	    likelihood(16)=FPQT(AF)
	    likelihood(17)=FPSAE(AF)
	    likelihood(18)=FSPMR(AF)
		CLOSE (443,STATUS='DELETE')
!	ENDIF
855   continue

!     --------------------------------------
!     COMPUTATION OF THE SCORES
!     --------------------------------------

!     --------------------------------------
!     1st SCORE
!     --------------------------------------

      likelihood(1)=SV2(1,4,2)

      IF (SV2(1,4,2) .LT. limcrit(1,1)) THEN
	    score(1)=0.0
	   ELSE
	    score(1)=(SV2(1,4,2)-limcrit(1,1))/(limcrit(2,1)-limcrit(1,1))
	END IF
      score(1)=min(1.0,score(1))

	write (444,'(A75)') zkcrit(1)
	write (444,'(A8,F6.3)') 'SCORE = ',score(1)
!     --------------------------------------
!     2nd SCORE
!     --------------------------------------

	STVAR=JE-JA+1-kaln
	DO I=1,STVAR
	   value(I)=SV2((I+1),4,2)
	END DO


      include 'osts.inc'

      likelihood(2)=wstats(6)

      IF (wstats(6) .GT. limcrit(1,2)) THEN
	    score(2)=0.0
	   ELSE
	    score(2)=1.0-max(0.0,(((wstats(6)-limcrit(2,2))/
     *             (limcrit(1,2)-limcrit(2,2)))))
	END IF

      score(2)=min(1.0,score(2))

	write (444,'(A75)') zkcrit(2)
      write (444,'(A8,F6.3)') 'STDEV = ',wstats(6)
	write (444,'(A8,F6.3)') 'SCORE = ',score(2)
!     --------------------------------------
!     3rd SCORE
!     --------------------------------------

	STVAR=JE-JA+1-kaln

	DO I=1,12
	   value(I)=SV2((I+STVAR+1),4,2)
	END DO
	stvar=12

      include 'osts.inc'

      likelihood(3)=wstats(6)

      IF (wstats(6) .GT. limcrit(1,3)) THEN
	    score(3)=0.0
	   ELSE
	    score(3)=1.0-max(0.0,(((wstats(6)-limcrit(2,3))/
     *             (limcrit(1,3)-limcrit(2,3)))))
	END IF

      score(3)=min(1.0,score(3))

	write (444,'(A75)') zkcrit(3)
      write (444,'(A8,F6.3)') 'STDEV = ',wstats(6)
	write (444,'(A8,F6.3)') 'SCORE = ',score(3)
!     --------------------------------------
!     4th SCORE
!     --------------------------------------

      likelihood(4)=SV2(1,4,3)

      IF (SV2(1,4,3) .LT. limcrit(1,4)) THEN
	    score(4)=0.0
	   ELSE
	    score(4)=(SV2(1,4,3)-limcrit(1,4))/(limcrit(2,4)-limcrit(1,4))
	END IF
      score(4)=min(1.0,score(4))

	write (444,'(A75)') zkcrit(4)
	write (444,'(A8,F6.3)') 'SCORE = ',score(4)
!     --------------------------------------
!     5th SCORE
!     --------------------------------------

	STVAR=JE-JA+1-kaln
	DO I=1,STVAR
	   value(I)=SV2((I+1),4,3)
	END DO

      include 'osts.inc'

      likelihood(5)=wstats(6)

      IF (wstats(6) .GT. limcrit(1,5)) THEN
	    score(5)=0.0
	   ELSE
	    score(5)=1.0-max(0.0,(((wstats(6)-limcrit(2,5))/
     *             (limcrit(1,5)-limcrit(2,5)))))
	END IF

      score(5)=min(1.0,score(5))

	write (444,'(A75)') zkcrit(5)
      write (444,'(A8,F6.3)') 'STDEV = ',wstats(6)
	write (444,'(A8,F6.3)') 'SCORE = ',score(5)
!     --------------------------------------
!     6th SCORE
!     --------------------------------------

	STVAR=JE-JA+1-kaln
	DO I=1,12
	   value(I)=SV2((I+STVAR+1),4,3)
	END DO
	stvar=12

      include 'osts.inc'

      likelihood(6)=wstats(6)

      IF (wstats(6) .GT. limcrit(1,6)) THEN
	    score(6)=0.0
	   ELSE
	    score(6)=1.0-max(0.0,(((wstats(6)-limcrit(2,6))/
     *             (limcrit(1,6)-limcrit(2,6)))))
	END IF

      score(6)=min(1.0,score(6))

	write (444,'(A75)') zkcrit(6)
      write (444,'(A8,F6.3)') 'STDEV = ',wstats(6)
	write (444,'(A8,F6.3)') 'SCORE = ',score(6)
!     --------------------------------------
!     7th SCORE
!     --------------------------------------

      likelihood(7)=ratio(1)

      IF (ratio(1) .GT. limcrit(1,7)) THEN
	    score(7)=0.0
	   ELSE
	    score(7)=1.0-max(0.0,(((ratio(1)-limcrit(2,7))/
     *             (limcrit(1,7)-limcrit(2,7)))))
	END IF
      score(7)=min(1.0,score(7))

	dstjoker(AF,1)=ratio(1)
      dstjoker(AF,2)=score(7)

	write (444,'(A75)') zkcrit(7)
	write (444,'(A8,F6.3)') 'SCORE = ',score(7)
!     --------------------------------------
!     8th SCORE
!     --------------------------------------

	STVAR=JE-JA+1-kaln
	DO I=1,STVAR
	   value(I)=ratio(I+1)
	END DO

      include 'osts.inc'

      likelihood(8)=wstats(6)

      IF (wstats(6) .GT. limcrit(1,8)) THEN
	    score(8)=0.0
	   ELSE
	    score(8)=1.0-max(0.0,(((wstats(6)-limcrit(2,8))/
     *             (limcrit(1,8)-limcrit(2,8)))))
	END IF

      score(8)=min(1.0,score(8))

	write (444,'(A75)') zkcrit(8)
      write (444,'(A8,F6.3)') 'STDEV = ',wstats(6)
	write (444,'(A8,F6.3)') 'SCORE = ',score(8)
!     --------------------------------------
!     9th SCORE
!     --------------------------------------

	STVAR=JE-JA+1-kaln
	DO I=1,12
	   value(I)=ratio(I+1+STVAR)
	END DO
      stvar=12

      include 'osts.inc'

      likelihood(9)=wstats(6)

      IF (wstats(6) .GT. limcrit(1,9)) THEN
	    score(9)=0.0
	   ELSE
	    score(9)=1.0-max(0.0,(((wstats(6)-limcrit(2,9))/
     *             (limcrit(1,9)-limcrit(2,9)))))
	END IF

      score(9)=min(1.0,score(9))

	write (444,'(A75)') zkcrit(9)
      write (444,'(A8,F6.3)') 'STDEV = ',wstats(6)
	write (444,'(A8,F6.3)') 'SCORE = ',score(9)

!     --------------------------------------
!     Finish
!     --------------------------------------

      wall(1)=wcrit(1)+wcrit(2)+wcrit(3)
      wall(2)=wcrit(4)+wcrit(5)+wcrit(6)
      wall(3)=wcrit(7)+wcrit(8)+wcrit(9)

!     -------------------------------------------
!     Weighted mean for criteria 1-3,4-6 and 7-9
!     -------------------------------------------

	scorew(1)=(score(1)*wcrit(1)+score(2)*wcrit(2)+
     /           score(3)*wcrit(3))/wall(1)
	scorew(2)=(score(4)*wcrit(4)+score(5)*wcrit(5)+
     /           score(6)*wcrit(6))/wall(2)
	scorew(3)=(score(7)*wcrit(7)+score(8)*wcrit(8)+
     /           score(9)*wcrit(9))/wall(3)

	scorew(1)=max(0.001,scorew(1))
	scorew(2)=max(0.001,scorew(2))
	scorew(3)=max(0.001,scorew(3))

!     --------------------------------------------------------
!     Overall acceptability after Seibert/McDonnell WRR 2002
!     Acceptability for ELI criteria - ELI-SCORE
!     Acceptability for ELN criteria - ELN-SCORE
!     Acceptability for SST/DST criteria - DST-SCORE
!     --------------------------------------------------------

	wall(4)=wall(1)+wall(2)+wall(3)
	wall(1)=wall(1)/wall(4)
	wall(2)=wall(2)/wall(4)
	wall(3)=wall(3)/wall(4)
	scorew(4)=scorew(1)**wall(1)*scorew(2)**wall(2)*scorew(3)**wall(3)
	FZZYSC(AF)=SCOREW(4) !*DV

	likelihood(10)=scorew(1)
	likelihood(11)=scorew(2)
	likelihood(12)=scorew(3)
	likelihood(13)=scorew(4)


!     -------------------------------------
!     Write summary and identify best run
!     -------------------------------------

	write (444,'(A14)') '[Total scores]'
	write (444,'(A12,F6.3)') 'ELI-SCORE = ',scorew(1)
	write (444,'(A12,F6.3)') 'ELN-SCORE = ',scorew(2)
	write (444,'(A12,F6.3)') 'DST-SCORE = ',scorew(3)
	write (444,'(A12,F6.3)') 'TOT-SCORE = ',scorew(4)
	write (444,'(A1)') ' '
	write (444,'(A14,4F6.3)') 'SCORES '//FILED(1:flln)//' = ',
     *                          (scorew(LN),LN=1,4)

      IF (OPT(1) .eq. 1) OKN4(1,AF)=FLOAT(OKI4(1,AF))
      IF (OPT(1) .eq. 2) OKN4(1,AF)=OKR4(1,AF)
	IF (OPT(2) .eq. 1) OKN4(2,AF)=FLOAT(OKI4(2,AF))
	IF (OPT(2) .eq. 2) OKN4(2,AF)=OKR4(2,AF)

!  Coefficients for taylor diagram
!	do ln=1,4
!	a=SV(1,2,7)/SV(1,1,7)
!	b=1.0
!	plopo=(a**2+b**2-2*a*b*scorew(LN))**0.5
!	wink=ACOS(scorew(LN))
!	tyx(ln)=a*scorew(LN)
!	tyy(ln)=a*sin(wink)
!	end do
!

	if (line(1) .lt. 0) then
	write (340,'(2X,I6.6,3X,I6.6,2X,I6.6,2(1X,I7),
     *               2(1X,E10.4),1X,13F6.3)')
     *             ITERI,STEPA,AF,LINE(1),LINE(2),
     *             OKN4(1,AF),OKN4(2,AF),(scorew(LN),LN=1,4)
	else
	write (340,'(2X,I2,3X,I2,2X,I2,2(1X,I7),
     *               2(1X,E10.4),1X,13F6.3)')
     *             ITERI,STEPA,AF,LINE(1),LINE(2),
     *             OKN4(1,AF),OKN4(2,AF),(scorew(LN),LN=1,4)
	end if

! Writing Taylor diagram ...
!     *             a,(tyx(ln),ln=1,4),(tyy(ln),ln=1,4)
	!writes info on all points that were probed


	best=max(best,scorew(4))

	write (aktzk(AF)(1:38),'(A14,4F6.3)') 'SCORES '//FILED(1:flln)//
     *                          ' = ',(scorew(LN),LN=1,4)

	if (best .eq. scorew(4)) then
	bestrun=AF

      write (bestzk(1:38),'(A14,4F6.3)') 'SCORES '//FILED(1:flln)//
     *                          ' = ',(scorew(LN),LN=1,4)
	end if
999   CONTINUE
      if (line(1) .lt. 0) exit
      END DO

	CLOSE (444)

      if (line(1) .lt. 0) goto 2000

1999  CONTINUE

!     -------------------
!     Accounting of strange cases
!     -------------------
!     Score(7) always less than 0.01 -> Best absolute ratio(1), stored in dstjoker(n,1)
      dstmin=200
      DO ln=1,4
	   if (dstjoker(ln,2) .gt. 0.01) goto 2000
	END DO

      DO ln=1,4
      if (dstmin .gt. dstjoker(ln,1)) dstbest=ln
	dstmin=min(dstmin,dstjoker(ln,1))
	END DO

      call messagerun (1,'Strange case - Volume Error too big')

	bestrun=dstbest
      bestzk(1:38)=aktzk(bestrun)(1:38)


2000  continue

!     -------------------------------------- !*DV
!     COMPUTATION OF PEAK FLOW SCORES
!     -------------------------------------------
	IF (.NOT.FDO) GOTO 9919
	FSC=0
	FSUM=0
	FTOTSCORE=0

	WRITE (TEXT,'(4(A,I2.2))')
     *  "ITER=",ITERI," STEP=",STEPA,
     * " LINE1=",LINE(1)," LINE2=",LINE(2)
	WRITE (448,'(A)') TRIM(TEXT)
	WRITE (TEXT,'(4(A,E10.4))')
     *  "VAL11=",OKN4(1,1)," VAL12=",OKN4(1,4),
     * " VAL21=",OKN4(2,1)," VAL22=",OKN4(2,4)
	WRITE (448,'(A)') TRIM(TEXT)

	IF (BESTOLD.NE.-2) THEN
		OPEN (499,FILE=DIROUTPUT(1:POS1)//"FLSSCORES.TMP")
		READ (499,*) FPSAE_LAST,FWAE_LAST,FNSE_LAST,
     *                 FPQT_LAST,FSPMR_LAST
		CLOSE (499)
	ENDIF

!     -------------------------------------- !*DV
!     peak flow score #1 (P-SAE) [0(best),infinity] = most important for peaks
!     -------------------------------------------
	call sort4(FPSAE(1),FPSAE(2),FPSAE(3),FPSAE(4),
     *           FRNK(1), FRNK(2), FRNK(3), FRNK(4), "D")
	DO I=1,4
		FSUM(I)=FSUM(I)+(FRNK(I))*FPSAE_WT
	ENDDO
	WRITE (448,'(A,F4.2,4(F12.2),4(I5))') "P-SAE     ",
     *       FPSAE_WT,(FPSAE(I),I=1,4),(FRNK(I),I=1,4)

!     -------------------------------------- !*DV
!     peak flow score #2 (WAE) [0(best),infinity]
!     -------------------------------------------
	call sort4(FWAE(1),FWAE(2),FWAE(3),FWAE(4),
     *           FRNK(1),FRNK(2),FRNK(3),FRNK(4),"D")
	DO I=1,4
		FSUM(I)=FSUM(I)+(FRNK(I))*FWAE_WT
	ENDDO
	WRITE (448,'(A,F4.2,4(F12.2),4(I5))') "WAE       ",
     *       FWAE_WT,(FWAE(I),I=1,4),(FRNK(I),I=1,4)

!     -------------------------------------- !*DV
!     peak flow score #3 (NSE) [-infinity,1(best)]
!     -------------------------------------------
	call sort4(FNSE(1),FNSE(2),FNSE(3),FNSE(4),
     *           FRNK(1),FRNK(2),FRNK(3),FRNK(4),"A")
	DO I=1,4
		FSUM(I)=FSUM(I)+(FRNK(I))*FNSE_WT
	ENDDO
	WRITE (448,'(A,F4.2,4(F12.2),4(I5))') "NSE       ",
     *       FNSE_WT,(FNSE(I),I=1,4),(FRNK(I),I=1,4)

!     -------------------------------------- !*DV
!     peak flow score #4 (POT-QT) [0(best),infinity] = most important for timing
!     -------------------------------------------
	call sort4(FPQT(1),FPQT(2),FPQT(3),FPQT(4),
     *           FRNK(1),FRNK(2),FRNK(3),FRNK(4),"D")
	DO I=1,4
		FSUM(I)=FSUM(I)+(FRNK(I))*FPQT_WT
	ENDDO
	WRITE (448,'(A,F4.2,4(F12.2),4(I5))') "POT-QT    ",
     *       FPQT_WT,(FPQT(I),I=1,4),(FRNK(I),I=1,4)

!     -------------------------------------- !*DV
!     peak flow score #5 (SPM-R) [-1,1(best)]
!     -------------------------------------------
	call sort4(FSPMR(1),FSPMR(2),FSPMR(3),FSPMR(4),
     *           FRNK(1), FRNK(2), FRNK(3), FRNK(4), "A")
	DO I=1,4
		FSUM(I)=FSUM(I)+(FRNK(I))*FSPMR_WT
	ENDDO
	WRITE (448,'(A,F4.2,4(F12.2),4(I5))') "SPM-R     ",
     *       FSPMR_WT,(FSPMR(I),I=1,4),(FRNK(I),I=1,4)

!     -------------------------------------- !*DV
!     Finish up peak flow scores
!     -------------------------------------------
	F_WTS=FNSE_WT+FWAE_WT+FPQT_WT+FPSAE_WT+FSPMR_WT

	DO I=1,4
		FSC(I)=FSUM(I)/F_WTS/3
		FTOTSCORE(I)=FSC(I)*FOVERALL_WT+FZZYSC(I)*(1-FOVERALL_WT)
	ENDDO

!     -------------------------------------- !*DV
!     Write log files and search for best run
!     -------------------------------------------
	WRITE (448,'(A10,TR52,4(I5))')  "SUM       ",
     *       (INT(FSUM(I)),I=1,4)
	WRITE (448,'(A,F4.2,4(F12.2))') "FLS_SC    ",
     *       FOVERALL_WT,(FSC(I),I=1,4)
	WRITE (448,'(A,F4.2,A,F10.2,3(F12.2))')  "FUZZY_SC (",
     *       1-FOVERALL_WT,") ",(FZZYSC(I),I=1,4)
	WRITE (448,'(A,TR4,4(F12.2))')  "OVERALL   ",
     *       (FTOTSCORE(I),I=1,4)

	BESTRUN=1
	DO I=1,4
		IF (FTOTSCORE(I).GT.FTOTSCORE(BESTRUN)) THEN
			BESTRUN=I
		ENDIF
		IF (FTOTSCORE(I).EQ.FTOTSCORE(BESTRUN)) THEN
		!In case two points receive similar score, decide either based on
		!FLS-score (overall weight>0.5) or FUZZY-Score (<=0.5)
			IF (FOVERALL_WT.GT.0.5) THEN
				IF (FSC(I).GT.FSC(BESTRUN)) THEN
					BESTRUN=I
				ENDIF
			ELSE
				IF (FZZYSC(I).GT.FZZYSC(BESTRUN)) THEN
					BESTRUN=I
				ENDIF
			ENDIF
		ENDIF
	ENDDO
	BEST=FTOTSCORE(BESTRUN)

	OPEN (499,FILE=DIROUTPUT(1:POS1)//"FLSSCORES.TMP")
	WRITE(499,'(5(F20.5))') FPSAE(BESTRUN),FWAE(BESTRUN),
     *           FNSE(BESTRUN),FPQT(BESTRUN),FSPMR(BESTRUN)
	CLOSE (499)

	WRITE (TEXT,'(A,I4,2(A,F5.2),A)')
     * "BEST RUN  ",BESTRUN,"  score",BEST," (last:",BESTOLD,")"
	WRITE (448,'(A)') TRIM(TEXT)
      CALL MESSAGERUN (1,TRIM(TEXT))

	IF (BESTOLD.GT.-2) THEN
	WRITE (TEXT,'(A,2(A,F6.1),A,F6.3,A,F6.1,A,F6.3)')
     *               "Improvements... ",
     *     "P-SAE:",-((FPSAE(BESTRUN)-FPSAE_LAST)/FPSAE_LAST)*100,
     *   "% / WAE:",-((FWAE(BESTRUN) -FWAE_LAST )/FWAE_LAST )*100,
     *   "% / NSE:", ((FNSE(BESTRUN) -FNSE_LAST )           ),
     * " / POT-QT:",-((FPQT(BESTRUN) -FPQT_LAST )/FPQT_LAST )*100,
     * "% / SPM-R:", ((FSPMR(BESTRUN)-FSPMR_LAST)           )
	WRITE (448,'(A)') TRIM(TEXT)
	ENDIF
	WRITE (448,'(A)')

	DO I=1,4
      WRITE (338,'(2X,I2,3X,I2,2X,I2,2(1X,I7),2(1X,E10.4),1X,13F6.3)')
     *             ITERI,STEPA,I,LINE(1),LINE(2),OKN4(1,I),OKN4(2,I),
     *             FSC(I),FZZYSC(I),0,FTOTSCORE(I)
	ENDDO
	!writes info on all points that were probed, along with flood measure only!


9919	CONTINUE

	if (line(1) .lt. 0) then
	CALL MESSAGERUN (1,aktzk(af)(1:38))
	return
	end if

!     -------------------
!     Declares best run
!     -------------------

      write (444,'(A1)') ' '
      write (444,'(A8)') 'BEST RUN'
      write (444,'(A38)') bestzk(1:38)
      write (339,'(2X,I2,3X,I2,6X,I2,2(1X,I7),2(1X,E10.4),1X,A24)')
     *             ITERI,STEPA,BESTRUN,LINE(1),LINE(2),
     *             OKN4(1,BESTRUN),OKN4(2,BESTRUN),bestzk(15:38)
	!writes info on best run that has been achieved

      call messagerun (1,'BEST RUN')
      write (messrun,
     *     '(2X,I2,3X,I2,6X,I2,2(1X,I7),2(1X,F))')
     *       ITERI,STEPA,BESTRUN,LINE(1),LINE(2),
     *       OKN4(1,BESTRUN),OKN4(2,BESTRUN)
      call messagerun (1,messrun)
      write (messrun,
     *     '(A17,1X,A24)') 'Best run scores: ',bestzk(15:38)
      call messagerun (1,messrun)
       write (messrun,'(A10,F6.3,A13,F6.3)')
     *             'Last step ',bestold,'   This step ',best
      call messagerun (1,messrun)

! decide wether furhter iterations will be done or not
! Fuzzy: improvement in score of at least 0.0025
! Flood: improvement of 0.005 in total score or at least 0.5% in one (weighted!) score component
	IF (.NOT.FDO) THEN !*DV
		IF (((best-bestold) .LT. 0.0025) .AND.
     *    ((best-bestold) .GT. 0.0))  THEN
		  call messagerun (1,'Further iterations for this step skipped')
			 stepa=ITLIM
			 best=-2.0
		END IF
	ELSE !*DV
		FABORT=.TRUE.
		IF ((BEST-BESTOLD).GT.0.005) FABORT=.FALSE.
		IF (BESTOLD.GT.-2) THEN
			IF (((FPSAE(BESTRUN)-FPSAE_LAST)/FPSAE_LAST).LT.0.005)
     *			FABORT=.FALSE.
			IF (((FWAE(BESTRUN) -FWAE_LAST )/FWAE_LAST ).LT.0.005)
     *			FABORT=.FALSE.
			IF (((FNSE(BESTRUN) -FNSE_LAST )/FNSE_LAST ).GT.0.005)
     *			FABORT=.FALSE.
			IF (((FPQT(BESTRUN) -FPQT_LAST )/FPQT_LAST ).LT.0.005)
     *			FABORT=.FALSE.
			IF (((FSPMR(BESTRUN)-FSPMR_LAST)/FSPMR_LAST).GT.0.005)
     *			FABORT=.FALSE.
		ENDIF
		IF (FABORT) THEN
			call messagerun
     *            (1,'Further iterations for this step skipped')
			WRITE (448,'(A)')
     *            'Further iterations for this step skipped'
			stepa=ITLIM
			best=-2.0
		ENDIF
	ENDIF !*DV



!      IF (best .gt. bestold)
      bestold=best

      IF (STEPA .ge. ITLIM) THEN
      write (446,'(I2)') ITER
      ELSE
      write (446,'(I2)') (ITER-1)
      END IF

      write (446,'(A11)') 'opt'//bestzk(10:11)//'.pin'
      write (446,'(A1)') ' '

      CLOSE (446)

      call messagerun (2,'[END REACHED]')
      call posnam (fileprev,1,pospin)
	OPEN (445, file=fileprev(1:pospin), status='old')
	OPEN (446, file=fileprev(1:pospin)//itt, status='unknown')

	DO I=1,1000
	   READ (445,'(A)',END=876) cppin
	   WRITE (446,'(A)') TRIM(cppin)
	END DO

876   CONTINUE
      CLOSE (446)
	CLOSE (445,status='delete')
	IF (diroutput(2:2) .eq. ':') THEN
	OPEN (445, file=diroutput(1:pos1)//
     *       'opt'//bestzk(10:11)//'.pin', status='old')
        ELSE
	OPEN (445, file='opt'//bestzk(10:11)//'.pin', status='old')
	END IF
      call posnam (fileprev,1,pospin)
	OPEN (446, file=fileprev(1:pospin), status='unknown')

	DO I=1,1000
	   READ (445,'(A100)',END=877) cppin
	   WRITE (446,'(A100)') cppin
	END DO
877   CONTINUE

      CLOSE (446)
	CLOSE (445,status='delete')
888   continue

	RETURN
      END

C -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
	SUBROUTINE SORT4(A1,A2,A3,A4,R1,R2,R3,R4,D)
	!Returns the ranks of 4 real values
	!Ascending  = smallest is 0, biggest 3
	!Descending = biggest is 3, smallest 0
	!Equal values are assigned same rank(s), skipping the subsequent rank(s)
	REAL A1,A2,A3,A4,A(4)		!VALUES, input
	INTEGER R1,R2,R3,R4,R(4)	!RANKS, output
	INTEGER P(4),P9				!POINTERS, temporary
	INTEGER I,J					!COUNTER
	CHARACTER*1 D				!DIRECTION, "A"=Ascending, "D"=Descending
	LOGICAL	X					!FLAG FOR CHANGE

	A(1)=A1;A(2)=A2;A(3)=A3;A(4)=A4
	P(1)=1;P(2)=2;P(3)=3;P(4)=4

43219	CONTINUE
	X=.FALSE.
	DO I=1,3
		DO J=I+1,4
			IF ((A(P(I)).GT.A(P(J))).AND.D.EQ."A") THEN
				P9=P(J);P(J)=P(I);P(I)=P9
				X=.TRUE.
			ENDIF
			IF ((A(P(I)).LT.A(P(J))).AND.D.EQ."D") THEN
				P9=P(J);P(J)=P(I);P(I)=P9
				X=.TRUE.
			ENDIF
		ENDDO
	ENDDO
	IF (X) GOTO 43219

	DO I=1,4
		DO J=1,4
			IF (P(J).EQ.I) THEN
				R(P(J))=J
			ENDIF
		ENDDO
	ENDDO

	DO I=2,4
		IF (A(P(I)).EQ.A(P(I-1))) THEN
			R(P(I))=R(P(I-1))
		ENDIF
	ENDDO

	R1=R(1)-1;R2=R(2)-1;R3=R(3)-1;R4=R(4)-1

	END SUBROUTINE
