!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2003 Joachim Gurtz, Massimiliano Zappa
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Enabling parallel execution with MPI

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

! ---------------------------------------------------------------------------------------------
      PROGRAM CALIPREVAH

!
!     Controls the calibration of PREVAH
!     Options:
!     DOCALI = 'y' or 'Y' -> full calibration
!     DOCALI = 'n' or 'N' -> full single run
!     DOCALI = 'c' or 'C' -> routing calibration
!     DOCALI = 'r' or 'R' -> routing single run
!     DOCALI = 'm' or 'M' -> multiple full runs
!     DOCALI = 'f' or 'F' -> forecast from file
!
!     Version 1.0 Without routing by M.Zappa. August-September 2003
!     Version 2.0 with routing by M.Zappa. November 2003
! ---------------------------------------------------------------------------------------------

use mpi
use realtime
use MPIvars
use fp_ops
!use malngam
!use mgamain
use auxroots
implicit none

!include 'mpif.h'  ! Declarations of MPI functions and constants


INTEGER:: I,LN,LINENAM,HRU
INTEGER:: J,RUN,TI,iter,itmax,rr,pos,bestrun,sta,pos3,pos4
INTEGER:: STEPA,ITLIM,nruns,WNEXT
integer,dimension(2,3):: dati
integer,dimension(2):: opt, line, oki
integer,dimension(3):: runs
integer,dimension(2,4):: oki4

! variable definition for reading program arguments
integer:: argnum,system,ires

REAL:: GEOBREITE,AELHG,mp1,mp2, bestold

real,dimension(2):: okr
real,dimension(20):: likelihood
real,dimension(2,3):: datr
real,dimension(2,4):: okr4

character:: dumc
character(len=1):: docali, idz
character(len=2):: chnr, itt
character(len=10):: pair
character(len=50):: gken
character(len=100):: string
character(len=256):: effini,ENAM,PNAM,CNAME,effnam,diroutput,NEXTNAM,FNAME,ONAME
character(len=256):: gknam,lnam,ernam
character(len=256),dimension(99):: mname


doParallel = .true. ! Temporary. This will need to be coordinated with make / run files.


! Initialization of MPI

root = 0
if (doParallel) then
  call MPIInit
else
  rank = root
end if

! Initialization of PREVAH: done by each process individually
	1     Continue
  sta=0

! read name of runfile from program argument. if not given, promt for filename
  argnum=1
  call getarg (argnum, CNAME)


if ((rank==root) .or. (doParallel==.false.)) then  ! Output to log file only by master
  if( CNAME == "")then
    call messageout ('Name of calibration control file (*.run)')
    READ (*,'(A)') CNAME
    Print*
  endif
end if


!	OPEN (336, file='read_ccf', status='old', err=211)
!	READ (336,'(A)') CNAME
!	CLOSE (336)
10    CONTINUE

  if ((rank==root) .or. (doParallel==.false.)) call MESSAGEFILE(cname)
  call posnam (CNAME,1,pos)

  OPEN (700, file=CNAME(1:pos), status='old', err=211)
  ITER=1
  OPEN (336, file='auto.opt', status='old', err=211)
  READ (336,*,end=211) iter

211   CONTINUE
  sta=sta+1
  CLOSE (336)
  write (itt,'(I2.2)') ITER

  READ (700,'(A1)') IDZ
  READ (700,'(A1)') docali
  READ (700,'(A)') FNAME


  call posnam (FNAME,1,pos)
  IF ((docali .eq. 'm') .or. (docali .eq. 'M')) then
    if ((rank==root) .or. (doParallel==.false.)) call MESSAGEFILE(fname)
    OPEN (339,FILE=FNAME(1:pos),STATUS='old', ERR=99)
    READ (339,*) nruns

    DO I=1,nruns
      READ (339,'(A)') MNAME(I)
    END DO

    CLOSE (339)
  END IF

  READ (700,'(A)') effini
  READ (700,*) ITMAX
  READ (700,*) ITLIM
  READ (700,*) HRU
  READ (700,*) TI
  READ (700,*) GEOBREITE
  READ (700,*) AELHG
  READ (700,*) LINENAM
  READ (700,*) switch_date

  IF ((docali .ne. 'y') .and. (docali .ne. 'Y') .and. &
  (docali .ne. 'c') .and. (docali .ne. 'C')) THEN
    PNAM=FNAME
    call posnam (FNAME,2,pos)
    do I=pos,1,-1
!	     if (fname(I:I) .eq. '\') exit
      if (fname(I:I) .eq. '/') exit
      pos4=I
    end do

    GKEN=FNAME(pos4:pos)//'_'

    ERNAM(1:11)='nocalib.out'
    GOTO 11
  END IF

  call posnam (diroutput,1,pos3)

if ((rank==root) .or.(doParallel == .false.)) then
  IF (sta .eq. 1) THEN
    OPEN (339,FILE=diroutput(1:pos3)//FNAME(1:4)//&
        'cali.best',STATUS='unknown', ERR=99)
    WRITE (339,'(A33,A46)') 'ITER STEP BESTRUN   LINE1   LINE2',&
	 '       PAR1       PAR2   ALIN  ALOG  ADST  ATOT'

    OPEN (340,FILE=diroutput(1:pos3)//FNAME(1:4)//&
	'cali.log',STATUS='unknown', ERR=99)
    WRITE (340,'(A29,A46)') 'ITER STEP RUN   LINE1   LINE2',&
	'       PAR1       PAR2   ALIN  ALOG  ADST  ATOT'
  END IF
end if

  IF (ITER .GT. ITMAX) GOTO 999

  nextnam=effnam

  DO RR=1,MIN(ITER+1,ITMAX)
    bestold=-2.0
    OKI(1)=0
    OKI(2)=0
    OKR(1)=0.0
    OKR(2)=0.0
    IF (RR .LE. ITER) THEN
      READ (700,'(A10)') PAIR
      print '(A10)',pair
      READ (700,'(A1)') IDZ
      READ (700,'(A)') EFFNAM

      DO I=1,2
	 opt(I)=2
	 READ (700,'(A1)') IDZ
        IF (IDZ .EQ. 'F') OPT(I)=2
	 IF (IDZ .EQ. 'f') OPT(I)=2
	 IF (IDZ .EQ. 'I') OPT(I)=1
	 IF (IDZ .EQ. 'i') OPT(I)=1
	 IF (OPT(I) .EQ. 1) READ (700,*) LINE(I),(DATI(I,LN),LN=1,3)
	 IF (OPT(I) .EQ. 2) READ (700,*) LINE(I),(DATR(I,LN),LN=1,3)
      END DO
    ELSEIF (RR .LT. ITMAX) then
      READ (700,'(A1)') dumc
      READ (700,'(A1)') dumc
      READ (700,'(A)') nextnam
    END IF
  END DO


  DO I=1,256
  WNEXT=0
    IF (NEXTNAM(I:I) .ne. EFFNAM(I:I)) THEN
      bestold=-2.0
      WNEXT=1
      EXIT
    END IF
  END DO

  CLOSE (700)

  RUNS(3)=32
  RUN=0

  STEPA=0

4     CONTINUE

  STEPA=STEPA+1
  run=0

  IF (STEPA .GT. ITLIM) GOTO 10

  IF (STEPA .GT. 1) THEN
    IF (BESTRUN .EQ. 1) THEN
			   IF (OPT(1) .EQ. 1) DATI(1,2)=DATI(1,2)-DATI(1,3)
			   IF (OPT(2) .EQ. 1) DATI(2,2)=DATI(2,2)-DATI(2,3)
			   IF (OPT(1) .EQ. 2) DATR(1,2)=DATR(1,2)-DATR(1,3)
			   IF (OPT(2) .EQ. 2) DATR(2,2)=DATR(2,2)-DATR(2,3)
    END IF

    IF (BESTRUN .EQ. 3) THEN
			   IF (OPT(1) .EQ. 1) DATI(1,1)=DATI(1,1)+DATI(1,3)
			   IF (OPT(2) .EQ. 1) DATI(2,2)=DATI(2,2)-DATI(2,3)
			   IF (OPT(1) .EQ. 2) DATR(1,1)=DATR(1,1)+DATR(1,3)
			   IF (OPT(2) .EQ. 2) DATR(2,2)=DATR(2,2)-DATR(2,3)
    END IF

    IF (BESTRUN .EQ. 2) THEN
			   IF (OPT(1) .EQ. 1) DATI(1,2)=DATI(1,2)-DATI(1,3)
			   IF (OPT(2) .EQ. 1) DATI(2,1)=DATI(2,1)+DATI(2,3)
			   IF (OPT(1) .EQ. 2) DATR(1,2)=DATR(1,2)-DATR(1,3)
			   IF (OPT(2) .EQ. 2) DATR(2,1)=DATR(2,1)+DATR(2,3)
    END IF

    IF (BESTRUN .EQ. 4) THEN
			   IF (OPT(1) .EQ. 1) DATI(1,1)=DATI(1,1)+DATI(1,3)
			   IF (OPT(2) .EQ. 1) DATI(2,1)=DATI(2,1)+DATI(2,3)
			   IF (OPT(1) .EQ. 2) DATR(1,1)=DATR(1,1)+DATR(1,3)
			   IF (OPT(2) .EQ. 2) DATR(2,1)=DATR(2,1)+DATR(2,3)
    END IF
  END IF

  IF (OPT(1) .EQ. 1) THEN
    DATI(1,3) = (DATI(1,2) - DATI(1,1))/3
    DATI(1,3) = MAX(1,DATI(1,3))
  END IF

  IF (OPT(1) .EQ. 2) DATR(1,3) = (DATR(1,2) - DATR(1,1))/3

  IF (OPT(2) .EQ. 1) THEN
    DATI(2,3) = (DATI(2,2) - DATI(2,1))/3
    DATI(2,3) = MAX(1,DATI(2,3))
  END IF

		  IF (OPT(2) .EQ. 2) DATR(2,3) = (DATR(2,2) - DATR(2,1))/3

		  IF (OPT(1) .EQ. 1) DATI(1,3) = (DATI(1,2) - DATI(1,1))/3
		  IF (OPT(1) .EQ. 2) DATR(1,3) = (DATR(1,2) - DATR(1,1))/3
		  IF (OPT(2) .EQ. 1) DATI(2,3) = (DATI(2,2) - DATI(2,1))/3
		  IF (OPT(2) .EQ. 2) DATR(2,3) = (DATR(2,2) - DATR(2,1))/3

		  IF (OPT(1) .EQ. 1) mp1=DATI(1,3)*0.5
		  IF (OPT(2) .EQ. 1) mp2=DATI(2,3)*0.5
		  IF (OPT(1) .EQ. 2) mp1=DATR(1,3)*0.5
		  IF (OPT(2) .EQ. 2) mp2=DATR(2,3)*0.5

		  IF (OPT(1) .EQ. 1) OKI(1)=DATI(1,1)
		  IF (OPT(1) .EQ. 2) OKR(1)=DATR(1,1)

	5     CONTINUE

		  IF (OPT(1) .EQ. 1) THEN
			  OKI(1)=OKI(1)+DATI(1,3)
			  IF (OKI(1) .GT. (DATI(1,2)-INT(mp1))) GOTO 99
		  END IF
		  IF (OPT(1) .EQ. 2) THEN
			  OKR(1)=OKR(1)+DATR(1,3)
			  IF (OKR(1) .GT. (DATR(1,2)-mp1)) GOTO 99
		  END IF

		  IF (OPT(2) .EQ. 1) OKI(2)=DATI(2,1)
		  IF (OPT(2) .EQ. 2) OKR(2)=DATR(2,1)

	6     CONTINUE

		  IF (OPT(2) .EQ. 1) THEN
			  OKI(2)=OKI(2)+DATI(2,3)
			  IF (OKI(2) .GT. (DATI(2,2)-int(MP2))) GOTO 5
		  END IF
		  IF (OPT(2) .EQ. 2) THEN
			  OKR(2)=OKR(2)+DATR(2,3)
			  IF (OKR(2) .GT. (DATR(2,2)-mp2)) GOTO 5
		  END IF

		  RUN=RUN+1

		  IF (run .gt. 4) GOTO 99

		  OKI4(1,RUN)=OKI(1)
		  OKI4(2,RUN)=OKI(2)
		  OKR4(1,RUN)=OKR(1)
		  OKR4(2,RUN)=OKR(2)

		  print *,'Calibration step ',ITER, ' of ',ITMAX
		  print *,'Iteration ',STEPA, ' of ',ITLIM
		  print *,'Calibration run ',RUN, ' of 4'

		  CALL CHARNR (RUN,CHNR)

		PNAM=diroutput(1:pos)//'opt'//CHNR//'.pin'
		  ONAME='out'//CHNR
		  gknam=effini
		  LNAM='gk'//CHNR//'.gut'
		  ERNAM='out'//CHNR//'.mit'
		  ENAM='givein.'//CHNR
		GKEN='ou'//CHNR


      if ((rank==root) .or. (doParallel==.false.)) call MESSAGEFILE(fname)
		  OPEN (334,FILE=FNAME,STATUS='old')
		  OPEN (335,FILE=PNAM,STATUS='unknown', ERR=99)

		  CLOSE (336)


		  DO I=1,10000
			 IF ((I .NE. LINE(1)) .AND. (I .NE. LINE(2)) .AND.&
		        (I .NE. LINENAM)) THEN
				  READ(334,'(A100)',END=9) STRING
				  WRITE(335,'(A100)') STRING
				  CYCLE
			 END IF

			 READ(334,'(A100)',END=9) STRING

			 IF (I .EQ. LINE(1)) THEN
			 IF (OPT(1) .EQ. 1) WRITE (335,*) OKI(1)
			 IF (OPT(1) .EQ. 2) WRITE (335,*) OKR(1)
		   CYCLE
			 END IF

			 IF (I .EQ. LINE(2)) THEN
			 IF (OPT(2) .EQ. 1) WRITE (335,*) OKI(2)
			 IF (OPT(2) .EQ. 2) WRITE (335,*) OKR(2)
		   CYCLE
			 END IF

			 IF ((docali .eq. 'y') .or. (docali .eq. 'Y')) THEN
			 IF (I .EQ. LINENAM) WRITE (335,'(A)') ONAME
		   ELSE
			 WRITE(335,'(A100)') STRING
		   END IF
		  END DO

	9     CONTINUE

		  CLOSE (334)
		  CLOSE (335)


	11    CONTINUE

if ((rank==root) .or. (doParallel==.false.)) print *,'*************'
IF ((docali .eq. 'm') .or. (docali .eq. 'M')) then
  do I=1,nruns
    if (doParallel) then
      call PREVAHO (TI,mname(I),effini,ernam,gken,diroutput,docali)
    end if
  end do
else
  call PREVAHO (TI,PNAM,effini,ernam,gken,diroutput,docali)
end if
if ((rank==root) .or. (doParallel==.false.)) print *,'*************'

		IF ((docali .ne. 'y') .and. (docali .ne. 'Y').and. &
		     (docali .ne. 'c') .and. (docali .ne. 'C')) GOTO 999

		  IF ((docali .ne. 'y') .and. (docali .ne. 'Y')) GOTO 6

		  call posnam (diroutput,1,pos)

		  OPEN (334,FILE=diroutput(1:pos)//'out'//CHNR//'.mit',&
		       STATUS='old', ERR=41)

	41    continue

		CLOSE (334,status='delete')
		  OPEN (334,FILE=diroutput(1:pos)//'out'//CHNR//'.pri',&
		       STATUS='old', ERR=42)
		CLOSE (334,status='delete')

	42    continue

		  OPEN (334,FILE=diroutput(1:pos)//'out'//CHNR//'.log',&
		       STATUS='old', ERR=43)
		CLOSE (334,status='delete')

	43    continue

		  OPEN (334,FILE=diroutput(1:pos)//'out'//CHNR//'.std',&
		       STATUS='old', ERR=44)
		CLOSE (334,status='delete')

	44    continue

		  OPEN (334,FILE=diroutput(1:pos)//'out'//CHNR//'.tmp',&
		       STATUS='old', ERR=45)
		CLOSE (334,status='delete')

	45    continue

		  OPEN (334,FILE=diroutput(1:pos)//'out'//CHNR//'.q',&
		       STATUS='old', ERR=46)
		CLOSE (334,status='delete')

	46    continue

		  goto 6

	99    CONTINUE

		  CLOSE (334)
		  CLOSE (335)
		  CLOSE (336)
		  CLOSE (337)

		print '(A,A)','Using '//effnam
		  call ANEFFIS (effnam,diroutput,bestrun,ITER,STEPA,OKI4,OKR4,&
		               bestold,ITLIM,LINE,OPT,likelihood)

		  if (stepa .lt. 9) GOTO 4

		  IF (ITER .LE. ITMAX ) GOTO 10

		  CLOSE (339)
		  CLOSE (340)

	999   CONTINUE


      ! Terminate the MPI execution environment.
      call MPI_FINALIZE(ierr)

      print *,'***************'
      print *,'* END REACHED *'
      print *,'***************'
      END

!* DISPLAY A MESSAGE (1)
!********************************************************************************
!* (1) - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
SUBROUTINE MESSAGEOUT(CONTENT)

CHARACTER(len=*):: CONTENT

      OPEN (999,FILE='PREVAH.LOG',POSITION="APPEND")
	WRITE (999,'(A)') TRIM(CONTENT)
	print* ,TRIM(CONTENT)
	CLOSE (999)

	END SUBROUTINE

!* DISPLAY A MESSAGE (1)
!********************************************************************************
!* (1) - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
SUBROUTINE MESSAGERUN(OPT,CONTENT)

CHARACTER(len=*) CONTENT
	INTEGER OPT

	IF (OPT .ne. 3) THEN
	print* ,TRIM(CONTENT)
      OPEN (999,FILE='PREVAH.LOG',POSITION="APPEND")
	WRITE (999,'(A)') TRIM(CONTENT)
	CLOSE (999)
	END IF
	END SUBROUTINE

!* DISPLAY A MESSAGE (1)
!********************************************************************************
!* (1) - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
SUBROUTINE MESSAGEFILE(FILETOTEST)

character(len=256):: content, filetotest
INTEGER:: ISFPOS

      call posnam (FILETOTEST,1,ISFPOS)

      OPEN (998,FILE=FILETOTEST(1:ISFPOS),status='old',err=100)
	CLOSE (998)

	CONTENT="OK File: "//FILETOTEST(1:ISFPOS)//' found - OK'

      OPEN (999,FILE='ISFILE.LOG',POSITION="APPEND")
	WRITE (999,'(A)') TRIM(CONTENT)
	print* ,TRIM(CONTENT)
	CLOSE (999)

	return

100   continue

	CONTENT="KO File: "//FILETOTEST(1:ISFPOS)//' NOT found - KO'

      OPEN (999,FILE='ISFILE.LOG',POSITION="APPEND")
	WRITE (999,'(A)') TRIM(CONTENT)
	print* ,TRIM(CONTENT)
	CLOSE (999)

	return

	END SUBROUTINE
