!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Reads in a grid with catchments; prepares output table
!-------------------------------------------------------------------------------

subroutine catchGridInit

  use prevahglobals, only: catch, nadgkw,ncol,nrow,namcatch, namprec, mcatch, pcatch, catchID, catchSize,&
  ecatch1, ecatch2, ecatch3, ecatch4, ecatch5, ecatch6, ecatch7,&
  namev1, namev2, namev3, namev4, namev5, namev6, namev7, rnodat
  use MPIvars, only: rank, root, doParallel

  implicit none

  integer :: posi1
  real, allocatable :: dom2d(:,:)
  integer :: ncatch, curMin, maxCatch
  integer :: xcoor, ycoor, i
  integer :: count, len
  ! Variables for input handling
  integer :: uwas, nxll, nyll, ngridsize, nodat, rloop
  real :: rcol, rrow, rxll, ryll, rgridsize
  real, dimension(6) :: wstats


!===============================

! Read in catchment grid
  call posnam(nadgkw,1,posi1)
  uwas = 731
  !OPEN (731, file=NADGKW(1:POSI1)//'.pwr', status='old',form='binary')
  OPEN (731, file=NADGKW(1:POSI1)//'.e03', status='old',form='binary')
  INCLUDE 'rwasim.inc'
  INCLUDE 'r2d.inc'
  catch = int(dom2d)
  deallocate(dom2d)

! Determine number of catchments
  ncatch = 0
  curMin = -1
  maxCatch = maxval(catch)
  do while(curmin .lt. maxCatch)
    curMin = minval(catch, mask = catch .gt. curMin)
    ncatch = ncatch + 1
  end do

  allocate(catchID(ncatch))
  allocate(catchSize(ncatch))
  catchID(1:ncatch) = 0
  catchSize(1:ncatch) = 0

  count = 1
  do ycoor = nrow, 1, -1
  do xcoor = 1,ncol
    if(any(catchID==catch(xcoor,ycoor))) cycle
    if(catch(xcoor,ycoor)==rnodat) cycle
    !print*, "catch", catch(xcoor,ycoor), rank
    catchID(count) = catch(xcoor,ycoor)
    count = count + 1
  end do
  end do

  do i = 1,size(catchID)
    do ycoor = nrow, 1, -1
    do xcoor = 1,ncol
      if(catch(xcoor,ycoor)==catchID(i)) catchSize(i) = catchSize(i) + 1
    end do
    end do
  end do

  if((rank == root) .or. (doParallel==.FALSE.)) then
    mcatch = 1987
    open(mcatch,file=namcatch,status="unknown")
    len = size(catchID)
    !write(mcatch,"(I3,X,I3,X,I5,X,<len>I8)")
    write(mcatch,"(A3,X,A3,X,A4,X,<len>I8)"), "day", "mon", "year",  catchID(1:size(catchID))

    pcatch = 1988
    open(pcatch,file=namprec,status="unknown")
    len = size(catchID)
    write(pcatch,"(A3,X,A3,X,A4,X,<len>I8)"), "day", "mon", "year",  catchID(1:size(catchID))

    ecatch1 = 1989
    open(ecatch1,file=namev1,status="unknown")
    write(ecatch1,"(A3,X,A3,X,A4,X,<len>I8)"), "day", "mon", "year",  catchID(1:size(catchID))

    ecatch2 = 1990
    open(ecatch2,file=namev2,status="unknown")
    write(ecatch2,"(A3,X,A3,X,A4,X,<len>I8)"), "day", "mon", "year",  catchID(1:size(catchID))

    ecatch3 = 1991
    open(ecatch3,file=namev3,status="unknown")
    write(ecatch3,"(A3,X,A3,X,A4,X,<len>I8)"), "day", "mon", "year",  catchID(1:size(catchID))

    ecatch4 = 1992
    open(ecatch4,file=namev4,status="unknown")
    write(ecatch4,"(A3,X,A3,X,A4,X,<len>I8)"), "day", "mon", "year",  catchID(1:size(catchID))

    ecatch5 = 1993
    open(ecatch5,file=namev5,status="unknown")
    write(ecatch5,"(A3,X,A3,X,A4,X,<len>I8)"), "day", "mon", "year",  catchID(1:size(catchID))

    ecatch6 = 1994
    open(ecatch6,file=namev6,status="unknown")
    write(ecatch6,"(A3,X,A3,X,A4,X,<len>I8)"), "day", "mon", "year",  catchID(1:size(catchID))

    ecatch7 = 1995
    open(ecatch7,file=namev7,status="unknown")
    write(ecatch7,"(A3,X,A3,X,A4,X,<len>I8)"), "day", "mon", "year",  catchID(1:size(catchID))
  end if




end subroutine catchGridInit


!===============================
! Writes daily average runoff [m3/s] from each catchment to file (.cat)
subroutine MPIcatchWrite(day, mon, year)

  use mpi
  use MPIvars, only: rank, root, nrowLocal, firstRow, lastRow,ierr,status, numslaves
  use prevahglobals, only: rgs2d, catchID, catchSize, catch,ncol, rgridsize, mcatch

  implicit none

  integer, intent(in) :: day, mon, year
  integer :: i, j
  real, allocatable :: locRSum(:), globRSum(:)
  real, allocatable :: rgTemp(:,:)
  integer, allocatable :: catchTemp(:,:)
  integer :: len

!------------------------

  allocate(locRSum(size(catchID)))
  locRSum = 0.0

  if(rank .ne. root) then
  ! Slaves: get daily sums for each catchment on their stripe
    allocate(rgTemp(ncol,nrowLocal))
    allocate(catchTemp(ncol,nrowLocal))
    rgTemp = rgs2d(:,firstRow:lastRow)
    catchTemp = catch(:,firstRow:lastRow)
   ! print*, "max, min runoff mm/day", rank, maxval(rgTemp), minval(rgTemp)
  ! Convert from mm/day to m3/s
    rgTemp = rgTemp * rgridsize*rgridsize / (1000.*86400.)
    !print*, "max, min runoff m3/s", rank, maxval(rgTemp), minval(rgTemp)
    do i = 1,size(catchID)
      locRSum(i) = sum(rgTemp, mask = catchTemp==catchID(i))
    end do
   ! print*, "locRSum", rank, locRSum
    call MPI_SEND(locRSum,size(catchID),MPI_REAL,root,21,MPI_COMM_WORLD,ierr)
  else
  ! Master
    allocate(globRSum(size(catchID)))
    globRSum = 0.0
    ! Sum of daily runoff over all stripes for each catchment
    do j = 1,numslaves
      call MPI_RECV(locRSum,size(catchID),MPI_REAL,j,21,MPI_COMM_WORLD,status,ierr)
      globRSum = globRSum + locRSum
    end do
    !print*, "globRSum", globRSum
    ! Write to file
    len = size(catchID)
    write(mcatch,"(I3,X,I3,X,I5,X,<len>F8.3)") day, mon, year, globRSum(1:len)
  end if  ! Slaves / master

  deallocate(locRSum)
  if(allocated(globRSum)) deallocate (globRSum)

end subroutine MPIcatchWrite


!===============================
! Writes daily  precipitation [mm/day] from each catchment to file (.prc)
subroutine MPIcatchPrec(day, mon, year)

  use mpi
  use MPIvars, only: rank, root, nrowLocal, firstRow, lastRow,ierr,status, numslaves
  use prevahglobals, only: prec2d, catchID, catchSize, catch,ncol, rgridsize, pcatch

  implicit none

  integer, intent(in) :: day, mon, year
  integer :: i, j
  real, allocatable :: locRSum(:), globRSum(:)
  real, allocatable :: rgTemp(:,:)
  integer, allocatable :: catchTemp(:,:)
  integer :: len

!------------------------

  allocate(locRSum(size(catchID)))
  locRSum = 0.0

  if(rank .ne. root) then
  ! Slaves: get daily sums for each catchment on their stripe
    allocate(rgTemp(ncol,nrowLocal))
    allocate(catchTemp(ncol,nrowLocal))
    rgTemp = prec2d(:,firstRow:lastRow)
    catchTemp = catch(:,firstRow:lastRow)
   ! print*, "max, min precip mm/day", rank, maxval(rgTemp), minval(rgTemp)
    do i = 1,size(catchID)
      locRSum(i) = sum(rgTemp, mask = catchTemp==catchID(i))
    end do
   ! print*, "locQSum", rank, locRSum
    call MPI_SEND(locRSum,size(catchID),MPI_REAL,root,21,MPI_COMM_WORLD,ierr)
  else
  ! Master
    allocate(globRSum(size(catchID)))
    globRSum = 0.0
    ! Sum of daily runoff over all stripes for each catchment
    do j = 1,numslaves
      call MPI_RECV(locRSum,size(catchID),MPI_REAL,j,21,MPI_COMM_WORLD,status,ierr)
      globRSum = globRSum + locRSum
    end do
    do i = 1,size(catchID)
      globRSum(i) = globRSum(i)/catchSize(i)
    end do
   ! print*, "globQSum", globRSum
    ! Write to file
    len = size(catchID)
    write(pcatch,"(I3,X,I3,X,I5,X,<len>F8.3)") day, mon, year, globRSum(1:len)
  end if  ! Slaves / master

  deallocate(locRSum)
  if(allocated(globRSum)) deallocate (globRSum)

end subroutine MPIcatchPrec

!=======================================================================
subroutine MPIcatchEvap(day,mon,year)

  use mpi
  use MPIvars, only: rank, root, nrowLocal, firstRow, lastRow,ierr,status, numslaves
  use prevahglobals, only: ecatch1, ecatch2, ecatch3, ecatch4, ecatch5, ecatch6, ecatch7,&
      pt2d,at2d,pes2d,aes2d,int2d,esn2d,enf2d,catchID, catchSize, catch,ncol, rgridsize,&
        namev1, namev2, namev3, namev4, namev5, namev6, namev7

  implicit none

  integer, intent(in) :: day, mon, year
  integer :: var, i, j
  real, allocatable :: locRSum(:), globRSum(:)
  real, allocatable :: rgTemp(:,:)
  integer, allocatable :: catchTemp(:,:)
  integer :: curFH
  integer :: len
!-----------------------------------------------------------------------
  allocate(locRSum(size(catchID)))
  locRSum = 0.0
  ! Loop over variables

  do var = 1,7

    if(rank .ne. root) then
      allocate(rgTemp(ncol,nrowLocal))
      allocate(catchTemp(ncol,nrowLocal))

      select case(var)
      case(1)
        rgTemp = pt2d(:,firstRow:lastRow)
      case(2)
        rgTemp = at2d(:,firstRow:lastRow)
      case(3)
        rgTemp = pes2d(:,firstRow:lastRow)
      case(4)
        rgTemp = aes2d(:,firstRow:lastRow)
      case(5)
        rgTemp = int2d(:,firstRow:lastRow)
      case(6)
        rgTemp = esn2d(:,firstRow:lastRow)
      case(7)
        rgTemp = enf2d(:,firstRow:lastRow)
      end select
      catchTemp = catch(:,firstRow:lastRow)
      do i = 1,size(catchID)
        locRSum(i) = sum(rgTemp, mask = catchTemp==catchID(i))
      end do
      call MPI_SEND(locRSum,size(catchID),MPI_REAL,root,21,MPI_COMM_WORLD,ierr)

      deallocate(rgTemp)
      deallocate(catchTemp)

    else  ! Master
      select case(var)
      case(1)
        curFH = ecatch1
      case(2)
        curFH = ecatch2
      case(3)
        curFH = ecatch3
      case(4)
        curFH = ecatch4
      case(5)
        curFH = ecatch5
      case(6)
        curFH = ecatch6
      case(7)
        curFH = ecatch7
      end select

      allocate(globRSum(size(catchID)))
      globRSum = 0.0
      do j = 1,numslaves
        call MPI_RECV(locRSum,size(catchID),MPI_REAL,j,21,MPI_COMM_WORLD,status,ierr)
        globRSum = globRSum + locRSum
      end do
      do i = 1,size(catchID)
        globRSum(i) = globRSum(i)/catchSize(i)
      end do
      ! Write to file
      len = size(catchID)
      write(curFH,"(I3,X,I3,X,I5,X,<len>F8.3)") day, mon, year, globRSum(1:len)
     ! print*, "curFH", curFH
      deallocate(globRSum)
    end if  ! Slaves/Master
  end do    ! Loop over variables

end subroutine MPIcatchEvap
