!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Auxiliary subroutine to assign land cover-specific parameter values to the
! current cell. For forested cells, some values will be overwritten in
! PREVAHCORE.
!-------------------------------------------------------------------------------

subroutine cellLCparams(land, mon, lai, zveg, bflux, rsmin, albedo, vFrac, SImax, awcSF, stock)

use prevahglobals, only: stdLai, stdZveg, stdRSmin, stdVfrac, &
       stdSImax, WHCsf, stdAlb
use forhycsglobals, only: TMtoPR

implicit none

integer, intent(in) :: land, mon
logical, intent(in) :: stock
real, intent(out) :: lai, zveg, bflux, rsmin, albedo, vFrac, SImax, awcSF
real, dimension(12), parameter :: albSoil = (/0.1, 0.1, 0.1, 0.1, 0.2, 0.2, 0.2, 0.2, 0.1, 0.1, 0.1, 0.1/)


select case (land)
  case(3,4,5,8,21,23,24,25,26)
    if((TMtoPR == 1).and.(stock==.true.)) then
      lai = stdLai(land,mon)
      zveg = stdZveg(land,mon)
      bflux = 0.3-0.03*lai
      rsmin = stdRSmin(land,mon)
      vFrac = stdVfrac(land,mon)
      SImax = stdSImax(land,mon)
    else
      lai = stdLai(land,mon)
      zveg = stdZveg(land,mon)
      bflux = 0.3-0.03*lai
      rsmin = stdRSmin(land,mon)
      vFrac = stdVfrac(land,mon)
      SImax = stdSImax(land,mon)
    end if

  case default
    lai = stdLai(land,mon)
    zveg = stdZveg(land,mon)
    bflux = 0.2
    rsmin = stdRSmin(land,mon)
    vFrac = stdVfrac(land,mon)
    SImax = stdSImax(land,mon)

  end select

    awcSF = whcSF(land)

! Albedo: averaged between vegetation and soil fractions.
  if(vFrac == 1.0) then
    albedo = stdAlb(land)
  else if (vFrac == 0.0) then
    albedo = albsoil(mon)
  else
    albedo = (vFrac * stdAlb(land)) + ((1-vFrac) * albsoil(mon))
  end if

  if((stdLai(land,mon)==0.0)) albedo = stdAlb(land)


end subroutine cellLCparams
