!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

! ---------------------------------------------------------------------
       SUBROUTINE CLIMATE (ISTK,NRK,MON,PRL,TEC,RGL,RGDTI,WIV,RLF,
     *          CLIMFILE )

! This routine assimilates climate scenarios from a file
!
! Author: M.Zappa 2004
! ---------------------------------------------------------------------
! Massimiliano Zappa, Dr. sc. nat. ETH 	- massimiliano.zappa@wsl.ch
! Eidgen�ssische Forschungsanstalt WSL
! Abteilung Wasser-, Erd- und Felsbewegungen
! Z�rcherstr. 111
! CH-8903 Birmensdorf
!
! Tel. ++41 1 739 24 33     Fax ++ 41 1 739 24 88
!
! ---------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------

      INTEGER ISTK,
     *        NRK,
     *        MON
      REAL    PRL,                             ! Precipitation
     *        TEC,                             ! Temperature
     *        RGL,                             ! Global radiation, daily mean
     *        RGDTI(24),                       ! Global radiation, hourly values
     *        WIV,                             ! Wind speed
     *        RLF                              ! relative humidity

	CHARACTER CLIMFILE*256,duchar*2


! ---------------------------------------------------------------------
!     Internal Variables
! ---------------------------------------------------------------------

      INTEGER I,
     *        RD,
     *        LN  ! Index for implicit loops in READ statements
      REAL    TEC1,                            ! backup temperature
     *        DAD,                             ! water vapour pressure
     *        DADM                             ! backup water vapour pressure

      REAL TS(12),PS(12),RGS(12),DADS(12),WIVS(12),rhg

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------
      IF (NRK .eq. 0) RD=0

      RD=RD+1
	RD=MIN(RD,2)

          IF (RD.EQ.1) THEN
          OPEN (777,file=CLIMFILE,status='unknown')
	    call messagerun (1,'Using climate scenario')

          READ (777, '(A2)') DUCHAR
          READ (777, '(A2)') DUCHAR
          READ (777, '(12(F,1X))')  (PS(LN),LN=1,12)
          READ (777, '(A2)') DUCHAR
          READ (777, '(12(F,1X))')  (TS(LN),LN=1,12)
          READ (777, '(A2)') DUCHAR
          READ (777, '(12(F,1X))')  (RGS(LN),LN=1,12)
          READ (777, '(A2)') DUCHAR
          READ (777, '(12(F,1X))')  (DADS(LN),LN=1,12)
          READ (777, '(A2)') DUCHAR
          READ (777, '(12(F,1X))')  (WIVS(LN),LN=1,12)
	    CLOSE (777)
          END IF

            PRL=PRL*PS(MON)
            TEC1=TEC
            TEC=TEC+TS(MON)
            IF (ISTK .EQ. 1) THEN
               RGL=RGL*RGS(MON)
	         RGL=MAX(0.,RGL)
               DO 2 I=1,24
                  RGDTI(I)=RGDTI(I)*RGS(MON)
	            RGDTI(I)=MAX(0.,RGDTI(I))
2              CONTINUE
            ENDIF
            WIV=WIV*WIVS(MON)
            RHG=100.0
            CALL VAPOUR (TEC1,RLF,DAD)
            DAD=DAD*DADS(MON)
            CALL VAPOUR (TEC,RHG,DADM)
            IF (DAD .GT. DADM) DAD=DADM
            RLF=(DAD/DADM)*100.0

! ---------------------------------------------------------------------
      RETURN
      END

! ---------------------------------------------------------------------
! PREVAH (Precipitation-Runoff-Evapotranspiration HRU model)
! Subroutine of Model core for WINDOWS, DOS, LINUX and SUN
! ---------------------------------------------------------------------
       SUBROUTINE DELTACLIMATE (ISTK,NRK,MON,PRL,TEC,RGL,RGDTI,WIV,RLF,
     *          CPRL,CRGMKDM,CWIV,CRLF,CTECK,
     *          deltavar,deltamethod)

! This routine assimilates climate scenarios after Bosshard et al. (2010
!
! Author: M.Zappa, T Bosshard, June 2010
! ---------------------------------------------------------------------
! Massimiliano Zappa, Dr. sc. nat. ETH 	- massimiliano.zappa@wsl.ch
! Eidgen�ssische Forschungsanstalt WSL
! Abteilung Wasser-, Erd- und Felsbewegungen
! Z�rcherstr. 111
! CH-8903 Birmensdorf
!
! Tel. ++41 1 739 24 33     Fax ++ 41 1 739 24 88
!
! ---------------------------------------------------------------------
      implicit none

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------

      INTEGER ISTK,
     *        NRK,
     *        MON
      REAL    PRL,                             ! Precipitation
     *        TEC,                             ! Temperature
     *        RGL,                             ! Global radiation, daily mean
     *        RGDTI(24),                       ! Global radiation, hourly values
     *        WIV,                             ! Wind speed
     *        RLF                              ! relative humidity

! ---------------------------------------------------------------------
!     Internal Variables
! ---------------------------------------------------------------------

      INTEGER I,STP

! ------------------------------------------------------------------------------
! Deltachange scenarios
	character deltavar(6),deltamethod(6)
	real CPRL,CRGMKDM,CWIV,CRLF,CTECK

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------

! [Precipitation]
      stp=1
      if (deltavar(stp). eq. 'Y') then
      if (deltamethod(stp). eq. 'M') then
	PRL=PRL*CPRL
	end if
      if (deltamethod(stp). eq. 'A') then
      PRL=PRL+CPRL
	PRL=max(0.0,PRL)
	end if
	end if

![Temperature]
      stp=2
      if (deltavar(stp). eq. 'Y') then
      if (deltamethod(stp). eq. 'M') then
      TEC=TEC*CTECK
      end if
      if (deltamethod(stp). eq. 'A') then
      TEC=TEC+CTECK
      end if
      end if

![Global Radiation]
      stp=3
      if (deltavar(stp). eq. 'Y') then
      if (deltamethod(stp). eq. 'M') then
      RGL=RGL*CRGMKDM
	RGL=MAX(0.,RGL)
      DO I=1,24
      RGDTI(I)=RGDTI(I)*CRGMKDM
	RGDTI(I)=MAX(0.,RGDTI(I))
      END DO
      end if
      if (deltamethod(stp). eq. 'A') then
      RGL=RGL+CRGMKDM
	RGL=MAX(0.,RGL)
      DO I=1,24
      RGDTI(I)=RGDTI(I)+CRGMKDM
	RGDTI(I)=MAX(0.,RGDTI(I))
      END DO
      end if
      end if

![Vapour Pressure / Humidity]
      stp=4
      if (deltavar(stp). eq. 'Y') then
      if (deltamethod(stp). eq. 'M') then
      RLF=RLF*CRLF
      end if
      if (deltamethod(stp). eq. 'A') then
      RLF=RLF+CRLF*100
	RLF=max(0.0,RLF)
      end if
      end if

![Wind speed]
      stp=5
      if (deltavar(stp). eq. 'Y') then
      if (deltamethod(stp). eq. 'M') then
      WIV=WIV*CWIV
      end if
      if (deltamethod(stp). eq. 'A') then
      WIV=WIV+CWIV
	WIV=max(0.0,WIV)
      end if
      end if
![Relative sunshine duration]
! Later on!

! ---------------------------------------------------------------------
      RETURN
      END
