!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Gives a daily value for atmospheric CO2 concentration. Linear interpolation
! between decadal modeled values as provided by the IPCC. For now - no
! seasonal cycling
!-------------------------------------------------------------------------------

subroutine co2(day, mon, year, ts, ca)

  implicit none

! Passed variables
  integer, intent(in) :: day, mon, year
  integer, intent(in) :: ts               ! Code indicating which time series to use
  real, intent(out) :: ca                 ! CO2 concentration

! Local variables
  real :: decadal(14)
  real :: relyear


  ! IPCC decadal values - values every 10 years from 1970 to 2100.
  ! Source: http://www.ipcc-data.org/observ/ddc_co2.html (accessed Nov. 28, 2017)

  real, parameter :: isam_a1b(14) = (/325.0, 337.0, 353.0, 369.0, 391.0, 420.0, &
                        454.0, 491.0, 532.0, 572.0, 611.0, 649.0, 685.0, 717.0/)
  real, parameter :: isam_a2(14) = (/325.0, 337.0, 353.0, 369.0, 390.0, 417.0, &
                        451.0, 490.0, 532.0, 580.0, 635.0, 698.0, 771.0, 856.0/)
  real, parameter :: bern_a1b(14) = (/325.0, 337.0, 352.0, 367.0, 388.0, 418.0, &
                        447.0, 483.0, 522.0, 563.0, 601.0, 639.0, 674.0, 703.0/)
  real, parameter :: bern_a2(14) = (/325.0, 337.0, 352.0, 367.0, 386.0, 414.0, &
                        444.0, 481.0, 522.0, 568.0, 620.0, 682.0, 754.0, 836.0/)

!=======================================================================

  select case(ts)
    case(11)
      decadal = isam_a1b
    case(12)
      decadal = isam_a2
    case(21)
      decadal = bern_a1b
    case(22)
      decadal = bern_a2
  end select

  if(year.ge.1970) then
    relyear = ((real(year)-1970.)/10.) + 1.0
    ca = decadal(floor(relyear)) + (decadal(ceiling(relyear)) - decadal(floor(relyear))) * modulo(relyear,1.0)
  else
    ca = 350.
  end if

return
end subroutine co2
