!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Addition of global parameters for the coupling in FORHYCS

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Calculates the annual De Martonne Index (de Martonne, 1926) for each cell.
! Modified to cover only the growing season instead of annual P and T
! (e.g. Rais et al., 2014 10.1007/s10342-014-0821-7)
!-------------------------------------------------------------------------------

subroutine deMartonne

use forhycsglobals, only: gsPrecCur, gsTempCur, gsDMI, gsLengthCur
use prevahglobals, only: ncol
use MPIvars, only: nrowLocal, firstRow, lastRow

implicit none

integer :: xcoor, ycoor
real :: gsl, psum, tavg


do ycoor = lastRow, firstRow, -1
do xcoor = 1, ncol
  if(gsLengthCur(xcoor,ycoor) == 0) cycle
  gsl = gsLengthCur(xcoor,ycoor)
  psum = gsPrecCur(xcoor,ycoor)
  tavg = gsTempCur(xcoor,ycoor) / gsl
  gsDMI(xcoor,ycoor) = psum/(tavg+10.)
end do
end do


end subroutine deMartonne
