!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE EFFCOEF (ininam,ernam,gken,dirmeteo,diroutput,docali,
     *                    dorout,AELHG,ZN,mhtml)
!--------------------------------------------------------------------------
!    Determinates statistical parameters for runoff simulations:
!    Input : Simulated Runoff (1 file), headers, number of columns
!            and column of runoff have to be specified
!    Input : Observed Runoff (1 file), headers, number of stations
!            and position of relevant station have to be specified
!            Format: Hourly: YY MM DD HH St1 St2 St..
!                    Daily:  YY MM DD St1 St2 St.....
!    Output : 5 Statistics Files, 4 for linear Values
!                                 1 for ln-Values
!
!    Literature: LEGATES, D.R., McCABE, G.J., 1999. Evaluating
!                   the use of goodness_of_fit measures in
!                   hydrologic and hydroclimatic model validation.
!                   Water Resources Research, vol. 35, no.1, pp 233-241.
!                DYCK, S., PESCHKE, G. (1995). Grundlagen der Hydrologie.
!                   Verlag f�r Bauwesen, 3. stark bearbeitete Auflage,
!                   Berlin, pp 123-127.
!
!    Version 1.0: Massimiliano Zappa, 30.7.2001
!    Version 2.0 Massimiliano Zappa, August 2003
!                Implemented into PREVAH in August 2003
!    Version 3.0 Massimiliano Zappa, Daniel Viviroli, February 2004
!				Flood statistic added
! ------------------------------------------------------------------------------
!
! Contact
! Massimiliano Zappa, Dr. sc. nat. ETH 	- massimiliano.zappa@wsl.ch
! Eidgen�ssische Forschungsanstalt WSL
! Abteilung Wasser-, Erd- und Felsbewegungen
! Z�rcherstr. 111
! CH-8903 Birmensdorf
!
! Tel. ++41 1 739 24 33     Fax ++ 41 1 739 24 88
!
! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------

      implicit none

      INTEGER I,LN1,LN2,MO,JA,JE,IGST,TA,HO,JJ,CT,step,NOW,mnow,flln
      INTEGER MONSEA(12),MS,SCOL,RSCOL,OCOL,ROCOL,SZK,OZK,SVID,JID
      INTEGER ybya,kaln,kalyy(100),pos,pos1,pos2,pos3,pos4,pos5,mhtml
      REAL DAYR,HOR,SIMOUT(50),SIMRD,L1S,L1O,SV(100,4,10)
      REAL SV2(100,5,3),OBSIN(50),so,dor,AELHG
      REAL SUMS(4),AVGS(4),VARZ(4),STABWS(4),NUMS(4),CO1(4),CO2(4)
      REAL TOP(4),BOT(4),EFFS(5,2),NOMS(5,2),DENS(5,2),exps(5)
      CHARACTER*512 fileh
	CHARACTER*256 filed,filedd,filetmp
	CHARACTER*1 erer
	CHARACTER*256 ininam
	CHARACTER*256 ernam,filehin
	Character*256 dirmeteo,diroutput
      CHARACTER waskomm*50,heada(5)*100,headb(5)*100,gken*8,ZN*32
      CHARACTER*2 atyp
      CHARACTER*1 seas,kaly,docali,dorout,dum,dofstat
      CHARACTER*8 addfa(3),addfb(2)
	CHARACTER*512 LISTSKIP_FILE
	INTEGER	LISTSKIP_N
	integer, allocatable ::  LISTSKIP_JJ(:),LISTSKIP_MO(:)
	integer, allocatable ::  LISTSKIP_TA(:),LISTSKIP_HO(:)
	REAL    LISTSKIP_NODATA

! ---------------------------------------------------------------------

	SVID=0

      exps(1)=0.25
	exps(2)=0.5
	exps(3)=1.0
	exps(4)=2.0
	exps(5)=4.0
	call posnam (ininam,1,pos)
      open (765,file=ininam(1:pos),status='old')

	flln=4
	IF ((docali .eq. 'g') .or. (docali .eq. 'G')) flln=8

! ---------------------------------------------------------------------
!    Input Parameters
! ---------------------------------------------------------------------
      read (765,*) dum
!      print* ,'Begin Year ? [YYYY]'
      read (765,*) JA

!      print* ,'End Jear ? [YYYY]'
      read (765,*) JE

      dofstat='N'
!      print* ,'Call Floodstat [Y,N] ?'
	read (765,'(A1)') dofstat

!      print* ,'Simulated File'
	read (765,'(A256)') filehin
	  DO I=1,256
	  filehin(I:I)=' '
	  END DO
	filehin=ernam

	call posnam (filehin,2,pos3)
	!call posnam (filehin,1,pos4)

	erer(1:1)=filehin((pos3+2):(pos3+2))

!	print* ,'Headers, Data Columns and Columns for Runoff?'
      read (765,*) SZK,SCOL,RSCOL

!      print* ,'Observed File'
	  DO I=1,512
	  fileh(I:I)=' '
	  END DO
	read (765,'(A)') fileh

!	print* ,'Headers, Data Columns and Columns for Runoff?'
      read (765,*) OZK,OCOL,ROCOL

!	print* ,'HH=Hourly Data,  DD=Daily Data,   DH= Daily Data from
!     * Hourly Data'
	read (765,'(A2)') atyp

	if (atyp .eq. 'HH') then
	szk=0
      scol=5
	RSCOL=5
	END IF

	if (atyp .eq. 'dd') then
      szk=2
	SCOL=14
	rscol=14
      END IF

	IF ((dorout .eq. 'y') .or. (dorout .eq. 'Y')) THEN
	call messagerun (2,'Reading routing file '//ernam(1:35))
	SZK=0
	SCOL=8
	RSCOL=8
	END IF

	if (atyp .eq. 'HH') then
	    if (erer(1:1) .eq. 'm') filehin=filehin(1:pos3)//'.q   '
	    if (erer(1:1) .eq. 'r') filehin=filehin(1:pos3)//'.rhh '
	end if


!	print* ,'4 Chars for Output File'
	read (765,'(A4)') filed(1:flln)
      IF ((docali .eq. 'y') .or. (docali .eq. 'Y') .or.
     *   (docali .eq. 'c') .or. (docali .eq. 'C') .or.
     *   (docali .eq. 'g') .or. (docali .eq. 'G'))
     *    filed(1:flln)=gken(1:flln)

!	print* ,'Season [Y,N]'
	read (765,'(A1)') seas

!      print* ,'How many months for the season ?'
	read (765,*) ms

!      print* ,'Which months for the season ? (space separated)'
	read (765,*) (MONSEA(I),I=1,MS)

!	print* ,'Exclude calibration years [Y,N], use List [L] or skip NODATA value [V]'
!     print* ,'List:   add path and name of list file after [L ]'
!     print* ,'Nodata: add nodata value after [V ]'
!     print* ,'        lowercase [l/v] still skips the specified calibration years'
	read (765,'(A1)') kaly

!	LISTKIP_N=0
	IF ((KALY.EQ.'l').OR.(KALY.EQ.'L')) THEN
		BACKSPACE (765)
	    allocate (LISTSKIP_JJ(99999))
		allocate (LISTSKIP_MO(99999))
		allocate (LISTSKIP_TA(99999))
	    allocate (LISTSKIP_HO(99999))
		read (765,'(A1,TR1,A512)') kaly,LISTSKIP_FILE
		OPEN (600,FILE=TRIM(LISTSKIP_FILE))
		DO I=1,99999
		IF (ATYP .EQ. 'HH') THEN
			READ (600,*,END=699) LISTSKIP_JJ(I),
     *              LISTSKIP_MO(I),LISTSKIP_TA(I),LISTSKIP_HO(I)
		ELSE
			READ (600,*,END=699) LISTSKIP_JJ(I),
     *              LISTSKIP_MO(I),LISTSKIP_TA(I)
		ENDIF
			LISTSKIP_N=LISTSKIP_N+1
		ENDDO
699		CLOSE (600)
	ENDIF

	LISTSKIP_NODATA=0
	IF ((KALY.EQ.'v').OR.(KALY.EQ.'V')) THEN
		BACKSPACE (765)
		read (765,*) kaly,LISTSKIP_NODATA
	ENDIF

!     print* ,'How many calibration years ?'
	read (765,*) kaln

!      print* ,'Which years ? (YYYY, space separated)'
	read (765,*) (kalyy(I),I=1,kaln)

      close (765)

! ---------------------------------------------------------------------
!    Setup of the analysis file
! ---------------------------------------------------------------------

      ybya=ja-1

9     CONTINUE

	ybya=ybya+1
	call messagerun (1,'Computing the model efficiency ...')
	call messagerun (1,'Setting up of the analysis file ...')
	if (ybya .gt. je) goto 12
	igst=1

      OPEN (331, file=filehin, status='old')
	call posnam(dirmeteo,1,pos)
	IF (fileh(2:2) .ne. ':') THEN
      OPEN (332, file=dirmeteo(1:pos)//fileh, status='old')
	ELSE
      OPEN (332, file=fileh, status='old')
      END IF
	call posnam(diroutput,1,pos2)
	filetmp=diroutput(1:pos2)//
     * filed(1:flln)//'eff.ddd'
	if (atyp .eq. 'HH') filetmp=diroutput(1:pos2)//
     * filed(1:flln)//'eff.hhh'

      IF ((docali .ne. 'y') .and. (docali .ne. 'Y') .and.
     *   (docali .ne. 'c') .and. (docali .ne. 'C').and.
     *   (docali .ne. 'g') .and. (docali .ne. 'G'))
     *    write(mhtml,'(A)')'<a href=file:///'//trim(filetmp)//
     *                  '>File used for efficiency analysis</a><br>'

      open (765,file=diroutput(1:pos2)//filed(1:flln)//'.eff',
     *     status='unknown')
      WRITE(765,'(a32)') ZN
	WRITE(765,'(F15.3)') AELHG
      WRITE(765,'(a2)') atyp
	call posnam (filetmp,1,pos5)
	WRITE(765,'(a)') filetmp(pos2+1:pos5)
	close (765)

	if (ybya .eq. ja) OPEN (334, file=Filetmp,
     *                        status='unknown')

      IF (SZK .GT. 0) THEN
	DO I=1,SZK
	   read (331,'(A50)') waskomm
	END DO
	END IF

      IF (OZK .GT. 0) THEN
	DO I=1,OZK
	   read (332,'(A50)') waskomm
	END DO
	END IF

10    continue

      DAYR=0
	SIMRD=0

      IF (atyp .eq. 'DD') GOTO 41

      DO LN2=1,24
454      continue
	   read (331,*,END=12) (simout(I),I=1,SCOL)


455      continue
         read (332,*,END=12) JJ,MO,TA,HO,(obsin(I),I=1,OCOL)
	   HOR=OBSIN(ROCOL)

         IF (JJ .lt. simout(1)) goto 455
	   IF (JJ .gt. simout(1)) then
	   backspace (332)
	   goto 454
         end if
	   IF (MO .ne. simout(2)) goto 455
         IF (TA .ne. simout(3)) goto 455

         IF (JJ .LT. JA) CYCLE
	   IF (JJ .GT. JE) CYCLE

         IF ((kaly .eq. 'Y') .or. (kaly .eq. 'y')
     *       .OR. (KALY.EQ.'L').OR.(KALY.EQ.'V')) THEN
	   DO I=1,kaln
            IF (JJ .EQ. kalyy(I)) GOTO 33
	   END DO
	   END IF

         IF ((KALY .EQ. 'l') .OR. (KALY .EQ. 'L')) THEN
		   DO I=1,LISTSKIP_N
			  IF (JJ .EQ. LISTSKIP_JJ(I).AND.
     *		      MO .EQ. LISTSKIP_MO(I).AND.
     *		      TA .EQ. LISTSKIP_TA(I).AND.
     *		      HO .EQ. LISTSKIP_HO(I)     ) GOTO 33
		   END DO
	   END IF

         IF ((KALY .EQ. 'v') .OR. (KALY .EQ. 'V')) THEN
			IF (OBSIN(ROCOL).EQ.LISTSKIP_NODATA) GOTO 33
	   ENDIF

         DAYR=DAYR+HOR
	   SIMRD=SIMRD+simout(rscol)

	   HOR=max(hor,0.0001)
         simout(rscol)=max(simout(rscol),0.0001)

         L1S=log(simout(rscol))
	   L1O=log(HOR)

	   IF (atyp .eq. 'HH') write (334,'(I5,3I3,6F9.4)') JJ,MO,TA,LN2,
     *                            HOR,simout(rscol),l1o,l1s

33    CONTINUE
      END DO

	   IF (JJ .LT. JA) GOTO 11
	   IF (JJ .GT. JE) GOTO 11

         IF ((kaly .eq. 'Y') .or. (kaly .eq. 'y')
     *       .OR. (KALY.EQ.'L').OR.(KALY.EQ.'V')) THEN
	   DO I=1,kaln
            IF (JJ .EQ. kalyy(I)) GOTO 11
	   END DO
	   END IF

         IF ((KALY .EQ. 'l') .OR. (KALY .EQ. 'L')) THEN
		   DO I=1,LISTSKIP_N
			  IF (JJ .EQ. LISTSKIP_JJ(I).AND.
     *		      MO .EQ. LISTSKIP_MO(I).AND.
     *		      TA .EQ. LISTSKIP_TA(I)     ) GOTO 11
		   END DO
	   END IF

         IF ((KALY .EQ. 'v') .OR. (KALY .EQ. 'V')) THEN
			IF (OBSIN(ROCOL).EQ.LISTSKIP_NODATA) GOTO 11
	   ENDIF

	   dayr=max(dayr,0.0001)
	   simrd=max(simrd,0.0001)
         L1S=log(simrd)
	   L1O=log(dayr)

	   IF (atyp .eq. 'DH') write (334,'(I5,2I3,6F9.4)') JJ,MO,TA,
     *                            DAYR,simrd,l1o,l1s

	   GOTO 11

41    CONTINUE

	   read (331,*,END=12) (simout(I),I=1,SCOL)
466   continue
         read (332,*,END=12) JJ,MO,TA,(obsin(I),I=1,OCOL)
	   DAYR=OBSIN(ROCOL)
         IF (JJ .lt. simout(1)) goto 466
	   IF (JJ .gt. simout(1)) then
	   backspace (332)
	   goto 41
         end if
	   IF (MO .ne. simout(2)) goto 466
         IF (TA .ne. simout(3)) goto 466

         IF (JJ .LT. JA) GOTO 11
	   IF (JJ .GT. JE) GOTO 11

         IF ((kaly .eq. 'Y') .or. (kaly .eq. 'y')
     *       .OR. (KALY.EQ.'L').OR.(KALY.EQ.'V')) THEN
	   DO I=1,kaln
            IF (JJ .EQ. kalyy(I)) GOTO 11
	   END DO
	   END IF

         IF ((KALY .EQ. 'l') .OR. (KALY .EQ. 'L')) THEN
		   DO I=1,LISTSKIP_N
			  IF (JJ .EQ. LISTSKIP_JJ(I).AND.
     *		      MO .EQ. LISTSKIP_MO(I).AND.
     *		      TA .EQ. LISTSKIP_TA(I)     ) GOTO 11
		   END DO
	   END IF

         IF ((KALY .EQ. 'v') .OR. (KALY .EQ. 'V')) THEN
			IF (OBSIN(ROCOL).EQ.LISTSKIP_NODATA) GOTO 11
	   ENDIF

	   SIMRD=simout(rscol)
	   dayr=max(dayr,0.0001)
	   simrd=max(simrd,0.0001)

	   L1S=log(simrd)
	   L1O=log(dayr)

	   write (334,'(I5,2I3,6F9.4)') JJ,MO,TA,
     *                            DAYR,simrd,l1o,l1s

11    CONTINUE

      GOTO 10

12    CONTINUE

      CLOSE (331)
	CLOSE (332)

	REWIND (334)

! ---------------------------------------------------------------------
!    DETERMINATION OF THE PERIOD TO COMPUTE
!    STEP 1: WHOLE PERIOD
!    STEP 2: YEAR BY YEAR
!    STEP 3: MONTHLY
!    STEP 4: Defined Season
! ---------------------------------------------------------------------
      step=0

	now=ja-1
	mnow=0

8     continue

      step=step+1

	IF (step .GT. 4) GOTO 900

! ---------------------------------------------------------------------
!    1. COMPLETE DATA SET

      if (step .eq. 1) goto 7

! ---------------------------------------------------------------------
!    2. YEAR BY YEAR


	if (step .eq. 2) then
	    now=now+1
	    if (now .gt. je) goto 8
	    step = 1
          IF ((kaly .eq. 'Y') .or. (kaly .eq. 'y')
     *       .OR. (KALY.EQ.'L').OR.(KALY.EQ.'V')) THEN

	    DO I=1,kaln
            IF (now .EQ. kalyy(I)) GOTO 8
	    END DO
	    END IF
	    close (334)
          OPEN (334, file=diroutput(1:pos2)//'tmp.ddd',
     *          status='unknown')
	    OPEN (335, file=filetmp, status='old')

110   continue

      IF (atyp .eq. 'HH') THEN

  	read (335,'(I5,3I3,6F9.4)',end=111) JJ,MO,TA,HO,
     *                            (nums(I),I=1,4)
      ELSE
	read (335,'(I5,2I3,6F9.4)',end=111) JJ,MO,TA,
     *                            (nums(I),I=1,4)
      END IF

	IF (now .eq. JJ) THEN
	write (334,'(I5,2I3,6F9.4)') JJ,MO,TA,
     *                            (nums(I),I=1,4)
	END IF

	goto 110

111   continue

	CLOSE (335)
	REWIND (334)

	GOTO 7

      END IF

! ---------------------------------------------------------------------
!    3. MONTH by MONTH

	if (step .eq. 3) then
	    mnow=mnow+1
	    if (mnow .gt. 12) goto 8
	    step = 2
	    close (334)
          OPEN (334, file=diroutput(1:pos2)//'tmp.ddd',
     *          status='unknown')
	    OPEN (335, file=filetmp, status='old')

120   continue

      IF (atyp .eq. 'HH') THEN

  	read (335,'(I5,3I3,6F9.4)',end=121) JJ,MO,TA,HO,
     *                            (nums(I),I=1,4)
      ELSE
	read (335,'(I5,2I3,6F9.4)',end=121) JJ,MO,TA,
     *                            (nums(I),I=1,4)
      END IF
	IF (mnow .eq. mo) THEN
	write (334,'(I5,2I3,6F9.4)') JJ,MO,TA,
     *                            (nums(I),I=1,4)
	END IF

	goto 120

121   continue

	CLOSE (335)
	REWIND (334)

	GOTO 7

      END IF

! ---------------------------------------------------------------------
!    4. Season

	if (step .eq. 4) then
	    if (seas .eq. 'n') GOTO 8
	    if (seas .eq. 'N') GOTO 8
	    close (334)
          OPEN (334, file=diroutput(1:pos2)//'tmp.ddd',
     *          status='unknown')
	    OPEN (335, file=filetmp, status='old')

130   continue

      IF (atyp .eq. 'HH') THEN

  	read (335,'(I5,3I3,6F9.4)',end=131) JJ,MO,TA,HO,
     *                            (nums(I),I=1,4)
      ELSE
	read (335,'(I5,2I3,6F9.4)',end=131) JJ,MO,TA,
     *                            (nums(I),I=1,4)
      END IF

	DO I=1,MS
	IF (monsea(I) .eq. mo) THEN
	write (334,'(I5,2I3,6F9.4)') JJ,MO,TA,
     *                            (nums(LN1),LN1=1,4)
	END IF
	END DO

	goto 130

131   continue

	CLOSE (335)
	REWIND (334)

	GOTO 7

      END IF

! ---------------------------------------------------------------------
!    COMPUTATION OF THE STATISTICAL
!    PARAMETERS
! ---------------------------------------------------------------------

7     continue

!    SETTING TO ZERO

      CT=0
      sums=0.
	avgs=0.
	varz=0.
	stabws=0.
	top=-10000000.
	bot=10000000.

! ---------------------------------------------------------------------
!    AVERAGE, SUM, MAX and MIN

20    CONTINUE

      IF (atyp .eq. 'HH') THEN
      	read (334,'(I5,3I3,6F9.4)',end=21) JJ,MO,TA,HO,
     *                            (nums(I),I=1,4)
      ELSE
      	read (334,'(I5,2I3,6F9.4)',end=21) JJ,MO,TA,
     *                            (nums(I),I=1,4)
      END IF

	CT=CT+1

	sums=sums+nums
	top=max(top,nums)
	bot=min(bot,nums)

      GOTO 20

21    CONTINUE
      avgs=sums/ct
      rewind (334)

! ---------------------------------------------------------------------
!    VARIANCE and STANDARD DEVIATION

22    CONTINUE

      IF (atyp .eq. 'HH') THEN
      	read (334,'(I5,3I3,6F9.4)',end=23) JJ,MO,TA,HO,
     *                            (nums(I),I=1,4)
      ELSE
      	read (334,'(I5,2I3,6F9.4)',end=23) JJ,MO,TA,
     *                            (nums(I),I=1,4)
      END IF

	VARZ=VARZ+(NUMS-AVGS)**2

	GOTO 22

23    CONTINUE

	if (ct.lt.2) then	!bugfix DV: only one value available?
		varz=0.
		stabws=0.
		co1=0.;co2=0.
		goto 29
      endif

	VARZ=VARZ/(CT-1)

	STABWS=VARZ**(0.5)
	CO1=AVGS-STABWS
	CO2=AVGS+STABWS

29	continue

      REWIND (334)

	NOMS=0
	EFFS=0
	DENS=0

! ---------------------------------------------------------------------
!    EFFCIENCY (LEGATES, D.R., McCABE, G.J., 1999) with
!    exponents 0.25, 0.5, 1.0, 2 and 4

24    CONTINUE

      IF (atyp .eq. 'HH') THEN
      	read (334,'(I5,3I3,6F9.4)',end=25) JJ,MO,TA,HO,
     *                            (nums(I),I=1,4)
      ELSE
      	read (334,'(I5,2I3,6F9.4)',end=25) JJ,MO,TA,
     *                            (nums(I),I=1,4)
      END IF

	DO I=1,5
	   NOMS(I,1)=NOMS(I,1)+(ABS(nums(1)-nums(2)))**exps(I)
         NOMS(I,2)=NOMS(I,2)+(ABS(nums(3)-nums(4)))**exps(I)

	   DENS(I,1)=DENS(I,1)+(ABS(nums(1)-avgs(1)))**exps(I)
	   DENS(I,2)=DENS(I,2)+(ABS(nums(3)-avgs(3)))**exps(I)
	END DO

	GOTO 24

25    CONTINUE

      DO I=1,5
         EFFS(I,1)=1.- NOMS(I,1)/DENS(I,1)
	   EFFS(I,2)=1.- NOMS(I,2)/DENS(I,2)
	END DO

      SVID=SVID+1

      DO I=1,2
	JID=NOW
	IF (STEP .EQ. 2) JID=MNOW
	IF (SVID .EQ. 1) JID=9999
	IF (STEP .EQ. 4) JID=8888
      SV(SVID,I,1)= JID
      SV(SVID,I,2)= top(I)
	SV(SVID,I,3)= bot(I)
	SV(SVID,I,4)= AVGS(I)
	SV(SVID,I,5)= sums(I)
	SV(SVID,I,6)= varz(I)
	SV(SVID,I,7)= stabws(I)
	SV(SVID,I,8)= co1(I)
	SV(SVID,I,9)= co2(I)
	END DO

	SV(SVID,3,1)= SV(SVID,2,1)

	DO I=2,9
	    SV(SVID,3,I)= SV(SVID,2,I)-SV(SVID,1,I)
	END DO

	DO I=1,5
      	JID=NOW
      	IF (STEP .EQ. 2) JID=MNOW
	    IF (STEP .EQ. 4) JID=8888
      	IF (SVID .EQ. 1) JID=9999
	    SV2(SVID,I,1)=JID
	    SV2(SVID,I,2)=EFFS(I,1)
	    SV2(SVID,I,3)=EFFS(I,2)
      END DO

      GOTO 8

! ---------------------------------------------------------------------
! Writing headers of output files
900   CONTINUE

      heada(1)='Statistics for Observed Runoff'
	heada(2)='Statistics for Simulated Runoff'
	heada(3)='Differences ((Sts Sim)- (Sts Obs))'
	heada(4)='Efficiency for linear data'
	heada(5)='Efficiency for log data'

	headb(1)=' ID   TYPE   Maximum   Minimum   Average    Summation'//
     * '  Variance    St.Dev    AV-STD    AV+STD'
	headb(2)=' ID   TYPE   Maximum   Minimum   Average    Summation'//
     * '  Variance    St.Dev    AV-STD    AV+STD'
	headb(3)=' ID   TYPE   Maximum   Minimum   Average    Summation'//
     * '  Variance    St.Dev    AV-STD    AV+STD'
	headb(4)=' ID   TYPE     E0.25     E0.50     E1.00     E2.00'//
     * '     E4.00'
	headb(5)=' ID   TYPE     E0.25     E0.50     E1.00     E2.00'//
     * '     E4.00'

      addfa(1)='_ost.dat'
      addfa(2)='_sst.dat'
	addfa(3)='_dst.dat'

	addfb(1)='_eli.dat'
      addfb(2)='_eln.dat'

! ---------------------------------------------------------------------
! Writing the results
      IF ((docali .ne. 'y') .and. (docali .ne. 'Y') .and.
     *   (docali .ne. 'c') .and. (docali .ne. 'C').and.
     *   (docali .ne. 'g') .and. (docali .ne. 'G'))
     *    write(mhtml,'(A)')'Results of efficiency module<br>'
      DO LN2=1,3
	   filedd=diroutput(1:pos2)//filed(1:flln)//addfa(LN2)
	   OPEN (333, file=filedd, status='unknown')
      IF ((docali .ne. 'y') .and. (docali .ne. 'Y') .and.
     *   (docali .ne. 'c') .and. (docali .ne. 'C').and.
     *   (docali .ne. 'g') .and. (docali .ne. 'G'))
     *    write(mhtml,'(A)')'<a href=file:///'//trim(filedd)//
     * '>'//trim(filedd)//'</a><br>'
	   WRITE (333,'(A100)') Heada(LN2)
	   WRITE (333,'(A100)') Headb(LN2)
	   DO LN1=1,SVID
	   WRITE (333,'(I3,2X,I5,1X,3(F9.4,1X),F12.2,1X,4(F9.4,1X))') LN2,
     *                  INT(SV(LN1,LN2,1)),(SV(LN1,LN2,I),I=2,9)
	   END DO
         WRITE (333, '(A1)') ' '
	   CLOSE (333)
	END DO


      DO LN2=2,3
	   filedd=diroutput(1:pos2)//filed(1:flln)//addfb(LN2-1)
	   OPEN (333, file=filedd, status='unknown')
      IF ((docali .ne. 'y') .and. (docali .ne. 'Y') .and.
     *   (docali .ne. 'c') .and. (docali .ne. 'C'))
     *    write(mhtml,'(A)')'<a href=file:///'//trim(filedd)//
     * '>'//trim(filedd)//'</a><br>'
	   WRITE (333,'(A100)') Heada(LN2+2)
	   WRITE (333,'(A100)') Headb(LN2+2)
	   DO LN1=1,SVID
	   DO I=1,5
	   SV2(LN1,I,LN2)=max(-999.999,SV2(LN1,I,LN2))
	   END DO
	   WRITE (333,'(I3,2X,I5,1X,5(F9.4,1X))') LN2,
     *                  INT(SV2(LN1,1,1)),(SV2(LN1,I,LN2),I=1,5)
	   END DO
         WRITE (333,'(A1)') ' '
	   CLOSE (333)
	END DO

      IF ((docali .ne. 'y') .and. (docali .ne. 'Y') .and.
     *   (docali .ne. 'c') .and. (docali .ne. 'C').and.
     *   (docali .ne. 'g') .and. (docali .ne. 'G'))
     *    call saneffi (ininam,diroutput,filedd,mhtml)
      IF (dofstat .eq. 'Y') THEN
	call messagerun (1,'Computing flood statistic')
	call floodstat (ininam,filetmp,filedd,mhtml)
      END IF

	IF ((KALY.EQ.'l').OR.(KALY.EQ.'L')) THEN
	    deallocate (LISTSKIP_JJ,LISTSKIP_MO)
		deallocate (LISTSKIP_TA,LISTSKIP_HO)
      END IF

	call messagerun (1,'Done')
	close (334)

999   CONTINUE

! ---------------------------------------------------------------------
      RETURN
      END
! ---------------------------------------------------------------------
