!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      subroutine eglob(nr,a,b)

!     This subroutine computes the incoming global radiation using the
!     computed maximum incoming global radiaiton and the relative
!     sunshine duration
!     special routine for some location in Germany
!--------------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------

      integer nr,i
      real a(12)
      real b(12)

! ---------------------------------------------------------------------
!     Internal Variables
! ---------------------------------------------------------------------

      real a1(12),a2(12),a3(12),a4(12),a5(12)
      real b1(12),b2(12),b3(12),b4(12),b5(12)


! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------

!     Coefficients for the determination of  the incoming global radiation
!     using the computed maximum incoming global radiaiton and the relative
!     sunshine duration
!     g = go * ( a + b * s/so )
!
!    Representative forthe period 1970-1991
!
!    nr   station number
!                        1 = Hamburg
!                        2 = Braunschweig
!                        3 = Trier
!                        4 = Wuerzburg
!                        5 = Hohenpei�enberg

! ---------------------------------------------------------------------

      data a1/0.172,0.181,0.189,0.187,0.175,0.160,0.175,0.195,0.188,
     * 0.192,0.172,0.146/,b1/0.629,0.645,0.608,0.582,0.565,0.573,0.528,
     * 0.485,0.550,0.564,0.633,0.733/
      data a2/0.208,0.198,0.233,0.177,0.197,0.238,0.224,0.284,0.200,
     * 0.221,0.174,0.185/,b2/0.578,0.687,0.513,0.629,0.551,0.410,0.444,
     * 0.300,0.532,0.477,0.660,0.607/
      data a3/0.175,0.194,0.199,0.212,0.188,0.219,0.189,0.217,0.194,
     * 0.181,0.197,0.161/,b3/0.719,0.685,0.625,0.498,0.551,0.483,0.544,
     * 0.490,0.590,0.674,0.626,0.753/
      data a4/0.191,0.241,0.258,0.236,0.206,0.230,0.215,0.266,0.226,
     * 0.193,0.217,0.182/,b4/0.842,0.597,0.531,0.526,0.565,0.491,0.514,
     * 0.398,0.546,0.677,0.587,0.779/
      data a5/0.305,0.277,0.263,0.217,0.198,0.186,0.190,0.225,0.223,
     * 0.226,0.208,0.227/,b5/0.472,0.591,0.586,0.596,0.570,0.563,0.535,
     * 0.475,0.540,0.594,0.660,0.645/


      do 10 i=1,12
      if(nr.eq.1) a(i)=a1(i)
      if(nr.eq.1) b(i)=b1(i)
      if(nr.eq.2) a(i)=a2(i)
      if(nr.eq.2) b(i)=b2(i)
      if(nr.eq.3) a(i)=a3(i)
      if(nr.eq.3) b(i)=b3(i)
      if(nr.eq.4) a(i)=a4(i)
      if(nr.eq.4) b(i)=b4(i)
      if(nr.eq.5) a(i)=a5(i)
      if(nr.eq.5) b(i)=b5(i)

  10  continue

! ---------------------------------------------------------------------
      return
      end
