!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE EINPUZDW (land,cland,btyp,cbtyp,nfcap,btief,
     *                   rlai,bhoehe,albedo,bflux,rsc,awacv,awacsol,
     *                   wurzelv,vbgf,simaxf)
! ---------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------
!     Variables

      integer j,			! Variable for monthly loop
     *        btyp,			! category of plant available field capacity of the actual HRU
     *        mbi,mdr,		! controls partial output
     *        land			! land use category of the actual HRU
      real rlai(12),			! parameterization of the leaf area index - monthly values
     *     albedo(12),			! parameterization of the albedo - monthly values
     *     albedob(12),			! parameterization of the albedo - monthly values
     *     bhoehe(12),			! parameterization of the vegetation height - monthly values
     *     bflux(12),			! parameterization of the soil heat flux - monthly values
     *     rsc(12),			! parameterization of the minimal stomata resistences - monthly values
     *     awacv(12),			! parameterization of the available water content - monthly values
     *     awacsol(12),			! parameterization of the available water content - monthly values
     *     wurzelv(12),			! parameterization of the root depth - monthly values
     *     vbgf(12),			! parameterization of the density of vegetated surfaces - monthly values
     *     albedov(30),			! parameterization of the albedo, values for each land use category
     *     simaxf(12),			! parameterization of the maximal interception storage - monthly values
     *     rlaim(30,12),		! parameterization of the leaf area index - monthly values / landuse
     *     bhoehem(30,12),		! parameterization of the vegetation height - monthly values / landuse
     *     wurzel(30,12),		! parameterization of the root depth - monthly values / landuse
     *     awac(30,12),			! parameterization of the available water content - monthly values / landuse
     *     rscm(30,12),			! parameterization of the minimal stomata resistences - monthly values / landuse
     *     vbgm(30,12),			! parameterization of the density of vegetated surfacesx - monthly values / landuse
     *     simaxm(30,12),		! parameterization of the maximal interception storage - monthly values / landuse
     *     nfcap,			! available field capacitry from spatial information of the actual HRU
     *     btief			! soil depth from spatial information of the actual HRU
      character*40 cland,		! land use type of the actual HRU - String
     *     cland30(30),			! land use type - all categories - String
     *     cbtyp3(3),			! category of plant available field capacity - all categories - String
     *     cbtyp			! category of plant available field capacity of the actual HRU - String

! ---------------------------------------------------------------------
!    Begin
! --------------------------------------------------------------------

! ---------------------------------------------------------------------
! Land use classes
      cland30(1)= ' Water                                 1'
      cland30(2)= ' Built-ip                              2'
      cland30(3)= ' Coniferous-forests                    3'
      cland30(4)= ' Deciduos forsts                       4'
      cland30(5)= ' Mixed forests                         5'
      cland30(6)= ' Cereals                               6'
      cland30(7)= ' Pasture                               7'
      cland30(8)= ' Bush                                  8'
      cland30(9)= ' Free                                  9'
      cland30(10)=' Free                                 10'
      cland30(11)=' Free                                 11'
      cland30(12)=' Free                                 12'
      cland30(13)=' Firn                                 13'
      cland30(14)=' Bare ice                             14'
      cland30(15)=' Rock                                 15'
      cland30(16)=' Free                                 16'
      cland30(17)=' Free                                 17'
      cland30(18)=' Fruits                               18'
      cland30(19)=' Vegetables                           19'
      cland30(20)=' Weheat                               20'
      cland30(21)=' Alpine vegetation                    21'
      cland30(22)=' Wetlands                             22'
      cland30(23)=' Rough pasture                        23'
      cland30(24)=' Sub-Alpine meadow                    24'
      cland30(25)=' Alpine meadow                        25'
      cland30(26)=' Bare soil vegetation                 26'
      cland30(27)='                                        '
      cland30(28)=' Corn                                   '
      cland30(29)=' Grapes                                 '
      cland30(30)='                                        '

! ---------------------------------------------------------------------
      cbtyp3(1)= ' Soil with low field capacity          1'
      cbtyp3(2)= ' Soil with middle field capacity       2'
      cbtyp3(3)= ' Soil with high field capacity         3'

! ---------------------------------------------------------------------
      MBI=6
      MDR=10

	if (land .lt. 0) print*, land

      cland=cland30(land)
      if (btyp .gt. 0) cbtyp(1:40)=cbtyp3(btyp)

! ---------------------------------------------------------------------
! Leaf area index
! ---------------------------------------------------------------------
      data (rlaim(1,j),j=1,12) /12*0.0/
      data (rlaim(2,j),j=1,12) /12*1.0/
      data (rlaim(3,j),j=1,12)
     +/6.0,6.0,6.0,6.0,7.0,8.0,8.0,8.0,8.0,7.0,6.0,6.0/
      data (rlaim(4,j),j=1,12)
     +/0.5,0.5,0.5,5.0,6.0,7.0,7.0,7.0,6.0,5.0,0.5,0.5/
      data (rlaim(5,j),j=1,12)
     +/2.0,2.0,2.0,3.5,6.0,6.0,6.0,6.0,6.0,5.5,2.0,2.0/
      data (rlaim(6,j),j=1,12)
     +/0.5,0.5,0.5,1.5,2.0,4.0,5.0,1.5,0.5,0.5,0.5,0.5/
      data (rlaim(7,j),j=1,12)
     +/0.5,0.5,1.0,3.0,5.0,5.0,5.0,4.0,3.0,2.0,0.5,0.5/
      data (rlaim(8,j),j=1,12)
     +/0.1,0.1,0.1,1.0,4.0,5.0,5.0,5.0,3.0,1.0,0.1,0.1/
      data (rlaim(9,j),j=1,12)
     +/0.2,0.3,0.4,0.5,0.7,1.0,1.0,1.0,0.7,0.5,0.3,0.2/
      data (rlaim(10,j),j=1,12)
     +/0.2,0.3,0.4,0.5,0.7,1.0,1.0,1.0,0.7,0.5,0.3,0.2/
      data (rlaim(11,j),j=1,12)
     +/0.2,0.3,0.4,0.5,1.0,1.5,2.0,1.5,1.0,0.5,0.3,0.2/
      data (rlaim(12,j),j=1,12)
     +/0.2,0.3,0.4,0.5,1.0,1.5,2.0,1.5,1.0,0.5,0.3,0.2/
      data (rlaim(13,j),j=1,12)
     +/0.2,0.3,0.4,0.5,1.0,1.5,2.0,1.5,1.0,0.5,0.3,0.2/
      data (rlaim(14,j),j=1,12) /12*0.0/
      data (rlaim(15,j),j=1,12) /12*0.0/
      data (rlaim(16,j),j=1,12) /12*0.0/
      data (rlaim(17,j),j=1,12) /12*0.0/
      data (rlaim(18,j),j=1,12)
     +/0.5,0.5,0.5,1.5,2.0,4.0,5.0,1.5,0.5,0.5,0.5,0.5/
      data (rlaim(19,j),j=1,12)
     +/0.20,0.20,0.2,1.0,5.0,5.0,5.0,5.0,4.0,2.5,0.25,0.25/
      data (rlaim(20,j),j=1,12)
     +/0.5,0.5,0.5,1.5,2.0,4.0,5.0,1.5,0.5,0.5,0.5,0.5/
      data (rlaim(21,j),j=1,12)
     +/0.1,0.1,0.1,0.1,1.5,3.0,4.0,4.0,3.0,1.0,0.1,0.1/
      data (rlaim(22,j),j=1,12)
     +/0.1,0.1,0.1,0.1,1.5,3.0,4.0,4.0,3.0,1.0,0.1,0.1/
      data (rlaim(23,j),j=1,12)
     +/0.1,0.1,0.1,0.1,1.5,3.0,4.0,3.5,2.5,1.0,0.1,0.1/
      data (rlaim(24,j),j=1,12)
     +/0.1,0.1,0.1,0.1,1.5,2.0,2.5,3.0,3.0,1.0,0.1,0.1/
      data (rlaim(25,j),j=1,12)
     +/0.1,0.1,0.1,0.1,0.5,1.5,2.0,2.0,2.0,1.0,0.1,0.1/
      data (rlaim(26,j),j=1,12)
     +/0.1,0.1,0.1,0.1,0.5,1.0,1.5,1.5,1.5,0.5,0.1,0.1/
      data (rlaim(27,j),j=1,12) /12*0.0/
      data (rlaim(28,j),j=1,12)
     +/0.10,0.10,0.1,0.1,0.1,1.0,3.0,5.0,5.0,5.0,0.1,0./
      data (rlaim(29,j),j=1,12)
     +/0.20,0.20,0.2,1.0,5.0,5.0,5.0,5.0,4.0,2.5,0.25,0.25/
      data (rlaim(30,j),j=1,12) /12*0.0/

      do 20 j=1,12
        rlai(j) = rlaim(land,j)

20    CONTINUE

! ---------------------------------------------------------------------
! Albedo
! ---------------------------------------------------------------------
      data albedov/0.05,0.10,0.12,0.17,0.15,0.25,0.25,0.25,0.10,0.20,
     *             0.30,0.05,0.10,0.40,0.12,0.70,0.50,0.25,0.20,0.25,
     *             0.20,0.20,0.23,0.23,0.25,0.20,0.20,0.20,0.20,0.20/

      data albedob/4*0.10,4*0.20,4*0.1/
      do 30 j=1,12
      if(rlaim(land,j).gt.4.or.rlaim(land,j).eq.0.0) then
	 albedo(j)= albedov(land)
      else
	 albedo(j)= albedob(j) + 0.25*(albedov(land)-albedob(j))*rlai(j)
      endif
30    continue

! ---------------------------------------------------------------------
! Vegetation Height
! ---------------------------------------------------------------------

      data (bhoehem(1,j),j=1,12)  /12*0.005/
      data (bhoehem(2,j),j=1,12)  /12*4.0/
      data (bhoehem(3,j),j=1,12)  /12*10.0/
      data (bhoehem(4,j),j=1,12)  /12*10.0/
      data (bhoehem(5,j),j=1,12)  /12*10.0/
      data (bhoehem(6,j),j=1,12)
     +/0.05,0.05,0.05,0.10,0.40,0.60,0.80,0.80,0.30,0.10,0.05,0.05/
      data (bhoehem(7,j),j=1,12)
     +/0.15,0.15,0.20,0.25,0.30,0.35,0.35,0.30,0.30,0.25,0.20,0.15/
      data (bhoehem(8,j),j=1,12)  /12*1.0 /
      data (bhoehem(9,j),j=1,12)  /12*0.05/
      data (bhoehem(10,j),j=1,12)  /12*0.05/
      data (bhoehem(11,j),j=1,12)  /12*0.05/
      data (bhoehem(12,j),j=1,12)  /12*0.05/
      data (bhoehem(13,j),j=1,12)  /12*0.05/
      data (bhoehem(14,j),j=1,12)  /12*0.00/
      data (bhoehem(15,j),j=1,12)  /12*0.05/
      data (bhoehem(16,j),j=1,12)  /12*0.05/
      data (bhoehem(17,j),j=1,12)  /12*0.05/
      data (bhoehem(18,j),j=1,12)
     +/0.10,0.10,0.10,0.15,0.40,0.60,0.80,0.80,0.30,0.10,0.10,0.10/
      data (bhoehem(19,j),j=1,12)
     +/0.30,0.30,0.50,1.00,2.5,3.00,3.0,3.0,2.0,1.0,0.50,0.30/
      data (bhoehem(20,j),j=1,12)
     +/0.05,0.05,0.05,0.05,0.05,0.05,0.2,0.6,0.8,0.05,0.05,0.05/
      data (bhoehem(21,j),j=1,12)  /12*0.5/
      data (bhoehem(22,j),j=1,12)
     +/0.10,0.10,0.10,0.10,0.15,0.20,0.25,0.30,0.25,0.15,0.10,0.10/
      data (bhoehem(23,j),j=1,12)
     +/0.10,0.10,0.10,0.10,0.15,0.20,0.25,0.30,0.25,0.15,0.10,0.10/
      data (bhoehem(24,j),j=1,12)
     +/0.10,0.10,0.10,0.10,0.15,0.20,0.25,0.30,0.25,0.15,0.10,0.10/
      data (bhoehem(25,j),j=1,12)
     +/0.10,0.10,0.10,0.10,0.15,0.20,0.25,0.30,0.25,0.15,0.10,0.10/
      data (bhoehem(26,j),j=1,12)
     +/0.05,0.05,0.05,0.10,0.15,0.20,0.25,0.30,0.25,0.15,0.10,0.10/
      data (bhoehem(27,j),j=1,12)  /12*0.05/
      data (bhoehem(28,j),j=1,12)
     +/0.05,0.05,0.05,0.05,0.05,0.25,1.25,2.50,2.50,2.50,0.10,0.10/
      data (bhoehem(29,j),j=1,12)  /12*2.00/
      data (bhoehem(30,j),j=1,12)  /12*0.05/

      do 40 j=1,12
        bhoehe(j)= bhoehem(land,j)
40    continue


! ---------------------------------------------------------------------
! Depth of rooths (m)
! ---------------------------------------------------------------------

      data (wurzel(1,j),j=1,12)  /12*0.000/
      data (wurzel(2,j),j=1,12)  /12*0.100/
      data (wurzel(3,j),j=1,12)  /12*1.5/
      data (wurzel(4,j),j=1,12)  /12*1.5/
      data (wurzel(5,j),j=1,12)  /12*1.5/
      data (wurzel(6,j),j=1,12)
     +/0.2,0.2,0.2,0.2,0.2,0.2,0.8,0.8,0.8,0.2,0.2,0.2/
      data (wurzel(7,j),j=1,12)   /12*0.6/
      data (wurzel(8,j),j=1,12)   /12*0.8/
      data (wurzel(9,j),j=1,12)   /12*0.2/
      data (wurzel(10,j),j=1,12)  /12*0.2/
      data (wurzel(11,j),j=1,12)  /12*0.2/
      data (wurzel(12,j),j=1,12)  /12*0.2/
      data (wurzel(13,j),j=1,12)  /12*0.2/
      data (wurzel(14,j),j=1,12)  /12*0.1/
      data (wurzel(15,j),j=1,12)  /12*0.1/
      data (wurzel(16,j),j=1,12)  /12*0.0/
      data (wurzel(17,j),j=1,12)  /12*0.0/
      data (wurzel(18,j),j=1,12)
     +/0.2,0.2,0.2,0.2,0.5,0.7,0.8,0.8,0.5,0.2,0.2,0.2/
      data (wurzel(19,j),j=1,12)  /12*0.8/
      data (wurzel(20,j),j=1,12)
     +/0.2,0.2,0.2,0.2,0.4,0.6,0.8,0.8,0.8,0.2,0.2,0.2/
      data (wurzel(21,j),j=1,12)  /12*0.5/
      data (wurzel(22,j),j=1,12)  /12*0.3/
      data (wurzel(23,j),j=1,12)  /12*0.3/
      data (wurzel(24,j),j=1,12)  /12*0.3/
      data (wurzel(25,j),j=1,12)  /12*0.3/
      data (wurzel(26,j),j=1,12)  /12*0.15/
      data (wurzel(27,j),j=1,12)  /12*0.2/
      data (wurzel(28,j),j=1,12)  /12*0.6/
      data (wurzel(29,j),j=1,12)  /12*0.8/
      data (wurzel(30,j),j=1,12)  /12*0.2/

      do 150 j=1,12
	 if (btyp .eq. 1) then
	    wurzelv(j)= (1.-0.25)*wurzel(land,j)
	 else if (btyp .eq. 2)  then
	    wurzelv(j)= wurzel(land,j)
	 else if (btyp .eq. 3) then
	    wurzelv(j)= (1.+0.25)*wurzel(land,j)
	 endif
150   continue

! ---------------------------------------------------------------------
!   available water capacity
! ---------------------------------------------------------------------
      data (awac(1,j),j=1,12)  /12*0.000/
      data (awac(2,j),j=1,12)  /12*10./ !0.000
      data (awac(3,j),j=1,12)  /12*175./
      data (awac(4,j),j=1,12)  /12*175./
      data (awac(5,j),j=1,12)  /12*175./
      data (awac(6,j),j=1,12)
     +/20,20,20,20,20,20,140,140,140,20,20,20/
      data (awac(7,j),j=1,12)   /12*125./
      data (awac(8,j),j=1,12)   /12*50./
      data (awac(9,j),j=1,12)   /12*20./
      data (awac(10,j),j=1,12)  /12*20./
      data (awac(11,j),j=1,12)  /12*20./
      data (awac(12,j),j=1,12)  /12*20./
      data (awac(13,j),j=1,12)  /12*20./
      data (awac(14,j),j=1,12)  /12*0.0/
      data (awac(15,j),j=1,12)  /12*0.0/
      data (awac(16,j),j=1,12)  /12*0.0/
      data (awac(17,j),j=1,12)  /12*0.0/
      data (awac(18,j),j=1,12)
     +/20,20,20,20,20,20,90,90,90,20,20,20/
      data (awac(19,j),j=1,12)  /12*140.0/
      data (awac(20,j),j=1,12)
     +/20.0,20.0,20.0,20.0,20,20.0,140.0,140.0,140.0,20.0,20.0,20.0/


      data (awac(21,j),j=1,12)  /12*20./
      data (awac(22,j),j=1,12)  /12*20./
      data (awac(23,j),j=1,12)  /12*20./
      data (awac(24,j),j=1,12)  /12*20./
      data (awac(25,j),j=1,12)  /12*20./
      data (awac(26,j),j=1,12)  /12*20./
      data (awac(27,j),j=1,12)  /12*20./
      data (awac(28,j),j=1,12)  /12*130.0/
      data (awac(29,j),j=1,12)  /12*130.0/
      data (awac(30,j),j=1,12)  /12*20./

      do 140 j=1,12
        awacsol(j)=nfcap*btief*10.0
        if ((wurzelv(j)+0.05) .lt. btief) then
           awacsol(j)=nfcap*(wurzel(13,j)+0.05)*10.0
        endif
        awacv(j)=nfcap*btief*10.0
        if ((wurzelv(j)+0.15) .lt. btief) then
           awacv(j)=nfcap*(wurzelv(j)+0.15)*10.0
        endif
        if (awac(land,j) .EQ. 0.0) awacv(j)=0.0
        if ((land.gt.2).and.(land.lt.14).and.(awacv(j).le.5.0)) then
           awacv(j)=10.0
        endif
        if ((land.gt.17) .and. (awacv(j).le.5.0)) awacv(j)=10.0
140   continue

! ---------------------------------------------------------------------
!   Density of vegetation
! ---------------------------------------------------------------------

      data (vbgm(1,j),j=1,12)  /12*0.0/
      data (vbgm(2,j),j=1,12)  /12*0.2/
      data (vbgm(3,j),j=1,12)
     * /0.92,0.92,0.93,0.94,0.95,0.97,1.00,1.00,1.00,0.98,0.97,0.96/
      data (vbgm(4,j),j=1,12)
     * /0.60,0.60,0.60,0.70,0.85,0.98,1.00,1.00,0.95,0.90,0.75,0.60/
      data (vbgm(5,j),j=1,12)
     * /0.76,0.76,0.78,0.80,0.87,0.98,1.00,1.00,0.97,0.94,0.82,0.80/
      data (vbgm(6,j),j=1,12)
     * /0.40,0.40,0.50,0.60,0.80,1.00,1.00,1.00,0.50,0.30,0.30,0.40/
      data (vbgm(7,j),j=1,12)
     * /0.90,0.90,0.92,0.94,0.96,0.98,0.98,0.98,0.95,0.92,0.90,0.90/
      data (vbgm(8,j),j=1,12)
     * /0.90,0.90,0.92,0.95,0.99,0.99,0.99,0.99,0.96,0.93,0.90,0.90/
      data (vbgm(9,j),j=1,12)   /12*0.1/
      data (vbgm(10,j),j=1,12)  /12*0.1/
      data (vbgm(11,j),j=1,12)  /12*0.1/
      data (vbgm(12,j),j=1,12)  /12*0.1/
      data (vbgm(13,j),j=1,12)  /12*0.1/
      data (vbgm(14,j),j=1,12)  /12*0.0/
      data (vbgm(15,j),j=1,12)  /12*0.1/
      data (vbgm(16,j),j=1,12)  /12*0.0/
      data (vbgm(17,j),j=1,12)  /12*0.0/
      data (vbgm(18,j),j=1,12)
     * /0.40,0.40,0.50,0.60,0.80,1.00,1.00,1.00,0.50,0.30,0.30,0.40/
      data (vbgm(19,j),j=1,12)
     * /0.70,0.70,0.75,0.80,0.90,0.95,0.95,0.95,0.95,0.90,0.80,0.75/
      data (vbgm(20,j),j=1,12)
     * /0.40,0.40,0.50,0.60,0.80,1.00,1.00,1.00,0.00,0.20,0.30,0.40/
      data (vbgm(21,j),j=1,12)
     * /0.80,0.80,0.80,0.80,0.80,0.90,0.90,0.90,0.85,0.80,0.80,0.80/
      data (vbgm(22,j),j=1,12)
     * /0.80,0.80,0.80,0.80,0.80,0.90,0.90,0.90,0.85,0.80,0.80,0.80/
      data (vbgm(23,j),j=1,12)
     * /0.70,0.70,0.70,0.70,0.70,0.80,0.80,0.80,0.75,0.70,0.70,0.70/
      data (vbgm(24,j),j=1,12)
     * /0.60,0.60,0.60,0.60,0.60,0.70,0.70,0.70,0.65,0.60,0.60,0.60/
      data (vbgm(25,j),j=1,12)
     * /0.50,0.50,0.50,0.50,0.50,0.60,0.60,0.60,0.55,0.50,0.50,0.50/
      data (vbgm(26,j),j=1,12)
     * /0.30,0.30,0.30,0.30,0.30,0.40,0.40,0.40,0.35,0.30,0.30,0.30/
      data (vbgm(27,j),j=1,12)  /12*0.1/
      data (vbgm(28,j),j=1,12)
     * /0.05,0.05,0.05,0.50,0.70,0.80,0.80,0.90,0.90,0.80,0.05,0.05/
      data (vbgm(29,j),j=1,12)
     * /0.50,0.50,0.55,0.70,0.90,0.90,0.90,0.90,0.90,0.80,0.60,0.50/
      data (vbgm(30,j),j=1,12)  /12*0.1/

      do 50 j=1,12
      vbgf(j)= vbgm(land,j)
50    continue

! ---------------------------------------------------------------------
!  Maximal interception storages
! ---------------------------------------------------------------------
      data (simaxm(1,j),j=1,12)  /12*0.0/
      data (simaxm(2,j),j=1,12)  /12*5.0/
      data (simaxm(3,j),j=1,12)
     * /2.5,2.8,2.8,2.5,3.5,4.3,4.5,4.3,4.1,3.9,3.0,2.5/
      data (simaxm(4,j),j=1,12)
     * /1.5,1.5,1.5,2.0,3.0,3.5,3.5,3.3,3.2,2.7,2.0,1.5/
      data (simaxm(5,j),j=1,12)
     * /2.0,2.2,2.2,2.3,3.3,3.9,4.0,3.8,3.6,3.2,2.5,2.0/
      data (simaxm(6,j),j=1,12)
     * /0.5,0.5,0.5,1.5,2.5,3.0,3.0,2.5,2.0,1.5,1.0,0.5/
      data (simaxm(7,j),j=1,12)
     * /1.5,1.5,1.8,2.3,3.0,3.5,3.5,3.2,2.8,2.0,1.5,1.5/
      data (simaxm(8,j),j=1,12)
     * /1.5,1.5,1.5,2.0,2.8,3.0,3.0,2.8,2.4,2.0,1.7,1.5/
      data (simaxm(9,j),j=1,12)   /12*0.5/
      data (simaxm(10,j),j=1,12)  /12*0.5/
      data (simaxm(11,j),j=1,12)  /12*0.5/
      data (simaxm(12,j),j=1,12)  /12*0.5/
      data (simaxm(13,j),j=1,12)  /12*0.5/
      data (simaxm(14,j),j=1,12)  /12*0.5/
      data (simaxm(15,j),j=1,12)  /12*4.0/
      data (simaxm(16,j),j=1,12)  /12*0.0/
      data (simaxm(17,j),j=1,12)  /12*0.0/
      data (simaxm(18,j),j=1,12)
     * /0.5,0.5,0.5,1.5,2.5,3.0,3.0,2.5,2.0,1.5,1.0,0.5/
      data (simaxm(19,j),j=1,12)
     * /1.5,1.5,1.6,2.1,3.0,3.5,3.5,3.2,3.0,2.4,1.7,1.5/
      data (simaxm(20,j),j=1,12)
     * /0.5,0.5,0.5,1.0,1.5,2.0,2.0,1.5,0.0,0.2,0.3,0.5/
      data (simaxm(21,j),j=1,12)
     * /0.5,0.5,0.5,0.8,0.8,1.4,1.5,1.5,1.4,0.5,0.5,0.5/
      data (simaxm(22,j),j=1,12)
     * /1.0,1.0,1.3,1.8,2.5,2.8,3.0,2.8,2.0,1.3,1.0,1.0/
      data (simaxm(23,j),j=1,12)
     * /1.0,1.0,1.3,1.8,2.5,2.8,3.0,2.8,2.0,1.3,1.0,1.0/
      data (simaxm(24,j),j=1,12)
     * /0.5,0.5,0.5,0.8,0.8,1.2,1.2,1.2,1.2,0.5,0.5,0.5/
      data (simaxm(25,j),j=1,12)
     * /0.5,0.5,0.5,0.6,0.8,1.2,1.2,1.2,1.2,0.5,0.5,0.5/
      data (simaxm(26,j),j=1,12)
     * /0.1,0.1,0.1,0.1,0.2,0.4,0.4,0.4,0.3,0.1,0.1,0.1/
      data (simaxm(27,j),j=1,12)  /12*1.5/
      data (simaxm(28,j),j=1,12)
     * /0.1,0.1,0.1,0.5,1.50,2.0,2.2,2.5,2.5,2.0,0.1,0.1/
      data (simaxm(29,j),j=1,12)
     * /0.5,0.5,0.6,1.0,2.0,2.5,2.5,2.2,2.0,0.9,0.7,0.5/
      data (simaxm(30,j),j=1,12)  /12*1.5/

      do 60 j=1,12
         simaxf(j)= simaxm(land,j)
60    continue

! ---------------------------------------------------------------------
!  Correction term for soil heat flux
! ---------------------------------------------------------------------
      do 70 j=1,12
        bflux(j)= 0.3-0.03*rlai(j)
        if (land.eq.7) bflux(j)=0.2
        if (land.le.2) bflux(j)=0.3
        if (land.ge.9 .and. land.le.17) bflux(j)=0.3
70    continue

! ---------------------------------------------------------------------
!  Stomata resistences
! ---------------------------------------------------------------------
      data (rscm(1,j),j=1,12) /12*0.0/
      data (rscm(2,j),j=1,12) /12*0.0/
      data (rscm(3,j),j=1,12)
     */55.0,55.0,55.0,50.0,40.0,40.0,40.0,45.0,55.0,55.0,55.0,55.0/
      data (rscm(4,j),j=1,12)
     */65.0,65.0,65.0,60.0,45.0,50.0,50.0,53.0,60.0,65.0,65.0,65.0/
      data (rscm(5,j),j=1,12)
     */63.0,63.0,63.0,60.0,43.0,48.0,48.0,51.0,60.0,63.0,63.0,63.0/
      data (rscm(6,j),j=1,12) /12*40.0/
      data (rscm(7,j),j=1,12)
     */70.0,70.0,50.0,35.0,25.0,25.0,30.0,40.0,45.0,60.0,70.0,70.0/
      data (rscm(8,j),j=1,12)
     */80.0,80.0,70.0,65.0,55.0,55.0,55.0,60.0,70.0,70.0,80.0,80.0/
      data (rscm(9,j),j=1,12) /12*-100.0/
      data (rscm(10,j),j=1,12) /12*-100.0/
      data (rscm(11,j),j=1,12) /12*-100.0/
      data (rscm(12,j),j=1,12) /12*-100.0/
      data (rscm(13,j),j=1,12) /12*-100.0/
      data (rscm(14,j),j=1,12) /12*0.0/
      data (rscm(15,j),j=1,12) /12*0.0/
      data (rscm(16,j),j=1,12) /12*0.0/
      data (rscm(17,j),j=1,12) /12*0.0/
      data (rscm(18,j),j=1,12) /12*40.0/
      data (rscm(19,j),j=1,12)
     * /80.0,80.0,80.0,70.0,55.0,55.0,60.0,65.0,70.0,75.0,80.0,80.0/
      data (rscm(20,j),j=1,12)
     */70.0,70.0,50.0,40.0,35.0,50.0,50.0,60.0,60.0,60.0,70.0,70.0/
      data (rscm(21,j),j=1,12)
     */75.0,75.0,60.0,52.5,50.0,57.5,57.5,65.0,65.0,65.0,75.0,75.0/
      data (rscm(22,j),j=1,12)
     */70.0,70.0,50.0,40.0,30.0,30.0,35.0,40.0,50.0,60.0,70.0,70.0/
      data (rscm(23,j),j=1,12)
     */75.0,75.0,55.0,45.0,35.0,35.0,40.0,45.0,55.0,65.0,75.0,75.0/
      data (rscm(24,j),j=1,12)
     */78.0,78.0,58.0,48.0,38.0,38.0,43.0,48.0,58.0,68.0,78.0,78.0/
      data (rscm(25,j),j=1,12)
     */80.0,80.0,60.0,50.0,40.0,40.0,45.0,50.0,60.0,70.0,80.0,80.0/
      data (rscm(26,j),j=1,12)
     */85.0,85.0,65.0,55.0,45.0,45.0,50.0,55.0,65.0,75.0,85.0,85.0/
      data (rscm(27,j),j=1,12) /12*40.0/
      data (rscm(28,j),j=1,12)
     * /80.0,80.0,80.0,70.0,55.0,55.0,60.0,65.0,65.0,75.0,80.0,80.0/
      data (rscm(29,j),j=1,12)
     * /80.0,80.0,80.0,70.0,55.0,55.0,60.0,65.0,70.0,75.0,80.0,80.0/
      data (rscm(30,j),j=1,12) /12*40.0/

      do 80 j=1,12
         rsc(j)=rscm(land,j)
80    continue

! ---------------------------------------------------------------------
      return
      end
! ---------------------------------------------------------------------
