!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2003 Joachim Gurtz, Massimiliano Zappa

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE ERAERO (u2,bhoehe,ra,idebug,land)

      implicit none

!-------------------------------------------------------------------------------
!     This subroutine computes the aerodynamic resistance RA
!     In FORHYCS, this subroutine is only used for non-forested vegetation
!     types. For aerodynamic resistance in forests, the new subroutine
!     eraeroCan is used.
!--------------------------------------------------------------------------

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------
      INTEGER idebug,                       ! controls the debugging process
     *        land                          ! land use
      REAL u2,                              ! wind speed at 2m height
     *     bhoehe,                          ! canopy height
     *     ra                               ! aereodynamic resistence

! ---------------------------------------------------------------------
!     Local Variables
! ---------------------------------------------------------------------
      REAL z0,                              ! roughness lenght
     *    d,u10,u10d

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------

      call EWIND (u2,3,bhoehe,1,u10)  ! Get wind speed at 10 m (output) from wind speed at 2 m (input)

      if (bhoehe.le.0.0) bhoehe=0.005
      z0=0.1*bhoehe
	if (land .eq. 1) z0=0.00137       ! z0 of water
      d=0.6*bhoehe

      if(z0.eq.0.0) write(6,*) ' Attention, error in SP ERAERO z0=0.0'
      if(z0.eq.0.0) write(42,*) ' Attention, error in SP ERAERO z0=0.0'

      u10d=u10
      if(z0.ge.0.9) u10d=0.6*u10
      if(u10d.eq.0.0) u10d=0.01
      ra= (6.25/u10d)*alog(10./z0)*alog(10./z0)
      if(z0.ge.0.9) ra=56.3/u10d

! ---------------------------------------------------------------------
!     output can be written if idebug > 0
! ---------------------------------------------------------------------
      if(idebug.gt.0) then
       write(42,'(''****Written by UP ERAERO****'')')
       write(42,'('' Wind in  2m Height  m/s '',f6.1)') u2
       write(42,'('' Wind in 10m Height  m/s '',f6.1)') u10
       write(42,'('' Canopy height      m   '',f6.2)') bhoehe
       write(42,'('' Resistance RA     s/m '',f10.2)') ra
      endif

      return
      end
