!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2003 Joachim Gurtz, Massimiliano Zappa
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Avoid NaNs

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE ERGKORZ (TAG,MONAT,JAHR,GEOBREITE,SLOPE,ASPECT,
     *                    S0,TSA,TSU,RGMAX,RGDTI,RGKDM)

      implicit none

! ---------------------------------------------------------------------
!      Potential incoming global radiation, the maximal sunshine
!      duration, surise, sunset and make a local adjustemnt of global
!      radiation
! ---------------------------------------------------------------------

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------

      REAL SLOPE,ASPECT				!	slope (deg) and aspect (deg) of the HRU
      REAL GEOBREITE				!	latitude (deg)
      INTEGER TAG,MONAT,JAHR			!	day, month year
      REAL RGDTI(24),RGMAXTI(24)		!	hourly observed and maximal incoming direct radiation
      REAL S0					!	maximal sunshine duration
      REAL RGMAX				!	potential incoming radiation
      REAL RGKDM				!	adjusted observed global radiation (daily average)
      REAL PI					!	3.1414......
      REAL DEKLINATION				!	Declination of the Aequator in Deg
      REAL JAHRESLAENGE				!	Length of the year (Days)
      REAL KF					!	Correction parameter
      REAL RM					!	Average distance earth-sun in millions of km
      REAL E					!	excentricity
      REAL I0					!	constant solar radiaiton
      REAL D					!	day length between sunrise and sunset in hours
      INTEGER JULDATUM				!	Julian date
      INTEGER SCHALTJAHR			!	variables to identify leap years
      INTEGER NORMDATUM				!	julian date of the 21st July in the previous year
      REAL WINKEL				!	Angle of sun at 21st June
      REAL W0					!	Angle of sun at sunset
      REAL AKTDEKLIN				!	actual declination
      REAL R					!	actual radius of earth ellipse
      INTEGER PERIHELDATUM			!	??????
      REAL PERIHEL				!	PERIHEL
      INTEGER TSA				!	time of sunrise
      INTEGER TSU				!	time of sunset
      REAL wsa					!	Angle at sunrise
      REAL wsu					!	angle at sunset
      real ws,stsa,stsu
      REAL zcos					!	Cosinus of zenith angle
      REAL zsin					!	Sinus of Zenith angle
      REAL as					!	Azimuth of sun
      REAL wslope
      real gws					!	summation of global radiation
      integer i,ii				!	varaibles for loops
      real sumg,wink
      real ssdr					!	relative sunshine duration
      REAL KORFAK,RGKDS,RGKOR			!	parameters for adjusting global radiation

      !integer xcoor, ycoor  ! DEBUG ONLY
! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------
! Declaration of constants
      PI=3.14159265359
      DEKLINATION=23.45
      JAHRESLAENGE=365.25
      KF=0.276666667
      RM=149.5
      E=0.01675
      I0=1367.0
      D =24.

      CALL EJULIANZ (JAHR,MONAT,TAG,JULDATUM)
      SCHALTJAHR=JAHR-MOD(JAHR,4)
      CALL EJULIANZ (SCHALTJAHR,6,21,NORMDATUM)
      WINKEL=360.*(JULDATUM-NORMDATUM)/JAHRESLAENGE+360.

    1 CONTINUE
      IF (WINKEL.LT.360.) GOTO 2
	WINKEL=WINKEL-360.
        GOTO 1
    2 CONTINUE

! ---------------------------------------------------------------------
!     Computation of the actual declination
! ---------------------------------------------------------------------
      AKTDEKLIN=DEKLINATION*COS(WINKEL*PI/180.)


! computation of the angle of sun above the horizon
      W0=180.*ACOS(-TAN(AKTDEKLIN*PI/180.)*TAN(GEOBREITE*PI/180.))/PI


! computaiton of the maximal sunshine duration
      S0=24.*W0/180.+KF


! Angle at 2nd of juli
      CALL EJULIANZ (JAHR,7,2,PERIHELDATUM)
      PERIHEL=360.*(JULDATUM-PERIHELDATUM)/JAHRESLAENGE+360.


! Distance between earth and sun
      R=RM*(1.-E*COS(PERIHEL*PI/180.))

! computation of sunrise and sunset
      wsu=w0
      stsu=(D/2.*wsu/180.)+12.
      stsa=12.-(stsu-12.)
      wsa=(stsa-12.)*2/D*180.
      tsa=int(stsa+0.25)
      tsu=int(stsu+0.75)

! ---------------------------------------------------------------------
! integration to houlrly steps
      ii=-1
      sumg=0.0
      RGKDS=0.0
      do 10 i=tsa,tsu-1
        ii=ii+1


! Hourly angle
        WS=(real(12-(tsa+ii+0.5)))*15.0


! Cosinus of zenith-angle
        ZCOS=
     *       SIN(AKTDEKLIN*PI/180.)*
     *       SIN(GEOBREITE*PI/180.)+
     +       COS(Ws*PI/180.)*
     *       COS(AKTDEKLIN*PI/180.)*
     *       COS(GEOBREITE*PI/180.)

! Sinus of zenith-angle
        WINK=acos(zcos)
	ZSIN=sin(WINK)

	 as=((SIN (AKTDEKLIN*PI/180.0)*COS (GEOBREITE*PI/180.0))-
     *        (COS (AKTDEKLIN*PI/180.0)*SIN (GEOBREITE*PI/180.0)*
     *         COS (WS*pi/180)))/zsin
         AS=ACOS(as)*180.0/PI
         IF (I .GE. 12) AS=360.0-AS
! cosinus of the sun angle and the angle normal to the slope
	 wslope=
     *       cos(slope*PI/180.0)*
     *       ZCOS+
     *       ZSIN*
     *       sin(slope*PI/180.0)*
     *       cos((aspect-AS)*PI/180.0)

! ---------------------------------------------------------------------
! slope correction
        !IF ((wslope .LE. 0.0) .OR. (zcos .LE. 0.0)) THEN
        IF (zcos .LE. 0.0) THEN
           KORFAK=0.0
           gws=0.0
        ELSE
           !KORFAK= wslope
            korfak = zcos    ! Was wrong in previous version of code (KORFAK was assigned WSLOPE instead of ZCOS)

! ---------------------------------------------------------------------
!   Adjustment of the maximal incoming global radiation
           gws=KORFAK*(I0*RM*RM/R/R)*(0.72**(1/zcos))
        ENDIF



        sumg=sumg+gws
	  RGMAXTI(I)=GWS

         IF (wslope .LE. 0.0) THEN
           KORFAK=0.0
           GOTO 6
         ENDIF
         SSDR=1.0

         IF (ZCOS .LE. 0.34) THEN
           KORFAK=WSLOPE*(1/0.34)
           GOTO 6
         ENDIF

! ---------------------------------------------------------------------
! Adjustment of observed global radiation
         KORFAK=WSLOPE*(1.0/ZCOS)
6        RGKOR=RGMAXTI(I)*(0.15+(0.85*KORFAK))


         RGKDS=RGKDS+RGKOR
	   RGDTI(I)=RGDTI(I)*RGKOR/RGMAXTI(I)
	   if(rgMaxTI(i)==0.0) rgdTI (i) = 0.0  ! Prevent NaN from dividing by zero
10    continue
! ---------------------------------------------------------------------
! Daily values
      RGMAX=sumg/24.0
      RGKDM=RGKDS/24.0

! ---------------------------------------------------------------------
      RETURN
      END
! ---------------------------------------------------------------------
