!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2003 Joachim Gurtz, Massimiliano Zappa

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

	SUBROUTINE ERSTOM01 (rlai,rsc,tl,t,elx,el,iart,land,awac,
     *                     awacsol,bfeucht,rs,idebug)

      implicit none

!-------------------------------------------------------------------------------
!     This subroutine computes the stomatal resistance RS
!     In FORHYCS, this subroutine is only used for non-forested vegetation
!     types. For stomatal resistance in forests, the new subroutine
!     erstomJarvis is used.
!--------------------------------------------------------------------------

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------
      INTEGER idebug			! controls output
	INTEGER iart			! control the computation of EP/ER
      INTEGER land			! Land use code
      REAL rlai				! Leaf area index
      REAL rsc				! minimal stomata resistence
      REAL tl					! lenght of a day
      REAL t					! temeperature in �C
      REAL elx				! staurated water vapour pressure hPa
      REAL el					! actual water vapour pressure hPa
      REAL rs					! actual stomata resistence
	REAL awac				! water capacity of vegetated soil
	REAL awacsol			! water capacity of non-vegetated soil
	REAL bfeucht			! soil moisture in mm

! ---------------------------------------------------------------------
!     Local Variables
! ---------------------------------------------------------------------
!     rss= surface resitance of bare soil in s/m
      REAL rss,rscnight,rsk,xbsoilm,ybsoilm,xvegm,yvegm,xveg,yveg
      REAL deltae,f,a,rsd,rsn

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------

      if (idebug.gt.0) then
       write(42,'(''****RUNNING SP ERSTOM01**** '')')
       write(42,'('' Leaf area index      m/m '',f6.2)') rlai
       write(42,'('' Stomata resistance   s/m '',f6.2)') rsc
       write(42,'('' Lenght of the day    h   '',f6.2)') tl
       write(42,'('' Temperature          C   '',f6.2)') t
       write(42,'('' Satur. water vapour  hpa '',f6.2)') elx
       write(42,'('' Water vapour         hpa '',f6.2)') el
      endif

      rss=100.
      rscnight=5000.
      rsk=10.**4.

! ----------------------------------------------------------------------
!  Stomata resistence for water rsc=0.0
! ----------------------------------------------------------------------
      if (rsc .eq. 0.0  .or.  iart .eq. 5) then
	 rs=0.0
	 if (idebug.gt.1)
     +    write(42,'('' Stomata resistence       '',f10.2)') rs
	 return
      endif

! ----------------------------------------------------------------------
!  Stomata resistence for bare soils rsc=-100.
! ----------------------------------------------------------------------
      if (rsc.eq.-100.) then
	 rs=rss
	 if (idebug.gt.1)
     +   write(42,'('' Stomata resistence        '',f10.2)') rs
	 return
      endif

! ----------------------------------------------------------------------
!  Accountig for temperature for coniferous forests
! ----------------------------------------------------------------------

      if (land .eq. 3) then
	 if (t.gt.-5.0 .and. t.le.20.0) rsc=25.*rsc / (t+5.0)
	 if (t.le.-5.) rsc=rsk
      if (idebug.gt.0)
     +write(42,'('' Stomata resistence, T     '',2f10.2)') rsc,t
      endif

! ----------------------------------------------------------------------
!  Accountig for water vapour pressure for coniferous forests
! ----------------------------------------------------------------------
      if (land .eq. 3) then
	 deltae=elx-el
	 if (deltae.le.20.0 .and. rsc.ne.rsk ) then
	    rsc =rsc / (1.0-0.05*deltae)
          else
	    rsc=rsk
	 endif
      endif

! ----------------------------------------------------------------------
!  Accounting of the soil moisture deficit
! ----------------------------------------------------------------------
      if (iart.gt.1) then
       xbsoilm=0.3 * awacsol
	 ybsoilm=0.5 * awacsol
	 xvegm= 0.3 *awac
	 yvegm= 0.5 *awac
	 if (bfeucht.le.yvegm) then
	    xveg=0.0
	    yveg=bfeucht
	 else
	    yveg=yvegm
	    xveg=bfeucht-yvegm
	 endif
	 if (land.lt.9 .or. land.gt.17) then
	     if (xveg.gt.0.0) then
		rsc=rsc
		rss=rss
		if (xvegm.le.xbsoilm) rss=rss*xvegm /(xveg+0.01 * xveg)
	     else
		rss=100000.0
                if (yveg.lt.1.0) yveg=0.5
                if (yvegm.gt.0.0)
     *   	rsc=rsc*(3.5*(1.-yveg/yvegm)+ exp(0.2*(yvegm/yveg-1.)))
	     endif
	 endif
	 if (land.ge.9 .and. land.le.14) then
	     if ((xbsoilm.gt.xveg) .and. (xveg.gt.0.0)) rss=rss
	     if ((xveg.eq.0.0) .and. (yvegm.gt.0.0)) then
               if (yveg.lt.1.0) yveg=0.5
               rss=rss*(3.5*(1.-yveg/yvegm) +exp(0.2*(yvegm/yveg-1.)))
             endif
	 endif
      endif

! ----------------------------------------------------------------------
!  Stomata resistence during the day rsd
! ----------------------------------------------------------------------
      f=0.7
      a=f**rlai
      if (rsc.le.0.0) rsc=0.001
      if (rss.le.0.0) rss=0.001
      rsd= (1.-a)/rsc + a/rss
      if (iart.eq.3) rsd=a/rss
      if (iart.eq.4) rsd=(1.-a)/rsc

! ----------------------------------------------------------------------
!  Stomata resistence during the day night  rsn
! ----------------------------------------------------------------------

      rsn=2.*rlai/rscnight + 1/rss
      if (iart.eq.3) rsn=1/rss
      if (iart.eq.4) rsn=2.*rlai/rscnight

! ----------------------------------------------------------------------
!  daily mean of the stomata resistence rs
! ----------------------------------------------------------------------
      rs = 1. / (tl/24.*rsd + (1.-tl/24.) * rsn)
      if (idebug.gt.0)
     +   write(42,'('' Stomata resistence TM     '',f10.2)') rs

! ----------------------------------------------------------------------
      return
      end
