!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE ESCIHELP (TAIR,PDT,PSNOW,PLIQ,PHASE,TGR,TTRANS,
     *                     SSO,LAND,CREDV)

!     This subroutine computes the phase of snow for ESCIMO
!--------------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------

      INTEGER LAND                                 ! land use
      REAL PHASE,                                  ! phase of precipitation
     *     TAIR,                                   ! phase of precipitation
     *     PDT,                                    ! precipitaiton in the timestep
     *     CREDV,                                  ! ask Achim
     *     TGR,                                    ! treshold temperature for snow phase
     *     SSO,                                    ! actual snow water equivalent
     *     TTRANS,                                 ! transition range for prec. phase
     *     PSNOW,                                  ! solid precipitation
     *     PLIQ,                                   ! liquid precipitation

! ---------------------------------------------------------------------
!     Internal Variables
! ---------------------------------------------------------------------

     *     PREC                                   ! backup precipitation

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------

      PSNOW=0.0
      PREC=PDT
      PLIQ=PREC


! ---------------------------------------------------------------------
!     This operations are processed only if CREDV>0.001, if snow is
!     or accumulation within the current time step.
!     No accumulation is possible on water surfaces (land = 1)
! ---------------------------------------------------------------------

      phase=0.0

      IF (CREDV .LT. 0.001)  GOTO 1202
      IF (LAND .EQ. 1) GOTO 1202

! no snow
      IF  ((SSO.LT.0.01) .and.
     *    ((PDT.LT.0.01) .or. (TAIR.GT.(TGR+TTRANS))) )  GOTO 1202

!     Adjustment of precipitation (both liquid and solid) occurs within
!     the main program

!     Just snow
      IF ((TAIR .LE. (TGR-TTRANS)) .or.
     *    ((TTRANS .LE. 0.01) .and. (TAIR .LE. (TGR+TTRANS)))) THEN
         PSNOW=PREC
         PLIQ=0.0
         phase=1.0
         GOTO 1202
      ENDIF

! Mixture of rain and snow
      IF ((TAIR.GT.(TGR-TTRANS)) .and. (TAIR.LT.(TGR+TTRANS))) THEN
         PSNOW=PREC*((TGR+TTRANS-TAIR)/(2*TTRANS))
         PLIQ=PREC-PSNOW
         IF (PLIQ .LT. 0.0) THEN
            PSNOW=PREC
            PLIQ=0.0
         ENDIF
      ENDIF

      IF (PSNOW.GT.PLIQ) PHASE=1.0

! ---------------------------------------------------------------------

1202  CONTINUE
      RETURN
      END
! ---------------------------------------------------------------------
