!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE EWIND (iwind,iunit,bhoehe,ounit,owind)

      implicit none

!     This subroutine computes the incoming wind speed into a new dimension
!     on the basis of the parameters iunit and ounit
!--------------------------------------------------------------------------

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------

      REAL iwind, owind, bhoehe
!     iunit:unit of iwind; ounit: unit of owind
      INTEGER iunit, ounit
!		0:BEAUFORT
!		1:M/S, at 10 m elevation
!		2:M/S, at 0.5 m elevation
!		3:M/S, at 2 m elevation
!		4:KM/H, at 2 m elevation

! ---------------------------------------------------------------------
!     Local Variables
! ---------------------------------------------------------------------
      REAL W
      REAL Z0

! -----------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------

      z0=0.1*bhoehe
      if (z0.le.0.0) z0=0.05
! ---------------------------------------------------------------------
!     from input to 10 meters in m/s
! ---------------------------------------------------------------------
      IF (iunit.EQ.0) W=0.836*iwind**(3./2.)
      if (iunit.eq.0.and.iwind.lt.0.3) w=0.1
      IF (iunit.EQ.1) W=iwind
      IF (iunit.EQ.2) W=iwind/0.7*1.3
      !IF (iunit.EQ.3) W=iwind*1.3
      IF (iunit.EQ.3) W=iwind*alog(10./z0)/alog(2./z0)
      IF (iunit.EQ.4) W=iwind*3.6

! ---------------------------------------------------------------------
!     from 10 meters in m/s to output
! ---------------------------------------------------------------------

      IF (ounit.EQ.0) owind=(W/0.836)**(2./3.)
      IF (ounit.EQ.1) owind=W
      IF (ounit.EQ.2) owind=w/1.3*0.7
      !IF (ounit.EQ.3) owind=W/0.786
      IF (ounit.EQ.3) owind=W*alog(2./z0)/alog(10./z0)
      IF (ounit.EQ.4) owind=W*alog(2./z0)/alog(10./z0)/3.6

! ---------------------------------------------------------------------
      RETURN
      END
