!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE EZENITZ (TAG,MONAT,JAHR,GEOBREITE,SLOPE,ASPECT,
     *                    DEBUG,S0,G0,TSA,TSU)

      implicit none
!
! Computes the potential incoming global radiation and the maximal�
! possible sunshine duration
! ------------------------------------------------------------------------------

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------

      REAL SLOPE,ASPECT				!	slope (deg) and aspect (deg) of the HRU
      REAL GEOBREITE				!	latitude (deg)
      INTEGER TAG,MONAT,JAHR			!	day, month year
      INTEGER DEBUG			!	controls fro partial output
      REAL S0					!	maximal sunshine duration
      REAL G0					!	potential incoming radiation
      REAL PI					!	3.1414......
      REAL DEKLINATION				!	Declination of the Aequator in Deg
      REAL JAHRESLAENGE				!	Length of the year (Days)
      REAL KF					!	Correction parameter
      REAL RM					!	Average distance earth-sun in millions of km
      REAL E					!	excentricity
      REAL I0					!	constant solar radiation
      REAL D					!	day length between sunrise and sunset in hours
      INTEGER JULDATUM				!	Julian date
      INTEGER SCHALTJAHR			!	variables to identify leap years
      INTEGER NORMDATUM				!	julian date of the 21st July in the previous year
      REAL WINKEL				!	Angle of sun at 21st June
      REAL W0					!	Angle of sun at sunset
      REAL AKTDEKLIN				!	actual declination
      REAL R					!	actual radius of earth ellipse
      INTEGER PERIHELDATUM			!	??????
      REAL PERIHEL				!	PERIHEL
      INTEGER TSA				!	time of sunrise
      INTEGER TSU				!	time of sunset
      REAL wsa					!	Angle at sunrise
      REAL wsu					!	angle at sunset
      real ws,stsa,stsu
      REAL zcos					!	Cosinus of zenith angle
      REAL zsin					!	Sinus of Zenith angle
      REAL as					!	Azimuth of sun
      REAL wslope
      real gws					!	summation of global radiation
      integer i,ii				!	varaibles for loops
      real sumg

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------
! Declaration of constants
      PI =3.14159265359
      DEKLINATION =23.45
      JAHRESLAENGE =365.25
      KF =0.276666667
      RM =149.5
      E =0.01675
      I0 =1367.0
      D =24.

      CALL EJULIANZ (JAHR,MONAT,TAG,JULDATUM)
      SCHALTJAHR=JAHR-MOD(JAHR,4)
      CALL EJULIANZ (SCHALTJAHR,6,21,NORMDATUM)
      WINKEL=360.*(JULDATUM-NORMDATUM)/JAHRESLAENGE+360.

    1 CONTINUE
      IF (WINKEL.LT.360.) GOTO 2
	WINKEL=WINKEL-360.
        GOTO 1
    2 CONTINUE

! ---------------------------------------------------------------------
!     Computation of the actual declination
! ---------------------------------------------------------------------
      AKTDEKLIN=DEKLINATION*COS(WINKEL*PI/180.)

! computation of the angle of sun above the horizon
      W0=180.*ACOS(-TAN(AKTDEKLIN*PI/180.)*TAN(GEOBREITE*PI/180.))/PI

! computaiton of the maximal sunshine duration
      S0=24.*W0/180.+KF

! Angle at 2nd of juli
      CALL EJULIANZ (JAHR,7,2,PERIHELDATUM)
      PERIHEL=360.*(JULDATUM-PERIHELDATUM)/JAHRESLAENGE+360.

! Distance between earth and sun
      R=RM*(1.-E*COS(PERIHEL*PI/180.))

! computation of sunrise and sunset
      wsu=w0
      stsu=(D/2.*wsu/180.)+12.
      stsa=stsu-s0
      wsa=(stsa-12.)*2/D*180.
      tsa=int(stsa+0.25)
      tsu=int(stsu+0.75)

! ---------------------------------------------------------------------
! integration to hourly step
      ii=-1
      sumg=0.0
      do 10 i=tsa,tsu-1
        ii=ii+1
! Hourly angle
        WS=(real(12-(tsa+ii+0.5)))*15.0

! Cosinus of zenith-angle
        ZCOS=
     *       SIN(AKTDEKLIN*PI/180.)*
     *       SIN(GEOBREITE*PI/180.)+
     +       COS(Ws*PI/180.)*
     *       COS(AKTDEKLIN*PI/180.)*
     *       COS(GEOBREITE*PI/180.)

! Sinus of zenith-angle
	 ZSIN=sin(acos(zcos))

	 as= (1./(sin(cos(aktdeklin*pi/180.)*
     *            sin(ws*pi/180)/zsin)))*180/pi

! cosinus of the sun angle and the angle normal to the slope
	 wslope=
     *       cos(slope*PI/180.)*
     *       zcos+
     +       zSIN*
     *       sin(slope*PI/180.)*
     *       cos((as-aspect)*PI/180.)

! ---------------------------------------------------------------------
! Summation

        gws=wslope*I0*RM*RM/R/R
        sumg=sumg+gws

10    continue
! ---------------------------------------------------------------------
! computing potential global radiation

      G0=sumg/24

      RETURN
      END
! ---------------------------------------------------------------------
