!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! A set of functions to compare floating-point values allowing for
! numerical imprecision, to ensure numerical stability.

! In each of these functions, a and b are two floating-point values
! and epsilon is the error tolerance

! See:
!
! Grosse-Kunstleve, R. W., N. K. Sauter and P. D. Adams (2003)
! Numerically stable algorithms for the computation of reduced unit cells.
! Acta Crystallographica Section A (60), 1-6.
! 10.1107/S010876730302186X
!-------------------------------------------------------------------------------

module fp_compare

contains

elemental function fp_eq(a,b,epsilon)
  real, intent(in) :: a,b
  real, intent(in) :: epsilon
  logical :: fp_eq

  !eq = (abs(a-b) .lt. epsilon)
  fp_eq = .not.((a .lt. b-epsilon) .or. (b .lt. a-epsilon))

end function fp_eq

!-------------------------------------

! see http://stackoverflow.com/a/12868306/2249007
elemental function fp_gt(a,b,epsilon)
  real, intent(in) :: a,b,epsilon
  logical:: fp_gt

  !gt = ((a-b .gt. epsilon) .and. (abs(a-b) .gt. epsilon))
  fp_gt = b .lt. a-epsilon

end function fp_gt

!-------------------------------------

elemental function fp_lt(a,b,epsilon)
  real, intent(in) :: a,b,epsilon
  logical:: fp_lt

  !lt = ((a-b .lt. epsilon) .and. (abs(a-b) .gt. epsilon))
  fp_lt = a .lt. b-epsilon

end function fp_lt

!-------------------------------------

elemental function fp_ge(a,b,epsilon)
  real, intent(in) :: a,b,epsilon
  logical:: fp_ge

  fp_ge = .not.(a .lt. b-epsilon)

end function fp_ge

!-------------------------------------

elemental function fp_le(a,b,epsilon)
 real, intent(in) :: a,b,epsilon
 logical:: fp_le

 fp_le = .not.(b .lt. a-epsilon)

end function fp_le

end module fp_compare
