!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE FROUTING (rout,ininam,ernam,storage,translation,AELHG,
     *           filetmp,mhtml,saveid,dirstate,savemmid,
     *              savemoreid,SAVEMORENN,SAVECHAR,loadrout,LOADCHAR,
     *           fortype,allinf)
!--------------------------------------------------------------------------
!     Operations: 1=Transformation of hydrograph from external file (Area of external region required)
!                 2=Addition of hydrograph from external file (for current catchment area)
!                 3=Subtraction of hydrograph from external file (for current catchment area)
!                 4=Transformation of actual hydrograph (for current catchment area)
!
!    Implemented in PREVAH by M.Zappa: 25-26 November 2003
!    Subroutine LINSTRA by J.Gurtz (adapted by M.Zappa 25 November 2003)
! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------

      implicit none

      INTEGER I,MO,TA,HO,JJ,rout,TAG,MON,JAHR,frdate(4,240),K
      INTEGER ROCOL,OZK,mhtml,fortype,frloop,frcount
      INTEGER pos,pos1,pos2,pos3,runit,operat
	INTEGER MOB,TAB,HOB,JJB,BEG
	REAL SMM,SLPS,RMM,RLPS,store,omm,olps
	REAL DSMM,DSLPS,DRMM,DRLPS,Domm,Dolps,tmm,tlps,dtmm,dtlps
	REAL obsin(99),simout(99),DIV,routensemble(9,240)
	REAL storage,translation,AELHG,AEEXT,HF(240),allinf,aeorig
      CHARACTER*256 fileh,filetmp
	CHARACTER*256 ininam
	CHARACTER*256 ernam
	Character*256 diroutput
      CHARACTER waskomm*50,stp*2,stold*2

! ------------------------------------------------------------------------------
!     Load and Save state
      integer monum(12),SAVEMORENN,NN
      character*256 dirstate,staternam
      character*1 LOADROUT,LOADBUP,SAVEID,SAVEMMID,SAVEMOREID
      character*8 LOADCHAR,DATUMCHAR,DATUMCHARO
      character*8 savechar(SAVEMORENN)

! ------------------------------------------------------------------------------
!     Date settings
	MONUM(1)=31
      MONUM(2)=28
      MONUM(3)=31
      MONUM(4)=30
      MONUM(5)=31
      MONUM(6)=30
      MONUM(7)=31
      MONUM(8)=31
      MONUM(9)=30
      MONUM(10)=31
      MONUM(11)=30
      MONUM(12)=31
	LOADBUP=LOADROUT

! ---------------------------------------------------------------------
!    Determinates routing
! ---------------------------------------------------------------------
      call messagerun (1,'Computing the routing ensembles ...')
      call charnr (rout,stp)
	call posnam (ininam,1,pos)
      open (765,file=ininam(1:pos),status='old')

! ---------------------------------------------------------------------
!    Reading the external control file
! ---------------------------------------------------------------------

!     print* ,'Header'
      read (765,'(A50)') waskomm
	call messagerun (2,waskomm(1:10))

	  DO I=1,256
	  fileh(I:I)=' '
	  END DO
!      print* ,'Data from external file'
	read (765,'(A256)') fileh

!	print* ,'Area?'
      read (765,*) AEEXT

!	print* ,'Headers, column to be used?'
      read (765,*) OZK, ROCOL

!	print* ,'Unit?'
      read (765,*) runit

!	print* ,'Operation?'
      read (765,*) operat

	DIV=1.0

!     Update of (sub-)catchment area
      aeorig=aelhg
	if (operat .eq. 2) AEEXT=AELHG
      if (operat .eq. 3) AEEXT=AELHG

	if (operat .eq. 2) DIV=2.0
      if (operat .eq. 3) DIV=2.0

	if (operat .eq. 4) AEEXT=AELHG
	if (operat .eq. 4) AELHG=0.0

      close (765)

! ---------------------------------------------------------------------
!    Opening the files
! ---------------------------------------------------------------------

	call posnam (ernam,1,pos3)

9     CONTINUE

!	call messageout (ininam(1:pos))
      IF (rout .eq. 1) THEN
	    OPEN (331, file=ernam(1:(pos3-1))//'fcast', status='old')
      ELSE
          call charnr ((rout-1),stold)
          filetmp=ernam(1:(pos3-2))//stold//'.frhh'
	    OPEN (331, file=filetmp, status='old')
	END IF


      if (operat .ne. 4) THEN
	call posnam(fileh,1,pos1)
      OPEN (332, file=fileh(1:pos1), status='old')
	  IF (OZK .GT. 0) THEN
	    DO I=1,OZK
	      read (332,'(A50)') waskomm
	    END DO
	  END IF
	end if

	filetmp=ernam(1:(pos3-2))//stp//'.frhh'
      OPEN (334, file=Filetmp,status='unknown')

	filetmp=ernam(1:(pos3-2))//stp//'.frdd'
      OPEN (335, file=Filetmp,status='unknown')

!     HTML Report
	write(mhtml,'(A)')'<a href=file:///'//ernam(1:(pos3-2))//stp//
     *              '.frhh >'//ernam(1:(pos3-2))//stp//'.frhh </a><br>'
	write(mhtml,'(A)')'<a href=file:///'//ernam(1:(pos3-2))//stp//
     *              '.frdd >'//ernam(1:(pos3-2))//stp//'.frdd </a><br>'

10    continue

      beg=0
	dsmm=0.0
	drmm=0.0
	domm=0.0

! ---------------------------------------------------------------------
!     Looking for start date in external file, unit conversions
! ---------------------------------------------------------------------
      DO FRLOOP=1,FORTYPE
	loadbup='Y'
	beg=0
	frcount=0
	rewind(331)
	rewind (332)
11    continue
      frcount=frcount+1

	       read (331,*,END=33) JJ,MO,TA,HO,(simout(I),I=1,fortype)
	       SMM=simout(FRLOOP)
	       SLPS=simout(FRLOOP)*AELHG*1000/(3.6)
	if (operat .eq. 4) SLPS=simout(FRLOOP)*AEEXT*1000/(3.6)

		   RMM=simout(FRLOOP)
	       RLPS=simout(FRLOOP)*AELHG*1000/(3.6)
	if (operat .eq. 4) RLPS=simout(FRLOOP)*AEEXT*1000/(3.6)


!     Definig yesterday -> done for correct assimiation inf LOAD option is chosen
      TAG=TA-1
      MON=MO
      JAHR=JJ

      MONUM(2)=28
      IF (JAHR.EQ.(INT(float(JAHR)/4.)*4)) MONUM(2)=29

      IF (TAG .eq. 0) THEN
          MON=MON-1
          IF (MON .eq. 0) THEN
          JAHR=JAHR-1
          MON=12
          TAG=31
          ELSE
          TAG=MONUM(MON)
          END IF
      END IF

      WRITE (DATUMCHAR,'(2I2.2,I4.4)') TAG,MON,JAHR
	DATUMCHARO=DATUMCHAR
      WRITE (DATUMCHAR,'(2I2.2,I4.4)') TA,MO,JJ
!     End load - save code

	   if (operat .eq. 4) THEN
	      SMM=0.0
	      SLPS=0.0
	      goto 114
         end if

111      continue

         read (332,*,END=33) JJB,MOB,TAB,HOB,(obsin(I),I=1,ROCOL)

         if (JJ .ne. JJB) goto 111
	   if (MO .ne. MOB) goto 111
	   if (TA .ne. TAB) goto 111
	   if (HO .ne. HOB) goto 111

113      continue

	   IF (runit .eq. 1) THEN
	       RMM=OBSIN(ROCOL)
	       RLPS=RMM*AEEXT*1000/(3.6)
	   END IF

	   IF (runit .eq. 2) THEN
	       RLPS=OBSIN(ROCOL)
	       RMM=RLPS/AEEXT/1000*(3.6)
	   END IF
	   IF (runit .eq. 3) THEN
	       RLPS=OBSIN(ROCOL)*1000
	       RMM=RLPS/AEEXT/1000*(3.6)
	   END IF

114   continue

      if (beg .eq. 0) THEN
      if ((loadbup .eq. 'y') .or. (loadbup .eq. 'Y')) THEN
           if (DATUMCHAR .eq. LOADCHAR) THEN
	         call posnam(dirstate,1,NN)
               staternam=dirstate(1:NN)//'routing'//stp//'state'//
     *                 datumcharo//'.sav'
               call messagerun (1,'Loading saved routing storages')
      open (9876, file=staternam, status='old', form='binary',err=998)
	read(9876) HF
	read(9876) STORE
      close (9876)
	         loadbup='n'
      end if
      beg=1
	else
		DO I=1,240
	       HF(I)=rlps
	    END DO
	    store = rlps
	    beg=1
	end if

      if ((loadbup .eq. 't') .or. (loadbup .eq. 'T')) THEN
	         call posnam(dirstate,1,NN)
               staternam=dirstate(1:NN)//'routing'//stp//'state'//
     *                 datumcharo//'.sav'
               call messagerun (1,'Loading saved routing storages')
      open (9876, file=staternam, status='old', form='binary',err=998)
	read(9876) HF
	read(9876) STORE
      close (9876)
	loadbup='n'
      beg=1
	else
		DO I=1,240
	       HF(I)=rlps
	    END DO
	    store = rlps
	    beg=1
	end if
998   continue

      end if


! ---------------------------------------------------------------------
!    Transformation
!    Operation 1: External file
!    Operation 4: Current simulation
! ---------------------------------------------------------------------
	if ((operat .eq. 1) .or. (operat .eq. 4)) then

	CALL LINSTRA (store,rlps,storage,INT(translation+0.5),HF,olps)

	tlps=slps+olps
	tmm=TLPS/(AEEXT+AELHG)/1000*(3.6)
      omm=OLPS/(AEEXT)/1000*(3.6)
	end if

! ---------------------------------------------------------------------
!    Addiction or subraction
! ---------------------------------------------------------------------

	if (operat .eq. 2) then
		olps=rlps
          tlps=slps+olps
	    omm=OLPS/(AELHG)/1000*(3.6)
          tmm=TLPS/(AELHG)/1000*(3.6)
	    rmm=0.
	    rlps=0.
	    store=0.
	end if

	if (operat .eq. 3) then
		olps=rlps
          tlps=slps-olps
	    omm=OLPS/(AELHG)/1000*(3.6)
          tmm=TLPS/(AELHG)/1000*(3.6)
		rmm=0.
	    rlps=0.
	    store=0.
	end if


	routensemble (frloop,frcount)=tmm
	frdate(1,frcount)=JJ
	frdate(2,frcount)=MO
	frdate(3,frcount)=TA
	frdate(4,frcount)=HO

! ---------------------------------------------------------------------
!    Daily sums
! ---------------------------------------------------------------------

	dsmm=dsmm+smm
	drmm=drmm+rmm
	domm=domm+omm

	if (ho .eq. 24) then
	    dslps=dsmm*AELHG*1000/(3.6)/24
	    drlps=drmm*AEEXT*1000/(3.6)/24
	    dolps=domm*AEEXT*1000/(3.6)/24
	    dtlps=dslps+dolps
	    dtmm=dTLPS/(AEEXT+AELHG)/1000*3.6*24*DIV
	    write (335,'(I5,3I3,4F9.3,5F13.1)') JJ,MO,TA,HO,
     *                     dsmm,drmm,domm,dtmm,dslps,drlps,dolps,
     *                     dtlps,store
	    dsmm=0.0
	    drmm=0.0
	    domm=0.0

! Saving state of routing storage

      if ((saveid .eq. 'D') .or. (saveid .eq. 'd')) THEN
      call posnam(dirstate,1,NN)
      staternam=dirstate(1:NN)//'routing'//stp//'state'//
     *                 datumchar//'.sav'
      open (9876, file=staternam, status='unknown', form='binary')
	write(9876) HF
	write(9876) STORE
      close (9876)
	end if

      if ((saveid .eq. 'y') .or. (saveid .eq. 'Y')) THEN
      call posnam(dirstate,1,NN)
      IF (TA .EQ. MONUM(MO)) THEN
      if ((savemmid .eq. 'y') .or. (savemmid .eq. 'Y')) THEN
         staternam=dirstate(1:NN)//'routing'//stp//'state'//
     *                 datumchar//'.sav'
      open (9876, file=staternam, status='unknown', form='binary')
	write(9876) HF
	write(9876) STORE
      close (9876)
	end if
	end if


      if ((savemoreid .eq. 'y') .or. (savemoreid .eq. 'Y')) THEN
	DO I=1,SAVEMORENN
         if (DATUMCHAR .eq. SAVECHAR(i)) THEN
         staternam=dirstate(1:NN)//'routing'//stp//'state'//
     *                 datumchar//'.sav'
      open (9876, file=staternam, status='unknown', form='binary')
	write(9876) HF
	write(9876) STORE
      close (9876)
      end if
      END DO
      end if
	end if

	end if

	goto 11

33    CONTINUE
      END DO

! ---------------------------------------------------------------------
!    Update of the Area
! ---------------------------------------------------------------------

      AELHG=(AEEXT+AELHG)/div
	   do I=1,(frcount-1)
	   write (334,'(I4,3(1X,I2),9(1X,F9.5))') frdate(1,I),
     *   frdate(2,I),frdate(3,I),frdate(4,I),
     *   (routensemble(K,I),K=1,fortype)
	   end do
	CLOSE (331)
	CLOSE (332)
	CLOSE (334)
	CLOSE (335)



999   CONTINUE

      call messagerun (2,'[END]')

! ---------------------------------------------------------------------
      RETURN
      END
! ---------------------------------------------------------------------
