!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2003 Joachim Gurtz, Massimiliano Zappa
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Adaptation for idealized climate change runs in FORHYCS case
! study

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE GIMETDTMIN (dayval,hourval,hharr,dtminVar,filenrVar,
     *                       mdatnVar)

      use mpi
	use realtime
       use gridmeteo
       use MPIVars  ! For testing
      use prevahglobals
      use fp_compare  ! Stable comparison of floating-point values
      use forhycsglobals, only: spinupYearsFH, doground, ghogyear,
     * futureyearsFH, deltaT, deltaP

      implicit none

!     This subroutine reads the incoming meteorological information
!     from grids
!--------------------------------------------------------------------------

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------

! Hourly values
	INTEGER lcol,lrow,hharr,pos2
      real,dimension(hharr,ncol,nrow):: hourval	! Hourly values
! Daily sums and averages
      real,dimension(ncol,nrow):: dayval
      integer:: dtminVar,mdatnVar  ! Value from arrays DTMIN(6) and mdatn(6) for the respective variable
      character(256):: filenrVar  ! Value from array FILENR(6) for the respective variable

! ---------------------------------------------------------------------
!     Internal Variables
! ---------------------------------------------------------------------

      INTEGER tryagain,
     *        hp1,hp2,
     *        p,loi,  ! parameters for loops
     *        spd
      REAL vak, hpdiv, ssdzeromax
	! Read values
	character*100 rdir,rnam,odir,onam
	character*256 rfile,orifile
	character*2 sa
	character*1 cg
	integer :: wyear   ! For groundhog spin-up in FORHYCS
       logical whatclim
      real:: tolerance  ! Error tolerance for floating-point comparisons
      real :: deltemp, delprec



! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------
      fileNrVar=fileNr(stp)
	CALL POSNAM (filenrVar,1,POS)
	CALL POSNAM (dirmeteo,1,POSI1)
       call posnam (diroutput,1,pos2)
       tryagain=0

       ! For spin-up: get data of bootstrapped years
       if(doground == 1) then
         wyear = ghogyear
       else
         if(jahr.lt.1971) then
           wyear = spinupYearsFH(472-(1971-jahr))
         !else if(jahr==1970) then  ! Defined 470 instead of 471 years - this is a quick fix before next compiling
         !  wyear = 1987
   !      else if(jahr.gt.2015) then
   !      ! Idealized climate runs: bootstrapped years from period 1981 to 2015 (without 2105), with linear
   !      ! increase of temperature delta until 2050, then constant delta.
   !      ! Hard-coded for now
   !        wyear = futureYearsFH(jahr-2015)
   !        deltemp = deltaT * min(1.0,(jahr-2015.0)/35.0)
   !        delprec = real(deltaP) * min(1.0,(jahr-2015.0)/35.0) * 0.01
         else
           wyear=jahr
         end if
       end if  ! doground or not

	sa='.a'
	if (stp .eq. 1) sa='.s'
	if (stp .eq. 6) sa='.s'

      SPD=INT(1440/dtminVar)

      if (spd .eq. 1) then
        if((rank==root).or.(doParallel==.false.)) then
	   write(rdir,'(I4.4,2(I2.2),A1)') wyear,Mon,tag,'//'
          write(rnam,'(I4.4,2I2.2,A2)') wyear,Mon,tag,'24'
	   write(odir,'(I4.4,A1,I4.4,2(I2.2),A1)') wyear,
     *    '//',wyear,Mon,tag,'//'
          write(onam,'(I4.4,2I2.2)') wyear,Mon,tag
	   rfile=diroutput(1:pos2)//trim(rnam)//'.'//filenrVar(1:pos)

500       continue
	   orifile=dirmeteo(1:posi1)//trim(odir)//filenrVar(1:pos)//
     *    trim(onam)//'.2km'
          INQUIRE (file=trim(orifile),EXIST=whatclim)
          if (.not. whatclim) then
	     orifile=dirmeteo(1:posi1)//trim(odir)//filenrVar(1:pos)//
     *      trim(onam)//'.grd'
            print*, "orifile:", orifile
            INQUIRE (file=trim(orifile),EXIST=whatclim)
            if (.not. whatclim) then
              print*, 'No Meteo file for date: ', wyear, mon, tag
              call sleep (10)
              tryagain=tryagain+1
              if (tryagain .eq. 10) stop
              goto 500
            end if
          end if
        end if  ! End of Master part

      call MPI_BCAST(orifile,len(orifile),MPI_CHARACTER,root,
     * MPI_COMM_WORLD,ierr)

1098   continue


       call GETGRIDS (orifile,filenrVar)

       ncol=bpcol
       nrow=bprow
       rnodat=bnodat

        if((rank.ne.root).or.(doParallel==.false.)) then

! SUNSHINE DURATION CONVERSION Transient scenarios
       if (stp .eq. 6) then
       !call SZEROMAX (tag,Mon,Jahr,rtbreite,ssdzeromax)
       call SZEROMAX (tag,Mon,wyear,geobreite,ssdzeromax)
       if (Jahr .ge. 1981) then
       if (fixssd .eq. 1) then
       do ycoor=lastrow,firstrow,-1
	do xcoor=1,ncol
       if (int(out2d(xcoor,ycoor)) .eq. int(rnodat)) then
	  cycle
	else
     	  out2d(xcoor,ycoor)=out2d(xcoor,ycoor)*ssdzeromax
	end if
 	end do
	END DO
       gstats(2)=gstats(2)*ssdzeromax
       gstats(3)=gstats(3)*ssdzeromax
       gstats(5)=gstats(5)*ssdzeromax
       gstats(6)=gstats(6)*ssdzeromax
       end if
       end if
       end if
! #######################################

          DAYVAL=out2d
          deallocate(out2d)

! Distribution of precipitation (daily sums) on 24 hours:
! If variability is high -> event is probably a thunderstorm -> probably taking place in the evening
! If variability is low -> probably a large-scale precipitation event

          if (wstats(5) .ne. 0) then
            vak=wstats(6)/wstats(5)  ! Coefficient of variation
          else
            vak=0.0
          end if


          hp1=1
          hp2=24
          hpdiv=24.

          tolerance = 0.001

          if (stp .eq. 1) then
            !if (vak .gt. 0.1) then
            if(fp_gt(vak,0.1,tolerance)) then
C            print*, "vak1", vak, wstats(6),wstats(5), rank
              hp1=4
              hp2=21
              hpdiv=18.
            end if

            !if (vak .gt. 0.25) then
            if(fp_gt(vak,0.25,tolerance)) then
C          print*, "vak2", vak, wstats(6),wstats(5), rank

              hp1=8
              hp2=19
              hpdiv=12.
            end if

            !if (vak .gt. 0.5) then
            if(fp_gt(vak,0.5,tolerance)) then
C          print*, "vak3", vak, wstats(6),wstats(5), rank

              hp1=15
              hp2=20
              hpdiv=6.
           end if

            !if (vak .gt. 1.0) then
            if(fp_gt(vak,1.0,tolerance)) then
C          print*, "vak4", vak, wstats(6),wstats(5), rank

              hp1=18
              hp2=20
              hpdiv=3.
            end if


          end if

          ! Climate runs - set temperature delta and precipitation reduction/augmentation
C         if(jahr.gt.2015.and.stp.eq.2) then
C           where(int(dayval).ne.int(rnodat)) dayval = dayval + deltemp
C         else if(jahr.gt.2015.and.stp.eq.1) then
C           where(int(dayval).ne.int(rnodat)) dayval = dayval +
C    * (dayval * delprec)
C         end if


          if (stp .gt. 1) then
            do loi=1,24
              do ycoor=lastRow,firstRow,-1
	       do xcoor=1,ncol
                hourval(loi,xcoor,ycoor)=0.
                if (int(dayval(xcoor,ycoor)) .eq. int(rnodat)) then
                  hourval(loi,xcoor,ycoor)=rnodat
	           cycle
	         else
     	           hourval(loi,xcoor,ycoor)=DAYVAL(xcoor,ycoor)
	           if (stp .eq. 6) then
                    hourval(loi,xcoor,ycoor)=hourval
     *              (loi,xcoor,ycoor)/24.
	           end if
	         end if  ! If nodat ... else ...
	       end do
	       end do  !Loop over cells
            end do  ! Loop over 24 hours
          else   ! Stp == 1 (Reading precipitation)
	     do loi=1,24
              do ycoor=lastRow,firstRow,-1
	       do xcoor=1,ncol
                hourval(loi,xcoor,ycoor)=0.
                if (int(dayval(xcoor,ycoor)) .eq. int(rnodat)) then
                  hourval(loi,xcoor,ycoor)=rnodat
	           cycle
	         else
                  hourval(loi,xcoor,ycoor)=DAYVAL(xcoor,ycoor)/hpdiv
                  if (loi .lt. hp1) hourval(loi,xcoor,ycoor)=0.0
                  if (loi .gt. hp2) hourval(loi,xcoor,ycoor)=0.0
	         end if
 	       end do
              end do
	     end do
          end if
	 end if  ! End of slaves part
      end if

112   CONTINUE

      RETURN

1999  CONTINUE


! ---------------------------------------------------------------------

      print*, 'File Not Found ',trim(rfile)

      STOP

      RETURN
      END
