!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2003 Joachim Gurtz, Massimiliano Zappa
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Adaptation for parallel execution

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

! ------------------------------------------------------------------------------
! Gridmeteo

      module gridmeteo
      real bpcol,bprow,bpxll,bpyll,bpsize,bnodat
      real lxll,lyll
      real wt
      real gstats(6)
      real out2D[ALLOCATABLE](:,:)
      real ww2D[ALLOCATABLE](:,:,:)
      integer id2D[ALLOCATABLE](:,:,:)
      integer mm5id[ALLOCATABLE](:)
      real mm5xx[ALLOCATABLE](:)
      real mm5yy[ALLOCATABLE](:)
      real mm5hh[ALLOCATABLE](:)
      real mm5data[ALLOCATABLE](:)
	end module
! ------------------------------------------------------------------------------
! ------------------------------
      SUBROUTINE SETGRIDMETEO
! ------------------------------
!     Created PREVAH GRIDS
!     Optimized for CCHYDRO setup
!
!    V.1.0 M_Zappa, WSL, April 2011
! ------------------------------

      use mpi
       use gridmeteo
       use prevahglobals
      use MPIVars

      implicit none

!      include 'mpif.h'

      integer nmm5,distflag(4),tnr,le,ccy,nprg
      character idwnam*256,rfile*256,ref*256,dij*1
      character*256 gpath
      real mindist(99),ldist,ws(4)
      logical exists,lres

	CALL POSNAM (NADGKW,1,POSI1)
       gpath=NADGKW(1:POSI1)

!	lbas=1
83    continue

! Reading co-ordinates of the radar points
      if((rank.ne.root).or.(doParallel==.false.)) then
	 open (77,file=trim(gpath)//'-grid2grid.xyz',status='old',err=84)   ! Name so far hard-coded

       nmm5=0 !nrow*ncol

88    continue
      read (77,*,end=89) ds
	nmm5=nmm5+1
	goto 88
89    continue

      allocate (mm5xx(nmm5))
      allocate (mm5yy(nmm5))
      allocate (mm5data(nmm5))
      allocate (mm5hh(nmm5))


      rewind(77)

      do i=1,nmm5
      read (77,*) mm5xx(i),mm5yy(i),mm5hh(i)
	end do

      close (77)

        if(doParallel==.false.) then
          nrowLocal=nrow
          lastRow=nrow
          firstRow=1
        end if
	ALLOCATE (ID2D(ncol,nrow,4))
	ALLOCATE (WW2D(ncol,nrow,4))

      end if

! --------------------------------------------------------------
! Looking for file with Downscaling weights
! --------------------------------------------------------------

      INQUIRE (FILE =trim(adjustl(gpath))//
     *  '.rgrid1', EXIST = exists)

	if (.not. exists) then  ! Parallel execution not tested yet!
! --------------------------------------------------------------
! RID File not found ...
! Computing Downscaling weights (nearesth neighbour)
! --------------------------------------------------------------
      if((rank==root).or.(doParallel==.false.)) then
         call messagerun (1,'RID NOT FOUND, computing weights')
      end if

      if((rank.ne.root).or.(doParallel==.false.)) then

      do ycoor=lastRow,firstRow,-1
	 do xcoor=1,ncol
	    mindist=99999.9
	    distflag=0

	    if (mezgrid(xcoor,ycoor) .eq. rnodat) cycle

	    lxll=RXLL+rgridsize*float(xcoor-1)
	    lyll=RYLL+rgridsize*float(ycoor-1)

! Searching the four nearest RADAR Points fore each Grid-point
! --------------------------------------------------------------
          ccy=0
	    do i=1,nmm5

          if (ccy .gt. 0) then
	    ccy=ccy-1
	    cycle
	    end if

	    ldist=((lxll-mm5xx(i))**2+(lyll-mm5yy(i))**2)**0.5
	    ldist=max(ldist,rgridsize/10.)

	    ccy=int(ldist/10000.)-1
	    ccy=max(ccy,0)

	    if (ldist .lt. mindist(1))then
	    do j=4,2,-1
          mindist(j)=mindist(j-1)
	    distflag(j)=distflag(j-1)
	    end do
          mindist(1)=ldist
	    distflag(1)=i
      	cycle
	    else if (ldist .lt. mindist(2))then
	    do j=4,3,-1
          mindist(j)=mindist(j-1)
	    distflag(j)=distflag(j-1)
	    end do
          mindist(2)=ldist
	    distflag(2)=i
      	cycle
	    else if (ldist .lt. mindist(3))then
	    j=4
          mindist(j)=mindist(j-1)
	    distflag(j)=distflag(j-1)
          mindist(3)=ldist
	    distflag(3)=i
      	cycle
	    else if (ldist .lt. mindist(4))then
          mindist(4)=ldist
	    distflag(4)=i
      	cycle
	    end if
	    end do

! computing weights for the four points (IDW-procedure)
! --------------------------------------------------------------
	    wt=0
	    do j=1,4
	    ws(j)=1/(mindist(j)/1000)**2.
	    wt=wt+ws(j)
	    end do

	    do j=1,4
          ws(j)=ws(j)/wt
	    ww2D(xcoor,ycoor,j)=ws(j)
	    id2d(xcoor,ycoor,j)=distflag(j)

          end do


	    end do
      end do

! --------------------------------------------------------------
! Writing file with Downscaling weights for later use
! --------------------------------------------------------------

      do j=1,4
	write(dij,'(I1)') j

      if (.not. allocated(write2d)) ALLOCATE (WRITE2D(ncol,nrow))

	do ycoor=firstRow,lastRow
	do xcoor=1,ncol
         !ycoorLocal=ycoor-(firstRow-1)
        if (int(mezgrid(xcoor,ycoor)) .ne. int(rnodat)) then
		write2D(xcoor,ycoor)=id2D(xcoor,ycoor,j)
	  else
	    write2D(xcoor,ycoor)=rnodat
	  endif
	end do
	end do

        open (78,file=trim(adjustl(gpath))//
     *   '.rgrid'//dij,status='unknown',form='BINARY')
          uwas=78
		include 'wstsr.inc'

	deallocate (write2d)

      if (.not. allocated(write2d)) ALLOCATE (WRITE2D(ncol,nrow))

	do ycoor=1,nrow
	do xcoor=1,ncol
        if (int(mezgrid(xcoor,ycoor)) .ne. int(rnodat)) then
		write2D(xcoor,ycoor)=ww2D(xcoor,ycoor,j)
	  else
	    write2D(xcoor,ycoor)=rnodat
	  endif
	end do
	end do

        open (78,file=trim(adjustl(gpath))//
     *  '.wgrid'//dij,status='unknown',form='BINARY')
          uwas=78
		include 'wstsr.inc'

	deallocate (write2d)

	end do

      end if  ! End of slaves part, Matthias

      call messagerun (1,'DONE RID')

      else
! --------------------------------------------------------------
! RID File found
! Reading WEIGTS, Array allocation
! --------------------------------------------------------------

        do j=1,4
	   write(dij,'(I1)') j

          if((rank.ne.root).or.(doParallel==.false.)) then
	     open (78,file=trim(adjustl(gpath))//
     *      '.rgrid'//dij,status='old',form='BINARY',
     *      access='sequential')

            uwas=78
          end if

          if((rank==root).or.(doParallel==.false.)) then
            call messagerun (1,'READING RID')
          end if

          if((rank.ne.root).or.(doParallel==.false.)) then

            if (allocated(write2d)) DEALLOCATE (WRITE2D)
	     include 'rwasim.inc'
	     include 'rw2Dr.inc'
	     do ycoor=firstRow,lastRow
	     do xcoor=1,ncol
              id2D(xcoor,ycoor,j)=write2D(xcoor,ycoor)
	     end do
	     end do

            DEALLOCATE(write2D)

	     open (78,file=trim(adjustl(gpath))//
     *  '.wgrid'//dij,status='old',form='BINARY',
     *  access='sequential')

            uwas=78

            if (allocated(write2d)) DEALLOCATE (WRITE2D)
	     include 'rwasim.inc'
	     include 'rw2Dr.inc'

	     do ycoor=firstRow,lastRow
	     do xcoor=1,ncol
	       ww2D(xcoor,ycoor,j)=write2D(xcoor,ycoor)
	     end do
	     end do

            DEALLOCATE(write2D)
	   end if
        end do

        call MPI_BARRIER(MPI_COMM_WORLD,ierr)
        if((rank==root).or.(doParallel==.false.)) then
          call messagerun (1,'DONE READING RID')
        end if
      end if

1044  continue

       return

84    continue
       STOP

! Bye bye
      end


! ------------------------------
      SUBROUTINE GETGRIDS (cnam,cvar)
! ------------------------------
!     Created PREVAH Tables from
!     MeteoSwiss radar data
!     Optimized for D-PHASE setup
!
!    V.1.0 M_Zappa, WSL, March 2007
!    V.2.0 M_Zappa, WSL, June 2008
! ------------------------------

       use gridmeteo
       use prevahglobals
      use MPIvars
      use mpi
      use fp_ops

      implicit none

      integer distflag(4),oo,le
	real llaps
      character*256 cnam
      character*256 cvar
      integer iicol,iirow
      real ws(4)
      real dhm2D[ALLOCATABLE](:,:)
      real rwrite2D[ALLOCATABLE](:,:)

      integer:: len
      real :: locSumSq(numslaves) ! TEST
  !    type(esum) :: locSum, globSum  ! Kahan summation
      real(kind=dblP), dimension(2) :: locSumArray, globSumArray
      real :: locSumr, globSumr
      real, dimension(ncol,nrow) :: squares
      real, allocatable :: reshVec(:)  ! Vector containing reshaped array
      real:: ssq

! Only for assessing difference between global sums with a different
! numbers of processors
	real:: dummysl1, dummysl2, dummysg1, dummysg2
      real:: dummyal1, dummyal2, dummyag1, dummyag2
	real:: dummysql1, dummysql2, dummysqg1, dummysqg2
	logical :: dumExist


      llaps=-0.0065  ! Constant lapse rate for temperature
      le=0
       bpcol=rcol
       bprow=rrow
       bpxll=rxll
       bpyll=ryll
       bpsize=rgridsize
       bnodat=rnodat

66    continue
      if((rank.ne.root).or.(doParallel==.false.)) then
        open (50, file=cnam,status='old',form='BINARY',
     *  access='sequential',err=67)
        goto 68
67      continue
        le=le+1
        call sleep (5)
        if (le .lt. 10) goto 66
        STOP
68      continue

	uwas=50
	include 'rwasim.inc'
! reads wasim grid


       ALLOCATE (WRITE2D(ncol,nrow))
       do ycoor=nrow,1,-1
          read (uwas) (write2D(xcoor,ycoor),xcoor=1,ncol)
       end do

       close (uwas)

! Cropping the global grid
!---------------------------------------

999   continue

!      check for NaNs, replace them with no data value
!	(usually -9999)

      where(isnan(write2d)) write2d = rnodat

!       ALLOCATE (RWRITE2D(ncol,nrow))

!       rwrite2D=write2d
!       deallocate (write2D)

       iirow=nrow
       iicol=ncol

1050  continue

1998  continue

! --------------------------------------------------------------
! Gridded output might be activated here
! --------------------------------------------------------------

600   continue


! --------------------------------------------------------------
! Reading the Meteorological Zones, MEZ
! --------------------------------------------------------------
       le=0
       rcol=bpcol
       rrow=bprow
       rxll=bpxll
       ryll=bpyll
       rgridsize=bpsize
       rnodat=bnodat

       ncol=int(rcol)
       nrow=int(rrow)
       nxll=int(rxll)
       nyll=int(ryll)

       ngridsize=int(rgridsize)
       nodat=int(rnodat)

	ALLOCATE (OUT2D(ncol,nrow))
	ALLOCATE (dhm2D(ncol,nrow))
	DHM2D=FHOEHE
	OUT2D=rnodat

! --------------------------------------------------------------
! Downscaling
! --------------------------------------------------------------


1897   continue
      i=0
	do ycoor=iirow,1,-1
	do xcoor=1,iicol
	i=i+1
	mm5data(i)=write2D(xcoor,ycoor)
	end do
	end do
!	deallocate (rwrite2d)

! Downscaling according to weights
      out2D=rnodat
      do ycoor=lastRow,firstRow,-1
        do xcoor=1,ncol
          if (mezgrid(xcoor,ycoor) .eq. rnodat) cycle

          do j=1,4
	     ws(j)=ww2D(xcoor,ycoor,j)
	     distflag(j)=id2D(xcoor,ycoor,j)
	   end do

	select case(cvar(1:4))

	  case('temp')

      	    out2D(xcoor,ycoor)=ws(1)*(mm5data(distflag(1))-
     *      llaps*(mm5hh(distflag(1))))+ws(2)*(mm5data(distflag(2))-
     *      llaps*(mm5hh(distflag(2))))+ws(3)*(mm5data(distflag(3))-
     *      llaps*(mm5hh(distflag(3))))+ws(4)*(mm5data(distflag(4))-
     *      llaps*(mm5hh(distflag(4))))
		 out2D(xcoor,ycoor)=out2D(xcoor,ycoor)+
     *      llaps*dhm2D(xcoor,ycoor)

	  case('prec')

      	    out2D(xcoor,ycoor)=ws(1)*mm5data(distflag(1))+
     *      ws(2)*mm5data(distflag(2))+
     *      ws(3)*mm5data(distflag(3))+
     *      ws(4)*mm5data(distflag(4))
           out2D(xcoor,ycoor)=max(0.,out2D(xcoor,ycoor))

	  case default

      	    out2D(xcoor,ycoor)=ws(1)*mm5data(distflag(1))+
     *      ws(2)*mm5data(distflag(2))+
     *      ws(3)*mm5data(distflag(3))+
     *      ws(4)*mm5data(distflag(4))

	end select

      end do
      end do

! Dummy stats, only for uncertainty assessment

C      print*, firstRow, lastRow, rank

      dummysl1 = 0.
      dummysl2 = 0.

      do ycoor=lastRow,firstRow,-1
        do xcoor=1,ncol
        dummysl1 = dummysl1 + out2d(xcoor,ycoor)
        end do
      end do

      dummysl2 = sum(out2d(:,firstRow:lastRow))

      len = count(out2d(:,firstRow:lastRow) .ne. rnodat)


      dummyal1 = dummysl1/len
      dummyal2 = dummysl2/len
C
C      print*, "dummysl1, 2:", dummysl1, dummysl2, rank
C      print*, "dummyal1, 2:", dummyal1, dummyal2, rank
C
      call MPI_ALLREDUCE(dummysl1,dummysg1,1,MPI_REAL,MPI_SUM,slaves,
     * ierr)
      call MPI_ALLREDUCE(dummysl2,dummysg2,1,MPI_REAL,MPI_SUM,slaves,
     * ierr)
C      call MPI_ALLREDUCE(dummyal1,dummyag1,1,MPI_REAL,MPI_SUM,slaves,
C     * ierr)
C      call MPI_ALLREDUCE(dummyal2,dummyag2,1,MPI_REAL,MPI_SUM,slaves,
C     * ierr)

      dummyag1 = dummysg1/(ncol*nrow)
      dummyag2 = dummysg2/(ncol*nrow)
C
C      print*, "dummysg1, 2:", dummysg1, dummysg2, rank
C      print*, "dummyag1, 2:", dummyag1, dummyag2, rank

      len = count(out2d(:,firstRow:lastRow) .ne. rnodat)

      gstats = 0
      gstats(1) = count(out2d .ne. rnodat)
      gstats(2) = minval(out2d, mask=out2d .ne. rnodat)
      gstats(3) = maxval(out2d, mask=out2d .ne. rnodat)
      !gstats(4) = sum(out2d, mask=out2d .ne. rnodat)
!      gstats(4) = fp_sum(reshape(out2d(:,firstRow:lastRow),(/len/)),len,
!     * 7, rnodat)

      allocate(reshVec(len))
      reshVec = reshape(out2d(:,firstRow:lastRow),(/len/))
      globSumr = kahan_sum(reshVec,len,rnodat)
      if(allocated(reshVec)) deallocate(reshVec)

      !gstats(5) = gstats(4)/gstats(1)

      !print*, "gstats(4)", gstats(4), rank


	  !print*, "gstats", gstats, rank

      wstats=gstats  ! Statistics for each part only - need to call MPIReduceStats below

      end if  ! End of slaves part

      if (allocated(dhm2d)) DEALLOCATE (dhm2D)
      if (allocated(rwrite2d)) DEALLOCATE (rwrite2D)
      if (allocated(write2d)) DEALLOCATE (write2D)

      if(rank==root) wstats=0

      if(rank.ne.root) then

       call MPIReduceStats(wstats)  ! Only for count, min, max

       !locSumArray(1) = locSum%sum
       !locSumArray(2) = locSum%cor

C       call MPI_ALLREDUCE(locSumArray,globSumArray,1,MPItwoDbl,kahanOP,
C     *  slaves,ierr)

      wstats(4) = globSumr
      wstats(5) = wstats(4)/wstats(1)

       ! Calculate sum of squares against reduced mean for each stripe
C        do ycoor=lastRow,firstRow,-1
C	    do xcoor=1,ncol
C          if (int(out2D(xcoor,ycoor)) .ne. int(nodat)) then
C		     gstats(6)=gstats(6)+(out2D(xcoor,ycoor)-wstats(5))**2
C	      endif
C	    end do
C	    end do

	    squares = rnodat

	    dummysql1 = 0.

	   do ycoor=lastRow,firstRow,-1
	    do xcoor=1,ncol
	    if (int(out2D(xcoor,ycoor)) .ne. int(nodat)) then
	      squares(xcoor,ycoor) = (out2d(xcoor,ycoor)-wstats(5))**2
	      dummysql1 = dummysql1 + squares(xcoor,ycoor)
	    end if
	    end do
	    end do

	    dummysql2 = sum(squares(:,firstRow:lastRow))

	    call MPI_ALLREDUCE(dummysql1,dummysqg1,1,MPI_REAL, MPI_SUM,
     *  slaves,ierr)
     	    call MPI_ALLREDUCE(dummysql2,dummysqg2,1,MPI_REAL, MPI_SUM,
     *  slaves,ierr)

      allocate(reshVec(len))
	reshVec = reshape(squares(:,firstRow:lastRow),(/len/))
	   ssq=kahan_sum(reshVec,len,rnodat)
	if(allocated(reshVec)) deallocate(reshVec)

	    ! Calculate standard deviation for the whole grid

C	    call MPI_ALLREDUCE(gstats(6),wstats(6),1,MPI_REAL,MPI_SUM,
C     *  slaves,ierr)
C
C      print*, "gstats(6)", gstats(6), rank
C
C      call MPI_ALLGATHER(gstats(6), 1, MPI_REAL, locSumSq, 1,
C     * MPI_REAL, slaves, ierr)
C
C      !wstats(6) = fp_sum(locSumSq,numslaves,7,rnodat)
C      locSumr = local_kahan_sum(locSumSq,numslaves,rnodat)
C      wstats(6) = locSumr

	   wstats(6) = ssq
	    gstats(6)=0

	    if (wstats(1) .gt. 1) then
		  wstats(6)=(wstats(6)/(wstats(1)-1))**(0.5)
		  dummysqg1=(dummysqg1/(wstats(1)-1))**(0.5)
		  dummysqg2=(dummysqg2/(wstats(1)-1))**(0.5)

	    else
		  wstats(6)=0.0
	    end if

C	    print*, "dummysqg1, 2", dummysqg1, dummysqg2

		!print*, 'wstats ', cvar(1:4),wstats, rank

C       ! Writing dummy results to file
C        if(rank == 1) then
C89     format(I5, I3, I3, A5, 9F9.4)
C       inquire(file = 'dummy.txt', exist = dumExist)
C       if(dumExist) then
C         open(2345,file='dummy.txt',status="old", position="append")
C       else
C         open(2345,file='dummy.txt',status='new')
C       end if
C       write(2345,89) jahr, mon, tag, cvar(1:4), dummysg1, dummyag1,
C     * dummysqg1, dummysg2, dummyag2, dummysqg2, wstats(4), wstats(5),
C     * wstats(6)
C       !print*, "wstats", rank, wstats
C       end if  ! rank == 1
C


      end if  ! End of slaves part
      RETURN

1044  continue

! Bye bye
      end
