!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

	SUBROUTINE GZERO (TAG,MONAT,JAHR,phi,P,RG)

! computes the potential incoming radiation
! similar to ergkorz and rtkorz
! Code provided by L.Menzel (PIK Potsdam)
! ------------------------------------------------------------------------------

      implicit none

! ------------------------------------------------------------------------------
! Variables
! ------------------------------------------------------------------------------

	INTEGER TAG,MONAT,JAHR,ihg,TAGNR,nz,k
	REAL phi,P
	REAL dekli,psi,rad,grad
	REAL x,y,z,hs,hsr,hss,tagl,dau
	REAL uhr(50),hsuhr(50),coZ(50),RGmax(50)
	REAL RGm,RG,rhg
	PARAMETER (rad=0.01745,grad=57.29578)
	CHARACTER*1 SJ

! ------------------------------------------------------------------------------
! Begin
! ------------------------------------------------------------------------------
	SJ='n'

	RHG=JAHR/4.0

      IHG=INT(RHG+0.01)

      IF (RHG.GT.IHG) SJ='Y'

! Cmputes the day number
	CALL TAGZAHL (TAG,MONAT,TAGNR,SJ)

	RGm=0.
	RG=0.

	z=COS(rad*360.*(float(TAGNR+10))/365.)
	dekli=-23.4*z

	psi=0.88+(0.05*z)

	x=SIN(rad*phi)*SIN(rad*dekli)
	y=COS(rad*phi)*COS(rad*dekli)
	hs=grad*ACOS(x/y)

	hsr=hs*3.8197/60.
	hss=24-hsr
	tagl=hss-hsr
	nz=INT(tagl+1)
	dau=tagl/nz

	DO 10 K=1,nz
	uhr(K)=hsr+K*dau
	hsuhr(K)=15*(12-uhr(K))
	coZ(K)=x+y*COS(rad*hsuhr(K))

	IF (grad*ACOS(coZ(K)) .GE. 90.) THEN
	   coZ(K)=0.001
	   RGmax(K)=0.
	ELSE
	   RGmax(K)=psi**((P/1013.)/coZ(K))
	   RGmax(K)=RGmax(K)*1367*coZ(K)*dau
	ENDIF

	IF (K .EQ. 1) THEN
	   RGm=RGmax(K)
	ELSE
	   RGm=RGmax(K)+RGm
	ENDIF
10	CONTINUE

	RG=RGM/24.
! ------------------------------------------------------------------------------
	RETURN
	END
! ------------------------------------------------------------------------------
