! Contains various helper functions for general tasks such as sorting, 
! calculating median values, etc.

! Code copied and adapted from external sources.
!
! Median: http://www.cs.mtu.edu/~shene/COURSES/cs201/NOTES/chap08/median.f90
! Quicksort: http://www.fortran.com/qsort_c.f95

module help

implicit none
public :: QsortC
private :: Partition

contains

!=======================================================================

! Median value of an array

real function median(A)
  real, dimension(:) :: A
  integer :: len
  
  call QsortC(A)
  
  len = size(A)
  IF (MOD(len,2) == 0) THEN           ! compute the median
    Median = (A(len/2) + A(len/2+1)) / 2.0
  ELSE
    Median = A(len/2+1)
  END IF

end function median

!=======================================================================

! Quicksort

recursive subroutine QsortC(A)
  real, intent(in out), dimension(:) :: A
  integer :: iq

  if(size(A) > 1) then
     call Partition(A, iq)
     call QsortC(A(:iq-1))
     call QsortC(A(iq:))
  endif
end subroutine QsortC

subroutine Partition(A, marker)
  real, intent(in out), dimension(:) :: A
  integer, intent(out) :: marker
  integer :: i, j
  real :: temp
  real :: x      ! pivot point
  x = A(1)
  i= 0
  j= size(A) + 1

  do
     j = j-1
     do
!     print*, "size(A), j, A(j), x", size(A), j, A(j), x
        if (A(j) <= x) exit
        j = j-1
     end do
     i = i+1
     do
        if (A(i) >= x) exit
        i = i+1
     end do
     if (i < j) then
        ! exchange A(i) and A(j)
        temp = A(i)
        A(i) = A(j)
        A(j) = temp
     elseif (i == j) then
        marker = i+1
        return
     else
        marker = i
        return
     endif
  end do

end subroutine Partition

!=======================================================================


end module help