!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE IEVAPARZ (IART,ZEIPM,CKURZ,KURZWSF,CLANG,LANGWSF,
     *                    VFEUCHTF,CVENT,ZLAND,ZBTYP,ZKWERT,ZEIKE,
     *                    MBI)
!
!    Reads from the control file the parameters of the evapotranspiration
!    module
!    Literature:
!    Gurtz et al., Hydrological processes (2003)
!    Zappa et al., Nordin Hydrology (2003)
!    Zappa and Gurtz, HESS (2003)
! ---------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------

      INTEGER L,IART,MBI,LANGWSF,KURZWSF,VFEUCHTF
      CHARACTER *56 ZEIKE,ZEIPM(6)
      CHARACTER*31 CKURZ(17),CLANG(10),CVENT(3)
      CHARACTER*40 ZLAND(30),ZBTYP(3),ZKWERT(3)

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------

! Definition of strings
! Estimation of short-wave radiation
      ckurz(1)=' Penman 1948 (S)            = 0'
      ckurz(2)=' Penman 1954 (S)            = 1'
      ckurz(3)=' Golchert 1981 (S)          = 2'
      ckurz(4)=' Karsten & Czeplak 1980 (N) = 3'
      ckurz(5)=' MORECS 1981 (S)            = 4'
      ckurz(6)=' Observed global radiation  = 5'
      ckurz(7)=' Schulla (S)                = 6'
      ckurz(12)=' Golchert Hamburg           =11'
      ckurz(13)=' Golchert Braunschweig      =12'
      ckurz(14)=' Golchert Trier             =13'
      ckurz(15)=' Golchert Wuerzburg         =14'
      ckurz(16)=' Golchert Hohenpeissenberg  =15'
      ckurz(17)='                               '

! Estimation of long-wave radiation
      clang(1)=' Penman 1948 (S)            = 0'
      clang(2)=' Penman 1954 (S)            = 1'
      clang(3)=' Angstroem 1923 (S)         = 2'
      clang(4)=' Angstroem 1923 (N)         = 3'
      clang(5)=' Feussner (S)               = 4'
      clang(6)=' Feussner (N)               = 5'
      clang(7)=' Czeplak (S)                = 6'
      clang(8)=' Czeplak (N)                = 7'
      clang(9)=' Brutsaert (S)              = 8'
      clang(10)= 'Schulla   (S)              = 9'

! Aerodynamic term in Penman equation
      cvent(1)=' Penman 1948                = 0'
      cvent(2)=' Penman 1954                = 1'
      cvent(3)=' Petznik 1988               = 2 '

      zeipm(1)=' Potential evapotranspiration after Penman    (iart=0) '
      zeipm(2)=' Pot. evapotranspiration after Penman-Monteith(iart=1) '
      zeipm(3)=' Act. evapotranspiration after Penman-Monteith(iart=2) '
      zeipm(4)=' Potential evapotranspiration after Hamon     (iart=3) '
      zeipm(5)=' Potential evapotranspiration after Turc      (iart=4) '
      zeipm(6)=' Potential evapotranspiration after Wendling  (iart=5) '

! ---------------------------------------------------------------------
! Land use classes
      zland(1)= ' Water                                 1'
      zland(2)= ' Built-ip                              2'
      zland(3)= ' Coniferous-forests                    3'
      zland(4)= ' Deciduos forsts                       4'
      zland(5)= ' Mixed forests                         5'
      zland(6)= ' Cereals                               6'
      zland(7)= ' Pasture                               7'
      zland(8)= ' Bush                                  8'
      zland(9)= ' Free                                  9'
      zland(10)=' Free                                 10'
      zland(11)=' Free                                 11'
      zland(12)=' Free                                 12'
      zland(13)=' Firn                                 13'
      zland(14)=' Bare ice                             14'
      zland(15)=' Rock                                 15'
      zland(16)=' Free                                 16'
      zland(17)=' Free                                 17'
      zland(18)=' Fruits                               18'
      zland(19)=' Vegetables                           19'
      zland(20)=' Weheat                               20'
      zland(21)=' Alpine vegetation                    21'
      zland(22)=' Wetlands                             22'
      zland(23)=' Rough pasture                        23'
      zland(24)=' Sub-Alpine meadow                    24'
      zland(25)=' Alpine meadow                        25'
      zland(26)=' Bare soil vegetation                 26'
      zland(27)='                                        '
      zland(28)=' Corn                                   '
      zland(29)=' Grapes                                 '
      zland(30)='                                        '

! ---------------------------------------------------------------------
      zbtyp(1)= ' Soil with low field capacity          1'
      zbtyp(2)= ' Soil with middle field capacity       2'
      zbtyp(3)= ' Soil with high field capacity         3'

      zkwert(1)= ' Soil with low conductivity            1'
      zkwert(2)= ' Soil with moderate conductivity       2'
      zkwert(3)= ' Soil with high conductivity           3'

7     FORMAT (A56)
! ---------------------------------------------------------------------
! A string is read from the control file
      ZEIKE='----------------------------------------------------'

149   FORMAT ( /,' EINGABE Art der Berechnung der potentiellen' ,
     *          'Evapotranspiration: ')
      WRITE (MBI,149)
      write(MBI,*) ' /0/ Penman         '
      write(MBI,*) ' /1/ Penman-Monteith pot.'
      write(MBI,*) ' /2/ Penman-Monteith actual'
      write(MBI,*) ' /3/ Hamon pot.'
      write(MBI,*) ' /4/ Turch pot.'
      write(MBI,*) ' /5/ Wendling pot.'
      write(MBI,*) iart

! Settings for computation of evapotranspiration are read from the control file
150   FORMAT ( A31,4X,A31 )
151   FORMAT ( /,' Parameterization of short-wave radiation',
     *           '/0 - 6,11-15/ : ')
      WRITE (MBI,151)
      L=1
82    WRITE (MBI,150) ckurz(L),ckurz(L+1)
      L=L+2
      IF (L.LT.6) GOTO 82
      L=12
84    WRITE (MBI,150) ckurz(L),ckurz(L+1)
      L=L+2
      IF (L.LT.15) GOTO 84
      write(MBI,*) kurzwsf

153   FORMAT ( /, ' Parameterization of long-wave radiaiton',
     *           '/0 - 9/ : ')
      WRITE (MBI,153)
      L=1
86    WRITE (MBI,150) clang(L), clang(L+1)
      L=L+2
      IF (L.LT.9) GOTO 86
      write(MBI,*) langwsf

155   FORMAT ( /, ' Aerodynamic term for Penman /0 - 2/ : ' )
      WRITE (MBI,155)
      L=1
152   FORMAT ( A31 )
88    WRITE (MBI,152) cvent(L)
      L=L+1
      IF (L.LE.3) GOTO 88
      write(MBI,*) vfeuchtf

! ---------------------------------------------------------------------
      RETURN
      END
! ---------------------------------------------------------------------
