!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE IMET10MIN (TI,TAG,MON,JAHR,
     *                    ERRA,TAGE,MONE,JAHRE,
     *                    TENVAL)

!     This subroutine reads the incoming meteorological information
!     from up to 999 meteorological zones.
!     Hourly values are read. Daily sums or averages are computed
!--------------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------

      INTEGER TI,			! Time step
     *        TAG,			! Actual day
     *        MON,			! Actual month
     *        JAHR,			! Actual year
     *        NTG,			! Number of meteorological zones
     *        NDTD,			! number of time steps
     *        ERRA,			! controls the output of error messages
     *        MDATN,			! identifier of the file
     *        MDR,			! pointer to the log-file
     *        MBI,			! pointer to to prompt
     *        TAGE,			! end day
     *        MONE,			! end month
     *        JAHRE			! end year

! Hourly values
      REAL TENVAL(24,6)			! Ten Minute values

! ---------------------------------------------------------------------
!     Internal Variables
! ---------------------------------------------------------------------

      INTEGER TAGI,			! begin day
     *        MONI,			! begin month
     *        JAHRI,			! begin year
     *        JAR,			! help parameter for year
     *        J,K,L,I,p,r	    ! parameters for loops
      REAL TAIR,RHUMIT,DAD,DADM,RL 	! Variables for the conversion of water vapour into relative humidity
      REAL dummy			! if data not available
	! Read values
      REAL, ALLOCATABLE :: READVAL(:,:)
	INTEGER DTMIN,SPD

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------
      SPD=INT(1440/60)

	NDTD=24
	MDATN=1234

      ERRA=0
      L=6

      ALLOCATE (READVAL(SPD,L))

      DO J=1,SPD

70      CONTINUE

73    CONTINUE

      READ (MDATN,*) JAR,MONI,TAGI,TI,(READVAL(J,K),K=1,L)

      JAHRI=JAR
      IF (JAR .LT. 100) JAHRI=JAR+1900 ! millenium check

! Test if input data are complete and agree with the internal date
      IF ((TAGI.EQ.TAG).AND.(MONI.EQ.MON).AND.(JAHRI.EQ.JAHR)) GOTO 75

      IF (JAHRI.LT.JAHR) GOTO 72

      IF ((JAHR.EQ.JAHRI).AND.(MONI.LT.MON)) GOTO 73

      IF ((JAHR.EQ.JAHRI).AND.(MON.EQ.MONI).AND.(TAGI.LT.TAG)) GOTO 73

111   FORMAT (/,' ATTENTION the current year ',I5,
     *          ' doesnt agree',
     *        /,'                 with the input year',I5)
114   FORMAT (/,' ATTENTION some days are missing',
     *        /,'           Current date is',
     *          I2,'.',I2,'.',I4,I4,'. File'
     *        /,'           read data carry the date ',
     *          I2,'.',I2,'.',I4,/ )

      IF (JAHRI.GT.JAHRE) GOTO 74
      IF ((JAHRI.EQ.JAHRE).AND.(MON.GT.MONE)) GOTO 74

      IF ((JAHRI.EQ.JAHRE).AND.(MONI.EQ.MONE).AND.(TAGI.GT.TAGE))
     * THEN
          IF ((I.EQ.1) .and. (J.EQ.1)) THEN
             TAG=TAGI
             MON=MONI
             JAHR=JAHRI
             GOTO 75
          ELSE
             ERRA=1
             GOTO 1999
          ENDIF
      ENDIF
72      CONTINUE
      GOTO 118

74    CONTINUE
118   ERRA=1
      GOTO 1999

75    CONTINUE
      END DO

      DO J=1,6
      DO K=1,24
        TENVAL(K,J)=READVAL(K,J)
      END DO
	END DO

      DEALLOCATE(READVAL)
      GOTO 1999

200   CONTINUE


1999  CONTINUE

! ---------------------------------------------------------------------

      RETURN
      END
