!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Inclusion of FORHYCS variables

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


!     Initialization of gridded output

      gkwfile='remap.tmp'
      dpunkt=':'
      grdakt='Y'
      opnssm=0
      allinf=1.0
	  isice=.false.
	  subcamax=0
      QmesDat=0
	  upfac(24)=1
	  jnmeas=0

!     Shape coefficients for temperature
      shapecoeffs(1,1)=-0.70711
      shapecoeffs(1,2)=-0.86603
      shapecoeffs(1,3)=-0.96593
      shapecoeffs(1,4)=-1.00000
      shapecoeffs(1,5)=-0.96593
      shapecoeffs(1,6)=-0.86603
      shapecoeffs(1,7)=-0.70711
      shapecoeffs(1,8)=-0.50000
      shapecoeffs(1,9)=-0.25882
      shapecoeffs(1,10)=0.00000
      shapecoeffs(1,11)=0.25882
      shapecoeffs(1,12)=0.50000
      shapecoeffs(1,13)=0.70711
      shapecoeffs(1,14)=0.86603
      shapecoeffs(1,15)=0.96593
      shapecoeffs(1,16)=1.00000
      shapecoeffs(1,17)=0.96593
      shapecoeffs(1,18)=0.86603
      shapecoeffs(1,19)=0.70711
      shapecoeffs(1,20)=0.50000
      shapecoeffs(1,21)=0.25882
      shapecoeffs(1,22)=0.00000
      shapecoeffs(1,23)=-0.25882
      shapecoeffs(1,24)=-0.50000


812   FORMAT (2(F10.2,1X),A1,1X,F15.5)

      ergrid(1)='.P'
      ergrid(2)='.ETP'
      ergrid(3)='.ETR'
      ergrid(4)='.EI'
      ergrid(5)='.EB'
      ergrid(6)='.RGS'
      ergrid(7)='.R0'
      ergrid(8)='.R1'
      ergrid(9)='.R2'
      ergrid(10)='.GL0'
      ergrid(11)='.GL1'
      ergrid(12)='.SWA'
      ergrid(13)='.GWN'
      ergrid(14)='.SSO'
      ergrid(15)='.SI'
      ergrid(16)='.SSM'
      ergrid(17)='.SUZ'
      ergrid(18)='.SLZ'
      ergrid(19)='.SL1'
      ergrid(20)='.SND'
      ergrid(21)='.FCP'
	  ! Matthias: increased the length of WHGRID and ERGRID to write more files.
	  ! Control files not modified for now - new WHGRID lines here:
	ergrid(22)='.PES'
	ergrid(23)='.AES'
	ergrid(24)='.ESN'
	ergrid(25)='.SMX'


      DO I=1,21
        whgrid(I)='N-N-N-N'
      END DO

!     First initialization

	  do ii=1,72
         HSUZHM(ii)=0
         HSLZHM(ii)=0
	  end do

      ntg=1
      iyr=0
      imont=0
      iday=0
      ihr=0
      imin=0
      isec=0
      i100th=0
      igst=0

      nof=0
      idruck=0
      iart=0
      turn= '    '
      form4='    '
	  PRECMIN=0.5

      zdim5='mm/d    '
      zpar5='PET     '
      dtd=24.0
      slope=0.0
      aspect=0.0
      bfeucht=0.0
      hoehe=0.0
      geobreite=47.0
      btyp=0
	  rice3=0.0
      asec=1.0
      MTA=5
      MBI=6
      MDTAG=8
      MDMON=9
      MDR=42
      MLOG=48
      MPRI=MDR
      MIDAT=10
      MDSTD=11
      MDGKW=12
      MDMIT=41
      MDTMP=40
	  MDHOU=13
	  MDQ=14
      MSHDH=43
	  MHTML=44
	  MFCAST=45
	  MDBULL=46

      MISB=50		!JS-2008-04
      MISBQ=47		!JS-2008-04
      MIDH=49		!JS-2008-04
      MXML=51		!JS-2008-04

      PNDS=0.0
      PAKDS=0.0
      WSADS=0.0
      EPDS=0.0
      ERDS=0.0
      EIDS=0.0
      EBDS=0.0
      R0DS=0.0
      R1DS=0.0
      R2DS=0.0
      RGEDS=0.0
      GLACDS=0.0
	  FIRNDS=0
      GLACTT=0.0
      AEGRID=0.01
      SLIQ=0.0
      SSNOW=0.0
      SSO=0.0
      SI=0.0
      SSZ=0.0
      SUZ=0.0
      SLZ1=0.0
      SLZ2=0.0
      SLZ3=0.0
      GEBIL=0.0

      DO i=1,48

      	   z(i:i)=' '
      	ENDDO

      	DO i=1,32
      	    zn(i:i)=' '
      	    zg(i:i)=' '
            ENDDO

            DO i=1,8
      	    zpar(i:i)=' '
      	    zdim(i:i)=' '
      	    behoe(i:i)=' '
            ENDDO

            DO i=1,6
                  filen(i)='            '
                  mdatn(i)=1111+i
      ENDDO


	  MONUM(1)=31
      MONUM(2)=28
      MONUM(3)=31
      MONUM(4)=30
      MONUM(5)=31
      MONUM(6)=30
      MONUM(7)=31
      MONUM(8)=31
      MONUM(9)=30
      MONUM(10)=31
      MONUM(11)=30
      MONUM(12)=31
