!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE INTLOZ (PI,SIMAX,SI,VBG,ETPDT,EIDT)

!     This subroutine computes the filling and depletion of the
!     interception storages SI as function of the precipitation intensity PI
!     the maximal storages SIMAX and potential evapotranspiration ETPDT.
!     The filling is computed as proposed by Menzel (1997)
! ---------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------
      REAL PI,SIMAX,SI,VBG,ETPDT,EIDT


! ---------------------------------------------------------------------
!     Internal Variables
! ---------------------------------------------------------------------
      REAL SIMAXI,SIA,SIE,CI
      REAL PIA,PIS,PIAS,SIPDT

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------

      SIA=SI

! Maximal inteception storage is reduced as function of the degree of
! vegetated surface VBG [0..1]
      SIMAXI=SIMAX*VBG

      CI=VBG*0.8
      SIPDT=0.0
      PIA=PI

764   IF ((CI.LE.0.0) .AND. (SIA.LE.0.0))  GOTO 772
      IF (CI .LT. 0.0) GOTO 772


c     Filling after Menzel (1997) if SIMAXI > 0.1

      IF (SIMAXI .LT. 0.1) GOTO 767

      SI= SIA+(SIMAXI-SIA)*(1-exp(-1/SIMAXI*PI))

      IF (SI.GE.SIMAXI) GOTO 766

      PI=PI-(SIMAXI-SIA)*(1-exp(-1/SIMAXI*PI))

      GOTO 768

c     Linear filling if SIMAXI < 0.1
767   SI=SIA+(CI*PI)
      IF (SI.GE.SIMAXI) GOTO 766
      PI=PI-CI*PI

      GOTO 768

766   SI=SIMAXI
      PI=PIA-(SIMAXI-SIA)

!     SI is depleted by evapotranspiration at potential rate
768   IF (SI.LT.0.01) GOTO 772
      IF (SI.LT.(ETPDT*VBG)) GOTO 769
      EIDT=ETPDT*VBG
      GOTO 770

769   EIDT=SI

770   SIE=SI-EIDT
      GOTO 774

772   EIDT=0.0
      SIE=0.0
      PI=PIA

774   PIS=PI
      PIAS=PIA
      SIPDT=PIA-PI

! ---------------------------------------------------------------------
!     Update of SI
502   SI=SIE

! ---------------------------------------------------------------------
      RETURN

      END
