!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa, J. Schwanbeck

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

	SUBROUTINE intsubbasin (workdir)

! ---------------------------------------------------------------------
!
!
! # Jan Schwanbeck <jansc@gmx.net>
! # Hydrology Group / Dept. of Geography, University of Berne
!
! VER 0.1, Build 16.04.2008
!
!      USE				DFLIB
!	  USE				DFLOGM
      use ifport  ! Contains function definition of CHANGEDIRQQ
                  ! (Matthias - added this after "implicit none")
! ---------------------------------------------------------------------
!   Common Variables - prevahglobals.for
! ---------------------------------------------------------------------
	  USE prevahglobals
!	  character*1   	isbyn
!	  character*256		isbpath
!	  real, allocatable :: areaisb(:)

      implicit none
! ---------------------------------------------------------------------
!   Local Variables
! ---------------------------------------------------------------------
	  logical				lval
   	  integer				ji, ncolj, nrowj, nxllj, nyllj, rloopj, xcoorj,ycoorj,validcell
	  real					rcolj, rrowj, rxllj, ryllj, rgridsizej, rnodatj, wstatsj(6)
	  character*256			f1, f2, f3, f4, isbdir, isbfile, workdir
	  real, allocatable		:: isbmap(:,:), idhmap(:,:)
	  integer, allocatable	:: num(:)
	  integer jii   ! Was not declared before, caught by "implicit none"
	  real tmpj  ! Was not declared before, caught by "implicit none". NOT SURE ABOUT TYPE
! ---------------------------------------------------------------------
!   Begin
! ---------------------------------------------------------------------
!	  Call messagerun (1,'Output for internal sub basins.')

! MZ: File Check already resolved!

	   call MESSAGEFILE(isbpath)
	OPEN (MISB, FILE=TRIM(isbpath),STATUS='OLD',form='binary', ERR=996) !*.isb
	CALL POSNAM (IDNAM,1,POSI1)
	call MESSAGEFILE(idnam)
	OPEN (MIDH, file=IDNAM(1:POSI1), status='old', form='binary', ERR=997)  !*.idh
	! ------------------------------------------------------------------------------------
	! reading headers and checking if extent equal
      READ (MISB) rcolj
      READ (MIDH) tmpj

      IF (rcolj .NE. tmpj) GOTO 998

      READ (MISB) rrowj
      READ (MIDH) tmpj
      IF (rrowj .NE. tmpj) GOTO 998
      READ (MISB) rxllj
      READ (MIDH) tmpj
      IF (rxllj .NE. tmpj) GOTO 998
      READ (MISB) ryllj
	  READ (MIDH) tmpj
	  IF (ryllj .NE. tmpj) GOTO 998
      READ (MISB) rgridsizej
	  READ (MIDH) tmpj
	  IF (rgridsizej .NE. tmpj) GOTO 998
      READ (MISB) rnodatj
	  READ (MIDH) tmpj
	  IF (rnodatj .NE. tmpj) GOTO 998

      ncolj=INT(rcolj)		        !transforming real to integer
      nrowj=INT(rrowj)
      nxllj=INT(rxllj)
      nyllj=INT(ryllj)

      DO rloopj=1,6				!reading statistics
       READ (MISB) wstats(rloopj)
       READ (MIDH) wstats(rloopj)
      END DO
	! -----------------------------------------------------------------------------------
	! reading data from gridmaps
	  validcell=0
	   ALLOCATE (isbmap(ncolj,nrowj))
	   ALLOCATE (idhmap(ncolj,nrowj))
	   DO ycoorj=1,nrowj
        DO xcoorj=1,ncolj
		 READ (MISB) isbmap(xcoorj,ycoorj)
         READ (MIDH) idhmap(xcoorj,ycoorj)
		 IF (isbmap(xcoorj,ycoorj).NE.rnodatj) validcell=validcell+1
		ENDDO
	   ENDDO
	CLOSE(MISB)
    CLOSE(MIDH)
    ! -----------------------------------------------------------------------------------
    ! extract valid cells of both maps and writes them "parallel" into two arrays
	  validcells=validcell
	  ALLOCATE (isb(validcells))
	  ALLOCATE (idh(validcells))
	  validcell=0
	  DO ycoorj=1,nrowj
       DO xcoorj=1,ncolj
        IF (isbmap(xcoorj,ycoorj).NE.rnodatj) THEN
	     validcell = validcell+1
		 isb(validcell)=int(isbmap(xcoorj,ycoorj))
		 idh(validcell)=int(idhmap(xcoorj,ycoorj))
		 IF (idh(validcell).EQ.int(rnodatj)) GOTO 998
		ENDIF
	   ENDDO
	  ENDDO
   	  DEALLOCATE (isbmap)
	  DEALLOCATE (idhmap)

	  numisb = MAXVAL(isb)-MINVAL(isb)+1
    ! -----------------------------------------------------------------------------------
    ! Determines area of internal sub basins (proportion of AELHG)

	  ALLOCATE(areaisb(numisb))
       DO jii=1,numisb
		areaisb(jii)=0
		DO ji=1,validcells
		 IF (isb(ji).EQ.jii) areaisb(jii)=areaisb(jii)+1
		ENDDO
	   ENDDO

      areaisb=areaisb*aegrid*aelhg/aeber
    ! -----------------------------------------------------------------------------------
    ! Opening output-file, writes headers

	  lval = CHANGEDIRQQ (TRIM(workdir))

      ALLOCATE (num(numisb))
	  ji=0
	  DO ji = 1,numisb
	   num(ji)=ji
	  ENDDO

201	  FORMAT ('YYYY MM DD HH ', <numisb>(X, 'Qisb', I3.3), X, 'Qtotal',' Ptotal',' Stotal',' Htotal')

	  ji=0
	  OPEN(MISBQ, FILE=NAMISBQ,STATUS='UNKNOWN',err=995)
      WRITE(mhtml,'(A)')'<a href=file:///'//namisbq//'>Houry runoff for internal sub catchments</a><br>'
	  WRITE(MISBQ,201) (num(ji),ji=1,numisb)

	  DEALLOCATE(num)

	  GOTO  999
    ! ----------------------------------------------------------------------------------
    ! Warnings and errors

995   CONTINUE
	  CALL messagerun (1,'Opening of output file *.qsb failed.')
	  GOTO 999

996   CONTINUE
	  CALL messagerun (1,'Error while opening *.ISB. STOP.')
	  GOTO 999

997   CONTINUE
	  CALL messagerun (1,'Error while opening *.IDH. STOP.')
	  GOTO 999

998   CONTINUE
	  CALL messagerun (1,'Gridmaps *.idh and *.isb are different. STOP.')
    ! --------------------------------------------------------------------------------
999   CONTINUE
	  lval = CHANGEDIRQQ (TRIM(workdir))
! ------------------------------------------------------------------------------------
!   End Processing
! ------------------------------------------------------------------------------------
      RETURN
	ENDSUBROUTINE
!*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
!
!*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

	SUBROUTINE procisb

! ---------------------------------------------------------------------
!
! ---------------------------------------------------------------------
!   Common Variables - prevahglobals.for
! ---------------------------------------------------------------------
	  USE prevahglobals
!	  real			   	aelhg, rges
!  	  integer	        tag, mon, jahr, i, kk
!	  character*1   	isbyn
!	  character*256		isbpath
!	  integer, allocatable :: isb(:) ! discharge for each internal sub basin
!	  real, allocatable :: qisb(:,:) ! discharge for each internal sub basin
! ---------------------------------------------------------------------
!   Local Variables
! ---------------------------------------------------------------------
   	  integer            ji
! ---------------------------------------------------------------------
!   Begin Processing
! ---------------------------------------------------------------------
	  DO ji=1,validcells
	   IF (idh(ji).EQ.kk) qisb(i,isb(ji))=qisb(i,isb(ji))+dble(rges)*aelhg/validcells/3.6 ! m3/s for each cell
	  ENDDO
! ------------------------------------------------------------------------------------
!   End Processing
! ------------------------------------------------------------------------------------
      RETURN
	ENDSUBROUTINE
!*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
!
!*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

	SUBROUTINE writeqisb

! ---------------------------------------------------------------------
!
! ---------------------------------------------------------------------
!   Common Variables - prevahglobals.for
! ---------------------------------------------------------------------
	  USE prevahglobals
!	  integer			numisb
!  	  integer	        tag, mon, jahr, i, kk
!	  real				aelhg
!	  real, allocatable :: qisb(:,:) ! discharge for each internal sub basin
! ---------------------------------------------------------------------
!   Begin
! ---------------------------------------------------------------------
101   FORMAT (I4,3(X,I2.2),<numisb+4>(X,F12.4))
! ---------------------------------------------------------------------
!   Begin Processing
! ---------------------------------------------------------------------
	  WRITE (MISBQ,101) jahr,mon,tag,i,(qisb(i,ji),ji=1,numisb),rgehm(i)*aelhg/(3.6),pakhm(i),ssohm(i),(ssmhm(i)+suzhm(i))/ssmb
! ------------------------------------------------------------------------------------
!   End Processing
! ------------------------------------------------------------------------------------
      RETURN
	ENDSUBROUTINE
!*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
