!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2003 Joachim Gurtz, Massimiliano Zappa
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Initialization of grids for new glacier module

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


! ------------------------------------------------------------------------------
! Reads the spatial information from the *.gkw catchment-ID file.
! ------------------------------------------------------------------------------



!  2D open issues
!   AETG(j)?
! Soil-depth and available field capacity can be adjusted
!      KORFAK=1.0
!      IF (NUGEB .EQ. 21) KORFAK=1.25
!      IF (NUGEB .EQ. 34) KORFAK=1.25
!      IF (NUGEB .EQ. 99) KORFAK=1.25
!      IF (NUGEB .EQ. 35) KORFAK=1.5

!      FBTIEF(xcoor,ycoor)=FBTIEF(xcoor,ycoor)*KORFAK
!      FNFC(xcoor,ycoor)=FNFC(xcoor,ycoor)*KORFAK


      SUBROUTINE ITFLAST2D (XY,ice)

	use realtime


! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------

      use prevahglobals
      use MPIvars, only: rank,root,doParallel, slaves,ierr
      use mpi  ! four countGlac only

      implicit none

      REAL    KORFAK				! adjustment factor

! ---------------------------------------------------------------------
!     Local Variables
! ---------------------------------------------------------------------
      INTEGER IHOEHE,IASPECT,ISLOPE,		! help for elevation, aspect and slope
     *     IAE,IHG1,IHG2			! dummy variables
      CHARACTER*72 ZEILE			! reads headers
      REAL LREAD(15),				! dummy variables
     *     STFL,STFL1,STFL2    	! dummy variables
      CHARACTER*1 XY				! Choice if more than one meteorological zone is available
	LOGICAL ice

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------

      AEBER=0.0
	ice = .false.
	areasubca=0.

	J=1
      L=0
      AETG=0.0

      glacmask = 0
      countGlacL = 0
      countGlac = 0

	!do l=1,13
	do l=1,14  ! Modified to include WHC grid - Matthias, Nov 2015
	CALL POSNAM (NADGKW,1,POSI1)
	if((rank==root).or.(doParallel==.false.))
     * print*, NADGKW(1:POSI1)//'.'//gkext(l)
	OPEN (731, file=NADGKW(1:POSI1)//'.'//gkext(l), status='old',
     *     form='binary',err=850)
	call posnam (NADGKW//'.'//gkext(l),2,posiid2)
		 do I=posiid2,1,-1
	     if (idnam(I:I) .eq. '\') exit
	     if (idnam(I:I) .eq. '/') exit
	     posiid1=I
	     end do
   	   uwas=731
	   INCLUDE 'rwasim.inc'
	   INCLUDE 'r2d.inc'
!		 nrtgfl=dom2d
      if (l .eq. 1) then
       FHOEHE=dom2d
	imid=int(max(0.,(wstats(2)-25.)))
	imad=int(min(10000.,(wstats(3)+25.)))
      end if
      if (l .eq. 2) LANDIN=dom2d
      if (l .eq. 3) FASPECT=dom2d
      if (l .eq. 4) FSLOPE=dom2d
      if (l .eq. 5) FNFC=dom2d
      if (l .eq. 6) FBTIEF=dom2d
      if (l .eq. 7) FCK=dom2d
      if (l .eq. 8) FTOPI=dom2d
      if (l .eq. 9) FBTOPI=dom2d
      !if (l .eq. 10) GGLM=dom2d
      if (l .eq. 10) rGLM=dom2d
      if (l .eq. 11) GEZG=dom2d
      if (l .eq. 12) MEZGRID=dom2d
      if (l .eq. 13) SCATCH=dom2d
      if (l .eq. 14) whc = dom2d

!         AETG(J)=AETG(J)+AETFL(xcoor,ycoor)
		 deallocate(DOM2D)
	end do   ! do l = 1,13

	AETFL=rgridsize*rgridsize/1000/1000
	AEBER=wstats(1)*rgridsize*rgridsize/1000/1000
         sso1500=0
      nsubca=0
	do ycoor=nrow,1,-1
	do xcoor=1,ncol

	  if((l==14).and.(xcoor==5).and.(ycoor==5)) then
	    print*, "WHC test: ", whc(xcoor, ycoor)
	  end if


        if (int(gezg(xcoor,ycoor)) .eq. int(rnodat)) cycle
! Increasing FNC by 33% to account for hardly-available soil moisture
           !FNFC(xcoor,ycoor)=FNFC(xcoor,ycoor)*1.33
! Basin Sub Areas
	    LREAD(10)=scatch(xcoor,ycoor)
	    IF (LREAD(10) .le. 1) then
           lread(10)=1
           scatch(xcoor,ycoor)=1
           END IF
          nsubca(lread(10))=nsubca(lread(10))+1
          areasubca(INT(LREAD(10)))=areasubca(INT(LREAD(10)))+
     *     AETFL(xcoor,ycoor)
	    scatch(xcoor,ycoor)=lread(10)
! DEM Stats
          IF (FHOEHE(xcoor,ycoor) .gt. 1500.) then
          sso1500(2,2)=sso1500(2,2)+1
          ELSE
          sso1500(2,1)=sso1500(2,1)+1
          END IF
! Glacier Areas
          !LREAD(2)=gglm(xcoor,ycoor)
          LREAD(2)=rglm(xcoor,ycoor)
      LN=LANDIN(xcoor,ycoor)

      if (ln .eq. 13) ice=.true.
      if (ln .eq. 14) ice=.true.

          if (gkext(10)(1:1) .ne. 'g') then
 	      if ((ln .eq. 13) .or. (ln .eq. 14)) THEN
              IF (INT(LREAD(2)) .eq. 2) LANDIN(xcoor,ycoor)=13
              IF (INT(LREAD(2)) .eq. 1) LANDIN(xcoor,ycoor)=14
       END IF
          else
 ! Glacier pixels determined from g** file
            IF (INT(LREAD(2)) .eq. 2) LANDIN(xcoor,ycoor)=13
            IF (INT(LREAD(2)) .eq. 1) LANDIN(xcoor,ycoor)=14

 ! No more glacier in accumulation area, then ROCK
            IF (LANDIN(xcoor,ycoor) .eq. 13) then
              IF (INT(LREAD(2)) .eq. 0) LANDIN(xcoor,ycoor)=15
            end if
 ! No more glacier in ablation area, then bare soil
            IF (LANDIN(xcoor,ycoor) .eq. 14) then
              IF (INT(LREAD(2)) .eq. 0) LANDIN(xcoor,ycoor)=26
            end if

           end if

! Decide whether the pixel is part of the glaciers considered for mass balance
! calculations (temporary - Matthias, Jan 2015)

      if(((landin(xcoor,ycoor)==13).or.(landin(xcoor,ycoor)==14))) then

           glacmask(xcoor,ycoor) = 1
           countGlac = countGlac + 1
         end if
!
! Determination of the category of available field capacity
         IF (FNFC(xcoor,ycoor) .LE. 8.0) FBTYP(xcoor,ycoor)=1
         IF ((FNFC(xcoor,ycoor) .GT. 8.0) .AND.
     *       (FNFC(xcoor,ycoor) .LT. 20.0)) FBTYP(xcoor,ycoor)=2
         IF (FNFC(xcoor,ycoor) .GE. 20.0) FBTYP(xcoor,ycoor)=3

! Determination of the category of hydraulic conductivity
         IF (FCK(xcoor,ycoor) .LE. 0.5) FKWERT(xcoor,ycoor)=1
         IF ((FCK(xcoor,ycoor) .GT. 0.5) .AND.
     *       (FCK(xcoor,ycoor) .LE. 10.0)) FKWERT(xcoor,ycoor)=2
         IF (FCK(xcoor,ycoor) .GE. 10.0) FKWERT(xcoor,ycoor)=3


	end do
	end do


! ---------------------------------------------------------------------
! Assigning the initial content of different storages
! ---------------------------------------------------------------------

      I=1

66    CONTINUE


        ! Test
        aeber = rgridsize * rgridsize

! Determining the Initial content of the runoff storage of the saturated zone SLZ
      IF (SLZ .LT. 0.0) THEN
         SLZ=0.0
         IF (QGA .LT. 0.0) THEN
            QGA=-QGA
! Transforming from l/s into mm/dt
            QGA=(QGA*(0.00006*DT))/AEBER
         ENDIF
! Filling of the storage accoring to the current water flow in l/s
         IF (K2 .GT. 0.0) SLZ=(QGA/K2)-QGA
      ENDIF



! ---------------------------------------------------------------------
      RETURN
850   continue
      print*, 'Error'
	print*, trim(NADGKW(1:POSI1)//'.'//gkext(l))
      print*, 'Not found'
      END
! ---------------------------------------------------------------------
