!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE KWLW1 (RGL,S,T,F,HOEHE,ALBEDO,GT,G0,TAG,
     *                     MONAT,JAHR,GEOBREITE,
     *                     LANGWSF,KURZWSF,QS,QL,
     *                     SLOPE,ASPECT,IRGL,PRL)

C      SUBROUTINE KWLW1 (RGL,S,T,F,HOEHE,ALBEDO,GT,TAG,
C     *                     MONAT,JAHR,GEOBREITE,
C     *                     LANGWSF,KURZWSF,qs,ql,
C     *                     SLOPE,ASPECT,IRGL,PRL)

! ---------------------------------------------------------------------
!      Computes long-wave and short-wave radiation
! ---------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------

      REAL RGL					!	Observed global radiation (W/M**2), daily sum
      REAL S					!	Sunshine duration (hours per day)
      REAL N					!	Cloudiness (1-8)
      REAL T					!	Air temperature (deg C)
	REAL F					!	Relative humidity (0..1)
	REAL HOEHE,SLOPE,ASPECT			!	elevation (m asl), slope (deg) and aspect (deg) of the HRU
	REAL ALBEDO				!	albedo
	REAL GEOBREITE				!	latitude (deg)
	REAL PRL				! Inflow in the runoff module
	INTEGER TAG,MONAT,JAHR			!	day, month year
	INTEGER KURZWSF,LANGWSF			!	control the computation of the short and long wave radiation
		!     LANGWSF		0:PENMAN 1948 (S)         1:PENMAN 1954 (S)
		!			2:ANGSTROEM 1923 (S)      3:ANGSTROEM 1923 (N)
		!			4:FEUSSNER (S)            5:FEUSSNER (N)
		!			6:CZEPLAK (S)             7:CZEPLAK (N)
		!			8:BRUTSAERT (S)
		!     KURZWSF		0:PENMAN 1948 (S)         1:PENMAN 1954 (S)
		!			2:GOLCHERT 1981 (S)       3:KARSTEN UND CZEPLAK 1980 (N)
		!			4:MORECS 1981 (S)         5:Mit gem. standortkorr. RGL
	INTEGER IRGL			!	controls the computation of incoming radiation
					!	IRGL	= -1 to 3   use obs observed incoming radiation
					!		= 4	without global radiaiton, with sunshine duration
					!	        = 5	withoud sunshine radiaiton, with cloudiness

! ---------------------------------------------------------------------
!     Local variables
! ---------------------------------------------------------------------

      INTEGER debugp				! Variable for debug
      !REAL TSA					!	time of sunrise
      !REAL TSU					!	time of sunset
      integer :: tsa, tsu
      REAL PT					!	air pressure at the elevation of the HRU hPa
      REAL GAMMA				!   psychometer constant hPa/K
      REAL ZETA					!	first derivative of saturated vapour pressure curve hPa/K
      REAL QS					!	short wave radiation balance (W/M^2)
      REAL QL					!	long wave radiation balance (W/M^2)
      REAL L					!	latent heat for evaporization for water (J/KG)
      REAL G0,GT				!   potential incoming clear sky dircet radiation (W/M^2)
      REAL S0					!	maximal sunshine duration (hours)
      REAL RHOL					!	density of air (KG/M^3)
      REAL RHOW					!	density of water (KG/M^3)
      REAL EL					!	water vapour pressure (hPa)
      REAL ELX				!	saturation water vapour pressure (hPa)

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------


!     Ezenitz computes the maximal sunshine SO (hours), the maximal incoming radiation
!     RO (W/m2), the time of sunrise TSA and the time of suinset TSU

      CALL EZENITZ (TAG,MONAT,JAHR,GEOBREITE,SLOPE,ASPECT,0,S0,G0,
     *              TSA,TSU)

      debugp=0
      CALL KWLW2 (RGL,S,N,T,F,HOEHE,ALBEDO,G0,GT,S0,MONAT,
     ,            LANGWSF,KURZWSF,GAMMA,
     ,            ZETA,QS,QL,ELX,EL,RHOL,RHOW,PT,L,IRGL,PRL)

      RETURN
      END
