!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------
      SUBROUTINE LINSTRA (S,F,CFH,ITRTH,HF,RDTF)
!
!     Linear storage and translation depending on:
!           CFH - Average storage time (H)
!           S   - storage content (l)
!           F   - current inflow into the storage (l/s)
!           Output:
!           RDTF - transformated discharge (l/s)
!    Subroutine LINSTRA by J.Gurtz (adapted by M.Zappa 25 November 2003)
! --------------------------------------------------------------------------------

      implicit none

      INTEGER J,ITRTH
      REAL F,RDTF,CFH,S,DT,HF(240)
      REAL SA,SE,CDT

      DT=1.0
      RDTF=0.0
      IF ((CFH.LE. 0.0) .and. (ITRTH .LE. 0)) THEN
         RDTF=F
         GOTO 5
      ENDIF
      IF (CFH .LE. 0.) THEN
         RDTF=F
         GOTO 2
      ENDIF

      SA=S
      CDT=EXP(-DT/CFH)
      SE=S*CDT+((1.0-CDT)*F*(CFH/DT))
      RDTF=F+SA-SE

      S=SE
2     CONTINUE
      J=1+ITRTH
      HF(J)=RDTF
      RDTF=HF(1)
      J=1
3     CONTINUE
      HF(J)=HF(J+1)
      J=J+1
      IF (J .LE. ITRTH) GOTO 3
5     CONTINUE
      RETURN
      END
