!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

! ---------------------------------------------------------------------

      SUBROUTINE LINTRA (S,F,CFH,ITRTH,HF,RDT )

!    SP LINTRA  -  Linear storages and translation member
!                 computes the outflow from a linear storages
!                 including a temporal shift of ITRTH h
!                accounting of:
!           S   - content of the storages (mm)
!           F   - inflow to the storages (mm/DT)
!           CFH - average storage coefficient (H)
!           ITRTH - shift time (DT-H) MAX=24
!           DT  - lenght of the time step (H) 1 h
!           HF  - array to do the temporal shift:
!           RDT - transformed discharge(mm/DT)
! ---------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------
C     INTEGER ITRTH
C     REAL F,RDT,CFH,S,HF(24)

      integer, intent(in) :: itrth
      real, intent(in) :: f, cfh
      real, intent(inout) :: s
      real, intent(inout), dimension(24) :: hf
      real, intent(out) :: rdt

! ---------------------------------------------------------------------
!     Internal Variables
! ---------------------------------------------------------------------
      INTEGER I,J
      REAL DT,SHF,RDTA
      REAL SA,SE,CDT

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------

      DT=1.0
      RDT=0.0
      SHF=0.0
      IF ((CFH .LE. 0.0) .and. (ITRTH .LE. 0)) THEN
         RDT=F
         GOTO 5
      ENDIF
      IF (ITRTH.GT.0) THEN
         DO I=1,24
            SHF=SHF+HF(I)
         ENDDO
         SA=S-SHF
         IF (SA.LE.0.0) SA=0.0
         GOTO 1
      ENDIF
      SA=S
1     CONTINUE
      IF (CFH .LE. 0.0) THEN
         RDT=F
         GOTO 2
      ENDIF

      CDT=EXP(-DT/CFH)
      SE=SA*CDT+((1.0-CDT)*F*(CFH/DT))
      RDT=F+SA-SE
      IF (RDT .LE. 0.0) RDT=0.0
      S=SE

2     CONTINUE
      IF (ITRTH.GT.0) THEN
         RDTA=RDT
         RDT=HF(1)
         DO J=1,23
            HF(J)=HF(J+1)
         ENDDO
         HF(24)=0.0
         J=ITRTH
         HF(J)=RDTA
         SHF=0.0
         DO I=1,24
            SHF=SHF+HF(I)
         ENDDO
         S=SE+SHF
         IF (SE.LE.0.0) SE=0.0
      ENDIF

5     CONTINUE

! ---------------------------------------------------------------------

      RETURN
      END
