!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

       SUBROUTINE OEVAPARZ (LAND,CLAND,RLAIM,BHOEHEM,ALBEDOM,RSCM,FLUXM,
     *                     WURZELV,AWACV,VBGF,SIMAXF,MDRU )

!     This subroutine writes the time changing parameters requested in the
!     evapotranspiration module into the log-file.
!     The values are specific for each HRU and land-use
!--------------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------
      INTEGER MDRU,
     *        LAND                                      ! Land use
      REAL RLAIM(12),                                   ! leaf area index [m2/m2]
     *        BHOEHEM(12),                              ! canopy height [m]
     *        FLUXM(12),                                ! soil heat flux [W/m2]
     *        ALBEDOM(12),                              ! albedo [-]
     *        RSCM(12),                                 ! minimal stomata resistences [s/m]
     *        WURZELV(12),                              ! root depths [m]
     *        AWACV(12),                                ! available water content [mm]
     *        VBGF(12),                                 ! vegetation density [0..1]
     *        SIMAXF(12)                                ! maximal interception storages [mm]
      CHARACTER*40 CLAND                                ! string of land use type


! ---------------------------------------------------------------------
!     Local Variables
! ---------------------------------------------------------------------

      INTEGER l

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------

2     FORMAT ( /,' Land use               ',i6,2x,a )
      WRITE  (MDRU,2) land,cland
4     FORMAT (   ' Leaf area index        ',12f9.3 )
      WRITE  (MDRU,4) (rlaim(l),l=1,12)
6     FORMAT (   ' Canopy height     (m)  ',12f9.3 )
      WRITE  (MDRU,6) (bhoehem(l),l=1,12)
8     FORMAT (   ' Albedo                 ',12f9.3 )
      WRITE  (MDRU,8) (albedom(l),l=1,12)
10    FORMAT (   ' Stomata resistence(s/m)',12f9.3 )
      WRITE  (MDRU,10) (rscm(l),l=1,12)
12    FORMAT (   ' Correction term BFLUSS ',12f9.3 )
      WRITE  (MDRU,12) (fluxm(l),l=1,12)
14    FORMAT (   ' Interception storage   ',12f9.3 )
      WRITE  (MDRU,14) (simaxf(l),l=1,12)
16    FORMAT (   ' Portion of vegetation  ',12f9.3 )
      WRITE  (MDRU,16) (vbgf(l),l=1,12)
18    FORMAT (   ' Root depth        (m)  ',12f9.3 )
      WRITE  (MDRU,18) (wurzelv(l),l=1,12)
20    FORMAT (   ' Avail. field capacity  ',12f9.3 )
      WRITE  (MDRU,20) (awacv(l),l=1,12)

! ---------------------------------------------------------------------
      RETURN
      END
