!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE OPARRSG2 (MDLOG,IGST,ISNOGL,PKORF,SNOKOR,
     *                     CREDV,TGR,TTRANS,T0,CRMFMAX,CRMFMIN,
     *                     CSNOMF,CASNO,
     *                     CWU1,CWA2,CWH,CEXLIQ,CRFR,AEGRID,
     *                     CBETA,CU,CRSZ,SGRLUZ,CPERC,K0H,K0,K1H,K1,K2H,
     *                     K2,CG1H,SLZ1MAX,CG3H,K3H,K3,
     *                     HELA,CICEMF,CAICE,KICEH,KICSH,KICFH,
     *                     TICEH,TICSH,TICFH )

!     This subroutine writes the used parameters for the actual model run
!     in the log-file
!    Literature:
!    Gurtz et al., Hydrological processes (2003)
!    Zappa et al., Nordin Hydrology (2003)
!    Zappa and Gurtz, HESS (2003)
! ---------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------

      INTEGER IGST,				! Controls the computation of the model performance
     *        MDLOG,				! Control the output to the prompt and to the log-files
     *        HELA,				! Can define the equilibrium line altitude of a glacier
     *        ISNOGL,				! Variable to control the snowmelt computation
     *        TICEH,				! Translation time for icemelt
     *        TICSH,				! Translation time for snowmelt
     *        TICFH				! Translation time for firn melt
      REAL PKORF,				! adjustment of rainfall %
     *     SNOKOR,				! adjustment of snowfall %
     *     CREDV,				! reduction factor - vegetated - unvegetated areas
     *     AEGRID,				! resolution of the grids in m2

! For snowmelt module
     *     TGR,					! treshold temperature snowfall
     *     TTRANS,				! transition temperature snow-rain
     *     T0,					! treshold temperature snowmelt
     *     CRMFMAX,				! maximal temperature-index melting factor
     *     CRMFMIN,				! minimal temperature-index melting factor
     *     CWU1,				! melting factor by wind 1
     *     CWA2,				! melting factor by wind 2
     *     CWH,					! retention factor for melting snow
     *     CEXLIQ,				! exponent for retention of melting snow
     *     CRFR,				! coefficien for re-freezin if Air temperature < 0 �C
     *     CSNOMF,				! constant temperature melt factor for snow
     *     CASNO,				! constant radiation melt factor for snow

!for runoff-generation module
     *     CBETA,				! non linearity parameter for inflitration module
     *     CU,					! treshold of ssm/ssmax (0..1) for reducin ETP
     *     CRSZ,				! 0..1 maximal percentage of total runoff that is generated by surface runoff
     *     SGRLUZ,				! treshold coefficient for surface runoff
     *     CPERC,				! Percolation rate
     *     K0H,					! Storage coefficient in hours for surface runoff
     *     K0,					! Storage coefficient for surface runoff as exponent
     *     K1H,					! Storage coefficient in hours for interflow
     *     K1,					! Storage coefficient for interflow as exponent
     *     K2H,					! Storage coefficient in hours for baseflow
     *     K2,					! Storage coefficient for delayed baseflow as exponent
     *     CG1H,				! Storage coefficient in hours for quick basflow
     *     SLZ1MAX,				! Maximal content of the quick baseflow storages SLZ1
     *     CG3H,				! 1/9 of CG1H after Schwarze
     *     K3H,					! Storage coefficient in hours for karst flow (not active)
     *     K3,					! Storage coefficient for karst flow as exponent (not active)

! for icemelt module
     *     CICEMF,				! constant temperature melt factor for ice
     *     CAICE,				! constant radiation melt factor for ice
     *     KICEH,				! Coefficient for runoff generation from ice storage in hours
     *     KICSH,				! Coefficient for runoff generation from snow storage in hours
     *     KICFH				! Coefficient for runoff generation from firn storage in hours

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------

302   FORMAT ( /,' resolution of the grids in m2                   :',
     *           F8.2 )
304   FORMAT ( /,' The snowmelt module used following parameters   :')
306   FORMAT ( /,' PKORF  - adjustment of rainfall %               :',
     *           F8.2,
     *         /,' SNOKOR - adjustment of snowfall %               :',
     *           F8.2,
     *         /,' CREDV  -reduction vegetated-unvegetated areas   :',
     *           F8.2,
     *         /,' TGR    - treshold temperature snowfall in deg C :',
     *           F8.2,
     *         /,' TTRANS - transition temperat. snow-rain in deg C:',
     *           F8.2,
     *         /,' T0     - treshold temperature snowmelt in deg C :',
     *           F8.2,
     *         /,' CRMFMAX- maximal temperature-index melting factor',
     *           ' in mm/(d*K)    :',F8.2,
     *         /,' CRMFMIN- minimal temperature-index melting factor',
     *           ' in mm/(d*K)    :',F8.2,
     *         /,' CWU1   - melting factor by wind 1 in mm/(d*K)   :',
     *           F8.2,
     *         /,' CWA2   - melting factor by wind 2 in mm/(d*K)   :',
     *           F8.2,
     *         /,' CWH    - retention factor for melting snow      :',
     *           F8.2,
     *         /,' CEXLIQ - exponent for retention of melting snow :',
     *           F8.2,
     *         /,' CRFR   - coefficien for re-freezin if T < 0 �C  :',
     *           F8.2     )
307   FORMAT (   ' CSNOMF - const. t-index for snow in mm/(d*K)    :',
     *           F8.2,
     *         /,' CASNO  - constant radiation melt factor for snow:',
     *           F8.2,
     *         /,' ISNOGL - control the snowmelt computation       :',
     *           I5  )
      WRITE (MDLOG,302) AEGRID
      WRITE (MDLOG,304)
      WRITE (MDLOG,306) PKORF,SNOKOR,CREDV,TGR,TTRANS,T0,CRMFMAX,
     *	              CRMFMIN,CWU1,CWA2,CWH,CEXLIQ,CRFR
      WRITE (MDLOG,307) CSNOMF,CASNO,ISNOGL

308   FORMAT ( /,' Parameters for the HBV-ETH runoff module        :')
309   FORMAT ( /,' CBETA  - parameter for inflitration module -    :',
     *           F8.2,
     *         /,' CU     - trshold for reducint ETP               :',
     *           F8.2,
     *         /,' CRSZ   - 0..1 maximal percentage of total runoff ',
     *         /,'           that is generated by surface runoff   :',
     *           F8.2,
     *         /,' SGRLUZ - treshold coeff. for surface runoff [mm]:',
     *           F8.2,
     *         /,' CPERC  - Percolation rate from SUZ in SLZ [mm/h]:',
     *           F8.2 )
310   FORMAT   ( ' K0H bzw. K0 - Storage coefficient surface runoff:',
     *           F8.2,F8.4,
     *         /,' K1H bzw. K1 - Storage coefficient interflow     :',
     *           F8.2,F8.4,
     *         /,' CG2H bzw.K2 - Storage coefficient slow baseflow :',
     *           F8.2,F8.4,
     *         /,' CG1H        - Storage coefficient quick baseflow:',
     *           F8.2,
     *         /,' SLZ1MAX     - Maximal content quick baseflow ',
     *         /,' storages SLZ1                                   :',
     *           F8.2,
     *         /,' CG3H     -  1/9 of CG1H after Schwarze          :',
     *           F8.2,
     *         /,' K3H bzw. K3 - Skarst flow (not active)          :',
     *           F8.2,F8.4,
     *         /,' IGST: Controls computation of model performance :',
     *           I4     )
      WRITE (MDLOG,308)
      WRITE (MDLOG,309) CBETA,CU,CRSZ,SGRLUZ,CPERC
      WRITE (MDLOG,310) K0H,K0,K1H,K1,K2H,K2,
     *               CG1H,SLZ1MAX,CG3H,K3H,K3,IGST
78    FORMAT ( /,' No glacier modelling HELA :', I8 )
      IF ( HELA .LT. 0 ) THEN
         WRITE (MDLOG,78) HELA
         GOTO 90
      ENDIF
80    FORMAT ( /,' The glacier module used following parameters     :')
82    FORMAT ( /,' HELA   - equilibrium line altitude.:',
     *         I8,
     *         /,'       If HELA = 0, then the glacier morphology is',
     *           'defined in the preprocessing',
     *         /,'       LAND= 13 above equilibrium line altitude',
     *         /,'       Land= 14 below equilibrium line altitude',
     *         /,'       IF HELA < 0, no glacier modelling',
     *         /,' CICEMF - temperature melt factor for ice         :',
     *         F8.3,
     *         /,' CAICE  - radiation melt factor for ice           :',
     *         F8.5,
     *         /,' KICEH  - Coefficient for runoff generation ',
     *         /,'         from ice storage:',
     *         F8.2  )
84    FORMAT (   ' KICSH  - Coefficient for runoff generation ',
     *         /,'          from snow storage below HELA :',
     *         F8.2,
     *         /,' KICFH  - Coefficient for runoff generation',
     *         /,'          from frin storage above HELA :',
     *         F8.2,
     *         /,' TICEH  - Translation time for icemelt:',
     *         I8,
     *         /,' TICSH  - Translation time for snowmelt:',
     *         I8,
     *         /,' TICFH  - Translation time for firnmelt:',
     *         I8 )
      WRITE (MDLOG,80)
      WRITE (MDLOG,82) HELA,CICEMF,CAICE,KICEH
      WRITE (MDLOG,84) KICSH,KICFH,TICEH,TICSH,TICFH

! ---------------------------------------------------------------------

90    CONTINUE
      RETURN
      END
