!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Parameterizes minimum stomatal resistance RSMIN
! for each species and height class
!-------------------------------------------------------------------------------

subroutine parstom

  use forhycsglobals, only: rsminStd
  use All_par, only: birht, maxhc, hcwidth, spec

  implicit none

  real, parameter :: gslope = -0.02
  real, parameter :: maxgs(3) = (/8.0, 5.5, 3.0/)
  integer, parameter :: gsclass(30) = (/2, 1, 3, 3, 3, 1, 3, &
                                    1, 2, 2, 1, 1, 1, 1, 1, 1, 3, 2, 1, 1, &
                                    1, 1, 1, 1, 2, 1, 1, 1, 2, 2/)
  real :: gs, gs0
  integer :: i, j
  real :: height


  !---------------------------------------------------------------------

  ! Array containing RSMIN values for each species and height class
  ! -> Will be used in subroutine vegPars to estimate RSMIN in each cell
  ! depending on species-size distribution

  allocate(rsminStd(30,0:maxhc))
  rsminStd(:,:) = 0.0

  ! Loop over species
  do i = 1,30
  ! Get maximum conductance (at height "zero") for this species
    gs0 = maxgs(gsclass(i))
    ! Loop over TreeMig height classes
    height = birht      ! Initial value: height of lowest height class
    do j = 1,spec(i)%maxhtc
    ! Get actual height corresponding to current height class
      gs = gs0 + gslope * height
      rsminStd(i,j) = 1/(gs*0.001)   ! gs in mm/s, RSMIN in s/m
      ! Height of next class
      height = MIN ((REAL (j) * hcwidth) + birht, REAL (spec(i)%khmax))
    end do
    ! Conductance for lowest height class
    rsminStd(i,0) = gs0
  end do



end subroutine parstom
