!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2003 Joachim Gurtz, Massimiliano Zappa
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Adaptation to the disaggregated soil evaporation and
! transpiration calculation.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------



      subroutine prevahcore

!!! Global variables and variables used in this subroutine
	     use prevahglobals
	     use mpi
	     use MPIVars
	     use outvars
	     use forhycsglobals, only: ddegsum5,muwint,meant,sump,ebjs2,
     *    TMtoPR,forhycs, drStr, underBioCrit, biomass2d,siutf,siumaxtf,
     *    fhla, vh100,vh99,vh95,snowoff,snowoffOld,snowdays,
     *      radCell, annprec, mmot,
     *    gsLengthCur,gsTpotCur,gsPrecCur,gsPrecDaysCur,gsTempCur,
     *    prevDayP,prevahBC,eeqrmean,pefrmean, gsPes, gsPesCur, meantC,
     *    readFH, gsPeCur
       use glacGlob, only: minSnowCover
       use All_par, only: stockability


	     implicit none

!!! Variables declaration
	      real dummysze,dummyszea(24)
	      real ssotr
	      integer ssodur,ssocum
        real cliadj_rain(6),cliadj_nrain(6)
        real, dimension(24) :: cf



! Updating procedure
	     Character*1 jnUp,okidoki,jjs		! Iterationsschalter y/n
	     INTEGER D1,D2,D3,D4,upi,uphour,upiter,uphourold,upacthour
	     INTEGER myi,tanrr,lass
	     REAL Qmes,jnm,rest,upfacold,Qmeso,dqmax
	     real restold,upfacnew
       CHARACTER(len=255) :: cwd
       character*4 :: chyr

	     REAL gdd, leafarea ! Marc's LAI stuff FORHYCS
	     integer :: chd
        REAL MHDEG   !mean hourly degree sums FORHYCS

! Hourly PET and AET terms
      real :: peTrans,peSnow,peSoil,peInt, aTrans, aESoil, eTrans,eSoil,
     *  eSnow, pe
      real :: eWater

! Daily sums of evaporation components, added during hourly loops
      real :: petDS, eSoilDS, eTransDS, eSnowDS, eIntDS,
     *  eWaterDS, peSoilDS, peTransDS, peDS

! Variables for parameterization of soil/understory evaporation
      real :: cumulRain  ! Cumulative rainfall over the whole day
      real, parameter :: critRain  = 0.5  ! Arbitrary
      integer :: ndrydays     ! Counter for number of consecutive dry days (used for soil evaporation)
      real :: eeqds           ! Rolling mean of soil equilibrium evaporation
      real :: wetFrac         ! Fraction of the canopy that is wet (used for interception evaporation)

! Auxiliary variables for radiation adjustment and sunshine duration
      real, dimension(24) :: ssdPot
      integer :: dd
      real :: rSSD
      real :: rgCor  ! Daily corrected global radiation (calculated as in the old PREVAH subroutine EPEMONTD)

! Auxiliary variables for reservoir state update
      real :: siOld
      real :: ssoOld
      real :: simaxi

! Grids of soil properties - for debug output only
      integer :: gridLoop
      real :: kwp  ! Funny conductivity from ronuff module
      real, dimension(ncol,nrow) :: kwp2d

! Vegetation properties
      logical :: gs  ! Growth season? Yes/no
      real :: vFrac  ! Second output of LeafPhenology (fractional canopy cover).
                     ! Replaces standard value of VBG for forested cells.
      real :: kInt  ! SIMAX parameter (Vegas-Galdos et al. 2012)


! TEMPORARY (glacier mass balance): keep storage contents at the beginning
! of the day to calculate change in storage content for each day
	     real :: tempsso, tempsi, tempssm, tempsuz, tempslz

! Standard land cover-dependent parameter values
      real :: sLAI, sZveg, sBflux, sRSmin, sAlb, sVfrac, sSImax, sawcSF

! Array for writing out dominant tree species number (after TM update)
      real, dimension(ncol,nrow) :: domSP
      integer :: domspec

! TreeMig stockability
      logical :: stock

! Available water capacity for soil. Helper variable for old stomatal regulation routine ERSTOM01
! Value set in soilPars.inc
      real :: awacsoil

      real :: rnk

! ------------------------------------------------------------------------------
! Dr. Thomas "Eddy" W�hling - Updating procedure
! September-October 2005
! Adapted by M.Zappa for use in every basin
! ------------------------------------------------------------------------------

      if (douptol) then
      include 'updating_core_1.inc'
	end if

6697  continue


! ------------------------------------------------------------------------------
! Assimilation of spatial fields
! M. Zappa, October-November 2006
! assimopt = 1 for snow cover maps, M. Zappa 4 November 2006
! ------------------------------------------------------------------------------

      INQUIRE (file=IDNAM(1:POSHRU)//'-assim.lst',EXIST=doassim)	! File controlling assimilation options

      if (doassim) then
	OPEN (3731, file=IDNAM(1:POSHRU)//'-assim.lst',
     * status='old')
	READ (3731,'(A)') kopf
      READ (3731,*) nassim		! Number of fields to be assimilated (binary rasters!)
	do lass=1,nassim
      READ (3731,'(A)') kopf
	READ (3731,'(A)') assimnam
      READ (3731,*) assimopt
	READ (3731,*) assimdate(1),assimdate(2),assimdate(3)
	READ (3731,*) assimlimit
	READ (3731,*) assimsource

! Assimsource
! 0 = Snow depth
! 1 = sub pixel snow cover

      if ((tag .eq. assimdate(1)) .and. (mon .eq. assimdate(2)) .and.
     *    (jahr .eq. assimdate(3))) then
	if (assimopt .eq. 1) then
	call messagerun (1, '[Assimilating snow data]')
!      call assimsnow (idnam)
	end if
	end if
	end do
	CLOSE (3731)
	end if

! Matthias: definition of parameters for climate adjustment
! for rainy and non-rainy days.
! Taken out of cell loop for performance.

      cliadj_rain = (/1.0, -1.5, 0.8, 1.1, 0.8, 1.0/)
      cliadj_nrain = (/1.0, 0.5, 1.1, 0.8, 1.1, 1.0/)


! ------------------------------------------------------------------------------
!    Begin of Daily CORE DO_LOOP
! ------------------------------------------------------------------------------

      if((rank.ne.root).or.(doParallel==.false.)) then
    	do ycoor=lastRow,firstRow,-1
    	do xcoor=1,ncol
    	if (int(gezg(xcoor,ycoor)) .eq. int(rnodat)) cycle


	dqmax=0

! Reset daily PET/AET sums
      petDS = 0.0
      eSoilDS = 0.0
      eTransDS = 0.0
      eSnowDS = 0.0
      eIntDS = 0.0
      eWaterDS = 0.0
      peSoilDS = 0.0
      peTransDS = 0.0
      peDS = 0.0
      eeqds = 0.0

      cumulRain = 0.0


! ------------------------------------------------------------------------------
!     load values for current cell (xcoor, ycoor)
! ------------------------------------------------------------------------------

       land=landin(xcoor,ycoor)           ! Land use type
       bfeucht=ssmtf(xcoor,ycoor)         ! Soil moisture
       aspect=faspect(xcoor,ycoor)        ! Aspect
       if (aspect .lt. 0.0) aspect=0.0
       slope=fslope(xcoor,ycoor)          ! Slope
       hoehe=fhoehe(xcoor,ycoor)          ! Elevation
       btyp=fbtyp(xcoor,ycoor)            ! Soil type (defined in itflast2d as a function of field capacity)
       btief=fbtief(xcoor,ycoor)          ! Soil depth
       nfcap=fnfc(xcoor,ycoor)            ! Field capacity
       snow=snowtf(xcoor,ycoor)           ! Snow storage
       m=mtf(xcoor,ycoor)
       sso=ssotf(xcoor,ycoor)


       ! TreeMig stockability
       if(stockability(ycoor,xcoor)==0.0) then
         stock = .false.
       else
         stock = .true.
       end if

! ------------------------------------------------------------------------------
!     Parameterization of CBETA
!     First Guess
! ------------------------------------------------------------------------------

      if (pardist) CBOPT=1

      IF (CBOPT .eq. 1) THEN
	CBETA=1.0+1500./max(min(hoehe,3000.),500.)+min(btief,2.)
!     Built Up Areas
      IF (land .eq. 2) CBETA=1.5
	END IF
! ------------------------------------------------------------------------------

      	if (land .le. 0) then
       land=26
       landin(xcoor,ycoor)=land
       end if


! Parameters are now set by subroutine cellLCparams further below


! ------------------------------------------------------------------------------
! Ev. regional adjustment of rainfall and snowfall
!      IF (precmokor .eq. 2) THEN
!         SNOKOR=regskor(NRTGFL(xcoor,ycoor))*100
!         PKORF=regrkor(NRTGFL(xcoor,ycoor))*100
!      END IF



      do i=1,NTG
	if (ntgid(i) .eq. mezgrid(xcoor,ycoor)) then
	K=i
	exit
	end if
	end do

      cliadjust(2)=0.0
	cliadjust(3)=1.0
	cliadjust(4)=1.0
	cliadjust(5)=1.0
	cliadjust(6)=1.0
! ------------------------------------------------------------------------------
!     Correction of climatology, for rainy - non rainy days
!     Only done if only precipiation is available on real time
      if (PNKDS(xcoor,ycoor) .gt. 2.) then
        where ((rmet .eq. 'c') .or. (rmet .eq. 'C'))
     *  cliadjust = cliadj_rain
      else
        where ((rmet .eq. 'c') .or. (rmet .eq. 'C'))
     *  cliadjust = cliadj_nrain
      end if

! ------------------------------------------------------------------------------
!     Assignment of the meteorological variables for the actual GRID CELL
! ------------------------------------------------------------------------------

      DO 81 I=1,NDTD
         IF (IRGL.LT.4) RGDTI(I)=RGDTG(I,xcoor,ycoor)
         IF (IRGL.LT.0) SDDTI(I)=SDDTG(I,xcoor,ycoor)
         TEDTI(I)=TEDTG(I,xcoor,ycoor)
         if ((rmet(2) .eq. 'c') .or. (rmet(2) .eq. 'C')) then
	   TEDTI(I)=TEDTI(I)+4*shapecoeffs(1,I)
	   end if
         if (dtmin(2) .eq. 1440) then
	   if (PNKDS(xcoor,ycoor) .gt. 10.) then
	   TEDTI(I)=TEDTI(I)+2*shapecoeffs(1,I)
	   else
	   TEDTI(I)=TEDTI(I)+4*shapecoeffs(1,I)
         end if
	   end if

! Random error for air temperature
      if (ranuse) then
	if (raopt .eq. 2) then
	TEDTI(I)=TEDTI(I)+ravar+rafipar(xcoor,ycoor)
	loadid='Y'
	end if
	end if

	   PNDTI(I)=PNDTG(I,xcoor,ycoor)
         RLDTI(I)=RLDTG(I,xcoor,ycoor) !*cliadjust(4)
         WVDTI(I)=WVDTG(I,xcoor,ycoor)
         SHADTI(I)=0.0
81    CONTINUE

      RGL=RGDM(xcoor,ycoor)
      x1=RGL

      PDS=PNKDS(xcoor,ycoor)
      x7=PDS
      SSD=SDDS(xcoor,ycoor)
      PRL=PDS

      BN=SSD



! ------------------------------------------------------------------------------
!     Local adjustment of air temperature and radiation
! ------------------------------------------------------------------------------
      CT=3.5



! Disaggregation of daily global radiation to hourly values and adjustment
! for slope and aspect.
      call rtkorz(tag,mon,jahr,geobreite,aspect,slope,
     *            hoehe,CT,MDR,DEBUG,rgl,tedti,sddti,shadti,
     *            tsa,tsu,ssdmax,shadm,tekdti,tekdm,rgdti,rgkdti,
     *                  rgkdm,rgmdti,rgmdm,rgmkdti,rgmkdm,rdmdti,
     *                  rdmdm,rdmkdti,rdmkdm,cf,xcoor,ycoor)

      ! Adjusted daily mean global radiation (used in evapDay)
      rgCor = rgl * rgmkdm/rgmdm



	  glkofa=RDMKDM/RDMDM

      ssdr=ssd/ssdmax

! Daily clearness index, may be used for estimating diffuse fraction of global radiation
! (currently not used)
      bn = 0.27 + ssdr*0.51


 !     if (tistep .eq. 24) SSD=SSD*SSDMX/24

      IF (IRGL .EQ. 4) GOTO 78
      IF (IRGL .LT. 0) GOTO 78
      IF (IRGL .EQ. 5) THEN
       SSD=(x1/8.0)*ssdmax
       GOTO 78
      ENDIF

78    CONTINUE
           TEC=TEDM(xcoor,ycoor)
           RLF=RLDM(xcoor,ycoor)
           WIV=WVDM(xcoor,ycoor)

! Random error for air temperature
      if (ranuse) then
	if (raopt .eq. 2) then
	TEC=TEC+ravar+rafipar(xcoor,ycoor)
	loadid='Y'
	end if
	end if

	! Update grid of mean May-Oct temperature
	if((mon.ge.5).and.(mon.le.10)) then
	  mmot(xcoor,ycoor) = mmot(xcoor,ycoor) + (tec/184.0)
	end if

! ------------------------------------------------------------------------------
!     If SHADST > 0. then shadowing is accounted
! ------------------------------------------------------------------------------
      IF (SHADST .GT. 0) THEN
!        Daily sums for each HRU
          IF (kk .EQ. 1)
     *   CALL SHADMORH (DS,TAG,MON,MSHDH,NTFL,SHADTFL,SHADETFL)
!     Interpolation to the single hours
          CALL SHADMOIH (TAG,MON,JAHR,DAT,DEBUG,MDR,NTFL,kk,
     *                   SHADTFL,SHADETFL,SHADTI,TSA,TSU)

          SHDJ=0
          CAICE=cagls
          CICEMF=caglt
          DO I=1,24
             SHDJ=SHDJ+SHADTI(I)
          ENDDO
          SHDFKT=(SHDJ-8)/16
          SHDFKT=1.4-SHDFKT
          IF (SHDFKT .LT. 1) SHDFKT=SHDFKT**4
          CICEMF=CICEMF*SHDFKT
          IF (SHDFKT .GT. 1) SHDFKT=1.0
          CAICE=CAICE*SHDFKT

        ELSE
          DO I=1,24
             SHADTI(I)=0.0
          ENDDO
      ENDIF



! ------------------------------------------------------------------------------
!    Assignment of climate scenario, if NRK > 0
! ------------------------------------------------------------------------------

      PRL=0.0
      ISTK=1

      if (dodelta .eq. 'Y') THEN
! Application of Deltachange scenarios after Bosshard et al. (2010)
      I=1
      include 'deltachange2d.inc'

         RLF=RLF*100.0
         CALL DELTACLIMATE (ISTK,NRK,MON,PRL,TEKDM,RGMKDM,
     *         RGMKDTI,WIV,RLF,CPRL,CRGMKDM,CWIV,CRLF,CTECK,
     *          deltavar,deltamethod)
	   NRK=1
         RLF=RLF*0.01
	END IF


      if ((szaktiv .eq. 'y') .or. (szaktiv .eq. 'Y')) THEN
         RLF=RLF*100.0
         CALL CLIMATE (ISTK,NRK,MON,PRL,TEKDM,RGMKDM,RGMKDTI,WIV,RLF,
     * CLIMFILE)
	   NRK=1
         RLF=RLF*0.01
	END IF

! Temperature scenario
      if (lscen) then
      TEKDM=TEKDM+TSCEN
	end if
! ------------------------------------------------------------------------------
!     Only for daily meteorological input
! ------------------------------------------------------------------------------

      IF (tistep .eq. 24) THEN


	END IF

! ------------------------------------------------------------------------------
!     Phenology module after LPJGUESS, Scherstjanoi et al. (FORHYCS)
!
!     Literature
!     Murray, N.B., Cannell,M.G.R.,Smith, I.(1989): Date of budburst of fifteen tree species
!      in Britain following climatic warming, J.appl.Ecol 26,693-700.
!
!     Sykes, M.T., Prentice I.C. and Cramer W. (1996): A bioclimatic model for the potential
!      distributions of north European tree species under present and future climates.
!      Journal of Biogeography 23: 209-233.
!
!     Scherstjanoi, M. (2013): Towards an efficient plant physiology-based moedling of
!      spatial forest dynamics, Dissertation, Ecole Polytechnique Federal de Lausanne
!
!     Scherstjanoi, M, Kaplan, J.O., Th�rig, E., Lischke, H. (2013):  GAPPARD: A computationally efficient
!      method of approximating gap-scale disturbance in vegetation models, Geosci. Model Dev. Discuss., 6, 1021-1084,
!      doi: 10.5194/gmdd-6-1021-2013.
!
! ------------------------------------------------------------------------------

! Land-cover specific and seasonal parameter values (used where forest/
! vegetation is not simulated dynamically). Replaces EINPUZDW.
      call cellLCparams(land, mon, slai, sZveg, sBflux,
     * sRSmin,sAlb, sVfrac, sSImax,sawcSF, stock)

      if (TMtoPR.eq.1.and.stock) then

      ! Determine dominant species (used to determine start and end of the
      ! growing season in the phenology module)
          domSP(xcoor,ycoor) =
     *     sum(real(maxloc(biomass2d(xcoor,ycoor,1:30))))
          domspec = int(domSP(xcoor,ycoor))

      ! Daily update of cumulative number of winter chill days and growing
      ! degree days. Both factors determine the timing of spring leaf
      ! phenology in the model of Murray (1989)
        CALL ChilldaysAndGDD5 (TEKDM,chd,gdd)

      ! Phenology for daily scaling of LAI and fractional canopy cover. This
      ! subroutine also determines whether the current day is part of the
      ! growing season or not (via the logical output argument GS)
        CALL LeafPhenology(chd, gdd, TEKDM, domspec, stock, leafarea,
     *   vFrac, gs)

        RLAI=max(leafarea,0.2)
        forhycs(xcoor,ycoor,1)=RLAI
        vbg = vFrac

      ! Potentially forested cells: set standard LAI/vFrac if simulated values
      ! are less than that (meaning that the forest is not (yet) growing well there)
      if((land==21).or.(land==23).or.(land==24).or.(land==25).or.
     * (land==26)) then
        rlai = max(leafarea,slai)
        vbg = max(vFrac,sVfrac)
      end if

      ! Interception storage capacity, parameterized following Menzel et al.
      ! (1996) and Vegas-Galdos et al. (2012)

        select case(land)
          case(3)
            kInt = 2
            SIMAX= (kInt * log10(1+RLAI))
          case(4)
            kInt = 1.6
            SIMAX= (kInt * log10(1+RLAI))
          case(5)
            kInt = 1.8  ! Interpolating between values for con and bro
            SIMAX= (kInt * log10(1+RLAI))
          case(8)
            kInt = 2.6
            SIMAX= (kInt * log10(1+RLAI))
          case(21,24,25,26)
            kInt = 1.8  ! For trees present in potentially forested areas
            SIMAX= (kInt * log10(1+RLAI))
        end select

        forhycs(xcoor,ycoor,2)=SIMAX

      ! Prescribed vegetation parameters (uninfluenced by coupling)
        bhoehe = stdZVeg(land,mon)
        fluxabsorb = sBflux
        albedo = sAlb

      else
      ! No coupling (either because this is an uncoupled run (TMtoPR==0)
      ! or because the land cover of this cell does not allow forest to grow).
      ! In this case, all vegetation parameters are prescribed, depending on
      ! LC class and month (mostly following standard PREVAH parameterization)
        rlai = slai
        bhoehe = sZveg
        fluxabsorb = sBflux
        rsc = sRSmin
        albedo = sAlb
        vbg = sVFrac
        SImax = sSImax

!     SIMAX, WURZELV and BHOEHE are linearly corrected between 1300 m a.s.l.
!     and 2500 m a.s.l.. After 2500 m a.s.l. values stay constant
        IF (((land.gt.2).and.(land.lt.9)).or.
     *    ((land.gt.17).and.(land.lt.20)).or. (land.eq.29)) THEN
          IF (HOEHE.GT.1300.0) THEN
            RHG=(HOEHE-1300.0)/1200.0		!(1200=2500-1300)
            IF (RHG.GT.1.0) RHG=1.0
            SIMAX=SIMAX*(0.4+(0.6*(1.0-RHG)))
            BHOEHE=BHOEHE*(0.5+(0.5*(1.0-RHG)))
            !WURZELV(MON)=WURZELV(MON)*(0.6+(0.4*(1.0-RHG)))
          ENDIF   ! Hoehe (elevation)
        ENDIF     ! Land
      end if      ! TMtoPR or stock

!     Albedo for glacerized HRU if no snow is present
      IF (( land .eq. 13) .or. (land .eq. 14)) THEN
	   isice=.true.
         IF (sso .lt. 0.1) albedo=0.27
      ENDIF


!     Ageing of albedo
      IF ((SNOEST .GT. 0 ) .and. (SNOEST .LE. 7)) THEN
         IF ( SSO .GT. 0.5) THEN
            DA=0.15
            ALBAS=0.40
	      ALBADDD=0.45
            RHG=M
            RHG=RHG/24.0
            albedo=ALBAS+ ALBADDD*(EXP(RHG*(-DA)))
         ENDIF
      ENDIF


! ------------------------------------------------------------------------------
!   The actual plant available field capacity SFC is assigned to each HRU.
!   BFEUCHT [mm] is parameterized as function of the assimilated average
!   plant available field capacity FNFC (Vol.%), the soil depth FBTIEF (m) and
!   the roots depth WURZELV (m)
!   For some land use types there is standard parametrerization of SFC
!
!   SGRLP=Treshold SFC for reducing the potential evapotranspiration
! ------------------------------------------------------------------------------


! SFC for non-forested cells
      include 'soilPars.inc'


      select case(land)
        case(1)
          sgrlp = 0.4*sfc
        case(2)
          sgrlp = cu*sfc
        case(13,14)
          sgrlp = 0.0
        case(15)
          sgrlp = cu*sfc
      end select

! Random error for soil moisture
      if (ranuse) then

        select case(raopt)

          case(1)
	     bfeucht=bfeucht*(1+ravar+rafipar(xcoor,ycoor))
	     bfeucht=min(bfeucht,SFC)
	     ssmtf(xcoor,ycoor)=bfeucht
	     loadid='Y'

          case(3)
            loadid='Y'

        end select
      end if
! -------------------------------------------

! FORHYCS spin up from bare soil: forested cells are initially given
! standard (low) values for vegetation parameters, until the canopy develops
! MS, March 2017
      if((readFH==0).and.(jahr.lt.(jahra+5))) then
      if((land==3).or.(land==4).or.(land==5).or.(land==8)) then
        rlai = max(rlai, 0.2)
        vbg = max(vbg,0.1)
        bhoehe = 2.0
      end if
      end if


! ------------------------------------------------------------------------------
! Update of bioclimatic indices for TreeMig/FORHYCS (degree-day sum and minimum winter temperature)
! Degree-day sum: computed with *hourly* values (see below)
! Marc: DDEGs old version, mean daily temperature; but correcture term in BioClim also accounts for hourly values!!!

! DDEGs new version
      MHDEG=0.0
      DO I=1,24
         IF (TEKDTI(I) .gt. 5.5) THEN
          MHDEG=MHDEG+(TEKDTI(I)-5.5)
         ENDIF
      ENDDO
      MHDEG=MHDEG/24.0
      ddegsum5(xcoor,ycoor)=ddegsum5(xcoor,ycoor)+MHDEG


!  Minimum winter temperature:
      IF (MON .eq. 11) THEN
        IF (TAG .eq. 1) THEN
          muwint(xcoor,ycoor,1)=0
          muwint(xcoor,ycoor,2)=0
          muwint(xcoor,ycoor,3)=0
          muwint(xcoor,ycoor,4)=100
        END IF
      END IF
      IF (MON .eq. 12) THEN
        muwint(xcoor,ycoor,1)=muwint(xcoor,ycoor,1)+TEKDM/31
      END IF
      IF (MON .eq. 1) THEN
        muwint(xcoor,ycoor,2)=muwint(xcoor,ycoor,2)+TEKDM/31
      END IF
      IF (((MON .eq. 2).AND.(JAHR.EQ.(INT(float(JAHR)/4.)*4))) .and.
     * (jahr.ge.1971).and.(jahr.le.2015)) THEN  ! Leap years (not considered during spinup and future runs)
        muwint(xcoor,ycoor,3)=muwint(xcoor,ycoor,3)+TEKDM/29
        if (TAG .eq. 29) THEN
          muwint(xcoor,ycoor,4)=min(muwint(xcoor,ycoor,1),
     *    muwint(xcoor,ycoor,2),muwint(xcoor,ycoor,3))
        END IF
      else if(mon .eq. 2) then  ! Non-leap year case for February
        muwint(xcoor,ycoor,3)=muwint(xcoor,ycoor,3)+TEKDM/28.
         if (TAG .eq. 28) THEN
          muwint(xcoor,ycoor,4)=min(muwint(xcoor,ycoor,1),
     *    muwint(xcoor,ycoor,2),muwint(xcoor,ycoor,3))
        END IF
      END IF

      meantC(xcoor,ycoor,MON)=meantC(xcoor,ycoor,MON)+TEKDM
      meant(xcoor,ycoor,MON)=meant(xcoor,ycoor,MON)+TEDM(xcoor,ycoor)
      sump(xcoor,ycoor,MON)=sump(xcoor,ycoor,MON)+PDS
      IF (((MON.eq.1).or.(MON.eq.3).or.(MON.eq.5).or.(MON.eq.7)
     *.or.(MON.eq.8).or.(MON.eq.10).or.(MON.eq.12))
     *.and.(TAG.eq.31)) THEN
        meant(xcoor,ycoor,MON)=meant(xcoor,ycoor,MON)/31.0
        meantC(xcoor,ycoor,MON)=meantC(xcoor,ycoor,MON)/31.0
      END IF
      IF (((MON.eq.4).or.(MON.eq.6).or.(MON.eq.9).or.(MON.eq.11))
     *.and.(TAG.eq.30)) THEN
        meant(xcoor,ycoor,MON)=meant(xcoor,ycoor,MON)/30.0
        meantC(xcoor,ycoor,MON)=meantC(xcoor,ycoor,MON)/30.0
      END IF
      !IF ((MON.eq.2).and.(TAG.eq.MONUM(2))) THEN
      IF((MON.eq.2).and.(jahr==int(float(jahr)/4.)*4).and.tag.eq.29)THEN

        meant(xcoor,ycoor,MON)=meant(xcoor,ycoor,MON)/29.
        meantC(xcoor,ycoor,MON)=meantC(xcoor,ycoor,MON)/29.
      else if(mon.eq.2.and.tag.eq.28) then
        meant(xcoor,ycoor,MON)=meant(xcoor,ycoor,MON)/28.
        meantC(xcoor,ycoor,MON)=meantC(xcoor,ycoor,MON)/28.

      END IF





	JAHRK=JAHR
	IF (JAHR. GT. 1999) JAHRK=JAHR-100
      DATUM=((JAHRK-1900)*100.0)+MON+(TAG*0.01)
      JAHRK=JAHRK-1900

      RLF=RLF*100.0  ! Relative humidity as a fraction instead of percentage


      IBILD=10
      IDRUST=DEBUG+10


! ------------------------------------------------------------------------------
!    Begin of hourly integration
! ------------------------------------------------------------------------------
      I=0
      RHG=NDTD*1.0

	IF (tcali .eq. 't') then
      IF (fnext .eq. 1) I=hoursb-2
	END IF

	if (educnr .eq. -99) then
	if (nwpuse) then
      IF (fnext .eq. 1) I=hoursb-2
	end if
	end if

      ibb=i+1

! This is the beginning of the hourly loop (GOTO statement sends back here)
120   I=I+1

! ------------------------------------------------------------------------------
! Dr. Thomas "Eddy" W�hling - Updating procedure
! September-October 2005
! Adapted by M.Zappa for use in every basin
! ------------------------------------------------------------------------------

      if (douptol) then
      include 'updating_core_2.inc'
	end if

! -----------------------------------


!   Assignment of hourly meteorological values
      jk=k

	IF ((tcali .eq. 't') .and. (fnext .eq. 1)) then
	if (I .eq. (hoursb-1)) PNDTG(I,xcoor,ycoor)=0
	END IF

      PNAKT(I)=PNDTG(I,xcoor,ycoor)
      TEC=TEDTG(I,xcoor,ycoor)

! Random error for air temperature
      if (ranuse) then
	if (raopt .eq. 2) then
	TEC=TEC+ravar+rafipar(xcoor,ycoor)
	loadid='Y'
	end if
	end if

      WIV=WVDTG(I,xcoor,ycoor)
      SSDHS=SDDTG(I,xcoor,ycoor)
      RLF=RLDTG(I,xcoor,ycoor)
      RGL=RGDTG(I,xcoor,ycoor)
      SSD=SDDTG(I,xcoor,ycoor)
      RGK=RGKDTI(I)
! "Hack" for consistency with PREVAH (and old RTKORZ) - Feb 2018
      if(i.gt.1) rgk = RGKDTI(I-1)
      if(i.gt.1) rdmk = RDMKDTI(I-1)

      rnk = rgdti(i)
      RDM=RDMDTI(I)
      RDMK=RDMKDTI(I)
      TECK=TEKDTI(I)

      ! Avoiding NaN in precipitation
      prl = max(0.0,pakt(i))
      if(prl.ne.prl) then
        prl = 0.0
        pakt(i) = 0.0
      end if

      if (dodelta .eq. 'Y') then
! Application of Deltachange scenarios after Bosshard et al. (2010)
      include 'deltachange2d.inc'
	end if


! Temperature scenario
      if (lscen) then
      TECK=TECK+TSCEN
	TEC=TEC+TSCEN
	end if


! Daily relative sunshine duration; to be updated every day at sunrise
      if(i == tsa) relSSD(xcoor,ycoor) = ssdr


! ------------------------------------------------------------------------------
!    Assignment of climate scenario, if NRK > 0
! ------------------------------------------------------------------------------


      if (dodelta .eq. 'Y') THEN
         RLF=RLF*100.0
         CALL DELTACLIMATE (ISTK,NRK,MON,PRL,TECK,RGMKDM,RGMKDTI,
     *          WIV,RLF,CPRL,CRGMKDM,CWIV,CRLF,CTECK,
     *          deltavar,deltamethod)
	   NRK=1
         RLF=RLF*0.01
	END IF

      if ((szaktiv .eq. 'y') .or. (szaktiv .eq. 'Y')) THEN
         IF (RLF.LT.1) RLF=RLF*100
         ISTK=0
!  Scenario of all variables, temperature corrected for exposition and slope
         CALL CLIMATE (ISTK,NRK,MON,PRL,TECK,RGMKDM,RGMKDTI,WIV,RLF,
     * CLIMFILE)
         RLF=RLF*0.01
         PAKT(I)=PRL
	END IF  ! szaktiv

! ------------------------------------------------------------------------------
!    Adjustment of precipitation
!
!    Literature
!    Gurtz, J., Baltensweiler, A., Lang, H., Menzel, L. and Schulla, J. (1997):
!     Auswirkungen von klimatischen Variationen von Wasserhaushalt und Abfluss
!     im Flussgebiet des Rheins. Schlussbericht NFP 31: Klima�nderungen und
!     Naturkatastrophen, vdf Hochschulverlag AG ETH Z�rich, 147 pp.
! ------------------------------------------------------------------------------

! Distributed parameters
      if (pardist) then
       pkorf=distpar(1,xcoor,ycoor)+fixpkorf
	snokor=distpar(2,xcoor,ycoor)+fixsnokor
	T0=distpar(3,xcoor,ycoor)+fixt0
	CSNOMF=distpar(4,xcoor,ycoor)*fixsnow
	CRMFMAX=distpar(4,xcoor,ycoor)*fixsnow*1.5
	CRMFMIN=distpar(4,xcoor,ycoor)*fixsnow*0.5
	CASNO=distpar(5,xcoor,ycoor)
       CICEMF=distpar(13,xcoor,ycoor)*fixicet
       CAICE=distpar(14,xcoor,ycoor)*fixicer
       SGRLUZ=distpar(6,xcoor,ycoor)
       K0=abfdist(1,xcoor,ycoor)
       K1=abfdist(2,xcoor,ycoor)
       K3=0.0
       CG1H=distpar(10,xcoor,ycoor)
       CG2H=distpar(9,xcoor,ycoor)
       CG3H=abfdist(4,xcoor,ycoor)
       SLZ1MAX=distpar(11,xcoor,ycoor)
       CPERC=distpar(12,xcoor,ycoor)
	end if

	! Matthias, Jan 2017: Adjusting some parameters to match simulated discharge
	! to observations in Gougra catchments
	sgrluz = sgrluz+60
	k0 = k0/8.
	cperc = cperc*2.
	!k1 = k1/2.
	csnomf = csnomf * 0.66
	!cg1h = cg1h+150
	casno = casno * 0.5
	!cg2h = cg2h + 600
	!cg3h = cg2h*0.11111 ! CG3H is always 1/9 of CG2H
	!slz1max = slz1max-50


	! -------------------------------------------

!     switch pkorf and snokor on/of depending on switch_date, also switch precmokor from 0 to 3 to avoid wind correction of forecast data (SJ)

      write(actual_date_char,'(I4.4,3I2.2)') JAHR,MON,TAG,I
      read(actual_date_char,'(I10)') actual_date

      ! to avoid any precip corrections set nugeb (in *.inp) to 10
      if(nugeb .eq. 10) then
        switch_date=actual_date
      end if

      if (actual_date .ge. switch_date ) then
        SNOKOR=0.0
	PKORF=0.0
        precmokor=3
      else
        SNOKOR=snokor_bp
	PKORF=pkorf_bp
      if (pardist) then
       pkorf=distpar(1,xcoor,ycoor)+fixpkorf
	snokor=distpar(2,xcoor,ycoor)+fixsnokor
      end if
      end if

!     No correction if forecast!
      IF ((docali .eq. 'f') .or. (docali .eq. 'F')) THEN
!     Include Here BIAS correction
	     SNOKOR=0.0
	     PKORF=0.0
	     PAKT(I)=PNAKT(I)
	ELSE

	If (precmokor .ne. 0) THEN
!     Old Snow-rain correction for special cases
!     Correction is monthly based precmokor=1
!     Correction is given for each HRU precmokor=2
!     No temperature scenario before correction rain-snow, to avoid difference in volume of precip !
      IF (TEC .LT. TGR) THEN
        PAKT(I)=PNAKT(I)*(1+(SNOKOR*0.01))
      ELSE
        PAKT(I)=PNAKT(I)*(1+(PKORF*0.01))
      ENDIF

	ELSE

!     New Snow-rain correction -> Activated if NUGEB=1
	IF (TEC .LT. TGR) THEN
        PAKT(I)=PNAKT(I)*(1+0.20*log(WIV+1.))
	  PAKT(I)=PAKT(I)*(1+SNOKOR*0.01)
      ELSE
        PAKT(I)=PNAKT(I)*(1+0.07*log(WIV+1.))
	  PAKT(I)=PAKT(I)*(1+PKORF*0.01)
      ENDIF

	END IF ! Close precmokor if structure
	END IF ! Close docali 'f' if structure

      if (dodelta .eq. 'Y') THEN
         RLF=RLF*100.0
         CALL DELTACLIMATE (ISTK,NRK,MON,PAKT(I),TEC,dummysze,dummyszea,
     *          dummysze,dummysze,CPRL,CRGMKDM,CWIV,CRLF,CTECK,
     *          deltavar,deltamethod)
	   NRK=1
         RLF=RLF*0.01
	END IF

      if ((szaktiv .eq. 'y') .or. (szaktiv .eq. 'Y')) THEN
         ISTK=0
!  Scenario only for temperature not corrected for exposition and slope (needed for snow accumulation)
	   CALL CLIMATE (ISTK,NRK,MON,PAKT(I),TEC,dummysze,dummyszea,
     *   dummysze,dummysze,CLIMFILE)
	END IF



! ------------------------------------------------------------------------------
! Accounting the millenium
      JAR=JAHR-1900
      IF (JAHR .GT. 1999) JAR=JAR-100

! ------------------------------------------------------------------------------
!    Switching to the current HRU - Storages
! ------------------------------------------------------------------------------

      SI=SITF(xcoor,ycoor)
      SSO=SSOTF(xcoor,ycoor)
      SSNOW=SSNOWTF(xcoor,ycoor)
      SLIQ=SLIQTF(xcoor,ycoor)
      snow=snowtf(xcoor,ycoor)
      M=mtf(xcoor,ycoor)
      VBGA=VBG
      prl = max(pakt(i),0.0)
      SSM = ssmtf(xcoor,ycoor)


! Update grid of annual precipitation sum
      annprec(xcoor,ycoor) = annprec(xcoor,ycoor) + prl


! TEMPORARY (glacier mass balance)

      tempsso = SSOTF(xcoor,ycoor)
      tempsi = SITF(xcoor,ycoor)
      tempssm = SSMTF(xcoor,ycoor)
      tempsuz = SUZTF(xcoor,ycoor)
      tempslz = SLZTF(xcoor,ycoor)

!     Wenn kein Schnee liegt, erfolgt fuer Siedlungs- und Felsflaechen
!          und fuer die Schutt- und Rohbodenvegetation die
!          Interzeption als direkte Verdunstung aus den Muldenspeichern
!          und von der benetzten Oberfl�che,
!          die mit 80 bzw. 90 % bzw. 65% der Flaeche festgelegt wird.
!          Vom restlichen Flaechenanteil ist Transpiration moeglich.


      IF ((SSO.LE.0.1) .or. (SNOEST.EQ.0)) THEN
        select case (land)
          case(2, 13, 14)
            vbga = 0.8
          case(15)
            vbga=0.9
          case(26)
            vbga=0.65
        end select
      ENDIF

! ------------------------------------------------------------------------------
! NEW: Hourly calculation of PET and AET
! ------------------------------------------------------------------------------

      rSSD = relSSD(xcoor,ycoor)



! Refill of SI must be calculated before PET and AET. The following was shifted from
! Interception module below, partially copied/adapted from INTLOZ.


! Filling after Menzel (1997) or linear, depending on size of SIMAX

      if(stock==.false.) then
        simaxi = simax * vbga
      else
        simaxi = simax
      end if

       siOld = si
       if(simaxi .lt. 0.1) then  ! linear
         si = siOld + prl*0.8
         if(si .ge. simaxi) then
           si = simaxi
           prl = prl-(simaxi-siOld)
         else
           prl = prl-prl*0.8
         end if
       else                     ! Menzel
         si = siOld + (simaxi-siOld)*(1-exp(-1/simaxi*prl))
         if(si .ge. simaxi) then
           si = simaxi
           prl = prl - (simaxi-siOld)
         else
           prl = prl - (simaxi-siOld)*(1-exp(-1/simaxi*prl))
         end if
       end if


       ! Numerical errors might lead to negative prl (causing NaNs further in
       ! the code, e.g. in snowmelt module)
       ! MS, Feb 2017
       if(prl.lt.0) then
         si = si+prl
         prl = 0.0
       end if

      ! Cumulative daily effective precipitation, used for soil evaporation reduction (see subroutine evap)
      cumulRain = cumulRain + prl
      if(cumulRain .gt. critRain) drydays(xcoor,ycoor) = 0
      ndrydays = drydays(xcoor,ycoor)

! Calculation of the different evaporation terms
      call evap(rgk,rSSD,teck,rlf,wiv,hoehe,bhoehe,geobreite,tag,mon,
     * jahr,i,sso,rlai,albedo,si,simaxi,ssm,sfc,fluxabsorb,kurzwsf,
     * langwsf,vfeuchtf,irgl,tsa,tsu,ndrydays,slope,aspect,
     * land,srsmin,vbga,peTrans,peSnow,peSoil,peInt,aTrans,aeSoil,pe,
     * stock,xcoor,ycoor)

       if((xcoor==69) .and. (ycoor==97) .and. (jahr==1676)) then
         print*, "------"
         print*, "After EVAP 69 97"
         print*, "ssm", ssm
         print*, "sfc", sfc
         print*, "peTrans", peTrans
         print*, "aTrans", aTrans
         print*, "peSoil", peSoil
         print*, "aeSoil", aeSoil
       end if

       if((xcoor==11) .and. (ycoor==164) .and. (jahr==1676)) then
         print*, "------"
         print*, "After EVAP 11 164"
         print*, "ssm", ssm
         print*, "sfc", sfc
         print*, "peTrans", peTrans
         print*, "aTrans", aTrans
         print*, "peSoil", peSoil
         print*, "aeSoil", aeSoil
       end if

       eeqds = eeqds + peSoil

      epdt = pe


! ------------------------------------------------------------------------------
!   Computation of hourly snow evaporation should be included here (SNOEST 3 or 4)
!   This feature is not yet implemented
! ------------------------------------------------------------------------------

! ------------------------------------------------------------------------------
!    Interception module
!
!    Literature
!    Gurtz, J., Baltensweiler, A., Lang, H., Menzel, L. and Schulla, J. (1997):
!     Auswirkungen von klimatischen Variationen von Wasserhaushalt und Abfluss
!     im Flussgebiet des Rheins. Schlussbericht NFP 31: Klima�nderungen und
!     Naturkatastrophen, vdf Hochschulverlag AG ETH Z�rich, 147 pp.
!    Menzel, L. (1997): Modellierung der Evapotranspiration im System Boden-
!     Pflanze-Atmosph�re. Z�rcher Geographische Schriften, Heft 67, ETH Z�rich
!     [available from the Institute for Atmospheric and Climate Science ETH,
!     Winterthurerstr. 190, CH-8057 Z�rich].
! ------------------------------------------------------------------------------

      prldifga=prl


! CALL INTLOZ (PRL,SIMAX,SI,VBGA,EPDT,EIDT)
C      if((xcoor == 35) .and. (ycoor == 161)) then
C        print*, "Before EI: si, peInt,eidt", si,peInt,eidt
C      end if

      select case(land)

      !if((land==2).or.(land==13).or.(land==14).or.(land==15)) then

      case(2, 13, 14, 15)

        ! Special land cover types: Penman evaporation is stored in the variable
        ! aeSoil (or peSnow if there is snow present). Energy from interception
        ! loss is taken from this variable
        if(sso .gt. 0.01) then
          eidt=min(si,peSnow)
          si=max(0.0,si-peSnow)
          sitf(xcoor,ycoor) = si
          peSnow = max(0.0,peSnow-eidt)
        else
          eidt=min(si,(aeSoil*vbga))
          si=max(0.0,si-(aeSoil*vbga))
          sitf(xcoor,ycoor) = si
          aeSoil = max(0.0,aeSoil-eidt)
        end if
      !else

       ! if(stock) then

      case(3:5,8,21,23:26)    ! Forested and potentially forested LC types

          !wetFrac = (SI/SIMAXi)**0.66
          !peInt = peInt * wetFrac
          !peTrans = peTrans * (1-wetFrac)
          !aTrans = aTrans * (1-wetFrac)


        ! Interception can only evaporate as much water as there is in the
        ! canopy storage
          eidt = min(si,peInt)
          si = max(0.0,si-peInt)
          sitf(xcoor,ycoor) = si

        ! The energy used for interception evaporation is no longer available
        ! for transpiration
         ! peTrans = max(0.0, peTrans-eidt)
          !aTrans = max(0.0, aTrans-eidt)


        !else

        case(6, 7, 18, 19, 20, 22, 28, 29)

        ! Non-forested, vegetated cells: as in stand-alone PREVAH, evaporative demand
        ! is represented here by a single variable (EPDT in PREVAH - the corresponding value
        ! has been assigned here to aTrans. Therefore, aTrans does not have the same meaning
        ! here as in forested cells)
          eidt=min(si,(aTrans*vbga))
          si=max(0.0,si-(aTrans*vbga))
          sitf(xcoor,ycoor) = si
          aTrans = max(0.0,aTrans-eidt)

        !end if

C       eidt = min(si,peInt)
C       si = max(0.0,si-peInt)
C       SITF(xcoor,ycoor)=SI

      !end if
      end select



        ! prldifga=prldifga-prl
       prldifga=0.0



! ------------------------------------------------------------------------------
!    Snowmelt module
!
!    Literature:
!    Anderson, E.A. (1973): National Weather Service River Forecast System-Snow
!     Accumulation and Ablation Model. NOAA Tech. Memo. NWS. U.S. Dep. of Commerce,
!     Silver Spring.
!    Braun, L. (1985): Simulation of Snowmelt-Runoff in Lowland and Lower Alpine
!     Regions of Switzerland. Z�rcher Geographische Schriften, Heft 21, ETH Z�rich
!     [available from the Institute for Atmospheric and Climate Science ETH,
!     Winterthurerstr. 190, CH-8057 Z�rich].
!    Hock, R. (1999): Distributed temperature-index ice- and snowmelt model including
!     potential direct solar radiation. J. Glaciol., 45, 101-111.
!    Pos, F. (2001): Spatially distributed modelling of snow accumulation and melt in
!     pre-alpine and alpine catchments. Diploma Thesis, Wageningen University. [available
!     from the Institute for Atmospheric and Climate Science ETH, Winterthurerstr.
!     190, CH-8057 Z�rich].
!    Strasser, U. and Mauser, W. (2001): Modelling the spatial and temporal variations
!     of the water balance for the Weser catchment 1965-1994. J. Hydrol., 254, 199-214.
!    Strasser, U., Etchevers, P. and Lejeune, Y. (2002): Intercomparision of two snow
!     models with different complexity using data from an alpine site. Nordic Hydrol.,
!     33, 15-26.
!    Zappa, M., Pos, F., Strasser, U. and J. Gurtz (2003): Seasonal water balance of
!     an alpine catchment as evaluated by different methods for spatially distributed
!     snow melt modelling. Nordic Hydrology, Vol. 34(3), 179-202.
! ------------------------------------------------------------------------------

! After snow redistribution you could have snow on water
! Here this snow is assigned as liquid precip to the water pixel
      IF (LAND .eq. 1) THEN
        IF (SSO .gt. 0) THEN
          PRL=PRL+SSO
          SSO=0.
          SSNOW=0.
          SLIQ=0.
C          print*, "water", xcoor,ycoor, i, prl, sso, ssnow, sliq
        END IF
      END IF

      IF (ISNOGL .LT. 10) THEN
        IF ((SNOEST .GT. 0) .and. (SNOEST .LE. 8)) THEN

! ------------------------------------------------------------------------------
!    Latent heat flux of snow if SNOEST
! ------------------------------------------------------------------------------
!     This subroutines compute evaporation/condensation from the snow cover
!     if the control parameter SNOEST is higher than 4

          if (snoest .LE. 6) then
C         CALL SNOEVAP1 (TECK,RLF,WIV,EPDT,ssdmax,
C     *	     SNOEST,ESNODT,ESKDT,EIDT,SSO,SSNOW,SLIQ)



! Copied / adapted from snoevap1
             ssoOld = sso
             peSnow = max(peSnow,0.0)
             if((sso .gt. 0.01) .and. (peSnow .gt. 0.0)) then
               eSnow = peSnow
               sso = sso-eSnow
               if(sso .lt. 0.0) then
                 eSnow = ssoOld
                 sso = 0.0
                 ssnow = 0.0
                 sliq = 0.0
               else
                 ssnow = ssnow*(sso/ssoOld)
                 sliq = sliq*(sso/ssoOld)
               end if
             else
               eSnow = 0.0
             end if


          else
	     SURFTEMP=surfsnot(xcoor,ycoor)
	     SURFTEMP=SURFTEMP-273.15

            CALL KWLW1 (RGK,SSDHS,TECK,RLF,HOEHE,ALBEDO,GT,G0,TAG,
     *            MON,JAHR,GEOBREITE,
     *            LANGWSF,KURZWSF,QS,QL,
     *            SLOPE,ASPECT,IRGL,PRL)

            CALL ESCIHELP (TEC,PRL,PSNOW,PLIQ,PHASE,TGR,TTRANS,
     *                     SSO,LAND,CREDV)

            emtfr=EMTF(xcoor,ycoor)
            SURFTEMP=surfsnot(xcoor,ycoor)

            if ((SSO .LT. 0.01).AND.(PSNOW.LT.0.001)) then
              SURFTEMP=273.16
              emtfr=0.0
	       sso=0.0
	       ssnow=0.0
	       sliq=0.0
            else
	       ssoa=sso
              CALL SNOWFLUX (SNOEST,PRL,TECK,WIV,RLF,RGK,QL,
     *              SSO,PHASE,WSB,SURFTEMP,WSA,emtfr,ALBEDO,vaps,vap)
	       kondens=wsb
	       IF (kondens .LT. 0.00001) kondens=0.0
	       WSB=WSB*(-1)
	          if (TECK .GT. T0) then
                SURFTEMP=273.16
              end if
		   !EIDT=0 ! COMMENTED OUT AS DEBUG - Matthias - Apr 15

C	       if (WSB .GT. 0) then
C                EIDT=WSB
C              end if
C
C	       if (EIDT .GT. EPDT) then
C                EPDT=EIDT
C	       end if

          EMTF(xcoor,ycoor)=emtfr
          surfsnot(xcoor,ycoor)=surftemp

            end if  ! sso .lt. 0.01 .and. psnow .lt. 0.001

            if (SSO .GE. 0.01) then
              SNOW=1
            else
              snow=0
            end if

  	        if ( SSO .LE. 0.0 ) then
              SSO=0.0
              SSNOW=0.0
              SLIQ=0.0
	         else if (ssoa .GT. 0) then
	           SSNOW=SSNOW*(SSO/SSOA)
               SLIQ=SLIQ*(SSO/SSOA)
             end if

          end if  ! snoest .le. 6 ... else
        end if  ! snoest between 0 and 8

! ------------------------------------------------------------------------------
!     Snowmelt
! ------------------------------------------------------------------------------
! Anderson - Braun - Hock are resolved in SNOMELT2



       if(prl.ne.prl) then
         prl = 0.0
         pakt(i) = 0.0
       end if

        CALL SNOMELT2 (TAG,MON,JAHR,LAND,ISNOGL,TEC,TECK,RDMK,RLF,WIV,
     *     PRL,NDTD,CREDV,TGR,TTRANS,T0,M,CRMFMAX,CRMFMIN,CWU1,CWA2,
     *     CRFR,CWH,CEXLIQ,PRECMIN,CSNOMF,CASNO,SNOW,SSO,SSNOW,SLIQ,
     *     WSA,WSB,DAT,DKAL,MDR,DEBUG,kk)


        if (WSB .LT. 0) then
C	      EIDT=EIDT-WSB
            eSnow = eSnow-wsb
	      WSB=0
	      end if



      end if  ! isnogl .lt. 10


! Escimo
      if (ISNOGL .EQ. 10) then
	 SURFTEMP=surfsnot(xcoor,ycoor)
	 SURFTEMP=SURFTEMP-273.15

        CALL KWLW1 (RGK,SSDHS,TECK,RLF,HOEHE,ALBEDO,GT,G0,TAG,
     *            MON,JAHR,GEOBREITE,
     *            LANGWSF,KURZWSF,QS,QL,
     *            SLOPE,ASPECT,IRGL,PRL)

        CALL ESCIHELP (TEC,PRL,PSNOW,PLIQ,PHASE,TGR,TTRANS,
     *                     SSO,LAND,CREDV)

        energy=snowen(xcoor,ycoor)
        emtfr=EMTF(xcoor,ycoor)
        SURFTEMP=surfsnot(xcoor,ycoor)

        if ((SSO .LT. 0.01).and.(PSNOW.LT.0.001)) then
          ENERGY=0
          SURFTEMP=273.16
          WSA=0.0
          emtfr=0.0
        else
          CALL ESCIMO1 (PRL,TECK,WIV,RLF,RGK,QL,
     *              SSO,PHASE,ENERGY,WSA,WSB,SURFTEMP,emtfr,A0,
     *              APOS,ANEG,ALBEDO,ALBADD)

          PRL=WSA
          if (WSB .LT. 0) then
	       EIDT=-WSB
	       WSB=0
	     end if
        end if   ! ((SSO .LT. 0.01).and.(PSNOW.LT.0.001))

        wsa = max(wsa, 0.0)

        EMTF(xcoor,ycoor)=emtfr
        snowen(xcoor,ycoor)=energy
        surfsnot(xcoor,ycoor)=surftemp
	  albhh(I)=albedo
        if (sso .ge. 0.01) then
          snow = 1
        else
          snow = 0
        end if
        sso = max(sso, 0.0)

      end if  ! isnogl==10

      MTF(xcoor,ycoor)=M
      SNOWTF(xcoor,ycoor)=SNOW
      SSOTF(xcoor,ycoor)=SSO
      SSNOWTF(xcoor,ycoor)=SSNOW
      SLIQTF(xcoor,ycoor)=SLIQ

! ------------------------------------------------------------------------------
!     Cumulative days with snow and Final Snow cover Day
!     Snow if SWE > 20 mm
!     Final snow ablation only if at least an equivalent of 25 days with snow
!     Final snow ablation date is fixed if after a certain date the threshold of
!     20 mm is not passed for 20 consecutive days

C     ssotr=5.0    !20.0
C     ssodur=20    !20.0
C     ssocum=10    !25
C     ssocum=(ssocum)*(ssocum+1)/2
C
C     if ((SSO .GT. ssotr).AND.(I .eq. 12)) then
C       snowdaystf(xcoor,ycoor,1)=snowdaystf(xcoor,ycoor,1)+1
C fsad_tf(xcoor,ycoor,1)=fsad_tf(xcoor,ycoor,1)+1
C fsad_tf(xcoor,ycoor,4)=fsad_tf(xcoor,ycoor,4)+1
C fsad_tf(xcoor,ycoor,5)=fsad_tf(xcoor,ycoor,5)+
C    *                       fsad_tf(xcoor,ycoor,4)
C
C     else if ((SSO .LE. ssotr).AND.(I .eq. 12)) then
C fsad_tf(xcoor,ycoor,4)=0
C     end if
C
C     if (fsad_tf(xcoor,ycoor,2) .LT. ssodur) then   ! 20 consecutive days with snow below 20 mm
C
C  if ((SSO .GT. ssotr).AND.(I .eq. 12)) then
C         fsad_tf(xcoor,ycoor,2)=0
C       end if
C
C       if ((SSO .LT. ssotr).AND.(I .eq. 19)) then
C
C         if (fsad_tf(xcoor,ycoor,5) .GT. ssocum) then  ! 25 consecutive days or an equivalent of it
C      jjs='N'
C      if ((TAG .eq. 29) .and. (MON .eq. 2)) jjs='Y'
C      call tagzahl (TAG,MON,TANRR,jjs)
C
C      if (MON .LE. 9) then
C             if (fsad_tf(xcoor,ycoor,2) .EQ. 0) then
C               fsad_tf(xcoor,ycoor,3)=tanrr
C             end if
C             fsad_tf(xcoor,ycoor,2)=fsad_tf(xcoor,ycoor,2)+1
C           end if
C
C    else
C           if (MON .LE. 9) then
C             fsad_tf(xcoor,ycoor,3)=0
C           end if
C    end if
C  end if
C     end if

! Get snow-off day (= day when snow gets below a fixed threshold, set here to 5 mm SWE).
! Snow-off day is updated after a period of at least 10 days above threshold.

      if((mon.le.9).and.(i==12)) then

      ssotr = 5.0
      ssodur = 10.0

      if(jahr==int(float(jahr)/4.)*4) then
        jjs = 'Y'
      else
        jjs = 'N'
      end if
C


      if((sso.gt.ssotr).and.(i==12)) then
        snowdays(xcoor,ycoor) = snowdays(xcoor,ycoor) + 1
      else if((sso .le. ssotr).and.(i==12)) then
        if(snowdays(xcoor,ycoor).ge.ssodur) then
          call tagzahl(tag,mon,tanrr,jjs)  ! Get DOY
          snowoff(xcoor,ycoor) = tanrr
          snowdays(xcoor,ycoor) = 0

        end if
      end if

      if((mon==9).and.(tag==30)) then
        if((snowoff(xcoor,ycoor)==0).and.(sso .gt. ssotr)) then
          snowoff(xcoor,ycoor) = 273
        end if
        snowoffold(xcoor,ycoor) = snowoff(xcoor,ycoor)
      end if

      end if !mon .le. 9

! ------------------------------------------------------------------------------
! Frost days risk
! Between April and July
! No snow
! Temperature < -2�C

!      IF ((SSO .LE. 20.0).AND.(I .eq. 12)) THEN
!	if (tekdm .lt. -2) then
!	if ((mon .ge. 4) .and. (mon .le. 7)) then
!	frost_tf(xcoor,ycoor)=frost_tf(xcoor,ycoor)+1
!	end if
!	end if
!	END IF


! ------------------------------------------------------------------------------
!    Icemelt module
!
!    Literature
!    Badoux, A. (1999): Untersuchung zur fl�chendifferenzierten Modellierung von
!     Abfluss und Schmelze in teil-vergletscherten Einzugsgebieten. Diploma thesis,
!     ETH Z�rich [available from the Institute for Atmospheric and Climate Science ETH,
!     Winterthurerstr. 190, CH-8057 Z�rich].
!    Hock, R. (1999): Distributed temperature-index ice- and snowmelt model including
!     potential direct solar radiation. J. Glaciol., 45, 101-111.
!    Klok, E.J., Jasper, K., Roelofsma, K.P., Badoux, A. and Gurtz, J. (2001): Distributed
!     hydrological modelling of a glaciated Alpine river basin. Hydrological Sciences
!     Journal, 46, 553-570.
!    Zappa, M., Badoux, A., and Gurtz, J. (2000): The application of a complex distributed
!     hydrological model in an highly glaciated alpine river catchment. In: Horvatic, J.
!     (Editor): Proceedings of the 33rd Conference of International Association for Danube
!     Research, Limnological Reports, 33, 23-28.
! ------------------------------------------------------------------------------

      SICE=SICETF(xcoor,ycoor)
      SICS=SICSTF(xcoor,ycoor)
      SICF=SICFTF(xcoor,ycoor)

      hfice(1:24) = htfice(xcoor,ycoor,1:24)
      hfics(1:24) = htfics(xcoor,ycoor,1:24)
      hficf(1:24) = htficf(xcoor,ycoor,1:24)

84    format (I3,7f7.1)


      if((glacID(xcoor,ycoor) .ne. int(rnodat))
     * .and. (jahr.ge.1971) .and. (glacSwitch==1)) then  ! New GLACMO routine: only for cells that are part of the transient glacier module, and not during groundhog spin-up

        call glacmoTrans(xcoor,ycoor,prl,teck,rdmk,cicemf,caice,erice,
     *      kiceh,kicsh,kicfh,ticeh,ticsh,ticfh,wice,wics,wicf,rice,
     *      rics,ricf,sso,sice,sics,sicf,hfice,hfics,hficf)

      else
        CALL GLACMO (LAND,HOEHE,HELA,DEBUG,MDR,SSO,
     *                    PRL,TECK,RDMK,CICEMF,CAICE,ERICE,
     *                    KICEH,KICSH,KICFH,TICEH,TICSH,TICFH,
     *                    SICE,SICS,SICF,HFICE,HFICS,HFICF,
     *                    WICE,WICS,WICF,RICE,RICS,RICF)

      end if



      SICETF(xcoor,ycoor)=SICE
      SICSTF(xcoor,ycoor)=SICS
      SICFTF(xcoor,ycoor)=SICF


      IF (rice3 .LT. 0.00001) r3=0.0

! GRID VERSION *ATFLK was originally here
      rices=rices+rice/wstats(1)

      htfice(xcoor,ycoor,1:24) = hfice(1:24)
      htfics(xcoor,ycoor,1:24) = hfics(1:24)
      htficf(xcoor,ycoor,1:24) = hficf(1:24)

! ------------------------------------------------------------------------------
!    Soil moisture and runoff-generation module
!
!    Literature
!    Bergstr�m, S. (1976): Development and Application of a Conceptual Runoff Model
!     for Scandinavian Catchments. Bulletin Series A, No. 52, University of Lund.
!    Bergstr�m, S. (1992): The HBV model - its structure and applications. SMHI, RH
!     No. 4, Norrk�ping, Sd.
!    Gurtz, J., Zappa, M., Jasper, K., Lang, H., Verbunt, M., Badoux, A. and
!     Vitvar, T. (2003): A Comparative Study in Modelling Runoff and its Components
!     in Two Mountainous Catchments. Hydrol. Processes, 17, 297-311.
!    Schwarze, R., Droege, W. and Opherden, K. (1999): Regional analysis and modelling
!     of groundwater runoff components from catchments in hard rock areas.
!     In: Diekkr�ger B, Kirkby MJ, Schr�der U (Editors): Regionalisation in hydrology,
!     IAHS Publication No. 254, 221-232.
!    Zappa, M. and Gurtz, J. (2003): Plot-scale simulation of soil moisture and
!     evapotranspiration during the 1999 MAP-Riviera campaign.
!     Hydrology and Earth System Sciences. Accepted.
! ------------------------------------------------------------------------------

      if (ISNOGL .EQ. 10) then
        epdt = max(epdt, eidt)
      end if

!      EPBDT=EPDT-EIDT

      IF (SSO .GT. 0.1) SGRLP=0.0     ! No transpiration if snow is present, Evaporation always at potential rate
      SSM=SSMTF(xcoor,ycoor)
      SSZ=SSZTF(xcoor,ycoor)
      SUZ=SUZTF(xcoor,ycoor)
      SLZ=SLZTF(xcoor,ycoor)
      SLZ1=SLZ1TF(xcoor,ycoor)
      SLZ2=SLZ2TF(xcoor,ycoor)
      SLZ3=SLZ3TF(xcoor,ycoor)
      TOPI=FTOPI(xcoor,ycoor)
      BTOPI=FBTOPI(xcoor,ycoor)
      KWERT=VKWERT(xcoor,ycoor)
      !kwert=fck(xcoor,ycoor)
      PERC=0

!      ASECA=ASEC
      ERICE=0.0

! Fractions of land and water for special land cover types (open water, firn,
! glacier, wetlands, built-up, rock...). For all other LC types, there is only
! land.
      select case(land)
        case(1)
          aseca = 0.1
          epbdt = epdt-eidt
        case(13, 14)
          aseca = 1.0
          epbdt = 0.0
        case(2, 15, 26)
          aseca = asec
          epbdt = epdt*vbga  ! VBG = vegetated surface (0 ... 1)
        case default
          aseca = asec
          epbdt = epdt-eidt
      end select


      !prldifga=prl+prldifga
      prldifga=0.0


      select case(land)
        case (1)
          asec = 0.1
        case (22)
          asec = 0.3
        case default
          asec = 1.0
      end select


      call abfgPatch(prl, rice, rics, ricf, aeSoil, aTrans,
     *     peInt, asec, ssm, sfc, btopi,
     *     topmin, topmit, kwert, kwminl, kwmitl,sgrlp, cperc,cbeta,
     *     sgrluz,k0,k1, k3, cg1h, cg2h, slz1max, ssz, suz,slz,slz1,
     *     slz2,slz3,rl,rges, r0, r1, r2, r3, rg1, rg2, rg3, eSoil,
     *     eTrans,eWater, perc, dsuz, iart, land, aelhg,xcoor,ycoor,kwp)

      kwp2d(xcoor,ycoor)= kwp

      if((xcoor==69) .and. (ycoor==97) .and. (jahr==1676)) then
        print*, "------"
        print*, "After ABFG 69 97"
        print*, "ssm", ssm
        print*, "sfc", sfc
        print*, "eTrans", eTrans
        print*, "eSoil", eSoil
        print*, "suz", suz
        print*, "dsuz", dsuz
      end if

      if((xcoor==11) .and. (ycoor==164) .and. (jahr==1676)) then
        print*, "------"
        print*, "After ABFG 11 164"
        print*, "ssm", ssm
        print*, "sfc", sfc
        print*, "eTrans", eTrans
        print*, "eSoil", eSoil
        print*, "suz", suz
        print*, "dsuz", dsuz
      end if




      IF (r0 .LT. 0.00001) r0=0.0
      IF (r1 .LT. 0.00001) r1=0.0
      IF (r2 .LT. 0.00001) r2=0.0
      IF (r3 .LT. 0.00001) r3=0.0

	IF (GRDAKT .EQ. 'S') THEN
	   XX(I,xcoor,ycoor)=dsuz
	   dsuz=0.0
      ENDIF

      SSMTF(xcoor,ycoor)=SSM
	SFCTF(xcoor,ycoor)=SFC
      SSZTF(xcoor,ycoor)=SSZ
      SUZTF(xcoor,ycoor)=SUZ
      SLZTF(xcoor,ycoor)=SLZ
      SLZ1TF(xcoor,ycoor)=SLZ1
      SLZ2TF(xcoor,ycoor)=SLZ2
      SLZ3TF(xcoor,ycoor)=SLZ3
      SUZ=SUZ+SSZ
      ebdt = aTrans + aeSoil
      ERDT=EIDT+EBDT



! cumulative monthly soil moisture deficit
      fsad_tf(xcoor,ycoor,6)=fsad_tf(xcoor,ycoor,6)+
     *                      max(0.,(SFC-SSM))
!

      IF (numohru .gt. 0) THEN
	 do llhru=1,numohru
	   IF (kk .ne. idnumhru(llhru)) cycle
	   unit=7500+llhru
          allocate(intArgs(6))
          intArgs=(/jahr,mon,tag,i,ncol,nrow/)
          allocate(writeArgs(19))
          allocate(writeArgOpts(19))
          writeArgs=(/pnakt(i),pakt(i),wsb,epdt,erdt,r0,r1,r2,rges,
     *                sso,si,ssm,suz,slz,ssz,perc,rice,rics,ricf/)
          writeArgOpts=1
          WRITE (unit,40) JAHR,MON,TAG,I,ncol,nrow,PNAKT(I),PAKT(I),
     *        WSB,EPDT,ERDT,R0,R1,R2,RGES,SSO,SI,SSM,SUZ,SLZ,SSZ,
     *        PERC,RICE,RICS,RICF
          deallocate(intArgs)
          deallocate(writeArgs)
          deallocate(writeArgOpts)
	 end do
      END IF

	dqmax=max(dqmax,RGES)



! ------------------------------------------------------------------------------
!    Writing hourly output for every HRU
! ------------------------------------------------------------------------------
40     FORMAT (  I5,3I3,2I6,9F8.3,6F8.2,4F8.4)
      IF ((ZIERG(9:9) .EQ. 'y') .OR. (ZIERG(9:9) .EQ. 'Y'))
     *  WRITE (MDHOU,40) JAHR,MON,TAG,I,ncol,nrow,PNAKT(I),PAKT(I),
     *        WSB,EPDT,ERDT,R0,R1,R2,RGES,SSO,SI,SSM,SUZ,SLZ,SSZ

	IF ((ZIERG(15:15) .EQ. 'y') .OR. (ZIERG(15:15) .EQ. 'Y')) THEN
	runsubca(i,max(1,scatch(xcoor,ycoor)))=
     *             runsubca(i,max(1,scatch(xcoor,ycoor)))+
     *             RGES*rgridsize*rgridsize/1000/1000/
     *             areasubca(max(1,scatch(xcoor,ycoor)))

	runsubca(i,999)=runsubca(i,999)+RGES*rgridsize*rgridsize/
     *               1000/1000/aelhg

	END IF
! ------------------------------------------------------------------------------
!    Writing hourly output for internal sub basins (JS:2008-04)
! ------------------------------------------------------------------------------

!	IF(isbyn) CALL procisb

! ------------------------------------------------------------------------------
!    End of day - Add to daily sums and build hourly values for the whole catchment
! ------------------------------------------------------------------------------


      include 'addday.inc'

      IF ((tcali .eq. 't') .or. (tcali .eq. 'T')) THEN
        IF ((JAHR.EQ.JAHRE).AND.(MON.EQ.MONE).AND.(TAG.EQ.TAGE)) THEN
	  IF (I .lt. hourse) goto 120
	  iee=hourse
        ELSE IF (I .LT. NDTD) THEN
	  iee=24
	  GOTO 120					! Hourly loop
        END IF
      ELSE IF (hourse .gt. 0) THEN
        IF ((JAHR.EQ.JAHRE).AND.(MON.EQ.MONE).AND.(TAG.EQ.TAGE)) THEN
          IF (I .lt. hourse) goto 120                   ! Hourly loop
          iee=hourse
        ELSE IF (I .LT. NDTD) THEN
	  iee=24
	  GOTO 120					! Hourly loop
        END IF
      ELSE IF (I .LT. NDTD) THEN
	iee=24
	GOTO 120					! Hourly loop
      END IF

! End of day
      DATUM=((JAHRK)*100.0)+MON+(TAG*0.01)

      if(cumulRain .lt. critRain) drydays(xcoor,ycoor) =
     *  drydays(xcoor,ycoor) + 1

      eeqrmean(xcoor,ycoor) = (eeqrmean(xcoor,ycoor)*15/16) + eeqds/16.
      pefrmean(xcoor,ycoor) =(pefrmean(xcoor,ycoor)*15/16)+cumulRain/16.


! ------------------------------------------------------------------------------
! For glacier melting season (1st June - 1st October): get minimum snow cover on
! firn cells.
! ------------------------------------------------------------------------------


      if ((land == 13) .or. (land == 14)) then
        if ((tag == 1) .and. (mon == 6)) then
          minSnowCover(xcoor,ycoor) = ssnowtf(xcoor,ycoor)
        else if((mon == 6) .or. (mon == 7) .or. (mon == 8) .or.
     *   (mon == 9)) then
          minSnowCover(xcoor,ycoor) = min(minSnowCover(xcoor,ycoor),
     *     ssnowtf(xcoor,ycoor))
        end if
      end if  ! land == 13

! ------------------------------------------------------------------------------
!     Updating daily, monthly, yearly and total sums adding subsequently the
!     values of each HRU (xcoor,ycoor).
! ------------------------------------------------------------------------------

C       CALL ADMSHYD (xcoor,ycoor,ATFL,ncol,nrow,PNDS,PAKDS,WSADS,EPDS,
C     *       ERDS,EIDS,EBDS,
C     *       R0DS,R1DS,R2DS,RGEDS,PNKSM,PAKSM,WSASM,EPSM,ERSM,
C     *       EISM,EBSM,R0SM,R1SM,R2SM,RGESSM,
C     *       PNKMS,PAKMS,WSAMS,EPMS,ERMS,EIMS,EBMS,R0MS,R1MS,R2MS,RGEMS,
C     *       PNKJS,PAKJS,WSAJS,EPJS,ERJS,EIJS,EBJS,EBJS2,R0JS,R1JS,R2JS,
C     *       RGEJS,PNKS,PAKS,WSAS,EPS,ERS,EIS,EBS,R0S,R1S,R2S,RGESS,
C     *       GWNBS,GWNBDS,GWNBJS,GWNBMS,GLACS,GLACMS,GLACJS,GLACDS,
C     *       GLACSS,wstats(1))

      call admshydPatch  (xcoor,ycoor,ATFL,ncol,nrow,
     *          PNDS,PAKDS,WSADS,EPDS,ERDS,EIDS,EBDS,R0DS,
     *          R1DS,R2DS,RGEDS,petDS,eSoilDS,eTransDS,
     *          eSnowDS,eIntDS,eWaterDS,peSoilDS,peTransDS,
     *          PNKSM,PAKSM,WSASM,EPSM,ERSM,EISM,EBSM,R0SM,R1SM,
     *          R2SM,RGESSM,
     *          PNKMS,PAKMS,WSAMS,EPMS,ERMS,EIMS,EBMS,R0MS,R1MS,
     *          R2MS,RGEMS,petMS,eSoilMS,eTransMS,eSnowMS,
     *          eIntMS,eWaterMS,peSoilMS, peTransMS,
     *          PNKJS,PAKJS,WSAJS,EPJS,ERJS,EIJS,EBJS,EBJS2,R0JS,
     *          R1JS,R2JS,RGEJS,petJS,eSoilJS,eTransJS,
     *          eSnowJS,eIntJS,eWaterJS, peSoilJS,peTransJS,
     *          PNKS,PAKS,WSAS,EPS,ERS,EIS,EBS,R0S,R1S,R2S,RGESS,
     *          GWNBS,GWNBDS,GWNBJS,GWNBMS,GLACS,GLACMS,GLACJS,
     *          GLACDS,GLACSS,wstats(1))
      ATFLK=ATFL(xcoor,ycoor)*0.01




! ------------------------------------------------------------------------------
! FORHYCS: Drought stress index  -  Matthias, July 2015
! ------------------------------------------------------------------------------

! If PT==0, set a minimum dummy value to prevent NaNs. Occurs e.g. if
! a cell is covered in snow all year. Not realistic to have trees grow there,
! but error is probably small (growth limited by low DDEGsum). MS, April 2017
      if(peTransJS(xcoor,ycoor).lt.0.1) then
        peTransJS(xcoor,ycoor) = 0.1
        eTransJS(xcoor,ycoor) = 0.1
      end if

      drstr(xcoor,ycoor) = 1.0-(eTransJS(xcoor,ycoor)
     * /peTransJS(xcoor,ycoor))


! FORHYCS: (Bio)climatic Indicators for root depth module
      if((stock).and.(TMtoPR==1)) then
      !if((land==3).or.(land==4).or.(land==5).or.(land==8)) then
        if(gs) then
          gsLengthCur(xcoor,ycoor) = gsLengthCur(xcoor,ycoor) + 1
          gsTpotCur(xcoor,ycoor) = gsTpotCur(xcoor,ycoor) + peTransDS
          gsPeCur(xcoor,ycoor) = gsPeCur(xcoor,ycoor) + peDS
          gsPrecCur(xcoor,ycoor) = gsPrecCur(xcoor,ycoor) + cumulRain
          gsPesCur(xcoor,ycoor) = gsPesCur(xcoor,ycoor) + peSoilDS
          if(cumulRain.gt.0.5) then
            if(prevDayP(xcoor,ycoor)==0) then
            ! Precipitation event defined as a series of consecutive
            ! rainy days. Increment event counter in case of a new event.
            gsPrecDaysCur(xcoor,ycoor) = gsPrecDaysCur(xcoor,ycoor) + 1
            end if
            prevDayP(xcoor,ycoor) = 1
          else
            prevDayP(xcoor,ycoor) = 0
          end if  ! Rainy day
          gsTempCur(xcoor,ycoor) = gsTempCur(xcoor,ycoor) + tekdm
        end if
      !end if
      end if ! (stock).and.(TMtoPR==1)

! ------------------------------------------------------------------------------
!    Determination of the water balance of each HRU.
! ------------------------------------------------------------------------------
      CALL BILANZ (GEBIL,PAKS(xcoor,ycoor),ERS(xcoor,ycoor),
     * RGESS(xcoor,ycoor),SSOB,SSOTF(xcoor,ycoor),
     * SSOD,SIB,SITF(xcoor,ycoor),SID,
     * FBFEUCHT(xcoor,ycoor),SSMTF(xcoor,ycoor),SSMD,
     * SUZB,SUZTF(xcoor,ycoor),SUZD,SLZB,SLZTF(xcoor,ycoor),SLZD)

! TEMPORARY (Matthias - Jan 2015): get daily mass balance of glaciers

! Got the following line out of IF block to get MTD balance for all cells,
! not only glaciers
       tempsnow(xcoor,ycoor) = ssotf(xcoor,ycoor)

      if(glacmask(xcoor,ycoor)==1) then
        CALL BILANZ (GEBIL,PAKdS,ERdS,
     * RGEdS,tempsso,SSOTF(xcoor,ycoor),
     * SSOD,tempsi,SITF(xcoor,ycoor),SID,
     * tempssm,SSMTF(xcoor,ycoor),SSMD,
     * tempsuz,SUZTF(xcoor,ycoor),SUZD,tempslz,SLZTF(xcoor,ycoor),SLZD)

       glacpd = glacpd + (PAKdS / countGlac)
       glaced = glaced + (ERdS / countGlac)
       glacqd = glacqd + (RGEdS / countGlac)
       glacdssd = glacdssd + (ssod / countGlac)
       glacdsid = glacdsid + (sid / countGlac)
       glacdsmd = glacdsmd + (ssmd / countGlac)
       glacdsuzd = glacdsuzd + (suzd / countGlac)
       glacdslzd = glacdslzd + (slzd / countGlac)
       glacbild = glacbild + (gebil / countGlac)

!       tempsnow(xcoor,ycoor) = ssotf(xcoor,ycoor)

       glacpy = glacpy + (pakds / countGlac)
       glacey = glacey + (erds / countGlac)
       glacqy = glacqy + (rgeds / countGlac)
       glacdssy = glacdssy + (ssod / countGlac)
       glacdsiy = glacdsiy + (sid / countGlac)
       glacdsmy = glacdsmy + (ssmd / countGlac)
       glacdsuzy = glacdsuzy + (suzd / countGlac)
       glacdslzy = glacdslzy + (slzd / countGlac)
       glacbily = glacbily + (gebil / countGlac)
       glacbily2d(xcoor,ycoor) = glacbily2d(xcoor,ycoor) + gebil



      end if



! ------------------------------------------------------------------------------
!    Writing daily output for every HRU - If wished clear ! and recompile
! ------------------------------------------------------------------------------

42      FORMAT ( I5,I3,I3,2I6,7F8.3,11F8.2)
      IF ((ZIERG(11:11) .EQ. 'y') .OR. (ZIERG(11:11) .EQ. 'Y'))
     * WRITE (MDTAG,42) JAHR,MON,TAG,nrow,ncol,PNDS,PAKDS,WSADS,EPDS,
     *    ERDS,EIDS,EBDS,R0DS,R1DS,R2DS,RGEDS,SSO,SI,SSM,SUZ,SLZ,GEBIL,
     *    GLACDS

      rgs2d(xcoor,ycoor) = rgeds
      prec2d(xcoor,ycoor) = pakds

      select case (land)
        case(3,4,5,8,21,23,24,25,26)  ! Forest or potential
          pt2d(xcoor,ycoor) = peTransDS
          at2d(xcoor,ycoor) = eTransDS
          pes2d(xcoor,ycoor) = peSoilDS
          aes2d(xcoor,ycoor) = eSoilDS
          int2d(xcoor,ycoor) = eIntDS
          esn2d(xcoor,ycoor) = eSnowDS
          enf2d(xcoor,ycoor) = 0.0

C         if(eTransDS.ne.eTransDS) then
C           print*, "NaNaNa", xcoor, ycoor, land
C         end if

        case(6,7,18,19,20,28,29)  ! Agricultural LC types
          pt2d(xcoor,ycoor) = 0.0
          at2d(xcoor,ycoor) = 0.0
          pes2d(xcoor,ycoor) = peTransDS
          aes2d(xcoor,ycoor) = eTransDS
          int2d(xcoor,ycoor) = eIntDS
          esn2d(xcoor,ycoor) = eSnowDS
          enf2d(xcoor,ycoor) = 0.0

        case default  ! Special LC types
          pt2d(xcoor,ycoor) = 0.0
          at2d(xcoor,ycoor) = 0.0
          pes2d(xcoor,ycoor) = 0.0
          aes2d(xcoor,ycoor) = 0.0
          int2d(xcoor,ycoor) = 0.0
          esn2d(xcoor,ycoor) = 0.0
          enf2d(xcoor,ycoor) = eSoilDS + eSnowDS + eIntDS
      end select

!    Setting up daily gridded output
      IF (GRDAKT .EQ. 'Y') THEN
         INCLUDE 'gridtin.inc'
      ENDIF

! ------------------------------------------------------------------------------
!    Setting daily sum for each HRU to 0, end of the day for the current HRU
! ------------------------------------------------------------------------------

      include 'setday.inc'
! ------------------------------------------------------------------------------
!    End of Daily CORE DO_LOOP
! ------------------------------------------------------------------------------

! Output for TreeMig/Bioclim: mean monthly temperature, based on daily values,
! adjusted for slope and exposition.

      if((mon==2) .and. (jahr==int(float(jahr)/4.)*4)) then  ! Leap years
        meanTOut(xcoor,ycoor,mon) = meanTOut(xcoor,ycoor,mon)+tekdm/29.
      else if(mon==2) then ! February, non-leap year
        meanTOut(xcoor,ycoor,mon) = meanTOut(xcoor,ycoor,mon)+tekdm/28.
      else
       meanTOut(xcoor,ycoor,mon) = meanTOut(xcoor,ycoor,mon)
     *  +tekdm/monum(mon)
      end if
! Output for TreeMig/Bioclim: monthly precip sums, based on daily values,
! with gauge correction and water balance adjustment.
      sumPOut(xcoor,ycoor,mon) = sumPOut(xcoor,ycoor,mon)+sum(pakt)/10.       ! Output in cm

	  end do
	  end do

	  end if  ! End of Slaves part
! ------------------------------------------------------------------------------
! Dr. Thomas "Eddy" W�hling - Updating procedure
! September-October 2005
! Adapted by M.Zappa for use in every basin
! ------------------------------------------------------------------------------

      if (douptol) then
      include 'updating_core_3.inc'
	end if


! Write annual maximal LAI for each year

      if((jahr.ge.1970) .and. (mon == 3) .and. (tag == 1)) then
      call MPI_BCAST(posiid1,1,MPI_INTEGER,1,MPI_COMM_WORLD,ierr)
      wrgrd = fhla(:,:,32)

      write(chyr,'(I4)') jahr

        if(rank .ne. root) then
          call MPI_SEND(wrgrd(:,firstRow:lastRow),ncol*nrowLocal,
     *    MPI_REAL,root,22,MPI_COMM_WORLD,ierr)
        else  ! Master
          wrgrd = rnodat

          do j=1,numslaves
            call MPI_RECV(wrgrd(:,firstRowArray(j):lastRowArray(j)),
     *          ncol*nrowLocalArray(j),MPI_REAL,j,22,
     *          MPI_COMM_WORLD,status,ierr)
          end do

              namgrd=diroutput(1:posi3)//idnam(posiid1:posiid2)//
     *          chyr//".LAI"
         OPEN (732,FILE=namgrd,STATUS='unknown',ERR=999,FORM='BINARY')
         uwas=732
	   include 'wstsp.inc'
        end if  ! Slaves / master
      call MPI_BARRIER(MPI_COMM_WORLD,ierr)

! Write vegetation height - maximum

      wrgrd = vh100

        if(rank .ne. root) then
          call MPI_SEND(wrgrd(:,firstRow:lastRow),ncol*nrowLocal,
     *    MPI_REAL,root,22,MPI_COMM_WORLD,ierr)
        else  ! Master
          wrgrd = rnodat

          do j=1,numslaves
            call MPI_RECV(wrgrd(:,firstRowArray(j):lastRowArray(j)),
     *          ncol*nrowLocalArray(j),MPI_REAL,j,22,
     *          MPI_COMM_WORLD,status,ierr)
          end do

              namgrd=diroutput(1:posi3)//idnam(posiid1:posiid2)//
     *          chyr//".VHE"
         OPEN (732,FILE=namgrd,STATUS='unknown',ERR=999,FORM='BINARY')
         uwas=732
	   include 'wstsp.inc'
        end if  ! Slaves / master

! Write vegetation height - 99th percentile
      wrgrd = vh99

        if(rank .ne. root) then
          call MPI_SEND(wrgrd(:,firstRow:lastRow),ncol*nrowLocal,
     *    MPI_REAL,root,22,MPI_COMM_WORLD,ierr)
        else  ! Master
          wrgrd = rnodat

          do j=1,numslaves
            call MPI_RECV(wrgrd(:,firstRowArray(j):lastRowArray(j)),
     *          ncol*nrowLocalArray(j),MPI_REAL,j,22,
     *          MPI_COMM_WORLD,status,ierr)
          end do

              namgrd=diroutput(1:posi3)//idnam(posiid1:posiid2)//
     *          chyr//".V99"
         OPEN (732,FILE=namgrd,STATUS='unknown',ERR=999,FORM='BINARY')
         uwas=732
	   include 'wstsp.inc'
        end if  ! Slaves / master

! Write vegetation height - 95th percentile
      wrgrd = vh95

        if(rank .ne. root) then
          call MPI_SEND(wrgrd(:,firstRow:lastRow),ncol*nrowLocal,
     *    MPI_REAL,root,22,MPI_COMM_WORLD,ierr)
        else  ! Master
          wrgrd = rnodat

          do j=1,numslaves
            call MPI_RECV(wrgrd(:,firstRowArray(j):lastRowArray(j)),
     *          ncol*nrowLocalArray(j),MPI_REAL,j,22,
     *          MPI_COMM_WORLD,status,ierr)
          end do

              namgrd=diroutput(1:posi3)//idnam(posiid1:posiid2)//
     *          chyr//".V95"
         OPEN (732,FILE=namgrd,STATUS='unknown',ERR=999,FORM='BINARY')
         uwas=732
	   include 'wstsp.inc'
        end if  ! Slaves / master

! Get dominant tree species and write grid - code block for getting dominant
! species shifted up (just above phenology module) to allow phenodates output

      domSP = 0.0

      if(rank .ne. root) then

C       do ycoor = lastRow, firstRow, -1
C       do xcoor = 1,ncol
C       if((landin(xcoor,ycoor) == 3).or.(landin(xcoor,ycoor) == 4).or.
C    * (landin(xcoor,ycoor) == 5).or.(landin(xcoor,ycoor) == 8)) then
C         domSP(xcoor,ycoor) =
C    *     sum(real(maxloc(biomass2d(xcoor,ycoor,1:30))))
C       end if
C
C       end do
C       end do

      end if  ! Slaves

        wrgrd = domSP

        if(rank .ne. root) then
          call MPI_SEND(wrgrd(:,firstRow:lastRow),ncol*nrowLocal,
     *    MPI_REAL,root,22,MPI_COMM_WORLD,ierr)
        else  ! Master
          wrgrd = rnodat

          do j=1,numslaves
            call MPI_RECV(wrgrd(:,firstRowArray(j):lastRowArray(j)),
     *          ncol*nrowLocalArray(j),MPI_REAL,j,22,
     *          MPI_COMM_WORLD,status,ierr)
          end do

              namgrd=diroutput(1:posi3)//idnam(posiid1:posiid2)//
     *          chyr//".DSP"
         OPEN (732,FILE=namgrd,STATUS='unknown',ERR=999,FORM='BINARY')
         uwas=732
	   include 'wstsp.inc'
        end if  ! Slaves / master

      end if  ! March 1st

! Write snow-off DOY
      if((jahr==1970) .and. (mon == 9) .and. (tag == 30)) then
        call MPI_BCAST(posiid1,1,MPI_INTEGER,1,MPI_COMM_WORLD,ierr)
        wrgrd = fsad_tf(:,:,3)

        write(chyr,'(I4)') jahr


              if(rank .ne. root) then
          call MPI_SEND(wrgrd(:,firstRow:lastRow),ncol*nrowLocal,
     *    MPI_REAL,root,22,MPI_COMM_WORLD,ierr)
        else  ! Master
          wrgrd = rnodat

          do j=1,numslaves
            call MPI_RECV(wrgrd(:,firstRowArray(j):lastRowArray(j)),
     *          ncol*nrowLocalArray(j),MPI_REAL,j,22,
     *          MPI_COMM_WORLD,status,ierr)
          end do

              namgrd=diroutput(1:posi3)//idnam(posiid1:posiid2)//
     *          chyr//".SOD"
         OPEN (732,FILE=namgrd,STATUS='unknown',ERR=999,FORM='BINARY')
         uwas=732
	   include 'wstsp.inc'
        end if  ! Slaves / master
      end if  ! Sept 30

! Write updated SFC
      if((jahr==1970) .and. (mon == 3) .and. (tag == 2)) then
        call MPI_BCAST(posiid1,1,MPI_INTEGER,1,MPI_COMM_WORLD,ierr)
        wrgrd = sfcG(:,:)

        write(chyr,'(I4)') jahr


              if(rank .ne. root) then
          call MPI_SEND(wrgrd(:,firstRow:lastRow),ncol*nrowLocal,
     *    MPI_REAL,root,22,MPI_COMM_WORLD,ierr)
        else  ! Master
          wrgrd = rnodat

          do j=1,numslaves
            call MPI_RECV(wrgrd(:,firstRowArray(j):lastRowArray(j)),
     *          ncol*nrowLocalArray(j),MPI_REAL,j,22,
     *          MPI_COMM_WORLD,status,ierr)
          end do

              namgrd=diroutput(1:posi3)//idnam(posiid1:posiid2)//
     *          chyr//".SFCu"
         OPEN (732,FILE=namgrd,STATUS='unknown',ERR=999,FORM='BINARY')
         uwas=732
	   include 'wstsp.inc'
        end if  ! Slaves / master
      end if  ! March 2

	RETURN
999   continue

      call messagerun (1, 'Error in model core')
	stop

	end
