!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2003 Joachim Gurtz, Massimiliano Zappa
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Adaptation for MPI parallel processing

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

! ------------------------------------------------------------------------------
!    End of the day in all HRUs - Determinating daily balance for the whole
!    catchment, writing daily results for the whole catchment
! ------------------------------------------------------------------------------

      subroutine prevahendday

	  use prevahglobals
	  use mpi
	  use MPIVars, only: firstRow, lastRow, rank, root, ierr, status,
     * nrowLocal
        use outvars, only: writeArgs, intArgs, writeArgOpts
        use fp_ops
        use forhycsglobals, only: pnksmp, paksmp, wsasmp, epsmp, ersmp,
     * eismp, ebsmp, r0smp, r1smp, r2smp, glacssp, rgessmp,
     * pnksmc, paksmc, wsasmc, epsmc, ersmc, eismc, ebsmc, r0smc,
     * r1smc, r2smc, glacssc, rgessmc

	  implicit none

	  integer intOuda
	  integer :: len
	  real, allocatable :: tempVec(:)

40     FORMAT (  I5,3I3,I6,9F8.3,5F8.2,4F8.4)

      allocate(writeArgs(18))

!    Daily average for the whole catchment
      if (rank .ne. root) then

!    Daily average for the whole catchment
C    	do ycoor=nrow,1,-1
C    	do xcoor=1,ncol
C        if (int(gezg(xcoor,ycoor)) .eq. int(rnodat)) cycle
C       ATFLK=ATFL(xcoor,ycoor)*0.01
C       SSOM=SSOM+(SSOTF(xcoor,ycoor)/wstats(1))
C       SIM=SIM+(SITF(xcoor,ycoor)/wstats(1))
C       SSMM=SSMM+(SSMTF(xcoor,ycoor)/wstats(1))
C       SUZM=SUZM+(SUZTF(xcoor,ycoor)/wstats(1))
C       SLZM=SLZM+(SLZTF(xcoor,ycoor)/wstats(1))
C      end do
C	end do

! Reduce sums for fluxes obtained previously in ADMSHYD
C      print*, "pnksmp,pnksmc", pnksmp,pnksmc,rank
      pnksm = kahanReduce(pnksmp,pnksmc)
C      print*, "pnksm", pnksm
      paksm = kahanReduce(paksmp,paksmc)
      wsasm = kahanReduce(wsasmp,wsasmc)
      epsm = kahanReduce(epsmp,epsmc)
      ersm = kahanReduce(ersmp,ersmc)
      eism = kahanReduce(eismp,eismc)
      ebsm = kahanReduce(ebsmp,ebsmc)
      r0sm = kahanReduce(r0smp,r0smc)
      r1sm = kahanReduce(r1smp,r1smc)
      r2sm = kahanReduce(r2smp,r2smc)
      glacss = kahanReduce(glacssp,glacssc)
      rgessm = kahanReduce(rgessmp,rgessmc)


! Mean values for state variables over the whole grid

      len = ncol*nrowLocal
      allocate(tempVec(len))
      tempVec = reshape(ssotf(:,firstRow:lastRow),(/len/))
      ssom = kahan_sum(tempVec,len,rnodat)
      tempVec = reshape(sitf(:,firstRow:lastRow),(/len/))
      sim = kahan_sum(tempVec,len,rnodat)
      tempVec = reshape(ssmtf(:,firstRow:lastRow),(/len/))
      ssmm = kahan_sum(tempVec,len,rnodat)
      tempVec = reshape(suztf(:,firstRow:lastRow),(/len/))
      suzm = kahan_sum(tempVec,len,rnodat)
      tempVec = reshape(slztf(:,firstRow:lastRow),(/len/))
      slzm = kahan_sum(tempVec,len,rnodat)

      ssom = ssom / wstats(1)
      sim = sim / wstats(1)
      ssmm = ssmm / wstats(1)
      suzm = suzm / wstats(1)
      slzm = slzm / wstats(1)

      end if  ! Slaves only



       CALL BILANZ (GEBIL,PAKSM,ERSM,RGESSM,SSOTB,SSOM,SSOD,SITB,SIM,
     *            SID,SSMTB,SSMM,SSMD,SUZTB,SUZM,SUZD,SLZTB,SLZM,SLZD )

C      print*, "ssotb,ssom,ssod", ssotb,ssom,ssod, rank

       K=0	! ID for catchment integration

      GLACSS=max(glacss,0.0)

      IF ((ZIERG(5:5) .EQ. 'y') .OR. (ZIERG(5:5) .EQ. 'Y')) then

              if(rank == 1) then
          writeArgs=(/PNKSM,PAKSM,WSASM,EPSM,ERSM,EISM,EBSM,R0SM,
     *              R1SM,R2SM,RGESSM,SSOM,SIM,SSMM,SUZM,SLZM,GEBIL,
     *              GLACSS/)
       call MPI_SEND(writeArgs,18,MPI_REAL,root,22,MPI_COMM_WORLD,ierr)

        else if (rank == root) then
        call MPI_RECV(writeArgs,18,MPI_REAL,1,22,MPI_COMM_WORLD,status,
     *  ierr)
          WRITE (MDMIT,'(3(I4,1X),7(F8.3,1X),15(F8.2,1X))') jahr,mon,
     * tag, writeArgs(1:18), rgb1dm,rgb2dm,rgb3dm,difgadm
        end if
C        allocate(intArgs(3))
C        allocate(writeArgs(22))
C        allocate(writeArgOpts(22))
C        intArgs=(/jahr,mon,tag/)
C        writeArgs=(/PNKSM,PAKSM,WSASM,EPSM,ERSM,EISM,EBSM,R0SM,
C     *              R1SM,R2SM,RGESSM,SSOM,SIM,SSMM,SUZM,SLZM,GEBIL,
C     *              GLACSS,rgb1dm,rgb2dm,rgb3dm,difgadm/)
C        writeArgOpts=1
C        call MPIWriteTable(mdmit,3)
C
C        deallocate(intArgs)
C        deallocate(writeArgs)
C        deallocate(writeArgOpts)

      end if
	  ssoma=ssom

	  deallocate(writeArgs)

	  if (ranuse) raerg(mon,tag,ranloop)=RGESSM

        rgb1dm=0.0
	  rgb2dm=0.0
	  rgb3dm=0.0
	  difgadm=0.0
! ------------------------------------------------------------------------------
!      Crating monthly and yearly sums for the whole catchment
! ------------------------------------------------------------------------------

      include 'addmmyy.inc'
      call MPI_BCAST(iee,1,MPI_INTEGER,1,MPI_COMM_WORLD,ierr)

! ------------------------------------------------------------------------------
!   Writing hourly output for the whole catchment
! ------------------------------------------------------------------------------

      DO 159 I=ibb,iee

! --------------------------------------------------------------------------

      do ln=72,2,-1
	   hsuzhm(ln)=hsuzhm(ln-1)
	   hslzhm(ln)=hslzhm(ln-1)
	end do
	   hsuzhm(1)=suzhm(i)
	   hslzhm(1)=slzhm(i)

! Not adapted for parallel execution yet
      IF ((ZIERG(3:3) .EQ. 'y') .OR. (ZIERG(3:3) .EQ. 'Y')) THEN
        K=0
        WRITE (MDSTD,40) JAHR,MON,TAG,I,K,PNKHM(I),PAKHM(I),
     *        WSAHM(I),EPHM(I),ERHM(I),R0HM(I),R1HM(I),
     *                 R2HM(I),RGEHM(I),SSOHM(I),SIHM(I),SSMHM(I),
     *                 SUZHM(I),SLZHM(I),RICEHM(I),RICSHM(I),RICFHM(I),
     *                 WICEHM(I)
       ENDIF


! ------------------------------------------------------------------------------
! Writing hourly runoff components for the whole catchemnt
! ------------------------------------------------------------------------------
201    FORMAT (  I4,3I3,F9.5,F11.2,6F9.5,F11.2,i5)			!formats(5)
2001    FORMAT (I4.4,2(A1,I2.2),1X,I2,A3,1X,F10.1,8(1X,F8.3),1X,F6.3)	!formats(6)
      IF ((ZIERG(1:1) .EQ. 'y') .OR. (ZIERG(1:1) .EQ. 'Y')) THEN
!      print*, "starting if zierg1", rank
      if(rank .ne. root) then
        if (ltrasl .gt. 0) then
	   do ln=2,ltrasl+1
	     rtrasl(ln-1)=rtrasl(ln)
          end do
          rtrasl(ltrasl+1)=RGEHM(I)*AELHG/(3.6)*1000
	 else
          rtrasl(1)=RGEHM(I)*AELHG/(3.6)*1000
	 end if
      else
        rtrasl(1)=0
      end if

	IF (AELHG .LT. 5.) THEN ! Small catchments in l/s
          allocate(intArgs(4))
          allocate(writeArgs(9))
          allocate(writeArgOpts(9))
          intArgs=(/jahr,mon,tag,i/)
          writeArgs=(/rgehm(i),rgehm(i)*aelhg*1000/(3.6),r0hm(i),
     *                r1hm(i),r2hm(i),rgb1hm(i),rgb2hm(i),rgb3hm(i),
     *                rtrasl(1)/)
          writeArgOpts=1

          call MPIWriteTable(mdq,5)

          deallocate(intArgs)
          deallocate(writeArgs)
          deallocate(writeArgOpts)


! XML-output JS2008-04
		IF (xmlyn.EQ.'Y') CALL xmlwrite !*1000 nur in m3/s

! ISB-output JS2008-04

   	IF (isbyn) CALL writeqisb

	else ! Big catchments in m3/s

          allocate(intArgs(4))
          allocate(writeArgs(10))
          allocate(writeArgOpts(10))
          intArgs=(/jahr,mon,tag,i/)
          writeArgs=(/rgehm(i),rgehm(i)*aelhg/(3.6),r0hm(i),
     *                r1hm(i),r2hm(i),rgb1hm(i),rgb2hm(i),rgb3hm(i),
     *                rtrasl(1)/1000,real(ltrasl)/)
          writeArgOpts=1


          !call MPIWriteTable(mdq,17)
          deallocate(intArgs)
          deallocate(writeArgs)
          deallocate(writeArgOpts)


! XML-output JS2008-04
! Not yet adapted for parallel execution
		IF (xmlyn.EQ.'Y') CALL xmlwrite !*1000 nur in m3/s

! ISB-output JS2008-04
! Not yet adapted for parallel execution
   	IF (isbyn) CALL writeqisb

	END IF
	END IF



	IF (lbulletin) then
      WRITE (MDBULL,2001) JAHR,'-',MON,'-',TAG,(I-1),':00',
     *               RGEHM(I)*AELHG*1000/(3.6),PAKHM(I),WSAHM(I),
     *               ERHM(I),R0HM(I),R1HM(I),R2HM(I),RGEHM(I),
     *               SSOHM(I),(SSMHM(I)+SUZHM(I))/ssmb
	end if

      IF ((ZIERG(15:15) .EQ. 'y') .OR. (ZIERG(15:15) .EQ. 'Y')) then
        allocate(intArgs(4))
        allocate(writeArgs(size(runsubca(i,:))))
        allocate(writeArgOpts(size(runsubca(i,:))))
        intArgs=(/jahr,mon,tag,i/)
        writeArgs=runsubca(i,:)
        writeArgOpts=1

        call MPIWriteTable(mdtmp,7)

        deallocate(intArgs)
        deallocate(writeArgs)
        deallocate(writeArgOpts)
      end if  ! zierg 15

	IF ((docali .eq. 'f') .or. (docali .eq. 'F')) then
	rgsensemble(forensemble,fnext,I)=rtrasl(1)/AELHG*3.6
      IF (AELHG .LT. 50.) then
	rgsensemble(forensemble,fnext,I)=
     *rgsensemble(forensemble,fnext,I)/1000
	END IF
	ENDIF

155   CONTINUE !JS-2008-04


! Setting hourly fields to 0
          PNKHM(i)=0.0
          PAKHM(i)=0.0
          WSAHM(i)=0.0
          EPHM(i)=0.0
          ERHM(i)=0.0
          EIHM(i)=0.0
          EBHM(i)=0.0
          R0HM(i)=0.0
          R1HM(i)=0.0
          R2HM(i)=0.0
          RGB1HM(i)=0.0
	  RGB2HM(i)=0.0
	  RGB3HM(i)=0.0
          RGEHM(i)=0.0
          SSMHM(i)=0.0
          SSOHM(i)=0.0
          SIHM(i)=0.0
          SUZHM(i)=0.0
          SLZHM(i)=0.0
	  WICEHM(I)=0.0
	  RICEHM(I)=0.0
	  RICSHM(I)=0.0
	  RICFHM(I)=0.0
          Rlake(I)=0.0   !JS-2008-04


159   CONTINUE


! Writing daily grids
! Not yet adapted for parallel execution
      IF (GRDAKT .EQ. 'Y') THEN
          INCLUDE 'gridt.inc'
      ENDIF

!	print*, 'ok'

	IF (GRDAKT .EQ. 'S') THEN
      DO POS=ibb,iee
	   call charnr(POS,chhh)
         namgrd=diroutput(1:posi3)//'dsuz'//datumchar(3:4)//
     *   datumchar(1:2)//chhh//'.d'//datumchar(5:8)

    	do ycoor=nrow,1,-1
    	do xcoor=1,ncol
        if (int(gezg(xcoor,ycoor)) .eq. int(rnodat)) then
		 WRGRD(xcoor,ycoor)=rnodat
		 cycle
		else
         WRGRD(xcoor,ycoor)=XX(POS,xcoor,ycoor)
		end if
        end do
		end do

        OPEN (732,FILE=namgrd,STATUS='UNKNOWN',ERR=999,FORM='BINARY')
        uwas=732
	  include 'wstsp.inc'

      END DO
      ENDIF

	RETURN

999   continue

      call messagerun (1, 'Error while opening the output files')
      stop
	end
