!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2003 Joachim Gurtz, Massimiliano Zappa
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Initialization of new soil variables

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Initialization of PREVAH model variables
!-------------------------------------------------------------------------------


!
!  2D open issues
!   ST -> to itflast? what's that
! ---------------------------------------------------------------------


      subroutine prevahinit

	  use prevahglobals
	  use multglob
	  use transglob
	  use mpi
	  use MPIVars
	  ! forhycsglobals: allocatable arrays are needed in allocate.inc
	  use forhycsglobals, only: gdd5,chilldays,dtemp31,mtemp,
     *      sen_day_count,aphen,oldphen,spcgdd0,ddegsum5,
     *      ddegsum5old,muwint,drstr,EBJSout,EPJSout,EIJSout,meant,
     *      meantold,sump,sumpold,drstr2,drstrold,drstrold2,
     *      fhla,ebjs2,tpswitch,PRtoTM,TMtoPR,prevahBC,forhycs,TMdir,
     *      lcChange, perCC, compBioClim, siuTF, siumaxTF, vh100,vh99,
     *      vh95,phenDoyBB,phenDoyMax,phenDoySen,phenDoyFall,
     *      phenBBOld, phenMaxOld, phenSenOld, phenFallOld,
     *      snowoff, snowoffOld, snowdays, spinupyearsFH, lcdyn,
     *      annprec, mmot, annprecold, mmotold,gsLength,gsTpot,
     *      gsPrec,gsPrecDays,gsTemp, gsLengthCur,gsTpotCur,
     *      gsPrecCur,gsPrecDaysCur,gsTempCur,sfcMax,prevDayP,
     *      eeqrmean, pefrmean, rootslag, dstr30,gdd30, curhcmax, gsDMI,
     *      Schill, Sforcing, doyBB, doySen, meanT4, dohmax, dolared,
     *      dosfc, dosmort, cdtol, doground, ghogyear, grsmin,
     *      gsPes, gsPesCur, readFH, writeFH, ssen3d, futureYearsFH,
     *      rsmin2d, deltaBio, meantc, meantcold, deltaT, deltaP, gsPe,
     *      gsPeCur
      use glacGlob, only: numGlac, pathsGlac, minSnowCover

	  implicit none


       integer dcyy,dcmm,dcdd
       real dcval(6)
       character(100) :: pathGlacConf
       character(12) :: glacStr
       integer :: posPath  ! Helper variable for file path handling for glacier parameter file
       integer :: posCatch1, posCatch2    ! Helper variables to extract catchment name in readpin2d.inc
       integer :: iGlac  ! Loop index

!     parameters to be initialized are declared in a include file
      include 'init.inc'

! ------------------------------------------------------------------------------
! Reads the control parameters
! ------------------------------------------------------------------------------
      snowupdate=0.
      CBOPT=0
	updt=0
	ranuse=.false.

      include 'readpin2d.inc'


       cleangrid='Y'
       cleansleep='N'
       if (szaktiv .eq. 'C') then
       szaktiv='N'
       cleangrid='N'
       end if
       if (pardist) then
       call pardistread
       CBETA=0.0
       end if

	IF (CBETA .le. 0.0) CBOPT=1
	allit=0
	ltrasl=0

!     Actualizing diroutput for multiple runs
      mmult=.false.

      IF (docali .eq. 'M') THEN
	call messagerun (1,'Assignig multiple run common parameters')
	diroutput=mdiro(1:LEN(TRIM(mdiro)))
	doeff=meff
	loadid=mload
	taga=mbegindays
	tage=menddays
	mona=mbeginmonths
	mone=mendmonths
	jahra=mbeginyears
	jahre=mendyears
	climfile=trim(fclimate)
	SZAKTIV=mclim
	do i=1,6
	rmet(i)=mrmet(i)
	end do
	if (mclim .eq. 'Y') saveid='N'
	if (mclim .eq. 'y') saveid='N'
!      saveid='N' ! activate for ice scenario
      WRITE (LOADCHAR,'(2I2.2,I4.4)') TAGA,MONA,JAHRA
	LOADROUT=LOADID
      mmult=.true.
	end if

!	call messagerun (1,diroutput)

      IF ((docali .eq. 'T') .or. (docali .eq. 't')) then
	Call messagerun (1,'Adapting Network unit ....'//prtunit)
	diroutput=trim(dirrt)
	diroutput(1:3)=prtunit
	CLIMFILE(1:3)=prtunit
      NADGKW(1:3)=prtunit
      dirstate(1:3)=prtunit
	dirmeteo(1:3)=prtunit
	IDNAM(1:3)=prtunit
	DO I=1,neffis
         effinams(i)(1:3)=prtunit
      END DO

	if (educnr .ne. -99) then
	IF (docali .eq. 'T') namaus='realtime'
      IF (docali .eq. 't') namaus='forecast'
	end if
      ENDIF

      call posnam (diroutput,1,posi3)

	open (987,file=diroutput(1:posi3)//
     *  'leave.pre',status='old',err=344)
	close (987,status='delete')
344   continue

!     Opening HTML Report File
	CALL POSNAM (NAMAUS,1,POS)

      if ((rank==root) .or. (doParallel==.false.)) then

	    open (mhtml,file=diroutput(1:posi3)//NAMAUS(1:POS)//'.html',
     *      status='unknown')

        write(mhtml,'(A)') '<html>'
        write(mhtml,'(A)') '<head>'
        write(mhtml,'(A)') '<title>PREVAH RUN - Summary File </title>'
        write(mhtml,'(A)') '</head>'
        write(mhtml,'(A)') '<body>'

        write(mhtml,'(A)')'<h1>PREVAH - Summary File </h1>'
        write(mhtml,'(A)')'<p><p>'
        write(mhtml,'(A)')'Settings<br>'
	    write(mhtml,'(A)')'<a href=file:///'//trim(namein)//
     *                  '>Control File used</a><br>'
	    write(mhtml,'(A)')'<a href=file:///'//trim(nadgkw)//
     *                  '>Spatial Information File used</a><br>'

      end if  ! End of Master part

	beg=0
	nrk=0

      if ((rank==root) .or. (doParallel==.false.)) then
        Call messagerun (1,'running PREVAH ....')
      end if

 	tcali='N'
	ibb=1

      IF (docali .eq. 'T') then
        if ((rank==root) .or. (doParallel==.false.)) then
          OPEN  (721,FILE="AREA.INC", position='append')
          WRITE (721,'(2F10.2,TR2,A)') AELHG, AE
	      CLOSE (721)
	    end if
      TAGA=begindays(2)
	MONA=beginmonths(2)
	JAHRA=beginyears(2)
      TAGE=enddays(2)
	MONE=endmonths(2)
	JAHRE=endyears(2)
	hoursb=beginhours(2)
	hourse=endhours(2)
	docali='N'
	doeff='N'
      tcali='T'
      ZIERG='Y-Y-Y-N-N-N-N-Y'
!	GRDAKT='N'
	SAVEID='D'
	IF (lscen) SAVEID='N'
	SAVEMMID='N'
	SAVEMOREID='N'
	LOADID='T'
	LOADROUT='T'
	if (educnr .ne. -99) then
	dorout='N'
	end if
	if (volumecorrection .gt. -101.) then
	pkorf=volumecorrection
	sgrluz=peakthreshold
      k0h=peakstorage
      IF (K0H .GT. 0.0) K0=1.0-EXP(-1.0/K0H)
	ltrasl=rttranslation
	end if
	END IF
      IF (docali .eq. 't') then
      TAGA=begindays(3)
	MONA=beginmonths(3)
	JAHRA=beginyears(3)
      TAGE=enddays(3)
	MONE=endmonths(3)
	JAHRE=endyears(3)
	hoursb=beginhours(3)
	hourse=endhours(3)
	docali='F'
	doeff='N'
      tcali='t'
      ZIERG='Y-Y-N-N-N-N-N-Y'
	GRDAKT='N'
	SAVEMMID='N'
	SAVEMOREID='N'
	LOADID='T'
	LOADROUT='t'
	if (educnr .ne. -99) then
	dorout='N'
	else
	fnext=1
	docali='T'
	tcali='T'
	end if
	lbulletin=.true.
	if (volumecorrection .gt. -101.) then
	sgrluz=peakthreshold
      k0h=peakstorage
      IF (K0H .GT. 0.0) K0=1.0-EXP(-1.0/K0H)
	ltrasl=rttranslation
	END IF
	END IF

      IF (lscen) then
        if ((rank==root) .or. (doParallel==.false.)) then
	      call messagerun (1,'[Adopting scenarios]')
	    end if
	  end if

      IF ((docali .eq. 'f') .or. (docali .eq. 'F')) then
        if ((rank==root) .or. (doParallel==.false.)) then
	      call messagerun (1,'[Loading forecast]')
	    end if
      include 'readfcast.inc'
      if (tcali .eq. 't') then
	fortype=1
	fnext=0
	end if
	end if

! ------------------------------------------------------------------------------
! Initialization - allocation of the arrays
! ------------------------------------------------------------------------------
! Arrays actually not necessary for root, except those of meteo inputs, to be passed to gimetdtmin
      INCLUDE 'allocate.inc'
      INCLUDE 'nullsetz.inc'

! ------------------------------------------------------------------------------
!  Choice for type of input
! ------------------------------------------------------------------------------

      IPDAT='2'
      IPNA='1'

      MTA= MIDAT
! ------------------------------------------------------------------------------
! Reads the control parameters
! ------------------------------------------------------------------------------

      Open (mlog,file='model.log',status='unknown')
128   FORMAT ( I4, A32 )
      WRITE (MLOG,128) NUGEB,ZG
! Declaration of the time step (60 minutes)
      DT=60.0
	first=0

132   FORMAT (A72)

      ZN=ZG
      NRGEB=NUGEB
      AEGES=AELHG
      IF (albadd .gt. 0.6) albadd=0.6

! ------------------------------------------------------------------------------
!   Rainfall and snowfall can be adjusted through monthly varying factors
!   the file 'monpkor.txt' has to be available.
!   SNOKOR andf PKORF in the control file must be set equal to -99.9

      precmokor=3
      if (nugeb .eq. 1) then
	    precmokor=0
          if ((rank==root) .or. (doParallel==.false.)) then
	        Call messagerun (1,'Using new precipitation correction!')
	      end if
        end if
	  if (nugeb .eq. 21) then
	   precmokor=0
        if ((rank==root) .or. (doParallel==.false.)) then
	      Call messagerun (1,'Using new precipitation correction!')
        end if
      end if


      IF ((SNOKOR .EQ. -99.9) .AND. (PKORF .EQ. -99.9)) THEN
         precmokor=1
         open (444, file='monpkor.txt', status='old',err=133)
         read (444,*) (snomokor(I), I=1,12)
         read (444,*) (remokor(I), I=1,12)
         close (444)
	   goto 134
133      continue
         stop 'Monthly correction file not found'
134      continue
      END IF

! ------------------------------------------------------------------------------
!   Input file with physiographic information is defined
!
!   Literature
!   Becker, A. and Braun, P. (1999): Disaggregation, aggregation and spatial scaling in
!    hydrological modelling. J. Hydrol., 217, 239-252.
!   Engel, B.A. (1996): Methodologies for development of hydrological response units
!    based on terrain, land cover and soils data. In GIS and environmental modeling.
!    New York: Oxford University Press, 123-128.
!   Fl�gel, W.A. (1995): Delineating hydrological response units by geographical information
!    system analyses for regional hydrological modelling using PRMS/MMS in the drainage basin
!    of the river Br�l, Germany. Hydrol. Process., 9, 423-436.
!   Fl�gel, W.A. (1997): Combining GIS with regional hydrological modelling using hydrologic
!    response units (HRUs): An application from Germany, Math. Comput. Simulat., 43, 297-304.
!   Gurtz, J., Baltensweiler, A. and Lang, H. (1999): Spatially distributed hydrotope-based
!    modelling of evapotranspiration and runoff in mountainous basins. Hydrol. Process., 13,
!    2751-2768.
!   Nemec, J. (1993): Comparison and selection of existing hydrological models for the
!    simulation of the dynamic water balance processes in basins of different sizes and on
!    different scales. International Commission for the Hydrology of the Rhine basin,
!    CHR/KHR-Report No. II-7.
! ------------------------------------------------------------------------------

      IPNA='1'

      CALL POSNAM (NADGKW,1,POSI1)


      CALL ITFLAST2D (ST,isice)

      ! Calculate number of non-No Data cells on the whole grid (to be used for
      ! calculating global averages)

      cntGlobal = count(int(gezg) .ne. int(rnodat))

      if(doParallel) then
        call MPI_BARRIER(MPI_COMM_WORLD,ierr)
        call MPIStripes
      end if

      CALL IEVAPARZ (IART,ZEIPM,CKURZ,KURZWSF,CLANG,LANGWSF,VFEUCHTF,
     *               CVENT,ZLAND,ZBTYP,ZKWERT,ZEIKE,MLOG )

      if((rank.ne.root).or.(doParallel==.false.)) then
        CALL MTDINIT
      end if
      call MPI_BARRIER(MPI_COMM_WORLD,ierr)
      if((rank.eq.root).or.(doParallel==.false.)) then
        call messagerun (1,'MTD initialized')
      end if

      CALL SETGRIDMETEO
      call MPI_BARRIER(MPI_COMM_WORLD,ierr)
      if((rank.eq.root).or.(doParallel==.false.)) then
        call messagerun (1,'GRIDMETEO initialized')
      end if

! Initialization of glacier information
      call glacInit
      if((rank.eq.root).or.(doParallel==.false.)) then
        call messagerun (1,'Glacier information initialized')
      end if

! Decide whether the pixel is part of the glaciers considered for mass balance
! calculations (temporary - Matthias, Jan 2015)	do ycoor=nrow,1,-1
C
Cdo ycoor=nrow,1,-1
Cdo xcoor=1,ncol
C       if(glacID(xcoor,ycoor) .ne. int(rnodat)) then
C
C         glacmask(xcoor,ycoor) = 1
C         countGlac = countGlac + 1
C       end if
C
C     end do
C     end do

	AE=AEBER
      DO I=1,NTG
          HOEHTG(I)=I
      END DO


! Set default values for land cover class dependent parameters (previously
! done in EINPUZDW)
      include 'LCparams.inc'


! Initialization of soil parameterization (available water capacity)
      if(rank .ne. root) then
        do ycoor = lastRow, firstRow, -1
        do xcoor = 1,ncol
          ! Simple parameterization of soil AWC as a function of elevation and slope:
          !call awcParam(xcoor,ycoor)

          ! Read in AWC after Remund & Augustin 2015:

          land = landin(xcoor,ycoor)
          select case(land)
            case(1)
            sfcg(xcoor,ycoor) = 2500.0

            case(2)
            sfcg(xcoor,ycoor) = 5.0

            case(3,4,5,8)
            ! Forested cells: Minimum value at initialization (for bare soil spin-up)
            sfcg(xcoor,ycoor) = 30.0

            case(13,14)
            sfcg(xcoor,ycoor) = 0.1

            case(15)
            sfcg(xcoor,ycoor) = 3.0

            case(-9999) ! rnodat; must be a constant expression
            sfcg(xcoor, ycoor) = rnodat

            case(0) ! rnodat; must be a constant expression
            sfcg(xcoor, ycoor) = rnodat

            case default
          sfcg(xcoor,ycoor) = whc(xcoor,ycoor) *
     *     whcSF(landin(xcoor,ycoor))
C           sfcg(xcoor,ycoor) = fnfc(xcoor,ycoor) * 10 *
C    *       whcSF(landin(xcoor,ycoor))

          end select
      ! Maximum soil water holding capacity, expressed as a fraction of soil volume.
      ! The data of Remund & Augustin give AWC for a soil thickness of 1 m.
      ! Needed for the root zone storage routine
          sfcMax(xcoor,ycoor) = whc(xcoor,ycoor) * 0.001
C         sfcMax(xcoor,ycoor) = fnfc(xcoor,ycoor) * 0.01
C         whc(xcoor,ycoor) = fnfc(xcoor,ycoor) * 10
        end do
        end do
      end if

! ------------------------------------------------------------------------------

      if((rank.eq.root).or.(doParallel==.false.)) then
        write (MLOG,*) ' Write debugging information '
        write (MLOG,*) ' No=0, Yes if >0 '
        write (MLOG,*) debug
      end if

      fehl=9999.0

      IF (IPDAT .EQ. '1') THEN
         MTA=5
      ENDIF

      nj=Jahre-Jahra+1

5     FORMAT (A75)
6     FORMAT (A256)
7     FORMAT (A56)
106    FORMAT (A1)
       IRGL=-1

      ANFS=1
      I=0
11    I=I+1
      IF (I .LT. 5) GOTO 11
      IF (( I .EQ. 5) .AND. (IRGL .LT. 0)) GOTO 11
      I=1
      IPNA='0'

! ------------------------------------------------------------------------------
!     Basic output name is defined

	CALL POSNAM (NAMAUS,1,POS)
	CALL POSNAM (diroutput,1,POSI3)
	NAMAUS=diroutput(1:posi3)//NAMAUS(1:POS)

! path to TreeMig directory
      TMdir=diroutput(1:posi3)//"treemig/"
! ------------------------------------------------------------------------------
!    Initializes the output of grids and hrus

      IF (GRDAKT .EQ. 'y') GRDAKT='Y'
	IF (GRDAKT .EQ. 's') GRDAKT='S'

      IF (GRDAKT .EQ. 'S') THEN
        if((rank.ne.root).or.(doParallel==.false.)) then
	   allocate (XX(24,ncol,nrow))
	   CALL POSNAM (IDNAM,1,POSI1)
	   OPEN (731, file=IDNAM(1:POSI1), status='old',form='binary')
	         call posnam (idnam,2,posiid2)
		 do I=posiid2,1,-1
	     if (idnam(I:I) .eq. '\') exit
	     if (idnam(I:I) .eq. '/') exit
	     posiid1=I
	     end do
   	   uwas=731
	   INCLUDE 'rwasim.inc'
	   INCLUDE 'r2d.inc'
        end if  ! End slaves part
      ENDIF

      IF (GRDAKT .EQ. 'Y') THEN
        if((rank.eq.root).or.(doParallel==.false.)) then
          call messagerun (1,'Grids will be written')
        else
	   allocate (XX(21,ncol,nrow))
	   CALL POSNAM (IDNAM,1,POSI1)
	   OPEN (731, file=IDNAM(1:POSI1), status='old',form='binary')
	         call posnam (idnam,2,posiid2)
		 do I=posiid2,1,-1
	     if (idnam(I:I) .eq. '\') exit
	     if (idnam(I:I) .eq. '/') exit
	     posiid1=I
	     end do
   	   uwas=731
	   INCLUDE 'rwasim.inc'
	   INCLUDE 'r2d.inc'
        end if  ! End slaves part
      ENDIF

      IF (GRDAKT .EQ. 'N') THEN
        if((rank.eq.root).or.(doParallel==.false.)) then
          call messagerun (1,'Gathering Grid Information')
        else
	   CALL POSNAM (IDNAM,1,POSI1)
	   OPEN (731, file=IDNAM(1:POSI1), status='old',form='binary')
	         call posnam (idnam,2,posiid2)
		 do I=posiid2,1,-1
	     if (idnam(I:I) .eq. '\') exit
	     if (idnam(I:I) .eq. '/') exit
	     posiid1=I
	     end do
   	   uwas=731
	   INCLUDE 'rwasim.inc'
	   close (uwas)
        end if  ! End slaves part
      ENDIF

!     Initialization of direct interpolation
! ------------------------------------------------------------------------------

!	dtctrfil='c:\modelling\swiss\dischma\meteo\dtidw.dir'
!      call ini_interpol (ncol,nrow,dtctrfil)

! ------------------------------------------------------------------------------


!     Decides the number of output tables
      if((rank.eq.root).or.(doParallel==.false.)) then

18    FORMAT ( /,' Tables with monthly and daily catchment averages ',
     *           ' will bewritten',
     *         /,' Do you wish additional output',
     *         /,' EINGABE = 0  No, thanks!',
     *         /,'         = 1  Hourly values',
     *         /,'         = 2  Daily values for each HRU',
     *         /,'         = 3  Monthly values for each HRU',
     *         /,'         = 4  Hourly and daily values for each HRU ',
     *         /,'         = 5  Daily and monthly values for each HRU',
     *         /,'         = 6  All the cited tables ' )

        WRITE (MLOG,18)
      end if

C      NAMDRU=NAMAUS(1:POS) // '.pri'	! Monthly results and statistics at end of period for the whole catchment
C      NAMTAG=NAMAUS(1:POS) // '.tag'	! Daily values for each HRU
C      NAMMON=NAMAUS(1:POS) // '.mon'	! Monthly values for each HRU
C      NAMHOU=NAMAUS(1:POS) // '.hou'	! Hourly values for each HRU
C      NAMSTD=NAMAUS(1:POS) // '.std'	! Hourly values for the whole catchment
C      giveuser(4)=namstd
C      NAMQ=NAMAUS(1:POS) // '.q'	! Hourly runoff and components for the whole catchment
C      giveuser(5)=namq
C      NAMISBQ=NAMAUS(1:POS) // '.qsb'	! Hourly runoff for internal sub basins
C      NAMTMP=NAMAUS(1:POS) // '.tmp'	! old file to have outputs for producing grids
C      NAMMIT=NAMAUS(1:POS) // '.mit' 	! Daily results and statistics for the whole catchment
C      NAMLOG=NAMAUS(1:POS) // '.log'	! Logfile
C	NAMBULL=NAMAUS(1:POS) // '.tdb'	! Logfile

      NAMDRU=trim(namaus) // '.pri'	! Monthly results and statistics at end of period for the whole catchment
      NAMTAG=trim(namaus) // '.tag'	! Daily values for each HRU
      NAMMON=trim(namaus) // '.mon'	! Monthly values for each HRU
      NAMHOU=trim(namaus) // '.hou'	! Hourly values for each HRU
      NAMSTD=trim(namaus) // '.std'	! Hourly values for the whole catchment
      giveuser(4)=namstd
      NAMQ=trim(namaus) // '.q'	! Hourly runoff and components for the whole catchment
      giveuser(5)=namq
      NAMISBQ=trim(namaus) // '.qsb'	! Hourly runoff for internal sub basins
      NAMTMP=trim(namaus) // '.tmp'	! old file to have outputs for producing grids
      NAMMIT=trim(namaus) // '.mit' 	! Daily results and statistics for the whole catchment
      NAMLOG=trim(namaus) // '.log'	! Logfile
	NAMBULL=trim(namaus) // '.tdb'	! Logfile
	namcatch = trim(namaus) // ".cat"   ! Daily runoff for catchments
	namprec = trim(namaus) // ".prc"   ! Daily precip for catchments
	namev1 = trim(namaus) // ".ptr"
	namev2 = trim(namaus) // ".atr"
	namev3 = trim(namaus) // ".pes"
	namev4 = trim(namaus) // ".aes"
	namev5 = trim(namaus) // ".int"
	namev6 = trim(namaus) // ".esn"
	namev7 = trim(namaus) // ".enf"


	IF (ERNAM(1:11) .eq. 'nocalib.out') THEN
	   ERNAM=NAMMIT
	ELSE
	   CALL POSNAM (ERNAM,1,POS)
	   ERNAM=diroutput(1:posi3)//ERNAM(1:POS)
	END IF

!     Looking for List to define Table output for selected HRUs
334   FORMAT (  'JJ MM DD TI FN NRTFL',2X,' P-uk',2X,'P-kor',2X,
     *          'P-SME',2X,' EPOT',2X,' EREA',2X,
     *          '   RO',2X,'   R1',2X,'   R2',2X,' RGES',2X,'S-SNO',
     *          2X,'   SI',2X,'  SSM',2X,'  SUZ',2X,'  SLZ',2X,'  SSZ' )

      call posnam (idnam,2,poshru)
	numohru=0
	OPEN (731, file=IDNAM(1:POSHRU)//'-hru.lst',
     * status='old',err=801)
	READ (731,'(A)',END=801,err=801) kopf
      READ (731,*,END=801,err=801) numohru

	do llhru=1,numohru
	read (731,*,END=801,err=801) idnumhru(llhru)
	unit=7500+llhru
	write (locnam,'(A,I4.4,A)') trim(namaus),idnumhru(llhru),'.std'
	open (unit,file=locnam,status='unknown')
      WRITE (unit,'(A30,1X,F12.3)') ZN(1:30),AELHG
      Write (unit,334)
	end do
	call messagerun (1, 'Single HRU output initialized')
      goto 802
801   continue
      numohru=0
802   continue
      close (731)

! ------------------------------------------------------------------------------
! Implementation of Deltachange sceanrios after Bosshard et al. (2010)
! M. Zappa, T. Bosshard, D Viviroli June 2010
! ------------------------------------------------------------------------------
      call messagerun (1, 'Looking for Deltachange file')
!      INQUIRE (file=IDNAM(1:POSHRU)//'-scenario.lst',EXIST=deltachange)	! File controlling assimilation options

       deltayy=1973

      if (szaktiv .eq. 'D') then
       szaktiv='N'
       cleansleep='Y'
      call messagerun (1, 'Deltachange file found')
	OPEN (3731, file=trim(CLIMFILE),status='old')
	READ (3731,'(A)') kopf
	READ (3731,'(A)') dodelta
	READ (3731,'(A)') cleangrid
	READ (3731,'(A)') filedelta
	READ (3731,*) deltayy
	do deltaloop=1,6
	READ (3731,'(A)')
	READ (3731,'(A1,1X,A1)') deltavar(deltaloop),deltamethod(deltaloop)
!	print*, trim(kopf),' ',deltavar(deltaloop),deltamethod(deltaloop)
      end do
	close (3731)

       if (dodelta .eq. 'Y') then
	OPEN (3731, file=trim(filedelta),status='old')
	READ (3731,'(A)') kopf

       do k=1,366
       READ (3731,*) dcyy,dcmm,dcdd,(dcval(i),i=1,6)
       cpndt(dcmm,dcdd)=dcval(1)
       ctedt(dcmm,dcdd)=dcval(2)
       crgdt(dcmm,dcdd)=dcval(3)
       crldt(dcmm,dcdd)=dcval(4)
       cwvdt(dcmm,dcdd)=dcval(5)
       csddt(dcmm,dcdd)=dcval(6)
       end do
855    continue

	close (3731)
       end if

	end if

! -------------------------------------------------------------------


! -------------------------------------------------------------------
!     Internal sub-basins from GRIs -> JS implementation
      isbyn=.false.
      call posnam (idnam,2,poshru)
      inquire (file=IDNAM(1:POSHRU)//'.isb',EXIST=isbyn)
      if (isbyn) then
      isbpath=IDNAM(1:POSHRU)//'.isb'
      Call messagerun (1,'Output for internal sub basins')
      Call messagerun (1,trim(isbpath))

      end if

! -------------------------------------------------------------------
!     Random perturbance runs
	ranuse=.false.

	OPEN (731, file='random.inp',
     * status='old',err=805)
	ranuse=.true.
      saveid='N'
	READ (731,'(A)',END=806,err=805) kopf

      READ (731,*,END=805,err=805) raopt
      READ (731,*,END=805,err=805) rapars
	do i=1,rapars
	READ (731,*,END=805,err=805) ramin(i),ramax(i),raminn(i),
     *  ramaxx(i)
	end do
	call messagerun (1, 'Random perturbance')
	if (raopt .eq. 1) then
	call messagerun (1, '.... SSM will be perturbed')
	else if (raopt .eq. 2) then
	call messagerun (1, '.... Temperature will be perturbed')
	else if (raopt .eq. 3) then
	call messagerun (1, '.... Random runs for selected parameters')
	ravarar(ranloop)=ranloop
	OPEN (998,FILE='RANDOMGLUE.LOG',POSITION="APPEND")
	Write (998,'(I4,2(1X,F6.2),7(1X,F6.3),1X,F8.6,4(1X,F7.3),
     *  3(1X,F7.2),1X,F6.4,1X,F6.3,1X,F7.3)') ranloop,
     *  PKORF,SNOKOR,TGR,TTRANS,T0,CRMFMAX,CRMFMIN,CRFR,CSNOMF,
     *  CASNO,CBETA,SGRLUZ,K0H,K1H,K2H,CG1H,SLZ1MAX,CPERC,
     *  CICEMF,CAICE
	CLOSE (998)

	end if
	call nurand
	print*, ranuse,ravar
      goto 806
805   continue
      ranuse=.false.
806   continue
      close (731)


! ------------------------------------------------------------------------------
      !if(prevahBC == 1) jahra = 1941   ! Groundhog spin-up for FORHYCS
C     spinupYearsFH=(/1973,1986,1985,1975,1981,1986,1979,1971,1981,1979,
C    * 1971,1979,1977,1978,1984,1979,1985,1976,1987,1981,1981,1986,1984,
C    * 1985,1975,1974,1984,1983,1974,1977/)

      spinupyearsFH = (/1994,2000,1981,1995,1997,1991,1986,1985,1993,
     * 1988,
     * 1985,1990,1986,1986,1988,1980,1986,1995,1983,1982,1982,1985,1980,
     * 1996,1997,1982,1998,1984,1982,2000,1996,1998,2000,2000,1994,1994,
     * 1986,1985,1999,1981,1988,1980,1989,1995,1988,1991,1983,1984,1998,
     * 2000,1984,2000,1999,1989,1985,1995,2000,1993,1998,1982,1986,1982,
     * 1993,1999,1993,1999,1987,1985,1990,1992,1997,1992,1997,1987,1989,
     * 1998,1985,1983,1994,1981,1994,1980,1985,1993,1990,1991,1988,1982,
     * 1981,1996,1984,1993,1998,1989,1990,1983,1988,1991,1981,1991,1989,
     * 1993,2000,1985,1997,1980,1997,1994,1980,1988,1993,1990,1981,1991,
     * 1982,1993,1983,1988,1986,1986,2000,1994,1985,1999,1988,1986,1993,
     * 1994,1983,1993,1997,1984,1996,1990,1996,1984,1982,1991,1990,1985,
     * 1993,1993,1984,1980,1995,1986,1990,1982,1992,1982,1982,1980,1997,
     * 2000,1984,1983,1980,1990,1990,1990,1988,2000,1989,1981,1993,1999,
     * 1984,1993,1992,1994,1994,1993,1992,1989,1997,1997,1993,1980,1995,
     * 1992,1998,1996,1988,1991,1981,1986,1989,1982,1983,1987,1984,1989,
     * 1986,1983,2000,1987,2000,1983,1982,1993,1980,1997,1984,1985,1995,
     * 1985,1982,1990,1990,1989,1997,1983,1996,1991,1986,1989,1999,1992,
     * 1989,1996,1993,1988,1985,1986,1991,1980,1989,1999,1982,1999,1981,
     * 1981,1988,1986,1989,1988,1986,1981,2000,1988,1996,1982,1996,1991,
     * 1995,1992,1985,1982,1986,1995,1993,1996,1994,2000,1999,1980,1996,
     * 1982,1996,1981,1985,1980,1984,1998,1983,1996,1983,1999,1986,1982,
     * 1989,1997,1998,1989,1989,1990,1990,1996,1996,1994,1988,1980,1993,
     * 1982,1982,1980,1983,1982,1990,1994,1995,1981,1986,1980,1986,1991,
     * 1996,1986,1997,2000,1982,1983,1985,1998,1985,1988,1994,1988,1989,
     * 1989,1998,1988,1983,1992,1988,1986,2000,1992,1982,1983,1993,1986,
     * 1994,1997,1982,1992,1997,1985,1995,1980,1993,1981,2000,1980,1998,
     * 1989,1994,1980,1982,1995,1985,1988,1995,1981,1987,1992,1987,1988,
     * 1989,1984,1993,1989,1982,1982,1986,1995,1987,1981,1983,1992,1980,
     * 2000,1991,1989,1999,1995,1980,1997,1998,1999,1992,1981,1982,2000,
     * 1983,1997,1980,2000,1982,1997,1986,1984,1986,1999,1986,1988,1992,
     * 1994,1985,1989,1993,1986,1995,1982,1991,1983,1988,1991,1982,1999,
     * 1999,1991,1992,1999,1987,1985,1982,1986,1981,1985,1990,1995,1994,
     * 1990,1998,1990,1991,1983,1994,1984,1991,1984,1985,1985,1988,1991,
     * 1988,1993,1994,1997,1995,1992,1995,1994,1998,1996,1989,1996,1981,
     * 1985,1995,1987,2000,1980,1980,1989,1999,1997,1999,1997,1993,1998,
     * 1991,1999,1998,1995,1982,1990,1994,1982,1990,1991,1988,1983,1996,
     * 1994,1982,1998,1982,1988,1987/)

      futureyearsFH=(/2010,2000,1981,2007,1996,2012,2011,2008,2015,1986,
     * 2014,2002,2005,2009,2006,1994,1995,1981,1986,1983,1983,1993,2012,
     * 1992,1989,1991,1994,2010,2006,1981,2011,2000,1990,1993,2011,1997,
     * 2010,2007,2009,1992,2006,1995,1996,1998,2002,1999,1998,2009,1991,
     * 2008,2013,1984,1986,2014,1991,1982,1990,2001,1987,2000,1984,2000,
     * 2010,1985,2014,1992,2013,2001,1990,2015,2005,2007,1983,2008,2010,
     * 1988,1995,1987,2012,1990,1989,1992,2010,2010,1983/)

	return

999   continue

      call messagerun (1, 'Error at initialization - File not found')
	stop

	end
