!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2003 Joachim Gurtz, Massimiliano Zappa
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Adaptation to MPI parallel execution

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


!-------------------------------------------------------------------------------
! Initialize PREVAH output files
!-------------------------------------------------------------------------------

      subroutine prevahoutput

	  use prevahglobals
	  use mpi
	  use MPIVars

	  implicit none

      real ssmb_tot, ssob_tot  !used during loop to calculate total soil moisture and snow
      real ssmb_totLocal, ssob_totLocal
      real lstats,globstats,lratio  ! Calculate ratio of non-No Data cells in each stripe
      								! to distribute initial storage contents.

      ssmb_tot=0.0
      ssob_tot=0.0
      ssmb_totLocal=0.0
      ssob_totLocal=0.0

! ------------------------------------------------------------------------------
!   Output tables are opened
! ------------------------------------------------------------------------------
      IF ((docali .eq. 'y') .or. (docali .eq. 'Y') .or.
     *  (docali .eq. 'g').or. (docali .eq. 'G')) zierg(9:15)='N-N-N-N'

      if ((rank==root) .or. (doParallel == .false.)) then

      write(mhtml,'(A)')'<p><p>'
      write(mhtml,'(A)')'Output Tables<br>'

      print*, "mlog, namlog", mlog, namlog
	OPEN (MLOG,FILE=NAMLOG,STATUS='UNKNOWN',err=999)

      IF ((ZIERG(1:1) .EQ. 'y') .OR. (ZIERG(1:1) .EQ. 'Y')) THEN
       OPEN (MDQ,FILE=NAMQ,STATUS='UNKNOWN',err=999)
	 write(mhtml,'(A)')'<a href=file:///'//namq//
     *   '>Houry runoff components</a><br>'
	END IF

      IF ((ZIERG(3:3) .EQ. 'y') .OR. (ZIERG(3:3) .EQ. 'Y')) THEN
       OPEN (MDSTD,FILE=NAMSTD,STATUS='UNKNOWN',err=999)
      WRITE (MDSTD,'(A30,1X,F12.3)') ZN(1:30),AELHG
      !--------------------------------------------------------------------------
	!    Special output (*.std) when the catchment is a lake (JS-2008-04)

	  IF (islake .eq. .true.) THEN
	   Call messagerun (1,'The Catchment seems to be a lake.')
	   Call messagerun (1,'Special output will be written.')

33       FORMAT (  ' JJ  MM DD TI ',2X,' P-uk',2X,'P-kor',2X,
     *          'P-SNO',2X,' EPOT',2X,' EREA',2X,
     *          'Rlake ')
	   Write (MDSTD,33)
	  ELSE
34       FORMAT (  'JJ MM DD TI FN NRTFL',2X,' P-uk',2X,'P-kor',2X,
     *          'P-SNO',2X,' EPOT',2X,' EREA',2X,
     *          '   RO',2X,'   R1',2X,'   R2',2X,' RGES',2X,'S-SNO',
     *          2X,'   SI',2X,'  SSM',2X,'  SUZ',2X,'  SLZ')
	   Write (MDSTD,34)
	  ENDIF
	!--------------------------------------------------------------------------
	 write(mhtml,'(A)')'<a href=file:///'//namstd//
     *                   '>Hourly Output</a><br>'
	END IF
	IF ((ZIERG(5:5) .EQ. 'y') .OR. (ZIERG(5:5) .EQ. 'Y')) THEN
       OPEN (MDMIT,FILE=NAMMIT,STATUS='UNKNOWN',err=999)
	 write(mhtml,'(A)')'<a href=file:///'//nammit//
     *                    '>Daily Output</a><br>'
	END IF

      IF ((ZIERG(7:7) .EQ. 'y') .OR. (ZIERG(7:7) .EQ. 'Y')) THEN
       OPEN (MDR,FILE=NAMDRU,STATUS='UNKNOWN',err=999)
	 write(mhtml,'(A)')'<a href=file:///'//namdru//
     *      '>Monthly and yearly Output</a><br>'
	END IF
      IF ((ZIERG(9:9) .EQ. 'y') .OR. (ZIERG(9:9) .EQ. 'Y')) THEN
       OPEN (MDHOU,FILE=NAMHOU,STATUS='UNKNOWN',err=999)
	write(mhtml,'(A)')'<a href=file:///'//namhou//
     *                 '>Hourly GIS Output</a><br>'
	END IF
      IF ((ZIERG(11:11) .EQ. 'y') .OR. (ZIERG(11:11) .EQ. 'Y')) THEN
       OPEN (MDTAG,FILE=NAMTAG,STATUS='UNKNOWN',err=999)
	 write(mhtml,'(A)')'<a href=file:///'//namtag//
     *  '>Daily GIS Output</a><br>'
	END IF
      IF ((ZIERG(13:13) .EQ. 'y') .OR. (ZIERG(13:13) .EQ. 'Y')) THEN
       OPEN (MDMON,FILE=NAMMON,STATUS='UNKNOWN',err=999)
	write(mhtml,'(A)')'<a href=file:///'//nammon//
     *  '>Monthly GIS Output</a><br>'
	END IF

      IF ((ZIERG(15:15) .EQ. 'y') .OR. (ZIERG(15:15) .EQ. 'Y')) THEN
       OPEN (MDTMP,FILE=NAMTMP,STATUS='UNKNOWN',err=999)
	 write(mhtml,'(A)')'<a href=file:///'//namtmp//
     *   '>Sub-catchment Output</a><br>'
	END IF

	end if  ! End of Master part

!-------------------------------------------------------------------------------
!	Opening XML - outputfile (JS:2008-04)
!-------------------------------------------------------------------------------

! No adaptations made for parallel execution yet.
	IF (xmlyn .EQ. 'Y') CALL xmloutput !xml_output.f90

!-------------------------------------------------------------------------------
!	Opening OUTPUT for Internal sub basins (JS:2008-04)
!-------------------------------------------------------------------------------
! No adaptations made for parallel execution yet.

	IF (isbyn) CALL intsubbasin (DIROUTPUT(1:POSI3))! intsubbasin.f90
! ------------------------------------------------------------------------------

      if((rank==root) .or. (doParallel==.false.)) then
      IF (lbulletin) THEN
       OPEN (MDBULL,FILE=NAMBULL,STATUS='UNKNOWN',err=999)
	 write(mhtml,'(A)')'<a href=file:///'//nambull//
     *   '>Realtime hourly bulletin</a><br>'
	END IF
! ------------------------------------------------------------------------------
!     IGST controls the opening of hourly and daily files:
!     IGST = 0 - no file is opened
!          = 1 - only daily files are opened
!          = 2 - daily and hourly files are opened
!          = 3 - only hourly files are opened
! ------------------------------------------------------------------------------
131   FORMAT ( /,' IGST controls the opening of hourly and daily files:',
     *         /,'  IGST = 0 - no file is opened',
     *         /,'       = 1 - only daily files are opened',
     *         /,'       = 2 - daily and hourly files are opened',
     *         /,'       = 3 - only hourly files are opened'  )
      WRITE (MLOG,131)

      end if  ! End of Master part


      if((rank.ne.root) .or. (doParallel==.false.)) then

!    Berechnung von Mittelwert und Extremwerten ausgewaehlter Parameter
      CALL PARMEX1 (FBTOPI,TOPMAX,TOPMIN,TOPMIT)
!    Berechnung der Logarithmen+1.0
!               von Mittelwert und Extremwerten ausgewaehlter Parameter
      CALL PARMEXL (KWMAXL,KWMINL,KWMITL)


! ------------------------------------------------------------------------------
!   The initial content of the storages are set for each HRU
! ------------------------------------------------------------------------------
      RHG=0.0        ! real
      FBFEUCHT=RHG   ! real(:,:)
      SSM=RHG        ! real
      SSOB=0.0       ! real
      SIB=0.0		 ! real
      SUZB=SUZ       ! real
      SSMB=SSM       ! real
      SLZB=SLZ       ! real
      SLZ2=SLZ       ! real
C      SSOTB=SSOB     ! real
      SITB=SIB       ! real
      SUZTB=SUZB     ! real
      SLZTB=SLZB     ! real


      ! Initial storage contents need to be distributed among the stripes
      ! Here: According to number of non-No Data cells per stripe.

      lstats=count(gezg(:,firstRow:lastRow) .ne. int(rnodat))
      globstats=count(gezg .ne. int(rnodat))
      lratio=lstats/globstats

C      ssotb=ssotb*lratio
      sitb=sitb*lratio
      suztb=suztb*lratio
      slztb=slztb*lratio

      end if  ! End of Slaves part


      call MPI_BARRIER(MPI_COMM_WORLD,ierr)

      if((rank==root) .or. (doParallel==.false.)) then
      Call messagerun
     *    (1,' Ready! PREVAH is computing following catchment:')
      WRITE (messtext,'(A,2I6)') zn,nugeb,nrgeb
      Call messagerun (1,messtext)
      Call messagerun (1,'Be prepared, it could be a long model run!')

! ------------------------------------------------------------------------------
! Headers for output files PRI,.MIT,.STD,.TAG und .MON
! ------------------------------------------------------------------------------

      CALL WDATHD1 ( IERG,NUGEB,AE,AELHG,IART,DT,NTFL,
     *           NTG,NTGFL,AETG,ATG,LANGWSF,KURZWSF,VFEUCHTF,
     *           TAGA,MONA,JAHRA,TAGE,MONE,JAHRE,GEOBREITE,
     *           ZG,ZN,CLANG,CKURZ,CVENT,ZEIKE,ZEIPM,
     *           MDR,MDMIT,MDSTD,MDTAG,MDMON,MLOG )

!  Writing into the logfile
      CALL OPARRSG2 (MLOG,IGST,ISNOGL,PKORF,SNOKOR,
     *               CREDV,TGR,TTRANS,T0,CRMFMAX,CRMFMIN,
     *               CSNOMF,CASNO,CWU1,CWA2,CWH,CEXLIQ,CRFR,AEGRID,
     *               CBETA,CU,CRSZ,SGRLUZ,CPERC,K0H,K0,K1H,K1,K2H,
     *               K2,CG1H,SLZ1MAX,CG3H,K3H,K3,
     *               HELA,CICEMF,CAICE,KICEH,KICSH,KICFH,
     *               TICEH,TICSH,TICFH )

      end if  ! End of Master part

      cagls=CAICE
      caglt=CICEMF

! ------------------------------------------------------------------------------
!  Initialization of the GRID Cells
! ------------------------------------------------------------------------------

      subcamax=-1

      if((rank.ne.root).or.(doParallel==.false.)) then

	do ycoor=lastRow,firstRow,-1
	do xcoor=1,ncol
         if (int(gezg(xcoor,ycoor)) .eq. int(rnodat)) cycle

! ------------------------------------------------------------------------------
!  Allocation of the land use parameters controlling evapotranspiration
! ------------------------------------------------------------------------------
       subcamax=max(subcamax,scatch(xcoor,ycoor))
       land=landin(xcoor,ycoor)
       aspect=faspect(xcoor,ycoor)
       slope=fslope(xcoor,ycoor)
       btyp=fbtyp(xcoor,ycoor)
       hoehe=fhoehe(xcoor,ycoor)
       suztf(xcoor,ycoor)=suz
       slztf(xcoor,ycoor)=slz
       slz2tf(xcoor,ycoor)=slz
       nfcap=fnfc(xcoor,ycoor)
       btief=fbtief(xcoor,ycoor)

      	if (land .le. 0) then
       land=26
       landin(xcoor,ycoor)=land
       end if

       CALL EINPUZDW (land,cland,btyp,cbtyp,nfcap,btief,
     *               rlai12,bhoehe12,albedo12,flux12,rsc12,
     *               awacv,awsolv,wurzelv,vbgf,simaxf)

! ------------------------------------------------------------------------------
!   The initial plant available field capacity BFEUCHT is assigned to each HRU.
!   BFEUCHT [mm] is parameterized as function of the assimilated average
!   plant available field capacity FNFC (Vol.%), the soil depth FBTIEF (m) and
!   the roots depth WURZELV (m)
!   For some land use types there is standard parametrerization of BFEUCHT
! ------------------------------------------------------------------------------

      select case(land)
        case(1) ! BFEUCHT for water surfaces: 2500 mm
          bfeucht=2500.0

        case(2) ! BFEUCHT for built-up areas: 5 mm
          bfeucht=5.0

        case(13,14) ! BFEUCHT for glaciated areas: 0.1 mm
          bfeucht=0.1

        case(15) ! BFEUCHT for rocky areas: 3 mm
          bfeucht=3.0

        case(3:12, 18:29) ! BFEUCHT for vegetated surfaces: at least 10 mm

!         If the soil depth exceeds the root depth by more than 0.15 m, then
!         the root depth limits the absolute value of BFEUCHT

C         if ((wurzelv(mona)+0.15) .LT. fbtief(xcoor,ycoor)) then
C           bfeucht=(wurzelv(mona)+0.15)*fnfc(xcoor,ycoor)*10.0
C         else
C           bfeucht=fnfc(xcoor,ycoor)*fbtief(xcoor,ycoor)*10.0
C         end if
C         bfeucht=max(bfeucht, 10.0)

      ! Initial SSM set to field capacity
          bfeucht = sfcg(xcoor,ycoor) * whcSF(land)

      end select

! ------------------------------------------------------------------------------
!   The initial parametrization of each GRID_CELL is assigned and written in the logfile
! ------------------------------------------------------------------------------

        fbfeucht(xcoor,ycoor)=bfeucht		! Initial plant available filed capacity
        ssmtf(xcoor,ycoor)=bfeucht

        I=nrtgfl(xcoor,ycoor)

        IF (land .eq. 1) THEN
           ssotf(xcoor,ycoor)=0.0			! No snwo on water
           ssnowtf(xcoor,ycoor)=0.0
        END IF

        sliqtf(xcoor,ycoor)=0.0				! No liquid snow at the beginning



!	if (docali .eq. 'N') THEN
!        CALL ODATTFDI
!        CALL OEVAPARZ
!     skipped in the GRID_Version

        ssmb_totLocal = ssmb_totLocal + fbfeucht(xcoor,ycoor)	! Soil moisture
        ssob_totLocal = ssob_totLocal + ssotf(xcoor,ycoor)		! Snow

        end do
      end do

! ------------------------------------------------------------------------------
!     Average moisture content of the whole catchment at model initialization
! ------------------------------------------------------------------------------

      call MPI_ALLREDUCE(ssmb_totLocal,ssmb_tot,1,MPI_REAL,
     * MPI_SUM,slaves,ierr)
      call MPI_ALLREDUCE(ssob_totLocal,ssob_tot,1,MPI_REAL,
     * MPI_SUM,slaves,ierr)

	SSMB = ssmb_tot/wstats(1)
	SSOB = ssob_tot/wstats(1)

      SSOTB=SSOB     ! real
      SSMTB=SSMB
      ssotb=ssotb*lratio
      ssmtb=ssmtb*lratio  ! Distribute initial storage content across stripes

	  end if  ! End of Slaves part

	if ((docali .eq. 'N').and.(ZIERG(7:7).eq.'Y')) THEN
      CLOSE (MLOG)
	ELSE
      CLOSE (MLOG,status='delete')
	end if



! ---------------------------------------------------------------------
! Additional temporary output for FLOODEFF: Catchment areas of OBS and SIM and Output directory
! Added by viviroli@giub.unibe.ch 07.03.2005
! ---------------------------------------------------------------------
	OPEN  (721,FILE="AREA.TMP")
      WRITE (721,'(F10.2,TR2,A)') AELHG,"catchment OBS [km**2]"
	WRITE (721,'(F10.2,TR2,A)') AE   ,"catchment SIM [km**2]"
	WRITE (721,'(A)'          ) ZN					!Gauge name
	CLOSE (721)

! ------------------------------------------------------------------------------
!  Additional headers for output files
! ------------------------------------------------------------------------------
      CALL WDATHD2 (IERG,MDMIT,MDSTD,MDTAG,MDMON, AELHG,ZN )

      return
999   continue
      call messagerun (1, namq)

      call messagerun (1, 'Error while opening the output files')
      stop
	end
