!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


! -----------------------------------------------------------------------------------------
c--
c rand.f: A sample program to generate a pseudorandom number sequence.
c This program prints ten numbers between 0.0 and 1.0, and it also
c prints a different sequence each time (unless you run it a LOT of
c times---the rand() functon is, after all, only pseudorandom).
c--
      subroutine  nurand
	  use prevahglobals

      implicit none

      real rand                 ! Declare the type of the rand() function
      integer irx,iry          ! Counts random numbers
      integer*4 timeArray(3)    ! Holds the hour, minute, and second
      integer ir  ! Matthias: added this after adding "implicit none"
c--
c In order to get a different sequence each time, we initialize the
c seed of the random number function with following combination of the current
c hour, minute, and second.
c--
65    continue
      call itime(timeArray)     ! Get the current time
      ir = rand ( timeArray(1)+timeArray(2)*100+timeArray(3)*10000 )
c--
c Calling rand() with an argument of zero generates the next number
c in sequence.
c--
      do irx=1,ncol
	do iry=1,nrow
      rafi(irx,iry)=rand(0)
	rafipar(irx,iry)=raminn(1)+rafi(irx,iry)*(ramaxx(1)-raminn(1))
	end do
	end do

	ravar=rand(0)
	ravar=ramin(1)+ravar*(ramax(1)-ramin(1))

	return

      end


! ------------------------------------------------------------------------------
      subroutine statzones (outfile, ouda, oovar)
!     Provides an average array of a value for each zone in zonefile according to
!     a input grid gridfile.
!     Maximum number of zones: 999999
! ------------------------------------------------------------------------------

      use mpi
      use prevahglobals
      use MPIvars,only: rank,root,ierr
      use outvars

      implicit none

      INTEGER zontg,nprg,lk,oovar
      REAL outarr(999999,6),imx,lmx,ssoavg
      DOUBLE PRECISION, allocatable :: zstat2D(:,:)
      INTEGER*4, allocatable :: zones(:,:),zoneslong(:),mzones(:,:)
      CHARACTER*256 gridfile,zonefile,outfile

!  Global data

	logical*4 lret,retval,sbytes

	real gmean
	integer checktid(9)
	REAL zonearr(999999,6)
	integer ouda  ! formerly character*8

      if(rank==root) then
	OPEN  (721,FILE=outfile, position='append')
      end if

! ------------------------------------------------------------------------------
!     Read the file to be ...zoned
! ------------------------------------------------------------------------------
      if(rank.ne.root) then
!      errlab=0
	nprg=0
	outarr=0
	lmx=0

	ALLOCATE (zstat2D(ncol,nrow))
	ALLOCATE (mzones(ncol,nrow))

      zstat2d = dble(wrgrd)
      lmx = maxval(wrgrd)

! ------------------------------------------------------------------------------
!     Read the zones
! ------------------------------------------------------------------------------

      imx = maxval(float(scatch))

      allocate(zoneslong(int(imx)))
	zoneslong=0

      where(scatch .eq. rnodat) dom2d=rnodat
      do j = 1,int(imx)
        zoneslong(j) = count(int(scatch) .eq. j)
      end do

      zontg=0
      zontg = count(zoneslong .ne. 0.)

      ALLOCATE (ZONES(zontg,3))

      J=0

      DO lk=1,INT(imx)
          IF (ZONESLONG(lk) .ne. 0.) then
              J=J+1
              ZONES(J,2)=ZONESLONG(lk)
              ZONES(J,1)=lk
              ZONES(J,3)=0.0
          end if
      END DO

      do ycoor=1,nrow
	   nprg=nprg+1
         do xcoor=1,ncol
           if (dom2D(xcoor,ycoor) .eq. rnodat) cycle
           DO lk=1,zontg
 	       outarr(lk,4)=1e8
	       outarr(lk,5)=-1e8
	       outarr(lk,6)=0.
	       if (int(scatch(xcoor,ycoor)) .eq. ZONES(lk,1)) then
		   mzones(xcoor,ycoor)=lk
		   end if
           end do
         enddo
      enddo


! ------------------------------------------------------------------------------
      sso1500(1,1)=0.
      sso1500(1,2)=0.

      do ycoor=1,nrow
	   nprg=nprg+1
      do xcoor=1,ncol
         if (dom2D(xcoor,ycoor) .eq. rnodat) cycle

          IF (FHOEHE(xcoor,ycoor) .gt. 1500.) then
          sso1500(1,2)=sso1500(1,2)+zstat2D(xcoor,ycoor)/
     *     (max(1.0,sso1500(2,2)))
          ELSE
          sso1500(1,1)=sso1500(1,1)+zstat2D(xcoor,ycoor)/
     *      (max(1.0,sso1500(2,1)))
          END IF

         J=mzones(xcoor,ycoor)
	   outarr(J,4)=min(outarr(J,4),sngl(zstat2D(xcoor,ycoor)))
	   outarr(J,5)=max(outarr(J,5),sngl(zstat2D(xcoor,ycoor)))
	   outarr(J,6)=outarr(J,6)+sngl(zstat2D(xcoor,ycoor))
      enddo
      enddo


      do J=1,zontg
         outarr(J,3)=sngl(dble(outarr(J,6))/dble(float(zones(J,2))))
	   outarr(J,1)=float(zones(J,1))
	   outarr(J,2)=float(zones(J,2))
      end do

      end if  ! End of slaves part

      call MPI_BCAST(imx,1,MPI_INTEGER,1,MPI_COMM_WORLD,ierr)
      allocate(intArgs(1))
      allocate(writeArgs(int(imx)))
      allocate(writeArgOpts(int(imx)))
      intArgs=ouda
      writeArgs=outarr(:,3)
      writeArgOpts=1
      call MPIWriteTable(721,15)
      deallocate(intArgs)
      deallocate(writeArgs)
      deallocate(writeArgOpts)

!	write (721,'(A8,999(1x,F10.3))') ouda,(outarr(J,3),j=1,zontg)  !formats(15)

      if(allocated(zstat2d)) DEALLOCATE (zstat2D)
      if(allocated(zones)) DEALLOCATE (zones)

      goto 1000
998   CONTINUE
      if(allocated(zstat2d)) DEALLOCATE (zstat2D)
      if(allocated(zones)) DEALLOCATE (zones)

999   CONTINUE

1000  CONTINUE
      close (721)
       if (oovar .eq. 14) then
       ssoavg=(sso1500(1,1)*sso1500(2,1)+sso1500(1,2)*sso1500(2,2))/
     *        (sso1500(2,1)+sso1500(2,2))
	OPEN  (721,FILE=trim(outfile)//'_1500', position='append')

        allocate(intArgs(1))
        allocate(writeArgs(3))
        allocate(writeArgOpts(3))
        intArgs=ouda
        writeArgs=(/sso1500(1,1),sso1500(1,2),ssoavg/)
        writeArgOpts=1
        call MPIWriteTable(721,15)
        deallocate(intArgs)
        deallocate(writeArgs)
        deallocate(writeArgOpts)

!	write (721,'(A8,999(1x,F10.3))') ouda,sso1500(1,1),sso1500(1,2),
!     *     ssoavg
       close (721)
       end if



	return

      END
