!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2003 Joachim Gurtz, Massimiliano Zappa
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Include glacier and soil variables

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      include 'initpar.inc'
! ------------------------------------------------------------------------------
! Reads the new control file
! Implemented by MZ in September 2003
! Added routing Module - MZ 25 November 2003
! Added compatibily to Windows GUI Version - MZ January 2004
! ------------------------------------------------------------------------------

      MDAT6=333

      MODUL(1)='[Identification]'
      MODUL(2)='[Tuneable Parameters]'
      MODUL(3)='[Catchment settings]'
      MODUL(4)='[Storage content]'
      MODUL(5)='[Date and save]'
      MODUL(6)='[Optional modules]'
      MODUL(7)='[Meteorological input]'
      MODUL(8)='[Climate scenario]'
      MODUL(9)='[Model output]'
      MODUL(10)='[Efficiency]'
	  MODUL(11)='[Routing]   '
      MODUL(12)='[Internal sub basins]'
      MODUL(13)='[Xml output]'

! ------------------------------------------------------------------------------
c     STANDARDWERTE
! ------------------------------------------------------------------------------

      NTG=1
      dorout='N'
	  xmlyn='N'! ------------------------------------------------------------------------------
c     Eingabe der Name der Inputdatei
! ------------------------------------------------------------------------------

      call posnam (NAMEIN,1,pos)
      if ((rank==root) .or. (doParallel==.false.))
     *  call MESSAGEFILE(NAMEIN)
      OPEN (MDAT6,FILE=trim(NAMEIN),STATUS='OLD',ERR=198)

! ------------------------------------------------------------------------------
c    Einlesen der Inputdatei
! ------------------------------------------------------------------------------

! This is actually a loop! GOTO 33 statement at the bottom
33    CONTINUE

      LN=-1

      READ (MDAT6,'(A78)',END=199,err=199) kopf

      DO I=1,13
         IF (kopf(1:12) .EQ. MODUL(I)(1:12)) THEN
            LN=I
            if((rank==root).or.(doParallel==.false.))
     *         call messagerun (2,MODUL(I)//' OK')
         ENDIF
      END DO


! ------------------------------------------------------------------------------
!     '[Identification]'
! ------------------------------------------------------------------------------

      IF (LN .EQ. 1) THEN
          READ(MDAT6,*,err=198) NUGEB
          READ(MDAT6,'(A32)',err=198) ZN
		  ZN(31:32)='  '
          ZG=ZN
          ZK=ZN
          READ(MDAT6,*,err=198) GEOBREITE
          READ(MDAT6,*,err=198) AELHG
          READ(MDAT6,*,err=198) AEGRID
      ENDIF


! ------------------------------------------------------------------------------
!     '[Tuneable Parameters]'
! ------------------------------------------------------------------------------

      IF (LN .EQ. 2) THEN
	     READ(MDAT6,*,err=198) pardist
            READ(MDAT6,*,err=198) CREDV

	     READ(MDAT6,*,err=198) PKORF


! ------------------------------------------------------------------------------
! Snowmelt module
         READ(MDAT6,*,err=198) SNOKOR
         READ(MDAT6,*,err=198) TGR
         READ(MDAT6,*,err=198) TTRANS
         READ(MDAT6,*,err=198) T0
         READ(MDAT6,*,err=198) CRMFMAX
         READ(MDAT6,*,err=198) CRMFMIN
         READ(MDAT6,*,err=198) CWU1
         READ(MDAT6,*,err=198) CWA2
         READ(MDAT6,*,err=198) CWH
         READ(MDAT6,*,err=198) CEXLIQ
         READ(MDAT6,*,err=198) CRFR
         READ(MDAT6,*,err=198) CSNOMF
         READ(MDAT6,*,err=198) CASNO

! backup PKORF and SNOKOR

         pkorf_bp=PKORF
         snokor_bp=SNOKOR

! ------------------------------------------------------------------------------
! Runoff-generation module
         READ(MDAT6,*,err=198) CBETA
         READ(MDAT6,*,err=198) CU
         READ(MDAT6,*,err=198) CRSZ
         READ(MDAT6,*,err=198) SGRLUZ
         READ(MDAT6,*,err=198) K0H
         READ(MDAT6,*,err=198) K1H
         READ(MDAT6,*,err=198) K2H
         READ(MDAT6,*,err=198) CG1H
         READ(MDAT6,*,err=198) SLZ1MAX
         READ(MDAT6,*,err=198) CPERC
! ------------------------------------------------------------------------------
! Transformation of storage coefficient from hours to exponential factor
      K0=0.0
      IF (K0H .GT. 0.0) K0=1.0-EXP(-1.0/K0H)
      K1=0.0
      IF (K1H .GT. 0.0) K1=1.0-EXP(-1.0/K1H)
      CG2H=K2H
      K2=0.0
      IF (K2H .GT. 0.0) K2=1.0-EXP(-1.0/K2H)
      K3=0.0
      CG3H=0.11111*CG2H			! After Schwarze

! ------------------------------------------------------------------------------
! Glacier module
         READ(MDAT6,*,err=198) HELA
         READ(MDAT6,*,err=198) CICEMF
         READ(MDAT6,*,err=198) CAICE
         READ(MDAT6,*,err=198) KICEH
         READ(MDAT6,*,err=198) KICSH
         READ(MDAT6,*,err=198) KICFH
         READ(MDAT6,*,err=198) TICEH
         READ(MDAT6,*,err=198) TICSH
         READ(MDAT6,*,err=198) TICFH

! ----------------------------------------------------
!     Educational - Exercise 4
      if (lschool) then
	 lscen=.true.
	  if (educnr .eq. 4) then
      if (educlog(1)) CICEMF=educparam(1)
      if (educlog(2)) CAICE=educparam(2)
      if (educlog(3)) KICEH=educparam(3)
	end if
	end if
! ----------------------------------------------------

! ------------------------------------------------------------------------------
! ESCIMO module
         A0=0.4
		 APOS=0.15
		 ANEG=0.05
		 ALBADD=0.5
         READ(MDAT6,*,err=33) A0
         READ(MDAT6,*,err=33) APOS
         READ(MDAT6,*,err=33) ANEG
         READ(MDAT6,*,err=33) ALBADD
      ENDIF

! ------------------------------------------------------------------------------
! adjustmend ditributed parameters
       fixpkorf=PKORF
       fixsnokor=SNOKOR
       fixperc=CPERC
       fixsgrluz=SGRLUZ
       fixicet=cicemf
       fixicer=caice
       fixsnow=CSNOMF
       fixt0=T0

! ------------------------------------------------------------------------------
!     '[Catchment settings]'
! ------------------------------------------------------------------------------


      IF (LN .EQ. 3) THEN
         READ(MDAT6,'(A256)',err=198) NADGKW
         READ(MDAT6,*,err=198) NTG
		 READ(MDAT6,'(A1)',err=198) zk1
		 do i=1,13
         READ(MDAT6,'(A3)',err=198) gkext(i)
		 end do

      print*, "NADGKW", nadgkw

      ! Matthias - Nov 2015
       gkext(14) = 'whc'
       ! Temporary workaround for CH2018 runs
       gkext(13) = 'mez'

	   CALL POSNAM (NADGKW,1,POSI1)
	   OPEN (731, file=NADGKW(1:POSI1)//'.'//gkext(1), status='old',
     *     form='binary')
	         call posnam (NADGKW//'.'//gkext(1),2,posiid2)
		 do I=posiid2,1,-1
	     if (idnam(I:I) .eq. '\') exit
	     if (idnam(I:I) .eq. '/') exit
	     posiid1=I
	     end do

	     ! MS, Feb 2019: Store name of catchment
	     ! Kludge to extract catchment name from directory path
	     posCatch2 = len(NADGKW)
	     do i = posCatch2, 1, -1
	       if(NADGKW(i:i).eq.'\') exit
	       if(NADGKW(i:i).eq.'/') exit
	     end do
	     posCatch1 = i
	     catchAbbr = NADGKW((posCatch1 + 1):posCatch2)
	     print*, "catchAbbr", catchAbbr

   	   uwas=731
	   INCLUDE 'rwasim.inc'
        !call MPI_BARRIER(MPI_COMM_WORLD,ierr)
          !if((doParallel==.false.).or.(rank.ne.root)) then
            INCLUDE 'r2d.inc'
	        ALLOCATE(nrtgfl(ncol,nrow))
	        nrtgfl=dom2d
	        deallocate(DOM2D)
          !end if



		 ST='Y'

      ENDIF

! ------------------------------------------------------------------------------
!     '[Storage content]'
! ------------------------------------------------------------------------------

      IF (LN .EQ. 4) THEN
         READ(MDAT6,'(A1)',err=198) zk1
         DO I=1,NTG
            READ(MDAT6,*,err=198) SSOTG(I)
            IF (SSOTG(1) .LT. 0.0) THEN
              DO J=1,NTG
                 SSOTG(J)=0.0
              END DO
              EXIT
            ENDIF
         END DO
         READ(MDAT6,'(A1)',err=198) zk1
         READ(MDAT6,*,err=198) ANFFC
         READ(MDAT6,*,err=198) SUZ
         READ(MDAT6,*,err=198) SLZ
         READ(MDAT6,*,err=198) QGA
      ENDIF


! ------------------------------------------------------------------------------
!     '[Date and save]'
! ------------------------------------------------------------------------------

      IF (LN .EQ. 5) THEN
      READ(MDAT6,*,err=198) TAGA,MONA,JAHRA
      READ (MDAT6,'(A1)',err=198) LOADID
	  LOADROUT=LOADID
      READ (MDAT6,'(A256)',err=198) dirstate
      READ(MDAT6,*,err=198) TAGE,MONE,JAHRE,HOURSE
      WRITE (LOADCHAR,'(2I2.2,I4.4)') TAGA,MONA,JAHRA
  	  DATUMCHAR='01011901'

      READ (MDAT6,'(A1)',err=198) SAVEID
      READ (MDAT6,'(A1)',err=198) SAVEMMID
      READ (MDAT6,'(A1)',err=198) SAVEMOREID
      READ (MDAT6,*,err=198) SAVEMORENN

      allocate (SAVEDD(SAVEMORENN))
      allocate (SAVEMM(SAVEMORENN))
      allocate (SAVEYY(SAVEMORENN))
      allocate (SAVECHAR(SAVEMORENN))

      DO I=1,SAVEMORENN
      READ (MDAT6,*,err=198) SAVEDD(i),SAVEMM(i), SAVEYY(i)
      WRITE (SAVECHAR(i),'(2I2.2,I4.4)') SAVEDD(i),SAVEMM(i), SAVEYY(i)
      END DO
      ENDIF


! ------------------------------------------------------------------------------
!     '[Optional modules]'
! ------------------------------------------------------------------------------

      IF (LN .EQ. 6) THEN
         READ(MDAT6,*,err=198) SNOEST
         READ(MDAT6,*,err=198) SHADST
         READ(MDAT6,*,err=198) ISNOGL
         READ(MDAT6,*,err=198) IART
         READ(MDAT6,*,err=198) KURZWSF
         READ(MDAT6,*,err=198) LANGWSF
         READ(MDAT6,*,err=198) VFEUCHTF
         READ(MDAT6,*,err=198) DEBUG
         READ(MDAT6,*,err=198) deltaT
         READ(MDAT6,*,err=198) deltaP
         READ(MDAT6,*,err=198) tpswitch


         PRtoTM=0
         TMtoPR=0
         prevahBC=0

         if (tpswitch(1:1) .eq. '1') PRtoTM=1
         if (tpswitch(2:2) .eq. '1') TMtoPR=1
         if (tpswitch(3:3) .eq. '1') prevahBC=1
         READ(MDAT6,*,err=198) LCDyn
         READ(MDAT6,*,err=198) glacSwitch
         READ(MDAT6,*,err=198) rootslag
         READ(MDAT6,*,err=198) dohmax
         READ(MDAT6,*,err=198) dolared
         READ(MDAT6,*,err=198) dosfc
         READ(MDAT6,*,err=198) dosmort
         READ(MDAT6,*,err=198) cdtol
         READ(MDAT6,*,err=198) doground
         READ(MDAT6,*,err=198) ghogyear
         READ(MDAT6,*,err=198) grsmin
         READ(MDAT6,*,err=198) readFH
         READ(MDAT6,*,err=198) writeFH


! ----------------------------------------------------
!     Educational - Exercise 2
      if (lschool) then
	lscen=.true.
	if (educnr .eq. 2) then
      if (educlog(1)) IART=0
      if (educlog(2)) IART=1
      if (educlog(3)) IART=2
      if (educlog(4)) IART=3
      if (educlog(5)) IART=4
      if (educlog(6)) IART=5
      if (educlog(7)) KURZWSF=5
      if (educlog(8)) KURZWSF=6
	end if

! ----------------------------------------------------
!     Educational - Exercise 3
	if (educnr .eq. 3) then
      if (educlog(6)) THEN
	  ISNOGL=0
	  CRMFMAX=2.0
	  CRMFMIN=1.25
	  end if
      if (educlog(7)) ISNOGL=3
      if (educlog(8)) ISNOGL=10
	end if
	end if
! ----------------------------------------------------

      ENDIF


 ! ------------------------------------------------------------------------------
 !     '[Meteorological input]'
 ! ------------------------------------------------------------------------------

      IF (LN .EQ. 7) THEN
      READ (MDAT6,'(A256)',err=198) dirmeteo
	  READ (MDAT6,'(A)',err=198) gebmet
      READ(MDAT6,*,err=198) NKZ
      READ(MDAT6,*,err=198) ANFS
      READ(MDAT6,*,err=198) DIGIEXT
34    continue
      READ (MDAT6,'(A78)',END=199,err=888) kopf
      if((rank==root).or.(doParallel==.false.))
     *  call messagerun (2,kopf(1:28))

      IF (kopf(1:15) .eq. '[Precipitation]') THEN
      READ (MDAT6,'(A1)',err=34) rmet(1)
      READ (MDAT6,*,err=34) dtmin(1)
	  rsplit='N'
	  IF (dtmin(1) .eq. 10) then
	  call messagerun (1,'Splitting rainfall to 10 minutes values')
	  rsplit='Y'
	  dtmin(1)=60
	  end if
      READ (MDAT6,'(A256)',err=34) filena(1)
! ----------------------------------------------------
!     Educational - Exercise 7
      if (lschool) then
	 lscen=.true.
	  if (educnr .eq. 7) filena(1)=trim(adjustl(eduprecnam))
	end if
      GOTO 34
      END IF

      IF (kopf(1:13) .eq. '[Temperature]') THEN
      READ (MDAT6,'(A1)',err=34) rmet(2)
      READ (MDAT6,*,err=34) dtmin(2)
      READ (MDAT6,'(A256)',err=34) filena(2)
      READ (MDAT6,*,err=34) TOPT
      GOTO 34
      END IF

      IF (kopf(1:18) .eq. '[Global Radiation]') THEN
      READ (MDAT6,'(A1)',err=34) rmet(3)
      READ (MDAT6,*,err=34) dtmin(3)
      READ (MDAT6,'(A256)',err=34) filena(3)
      GOTO 34
      END IF

      IF (kopf(1:28) .eq. '[Vapour Pressure / Humidity]') THEN
      READ (MDAT6,'(A1)',err=34) rmet(4)
      READ (MDAT6,*,err=34) dtmin(4)
      READ (MDAT6,'(A256)',err=34) filena(4)
      READ (MDAT6,*,err=34) RHOPT
      GOTO 34
      END IF

      IF (kopf(1:12) .eq. '[Wind speed]') THEN
      READ (MDAT6,'(A1)',err=34) rmet(5)
      READ (MDAT6,*,err=34) dtmin(5)
      READ (MDAT6,'(A256)',err=34) filena(5)
      GOTO 34
      END IF

      IF (kopf(1:28) .eq. '[Relative sunshine duration]') THEN
      READ (MDAT6,'(A1)',err=34) rmet(6)
      READ (MDAT6,*,err=34) dtmin(6)
	  if (dtmin(6) .gt. 60) tistep=24
      READ (MDAT6,'(A256)',err=34) filena(6)
      GOTO 34
      END IF

888   continue
      backspace (MDAT6)
      IF ((docali .eq. 'T') .or. (docali .eq. 't')) then
	  dirmeteo(1:3)=prtunit
	  end if
	  DO I=1,6
	    filenr(i)=filena(I)
	  END DO

! Changing Meteopath to NWP-File - Only precip so far
      if (educnr .eq. -99) then
      if (nwpuse) then
      dtmin(1)=nwpdt
	  filena(1)=nwpnam(3)
      end if
      end if
      END IF


 ! ------------------------------------------------------------------------------
 !     '[Climate scenario]'
 ! ------------------------------------------------------------------------------

      IF (LN .EQ. 8) THEN
         READ(MDAT6,'(A1)',err=198) szaktiv
		 NRK=0
         READ(MDAT6,'(A)') CLIMFILE
      ENDIF


 ! ------------------------------------------------------------------------------
 !     '[Model output]'
 ! ------------------------------------------------------------------------------

      IF (LN .EQ. 9) THEN
         READ (MDAT6,'(A256)',err=198) diroutput
         READ(MDAT6,'(A256)',err=198) NAMAUS
         READ(MDAT6,'(A15)',err=198) ZIERG
		 IERG=0
         READ(MDAT6,'(A1)',err=198) GRDAKT
         READ (MDAT6,'(A256)',err=198) IDNAM
         DO I=1,21
            READ (MDAT6,'(A7)') WHGRID(I)
         END DO
	  DO I=1,6
	    CALL POSNAM (filena(I),1,POS)
	    CALL POSNAM (diroutput,1,POSI1)
	    filena(I)=diroutput(1:posi1)//filena(I)(1:POS)
	    CALL POSNAM (filena(I),1,POS)
	  END DO

	  ! Matthias: increased the length of WHGRID and ERGRID to write more files.
	  ! Control files not modified for now - new WHGRID lines here:

	  whgrid(22) = 'N-N-N-N'
	  whgrid(23) = 'N-N-N-N'
	  whgrid(24) = 'N-N-N-N'
	  whgrid(25) = 'N-N-N-N'

      ENDIF



 ! ------------------------------------------------------------------------------
 !     '[Efficiency]'
 ! ------------------------------------------------------------------------------
      IF (LN .EQ. 10) THEN
	     READ(MDAT6,*,err=231) neffis
		 goto 232
231   CONTINUE
         BACKSPACE (MDAT6)
         READ (MDAT6,'(A1)',err=198) doeff
		 READ(MDAT6,*,err=198) neffis
232   CONTINUE
         DO I=1,neffis
            READ (MDAT6,'(A256)',err=198) effinams(i)
         END DO
      ENDIF

 ! ------------------------------------------------------------------------------
 !     '[Routing]'
 ! ------------------------------------------------------------------------------

      IF (LN .EQ. 11) THEN
	     READ (MDAT6,'(A1)') dorout
         READ(MDAT6,*,err=198) nrouts
         DO I=1,nrouts
            READ (MDAT6,'(A256)',err=198) routnams(i)
            read (MDAT6,*) translation(I)
            read (MDAT6,*) storage(I)
         END DO
      ENDIF

      goto 33

  ! ------------------------------------------------------------------------------
  !     '[Xml output]'
  ! ------------------------------------------------------------------------------

       IF (LN .EQ. 13) THEN
 		 READ (MDAT6,'(A1)',err=198) xmlyn
 		 IF ((xmlyn .EQ. 'y') .OR. (xmlyn .EQ. 'Y')) THEN
 			xmlyn='Y'
 			READ (MDAT6,'(A)',err=198) xmlpath
 			READ (MDAT6,'(A)',err=198) gaugeid
 			READ (MDAT6,'(A)',err=198) runname
 		 ENDIF
 	 ENDIF

 	  goto 33


198   CONTINUE


      IF (LN .GT. 0) THEN
      call messagerun (2,'Error in module '//MODUL(LN))
      ENDIF
      call messagerun (2,' ')
      call messagerun (2,'Error in input file:')
      call messagerun (2,'- REAL instead of INTEGER')
      call messagerun (2,'- NUMBER instead of CHARACTER or vice versa')
      call messagerun (2,'- Filename longer than 12 characters')
199   CONTINUE

      CLOSE(MDAT6)



! ------------------------------------------------------------------------------
! Reading in the glacier parameter file (file path hard-coded for now)

      posPath = pos+8
      !glacStr = "glac"
!      write(glacStr,*) '/Got200.glac'
      pathGlacConf= namein(1:posPath)//'.glac'

      print*, namein
      print*, "pos", pos
      print*, "posPath ", posPath
      print*, pathGlacConf

      open(555,file=trim(pathGlacConf),status='old',access='sequential')

      read(555,*) numGlac

      print*, numGlac

      allocate(pathsGlac(numGlac))

      do iGlac = 1,numGlac
        read(555,'(A)') pathsGlac(iGlac)
        print*, pathsGlac(iGlac)
      end do
      close(555)

      print*, 'readpin2d done'
