!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!     -----------------------------------------------------------------
      SUBROUTINE SANEFFI (fileeffi,diroutput,fileout,mhtml)

!     -----------------------------------------------------------------
!     Analyses statistical parameters for runoff simulations:
!     Input : 5 Statistics Files, 4 for linear Values, 1 for ln-Values
!     Output : Score for this model run
!
!     Literature: LETGATES, D.R., McCABE, G.J., 1999. Evaluating
!                    the use of goodness_of_fit measures in
!                    hydrologi! and hydroclimati! model validation.
!                    Water Resources Research, vol. 35, no.1, pp 233-241.
!                 DYCK, S., PESCHKE, G. (1995). Grundlagen der Hydrologie.
!                    Verlag f�r Bauwesen, 3. stark bearbeitete Auflage,
!                    Berlin, pp 123-127.
!                 SEIBERT, J. - MCDONNELL, J.J.: On the dialog between
!                    experimentalist and modeler in catchment hydrology:
!                    Use of soft data for multi-criteria model calibration.
!                    Water Resour. Res., 38(11), 2002,
!                    DOI:10.1029/2001WR000978.
!
!     Version 1.0: Massimiliano Zappa, May 2003
!     Actual Version 2.0 Massimiliano Zappa, September 2003
!     Fully implemented in PREVAH in September 2003
!     Version 2.1, adapted to Windows GUI requirments - January 2004
!     -----------------------------------------------------------------

      implicit none

      INTEGER I,LN,LN1,LN2,JA,JE,iteri,itlim,wnext,mhtml
      INTEGER MONSEA(12),MS,SCOL,RSCOL,OCOL,ROCOL,SZK,OZK,SVID,stvar
      INTEGER kaln,kalyy(100),crit,pairs,docrit(9),LN3,OPT(2),posf
      INTEGER iter,pos,af,bestrun,stepa,OKI4(2,4),LINE(2),pos1,pospin
      REAL SV(100,4,10),SV2(100,5,3),wcrit(9),limcrit(2,9)
      REAL best,OKR4(2,4),bestold,OKN4(2,4)
      REAL SCORE(9),ratio(100),wstats(6),value(100),scorew(4),wall(4)
      CHARACTER*256 filehin,fileh,fileeffi
      CHARACTER*256 fileprev,fileout,effnam
      CHARACTER heada(5)*100,headb(5)*100,zkcrit(9)*75
      CHARACTER*100 bestzk,messrun
      CHARACTER*256 diroutput,filed,filedd,cppin,nextnam
      CHARACTER*2 atyp,itt
      CHARACTER*1 seas,yby,kaly,dm
      CHARACTER*8 addfa(3),addfb(2)

!     --------------------------------------
!     Input Parameters
!     --------------------------------------
      CALL POSNAM (diroutput,1,pos1)

      call posnam (fileout,1,posf)
	OPEN (444,FILE=TRIM(fileout(1:(posf-8))//"_sco.dat"),
     *      status='unknown')
	write(mhtml,'(A)')'Standard scores<br>'
      write(mhtml,'(A)')'<a href=file:///'//TRIM(fileout(1:(posf-8))//
     * "_sco.dat")//'>'//TRIM(fileout(1:(posf-8))//"_sco.dat")//
     * ' </a><br>'

      crit=9

      limcrit(1,1)=0.2500000
	limcrit(2,1)=0.9500000
	wcrit(1)=6.0000000
      limcrit(1,2)=0.1000000
	limcrit(2,2)=0.0100000
	wcrit(2)=4.0000000
      limcrit(1,3)=0.5000000
	limcrit(2,3)=0.0200000
	wcrit(3)=2.0000000
      limcrit(1,4)=0.2500000
	limcrit(2,4)=0.9500000
	wcrit(4)=3.0000000
      limcrit(1,5)=0.1000000
	limcrit(2,5)=0.0100000
	wcrit(5)=2.0000000
      limcrit(1,6)=0.5000000
	limcrit(2,6)=0.0200000
	wcrit(6)=1.0000000
      limcrit(1,7)=0.1000000
	limcrit(2,7)=0.0100000
	wcrit(7)=10.0000000
      limcrit(1,8)=0.1500000
	limcrit(2,8)=0.0200000
	wcrit(8)=4.0000000
      limcrit(1,9)=0.1000000
	limcrit(2,9)=0.0200000
	wcrit(9)=3.0000000

	do I=1,crit
      docrit(I)=1
	end do

      call messagerun (1,'Computing Score')

!     --------------------------------------
!     Efficiency Parameters
!     --------------------------------------
      call posnam (fileeffi,1,pos)
      OPEN (443, file=fileeffi(1:pos), status='old')
      read (443,'(A1)') dm
!      print* ,'Begin Year ? [YYYY]'
      read (443,*) JA

!      print* ,'End Year ? [YYYY]'
      read (443,*) JE

      yby='N'
!      print* ,'Input files Year by Year [Y,N] ?'
	read (443,'(A1)') yby

!      print* ,'Simulated File'
	read (443,'(A)') filehin

!	print* ,'Headers, Data Columns and Columns for Runoff?'
      read (443,*) SZK,SCOL,RSCOL

!      print* ,'Observed File'
	read (443,'(A)') fileh

!	print* ,'Headers, Data Columns and Columns for Runoff?'
      read (443,*) OZK,OCOL,ROCOL

!	print* ,'HH=Hourly Data,  DD=Daily Data,   DH= Daily Data from
!     * Hourly Data'
	read (443,'(A2)') atyp

!	print* ,'4 Chars for Output File'
	read (443,'(A4)') filed(1:4)

!	print* ,'Season [Y,N]'
	read (443,'(A1)') seas

!      print* ,'How many months for the season ?'
	read (443,*) ms

!      print* ,'Which months for the season ? (space separated)'
	read (443,*) (MONSEA(I),I=1,MS)

!	print* ,'Exclude calibration years [Y,N]'
	read (443,'(A1)') kaly

!      print* ,'How many calibration years ?'
	read (443,*) kaln

	IF (kaly .eq. 'n') GOTO 900
	IF (kaly .eq. 'N') GOTO 900

!      print* ,'Which years ? (YYYY, space separated)'
	read (443,*) (kalyy(I),I=1,kaln)
	close (443)

!     --------------------------------------
!     Read the analysis file
!     --------------------------------------


900   CONTINUE

      SVID=JE-JA+1-kaln+2+12
	IF (seas .eq. 'N') svid=svid-1

      addfa(1)='_ost.dat'
      addfa(2)='_sst.dat'
	addfa(3)='_dst.dat'

	addfb(1)='_eli.dat'
      addfb(2)='_eln.dat'

      DO LN2=1,3
	   filedd=diroutput(1:pos1)//filed(1:4)//addfa(LN2)
	   OPEN (443, file=filedd, status='old',ERR=999)
	   READ (443,'(A100)') Heada(LN2)
	   READ (443,'(A100)') Headb(LN2)
	   DO LN1=1,SVID
	   READ (443,*) LN3,(SV(LN1,LN2,1)),(SV(LN1,LN2,I),I=2,9)
	   END DO
	   CLOSE (443)
	END DO

      write (444,'(A13)') '[SCORES '//FILED(1:4)//']'

	DO LN1=1,SVID
	   ratio(LN1)=ABS(SV(LN1,2,5)/SV(LN1,1,5)-1)
      END DO


      DO LN2=2,3
	   filedd=diroutput(1:pos1)//filed(1:4)//addfb(LN2-1)
	   OPEN (443, file=filedd, status='old')
	   READ (443,'(A100)') Heada(LN2+2)
	   READ (443,'(A100)') Headb(LN2+2)
	   DO LN1=1,SVID
	   READ (443,*) LN3,(SV2(LN1,1,1)),(SV2(LN1,I,LN2),I=1,4)
	   END DO
	   CLOSE (443)
	END DO

!     --------------------------------------
!     COMPUTATION OF THE SCORES
!     --------------------------------------

!     --------------------------------------
!     1st SCORE
!     --------------------------------------

      IF (SV2(1,4,2) .LT. limcrit(1,1)) THEN
	    score(1)=0.0
	   ELSE
	    score(1)=(SV2(1,4,2)-limcrit(1,1))/(limcrit(2,1)-limcrit(1,1))
	END IF
      score(1)=min(1.0,score(1))
      write (444,'(A)') '[Linear efficiency - full period]'
	write (444,'(A8,F6.3)') 'ELN   = ',SV2(1,4,2)
	write (444,'(A8,F6.3)') 'SCORE = ',score(1)
!     --------------------------------------
!     2nd SCORE
!     --------------------------------------

	STVAR=JE-JA+1-kaln
	DO I=1,STVAR
	   value(I)=SV2((I+1),4,2)
	END DO

      include 'osts.inc'

      IF (wstats(6) .GT. limcrit(1,2)) THEN
	    score(2)=0.0
	   ELSE
	    score(2)=1.0-max(0.0,(((wstats(6)-limcrit(2,2))/
     *             (limcrit(1,2)-limcrit(2,2)))))
	END IF

      score(2)=min(1.0,score(2))

	write (444,'(A)') '[Linear efficiency - year-by-year]'
      write (444,'(A8,F6.3)') 'STDEV = ',wstats(6)
	write (444,'(A8,F6.3)') 'SCORE = ',score(2)
!     --------------------------------------
!     3rd SCORE
!     --------------------------------------

	STVAR=JE-JA+1-kaln
	DO I=1,12
	   value(I)=SV2((I+STVAR+1),4,2)
	END DO
	stvar=12

      include 'osts.inc'

      IF (wstats(6) .GT. limcrit(1,3)) THEN
	    score(3)=0.0
	   ELSE
	    score(3)=1.0-max(0.0,(((wstats(6)-limcrit(2,3))/
     *             (limcrit(1,3)-limcrit(2,3)))))
	END IF

      score(3)=min(1.0,score(3))

	write (444,'(A)') '[Linear efficiency - month-by-month]'
      write (444,'(A8,F6.3)') 'STDEV = ',wstats(6)
	write (444,'(A8,F6.3)') 'SCORE = ',score(3)
!     --------------------------------------
!     4th SCORE
!     --------------------------------------

      IF (SV2(1,4,3) .LT. limcrit(1,4)) THEN
	    score(4)=0.0
	   ELSE
	    score(4)=(SV2(1,4,3)-limcrit(1,4))/(limcrit(2,4)-limcrit(1,4))
	END IF
      score(4)=min(1.0,score(4))

	write (444,'(A)') '[Logarithmic efficiency - full period]'
	write (444,'(A8,F6.3)') 'ELN   = ',SV2(1,4,3)
	write (444,'(A8,F6.3)') 'SCORE = ',score(4)
!     --------------------------------------
!     5th SCORE
!     --------------------------------------

	STVAR=JE-JA+1-kaln
	DO I=1,STVAR
	   value(I)=SV2((I+1),4,3)
	END DO

      include 'osts.inc'

      IF (wstats(6) .GT. limcrit(1,5)) THEN
	    score(5)=0.0
	   ELSE
	    score(5)=1.0-max(0.0,(((wstats(6)-limcrit(2,5))/
     *             (limcrit(1,5)-limcrit(2,5)))))
	END IF

      score(5)=min(1.0,score(5))

	write (444,'(A)') '[Logarithmic efficiency - year-by-year]'
      write (444,'(A8,F6.3)') 'STDEV = ',wstats(6)
	write (444,'(A8,F6.3)') 'SCORE = ',score(5)
!     --------------------------------------
!     6th SCORE
!     --------------------------------------

	STVAR=JE-JA+1-kaln
	DO I=1,12
	   value(I)=SV2((I+STVAR+1),4,3)
	END DO

	stvar=12

      include 'osts.inc'

      IF (wstats(6) .GT. limcrit(1,6)) THEN
	    score(6)=0.0
	   ELSE
	    score(6)=1.0-max(0.0,(((wstats(6)-limcrit(2,6))/
     *             (limcrit(1,6)-limcrit(2,6)))))
	END IF

      score(6)=min(1.0,score(6))

	write (444,'(A)') '[Logarithmic efficiency - month-by-month]'
      write (444,'(A8,F6.3)') 'STDEV = ',wstats(6)
	write (444,'(A8,F6.3)') 'SCORE = ',score(6)
!     --------------------------------------
!     7th SCORE
!     --------------------------------------

      IF (ratio(1) .GT. limcrit(1,7)) THEN
	    score(7)=0.0
	   ELSE
	    score(7)=1.0-max(0.0,(((ratio(1)-limcrit(2,7))/
     *             (limcrit(1,7)-limcrit(2,7)))))
	END IF
      score(7)=min(1.0,score(7))

	write (444,'(A)') '[Volume error - full period]'
	write (444,'(A8,F6.3)') 'SCORE = ',score(7)
!     --------------------------------------
!     8th SCORE
!     --------------------------------------

	STVAR=JE-JA+1-kaln
	DO I=1,STVAR
	   value(I)=ratio(I+1)
	END DO

      include 'osts.inc'

      IF (wstats(6) .GT. limcrit(1,8)) THEN
	    score(8)=0.0
	   ELSE
	    score(8)=1.0-max(0.0,(((wstats(6)-limcrit(2,8))/
     *             (limcrit(1,8)-limcrit(2,8)))))
	END IF

      score(8)=min(1.0,score(8))

	write (444,'(A)') '[Volume error - year-by-year]'
      write (444,'(A8,F6.3)') 'STDEV = ',wstats(6)
	write (444,'(A8,F6.3)') 'SCORE = ',score(8)
!     --------------------------------------
!     9th SCORE
!     --------------------------------------

	STVAR=JE-JA+1-kaln
	DO I=1,12
	   value(I)=ratio(I+1+STVAR)
	END DO


	stvar=12

      include 'osts.inc'

      IF (wstats(6) .GT. limcrit(1,9)) THEN
	    score(9)=0.0
	   ELSE
	    score(9)=1.0-max(0.0,(((wstats(6)-limcrit(2,9))/
     *             (limcrit(1,9)-limcrit(2,9)))))
	END IF

      score(9)=min(1.0,score(9))

	write (444,'(A)') '[Volume error - month-by-month]'
      write (444,'(A8,F6.3)') 'STDEV = ',wstats(6)
	write (444,'(A8,F6.3)') 'SCORE = ',score(9)
!     --------------------------------------
!     Finish
!     --------------------------------------

      wall(1)=wcrit(1)+wcrit(2)+wcrit(3)
      wall(2)=wcrit(4)+wcrit(5)+wcrit(6)
      wall(3)=wcrit(7)+wcrit(8)+wcrit(9)

!     -------------------------------------------
!     Weighted mean for criteria 1-3,4-6 and 7-9
!     -------------------------------------------

	scorew(1)=(score(1)*wcrit(1)+score(2)*wcrit(2)+
     /           score(3)*wcrit(3))/wall(1)
	scorew(2)=(score(4)*wcrit(4)+score(5)*wcrit(5)+
     /           score(6)*wcrit(6))/wall(2)
	scorew(3)=(score(7)*wcrit(7)+score(8)*wcrit(8)+
     /           score(9)*wcrit(9))/wall(3)

	scorew(1)=max(0.001,scorew(1))
	scorew(2)=max(0.001,scorew(2))
	scorew(3)=max(0.001,scorew(3))

!     --------------------------------------------------------
!     Overall acceptability after Seibert/McDonnell WRR 2002
!     Acceptability for ELI criteria - ELI-SCORE
!     Acceptability for ELN criteria - ELN-SCORE
!     Acceptability for SST/DST criteria - DST-SCORE
!     --------------------------------------------------------

	wall(4)=wall(1)+wall(2)+wall(3)
	wall(1)=wall(1)/wall(4)
	wall(2)=wall(2)/wall(4)
	wall(3)=wall(3)/wall(4)
	scorew(4)=scorew(1)**wall(1)*scorew(2)**wall(2)*scorew(3)**wall(3)

!     -------------------------------------
!     Write summary and identify best run
!     -------------------------------------

	write (444,'(A14)') '[Total scores]'
	write (444,'(A12,F6.3)') 'ELI-SCORE = ',scorew(1)
	write (444,'(A12,F6.3)') 'ELN-SCORE = ',scorew(2)
	write (444,'(A12,F6.3)') 'DST-SCORE = ',scorew(3)
	write (444,'(A12,F6.3)') 'TOT-SCORE = ',scorew(4)

999   CONTINUE

	CLOSE (444)

	RETURN
      END
