!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Set soil parameters (available water capacity). Done at the beginning of each month
! for land use classes 6 (Cereals), 18 (Orchards) and 20 (Wheat) due to temporally varying
! rooting depths. Also done after each TreeMig run and subsequent land use update for cells where land use
! class has changed.
!-------------------------------------------------------------------------------


      if((stock==.false.).or.(TMtoPR==0)) then
        sfc = whc(xcoor,ycoor)
      else
! The grid sfcG contains the rooting depths calculated with the Guswa model
! for forested cells.
        sfc = sfcG(xcoor,ycoor)
      end if

      select case(land)
        case(1) ! SFC for water surfaces: 2500 mm
          sfc = max(sfc,2500.0)
          !sgrlp = 0.4*sfc


        case(2) ! SFC for built-up areas: 5 mm
          sfc = 5.0
          !sgrlp = cu*sfc

        case(13,14) ! SFC for glaciated areas: 0.1 mm
          sfc = 0.1
          !sgrlp = 0.0

        case(15) ! SFC for rocky areas: 3 mm
          sfc = 3.0
          !sgrlp = cu*sfc

        !case(3:12, 18:29)
        case(3:5, 8, 21, 23, 24, 25, 26)  ! Potentially forested vegetation types

          ! For vegetated surfaces: the effective soil depth is given by the
          ! minimum of soil depth and land cover type specific rooting depth.
          ! To account for capillary rise the effective soil depth is increased by 15 cm.

          ! Only done on cells where forest is not simulated with TreeMig (in those
          ! cells, rooting depth is calculated with Guswa's model)

          ! also includes the parameters AWAC and awacsoil, needed for the old stomatal
          ! closure routine ERSTOM01 (in non-forested cells)


        if((stock==.false.).or.(TMtoPR==0)) then
          sfc = sfc * whcSF(land)
          sfc = max(sfc, 10.)

          awacsoil = whc(xcoor,ycoor) * 0.25

        end if


        case(6, 7, 18, 19, 20, 22, 28, 29)
        ! Non-forested vegetation types (agriculture)

          if ((stdZroot(land,mon)+0.15) .LT. fbtief(xcoor,ycoor)) then
            sfc=(stdZroot(land,mon)+0.15)*fnfc(xcoor,ycoor)*10.0
          else
            sfc=fnfc(xcoor,ycoor)*fbtief(xcoor,ycoor)*10.0
          end if

          sfc=max(sfc, 10.0)
          sgrlp = cu*sfc

          if((stdZroot(land,mon)+0.05).lt.fbtief(xcoor,ycoor)) then
            awacsoil = fbtief(xcoor,ycoor) * 0.25 * 10.0     ! As in EINPUZDW.for
          else
            awacsoil = fnfc(xcoor,ycoor)*fbtief(xcoor,ycoor)*10.0
          end if

      end select
