!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE SZEROMAX (TAG,MONAT,JAHR,GEOBREITE,S0)
       !PROGRAM SZEROMAX
! ---------------------------------------------------------------------
!      Potential incoming global radiation, the maximal sunshine
!      duration, surise, sunset and make a local adjustemnt of global
!      radiation and air temperature (hourly and daily values)
!      Shadowing can be accounted
! ---------------------------------------------------------------------

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------

      REAL SLOPE,ASPECT,HHOEHE			!	slope (deg), aspect (deg) and elevation (m asl)of the HRU
      REAL GEOBREITE				!	latitude (deg)
      INTEGER TAG,MONAT,JAHR			!	day, month year
      INTEGER DEBUG,MDR			!	controls for partial output
      REAL S0					!	maximal sunshine duration
      REAL RGKDM				!	adjusted observed global radiation (daily average)
      REAL PI					!	3.1414......
      REAL DEKLINATION				!	Declination of the Aequator in Deg
      REAL JAHRESLAENGE				!	Length of the year (Days)
      REAL KF					!	Correction parameter
      REAL RM					!	Average distance earth-sun in millions of km
      REAL E					!	excentricity
      REAL I0					!	constant solar radiaiton
      REAL A					!	unit transformation from m/s to mm/d
      REAL D					!	day length between sunrise and sunset in hours
      INTEGER JULDATUM				!	Julian date
      INTEGER SCHALTJAHR			!	variables to identify leap years
      INTEGER NORMDATUM				!	julian date of the 21st July in the previous year
      REAL WINKEL				!	Angle of sun at 21st June
      REAL W0					!	Angle of sun at sunset
      REAL AKTDEKLIN				!	actual declination
      REAL R					!	actual radius of earth ellipse
      INTEGER PERIHELDATUM			!	??????
      REAL PERIHEL				!	PERIHEL
      Integer TSA				!	time of sunrise
      integer TSU				!	time of sunset
      REAL wsa					!	Angle at sunrise
      REAL wsu					!	angle at sunset
      real ws,stsa,stsu
      REAL zcos					!	Cosinus of zenith angle
      REAL zsin					!	Sinus of Zenith angle
      REAL as					!	Azimuth of sun
      REAL wslope
      integer i,ii,k,std			!	varaibles for loops
      real sumg,sumgk,sumd,sumdk,wink		!	Help values
      REAL KORFAK,RGKDS,RGKOR			!	parameters for adjusting global radiation
      REAL CT					!	minimal positive temperature adjustment in deg Celsius

      REAL TEDTI(24)				!	observed temperature - hourly values
      REAL TEDM					!	observed temperature - daily average
      REAL TEDS			    	!	observed temperature - daily sums
      REAL TEKDS					!	adjusted temperature - daily sums
      REAL SSDTI(24)				!	observed sunshine duration - hourly values [0..1]
      REAL SSDDS				!	observed sunshine duration - daily sum in hours
      REAL SHADTI(24)				!	Hourly shadowing in current HRU
      REAL SHADM				!	Average shadowing in current HRU between sunrise and sunset
      REAL RGDTI(24)				!	hourly observed incoming direct radiation
      REAL RGDM					!	daily average of observed global radiation

      REAL TEKDTI(24)				!	adjusted temperature - hourly values
      REAL TEKDM				!	adjusted temperature - daily average
      REAL RGKDTI(24)				!	adjusted hourly observed incoming direct radiation
      REAL RGMDTI(24)				!	hourly potential incoming radiation
      REAL RGMDM				!	daily average of potential incoming radiation
      REAL RGKMDTI(24)				!	adjusted hourly potential incoming radiation
      REAL RGKMDM				!	adjusted daily average of potential incoming radiation
      REAL RDMDTI(24)				!	hourly potential incoming direct radiation
      REAL RDMDM				!	daily average of potential incoming direct radiation
      REAL RDKMDTI(24)				!	adjusted hourly potential incoming direct radiation
      REAL RDKMDM				!	adjusted daily average of potential incoming direct radiation
      REAL SHADS				!	daily sum of shadowing at current HRU
      REAL RGDS					!	daily sum of observed incoming direct radiation
      REAL P0					!	Air pressure at sea level 1013 hPa

!	Help variables
      REAL RHG, KFAK				!	Help values
      REAL FAKTOR(24)				!	Help array for temperature adjustment
      REAL PHOEHE				!	Air pressure at curret elevation
      REAL RHG1,SHAD,HFAK			!	Help values

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------


! Declaration of constants
      PI=3.14159265359
      DEKLINATION=23.45
      JAHRESLAENGE=365.25
      KF=0.276666667
      RM=149.5
      E=0.01675
      I0=1367.0
      A=8.64E7
      D =24.
      P0=1013.0

      CALL EJULIANZ (JAHR,MONAT,TAG,JULDATUM)
      SCHALTJAHR=JAHR-MOD(JAHR,4)
      CALL EJULIANZ (SCHALTJAHR,6,21,NORMDATUM)
      WINKEL=360.*(JULDATUM-NORMDATUM)/JAHRESLAENGE+360.

    1 CONTINUE
      IF (WINKEL.LT.360.) GOTO 2
	WINKEL=WINKEL-360.
        GOTO 1
    2 CONTINUE

! ---------------------------------------------------------------------
!     Computation of the actual declination
! ---------------------------------------------------------------------
      AKTDEKLIN=DEKLINATION*COS(WINKEL*PI/180.)

! computation of the angle of sun above the horizon
      RHG=-TAN(AKTDEKLIN*PI/180.0)
      W0=180.*ACOS(-TAN(AKTDEKLIN*PI/180.)*TAN(GEOBREITE*PI/180.))/PI

! computaiton of the maximal sunshine duration
      S0=24.*W0/180.+KF

! Angle at 2nd of juli
      CALL EJULIANZ (JAHR,7,2,PERIHELDATUM)
      PERIHEL=360.*(JULDATUM-PERIHELDATUM)/JAHRESLAENGE+360.

! Distance between earth and sun
      R=RM*(1.-E*COS(PERIHEL*PI/180.))

! computation of sunrise and sunset
      wsu=w0
      stsu=(D/2.0*wsu/180.0)+12.0
      stsa=stsu-s0
      wsa=(stsa-12.0)*2/D*180.0
      tsa=int(stsa+0.25)
      tsu=int(stsu+0.75)

!      print*, S0,stsu,stsa

! ---------------------------------------------------------------------
      RETURN
      END

! ---------------------------------------------------------------------
