!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

! ---------------------------------------------------------------------
! PREVAH (Precipitation-Runoff-Evapotranspiration HRU model)
! Subroutine of Model core for WINDOWS, DOS, LINUX and SUN
! ---------------------------------------------------------------------

      SUBROUTINE TVERT ( DATUM,DT,EPDS,SSD,SSDMX,TSA,TSU,EPDT,EDSU,
     *                   DEBUG,MPRI )

C      SUBROUTINE TVERT ( DATUM,DT,EPDS,SSD,SSDMX,TSA,TSU,EPDT,
C     *                   DEBUG,MPRI )

!     SUBROUTINE for the disaggregation of the daily sums of evapotranspiration
!     into the single time steps (hours) of a day between the sunrise TSA and
!    the sunset tsu. The maximal sunshine duration SSDM is equal to TSU-TSA.
!    The disaggregation depends on the sunshine duration SSD, or on the relative
!    sunshine duration

!    SSDR=SSD/SSDMX
!    1h<=DT<24h DT only as full hour 60,120,180,240...min
!    We distinguish:       Cloudy day SSDR <  0.75
!                          Radiation day  SSDR >= 0.75
!
!    Programmed by J. Gurtz (ETHZ) on the basis of P.M. Fleming (1970)
! ---------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------
!     Common Variables

      INTEGER TSA,TSU                            ! time of sunrise, time of sunset
      INTEGER DEBUG,MPRI                         ! controls output during debugging
      REAL DT,                                   ! time step
     *     DATUM,                                ! date
     *     EPDS,                                 ! evapotransiopration (mm), daily sum
     *     SSD,                                  ! actual sunshine duration (h)
     *     SSDMX,                                ! maximal sunshine duration (h)
     *     EDSU,                                 ! evapotransiopration (mm), daily sum, control
     *     EPDT(24)                              ! evaptranspiration, hourly values

! ---------------------------------------------------------------------
!     Internal Variables
! --------------------------------------------------------------------
      INTEGER I,L,SSDM,ISTU,IDT,NDTD,TI
      REAL EVI,EVIS,AEVI,DTER,DHOUR,SSDR

! ---------------------------------------------------------------------
!     Initialization
! --------------------------------------------------------------------

      EDSU=0.0
      L=1
      ISTU=1
      TI=1
      SSDM=TSU-TSA
      DTER=0.


! relative sunshine duration [-] SSDR
      SSDR=SSD/SSDMX

! variables for the integration
      NDTD=INT((1440.0/DT)+0.5)
      IDT=INT((DT/60.0)+0.5)

! --------------------------------------------------------------------
2     EVIS=0.0
      IF (TI .LE. TSA) THEN
       EVI=0
       ISTU=0
       GOTO 6
      ENDIF
      IF (TI .GT. (TSU+IDT)) THEN
       EVI=0.0
       GOTO 6
      ENDIF
      DTER=EPDS/(float(SSDM))
4     DHOUR=ISTU/(float(SSDM))
      IF (SSDR .GE. 0.75) THEN
       AEVI=31.4*(DHOUR**2.0)-33.6*DHOUR+2.23
       EVI= 2.78-(7.73+AEVI)**0.5
       GOTO 6
      ENDIF
      IF (DHOUR .LE. 0.06) THEN
       EVI=0.0
      ELSE
       EVI=6.75*(DHOUR-0.06)
      ENDIF
      IF (DHOUR .GT. 0.26) EVI=1.35
      IF (DHOUR .GT. 0.80) EVI=1.35-6.75*(DHOUR-0.8)
6     CONTINUE
      IF (EVI .LE. 0.0) EVI=0.0
      EVIS=EVIS+EVI
      ISTU=ISTU+1
      IF (TI .GE. (TSA+ISTU)) GOTO 4
      EPDT(L)=EVIS*DTER
      EDSU=EDSU+EPDT(L)
      L=L+1
      IF (TI .LT. 24) THEN
       TI=TI+IDT
       GOTO 2
      ENDIF
8     CONTINUE

! ---------------------------------------------------------------------
!     Write local output if DEBUG > 0
! --------------------------------------------------------------------
10    FORMAT ( ' TVERT-ERG.: DATE     DT    EPDS    EDSU     SSD  SSDR',
     *         ' SSDM TSA TSU - EPDT',/,11X,6F7.2,3I4 )
12    FORMAT ( 24F5.2)
      IF (DEBUG .GT. 0) THEN
       WRITE (MPRI,10) DATUM,DT,EPDS,EDSU,SSD,SSDR,SSDM,TSA,TSU
       WRITE (MPRI,12) ( EPDT(I), I=1,NDTD )
      ENDIF

      RETURN

      END
