!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE VAPOUR (TAIR,RHUMIT,VAP)

!     This subroutine computes the water vapour pressure  (hPa) at given
!     air temperature TAIR (�C) and the relative humidity RHUMIT (%)
!     after LOWE (1977). The original equation provides VAP in Pa. Here VAP
!     is trans formed into hPa
! ---------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------
!     Common Variables

      REAL VAP,RHUMIT,TAIR

! ---------------------------------------------------------------------
!     Internal Variables
! ---------------------------------------------------------------------
      REAL  A0,A1,A2,A3,A4,A5,A6,T

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------

      A0=6.107799961
      A1=4.436518521E-1
      A2=1.428945805E-2
      A3=2.650648471E-4
      A4=3.031240396E-6
      A5=2.034080948E-8
      A6=6.136820929E-11
      T=TAIR
      VAP=RHUMIT*(A0+T*(A1+T*(A2+T*(A3+T*(A4+T*(A5+A6*T))))))

! ---------------------------------------------------------------------
!     Transformation from Pa to hPa
! ---------------------------------------------------------------------
      VAP=VAP*0.01

	RETURN

	END
