!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa, J. Schwanbeck et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

	SUBROUTINE xmloutput

! ------------------------------------------------------------------------------------
!	Writes RESULTS (hourly runoff m3/s) into *.XML-FILE (exchange-format with FLORIS)
!	--> in multiple run mode, all results will be written into the same file
!
!
!	# Jan Schwanbeck <jansc@gmx.net>
!	# Hydrology Group / Dept. of Geography, University of Berne
!
!	VER 0.1, Build 15.04.2008
!
! ------------------------------------------------------------------------------------
!   Common Variables - prevahglobals.for
! ------------------------------------------------------------------------------------
	  USE	prevahglobals

      implicit none
! ------------------------------------------------------------------------------------
!   Local Variables
! ------------------------------------------------------------------------------------
	  logical			lval
   	  integer           ji,jii
	  character*256		f1, f2, f3, f4, xmldir, xmlfile, workdir, dumpj
	  character*16		datentest(10), valchar, timestep
	  character*6	    qunit,punit,hunit,sunit
          character*12      qname,pname,hname,sname
          character*120     znisb
	  real				aeisb
	  integer lastline   ! Not declared before, caught by "implicit none"
! ------------------------------------------------------------------------------------
!   Begin Processing
! ------------------------------------------------------------------------------------
	  CALL messagerun (1,'XML-output will be written.')
!
!	 checking, if xml-output-path does already exist, if not it will be created
!

	  INQUIRE(FILE=xmlpath,EXIST=lval)
	  IF (.NOT.lval) THEN
		OPEN(MXML, FILE=TRIM(xmlpath),STATUS='unknown')
		WRITE(MXML,'(A)') TRIM('<?xml version="1.0" encoding="UTF-8"?>') ! header --> xml-conventions
		WRITE(MXML,'(A)') ''
                !runname = "To_be_defined" defined in *.inp
		write(MXML,'(A)') '<run name="'//TRIM(runname)//'">'		! header --> id of prevah-run (name of xml-file (=f3))
	  ELSE
		CALL CountLines(xmlpath,lastline)
		OPEN(MXML, FILE=TRIM(xmlpath),STATUS='OLD', POSITION='APPEND')
		BACKSPACE(MXML)
		READ(MXML,'(A)') dumpj
		IF (TRIM(dumpj).eq.'</run>') BACKSPACE(MXML)
	  ENDIF
!
!	 writing header
!
	  CALL xmlheader(gaugeid, zn, aelhg)
! ------------------------------------------------------------------------------------
!   End Processing
! ------------------------------------------------------------------------------------
      RETURN
      ENDSUBROUTINE
!*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
!
!*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

	SUBROUTINE xmlheader(idbasin, namebasin, aebasin)

! ------------------------------------------------------------------------------------
!   writes the header of the current xml-output (timeseries)
!
! ------------------------------------------------------------------------------------
!   Common Variables - prevahglobals.for
! ------------------------------------------------------------------------------------
	  USE	prevahglobals

	  implicit none
!	  integer			tistep
!	  logical			islake
! ------------------------------------------------------------------------------------
!   Local Variables
! ------------------------------------------------------------------------------------
	  character*6	    qunit,punit,hunit,sunit
          character*12      qname,pname,hname,sname
	  real				aebasin				! area of basin
	  character*32  	idbasin, namebasin	! identifier and name of basin
! ------------------------------------------------------------------------------------
!	Formats
! ------------------------------------------------------------------------------------
101   FORMAT(T2,'<station no="',A,'" name="',A,'" basin-area="',F6.1,' km2">')
102   FORMAT(T3,'<hydrograph timestep="',I2.2,' h" q-name="',A,'" q-unit="',A,'" p-name="',A,'" p-unit="',A,'" s-name="',A,'" s-unit="',A,'" h-name="',A,'" h-unit="',A,'">')
! ------------------------------------------------------------------------------------
!   Begin Processing
! ------------------------------------------------------------------------------------
          qname='runoff'
	  qunit='m3/s'
!          IF (AELHG .LT. 50.) qunit='l/s' !JS:2008-04 global character varaible for xml-output.for
	  IF (islake .EQ. .true.) qunit='m3/s'
          pname='precip'
	  punit='mm/h'
          sname='snow'
	  sunit='mm'
          hname='humidity'
	  hunit='mm/mm'

	  WRITE(MXML,101) TRIM(idbasin), TRIM(namebasin), aebasin
	  WRITE(MXML,102) tistep,TRIM(qname),TRIM(qunit),TRIM(pname),TRIM(punit),TRIM(sname),TRIM(sunit),TRIM(hname),TRIM(hunit)
! ------------------------------------------------------------------------------------
!   End Processing
! ------------------------------------------------------------------------------------
      RETURN
    ENDSUBROUTINE
!*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
!
!*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

	SUBROUTINE xmlwrite

! ------------------------------------------------------------------------------------
!   writes current xml-output (timeseries)
!
! ------------------------------------------------------------------------------------
!   Common Variables - prevahglobals.for
! ------------------------------------------------------------------------------------
	  USE	prevahglobals

	  implicit none
!	  real			aelhg
!  	  INTEGER       tag, mon, jahr, i
!	  real			rgehm(24)
! ------------------------------------------------------------------------------------
!	Formats
! ------------------------------------------------------------------------------------
!101	  FORMAT(T4,'<wert dat="',I2.2,'.',I2.2,'.',I4,X,I2.2,':00" val="',F12.4,'" />')
101	  FORMAT(T4,'<wert dat="',I2.2,'.',I2.2,'.',I4,X,I2.2,':00" q="',F12.4,'" p="',F12.4,'" s="',F12.4,'" h="',F12.4,'"/>')
! ------------------------------------------------------------------------------------
!   Begin Processing
! ------------------------------------------------------------------------------------
!	  WRITE (MXML,101) tag,mon,jahr,i,rgehm(i)*aelhg/(3.6)  !*1000 nur in m3/s
	  WRITE (MXML,101) tag,mon,jahr,i,rgehm(i)*aelhg/(3.6),pakhm(i),ssohm(i),(ssmhm(i)+suzhm(i))/ssmb
! ------------------------------------------------------------------------------------
!   End Processing
! ------------------------------------------------------------------------------------
      RETURN
	ENDSUBROUTINE
!*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
!
!*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

	SUBROUTINE xmlclose

! ------------------------------------------------------------------------------------
!   writes current xml-output (timeseries)
!
! ------------------------------------------------------------------------------------
!   Common Variables - prevahglobals.for
! ------------------------------------------------------------------------------------
	  USE		prevahglobals

	  implicit none
!	  real		aelhg
!     logical	lastmultirun
! ------------------------------------------------------------------------------------
!   Begin Processing
! ------------------------------------------------------------------------------------
	  write(MXML,'(T3,A)') TRIM('</hydrograph>')
	  write(MXML,'(T2,A)') TRIM('</station>')
	  IF (isbyn) CALL isb2xml
	  IF (docali .NE. 'M') write(MXML,'(A)') TRIM('</run>')
	  IF ((docali .EQ. 'M') .AND. (lastmultirun .EQ. .true.)) write(MXML,'(A)') TRIM('</run>')
	  CLOSE(MXML)
! ------------------------------------------------------------------------------------
!   End Processing
! ------------------------------------------------------------------------------------
      RETURN
      ENDSUBROUTINE
!*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
!
!*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

	SUBROUTINE isb2xml

! ------------------------------------------------------------------------------------
!   adds hydrographs of internal sub basins to xml-file

! ------------------------------------------------------------------------------------
!     Common Variables - prevahglobals.for
! ------------------------------------------------------------------------------------
	  USE		prevahglobals

	  implicit none
!	  integer	numisb, MXML,
!
! ------------------------------------------------------------------------------------
!	Local variables
! ------------------------------------------------------------------------------------
	  integer			ji, jii, numlines
	  character			dumpj(4)
	  character(LEN=32)	znisb
	  character(LEN=6)	qunit
	  real				aeisb
!
	  character(LEN=32), allocatable :: isbname(:)
	  real, allocatable :: isbdata(:,:)
!---------------------------------------------------------
!	Formats
!---------------------------------------------------------
101   FORMAT (T4,'<wert dat="',I2.2,'.',I2.2,'.',I4,X,I2.2,':00" q="',F12.4,'" p="',F12.4,'" s="',F12.4,'" h="',F12.4,'"/>')
!---------------------------------------------------------
!	Begin Processing
!---------------------------------------------------------
	  CALL CountLines(NAMISBQ,numlines)
      numlines = numlines-1				! -1 = header

	  ALLOCATE(isbname(numisb+8))
	  ALLOCATE(isbdata(numlines,numisb+8))

	  OPEN(MISBQ, FILE=NAMISBQ, STATUS='OLD')
	   READ(MISBQ,*) (dumpj(ji),ji=1,4), (isbname(ji),ji=1,numisb+4) !header
	   DO jii=1,numlines
	    READ(MISBQ,*) (isbdata(jii,ji),ji=1,numisb+8) !data
	   ENDDO
	  CLOSE(MISBQ)

	  DO ji = 1, numisb  ! last column (Qtotal) will not be processed, is already done
!	    writing header
		znisb =TRIM(ZN)//' ('//TRIM(isbname(ji))//')'
	    CALL xmlheader(isbname(ji),znisb,areaisb(ji))
		  DO jii = 1, numlines
		    WRITE (MXML,101) int(isbdata(jii,3)),int(isbdata(jii,2)),int(isbdata(jii,1)),int(isbdata(jii,4)),isbdata(jii,ji+4),isbdata(jii,numisb+6),isbdata(jii,numisb+7),isbdata(jii,numisb+8)  !*1000 nur in m3/s
		  ENDDO
		  write(MXML,'(T3,A)') TRIM('</hydrograph>')
		  write(MXML,'(T2,A)') TRIM('</station>')
	  ENDDO
	  DEALLOCATE(isbname)
	  DEALLOCATE(isbdata)
! ------------------------------------------------------------------------------------
!   End Processing
! ------------------------------------------------------------------------------------
      RETURN
      ENDSUBROUTINE
!*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
!
!*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

	  SUBROUTINE CountLines(filenamj,linj)

! ------------------------------------------------------------------------------------
!	  Counts number of line in file

! ------------------------------------------------------------------------------------
!	  Local Variables
! ------------------------------------------------------------------------------------
	  implicit none
	  character  filenamj*256, dumpj*1
	  integer    linj
! ------------------------------------------------------------------------------------
!   Begin Processing
! ------------------------------------------------------------------------------------
	  PRINT *,TRIM(filenamj)

	  OPEN(969,FILE=TRIM(filenamj), status='old')
       linj=0
9000   CONTINUE
	   READ(969,'(A1)',END=9100) dumpj
	   linj=linj+1
       GOTO 9000
9100   CONTINUE
      CLOSE (969)
! ------------------------------------------------------------------------------------
!   End Processing
! ------------------------------------------------------------------------------------
      RETURN
      ENDSUBROUTINE
!*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
